@@ -20,6 +20,8 @@
 #include "maininfo.h"
 #include "arcfg.h"
 #include "stl.h"
+#include "buffer.h"
+#include "core.h"
 
 #include <qpixmap.h>
 #include <qlistview.h>
@@ -174,7 +176,7 @@ void PluginItem::apply()
             m_widget = NULL;
         }
     }
-    Event e(EventApplyPlugin, m_info->name);
+    Event e(EventApplyPlugin, (char*)m_info->name.c_str());
     e.process();
 }
 
@@ -224,7 +226,7 @@ void ClientItem::init()
         setText(0, i18n(m_cmd->text));
     }
     if (m_cmd->icon)
-        setPixmap(0, Pict(m_cmd->icon));
+        setPixmap(0, Pict(m_cmd->icon, listView()->colorGroup().base()));
 }
 
 QWidget *ClientItem::getWidget(ConfigureDialog *dlg)
@@ -250,7 +252,7 @@ ARItem::ARItem(QListViewItem *item, const CommandDef *d)
 {
     m_status = d->id;
     setText(0, i18n(d->text));
-    setPixmap(0, Pict(d->icon));
+    setPixmap(0, Pict(d->icon, listView()->colorGroup().base()));
 }
 
 QWidget *ARItem::getWidget(ConfigureDialog *dlg)
@@ -271,7 +273,7 @@ MainInfoItem::MainInfoItem(QListView *view, unsigned id)
         : ConfigItem(view, id)
 {
     setText(0, i18n("User info"));
-    setPixmap(0, Pict("info"));
+    setPixmap(0, Pict("info", listView()->colorGroup().base()));
 }
 
 QWidget *MainInfoItem::getWidget(ConfigureDialog *dlg)
@@ -279,7 +281,7 @@ QWidget *MainInfoItem::getWidget(ConfigureDialog *dlg)
     return new MainInfo(dlg, NULL);
 }
 
-};
+}
 
 using namespace ConfigDlg;
 
@@ -291,9 +293,9 @@ ConfigureDialog::ConfigureDialog()
     setButtonsPict(this);
     setTitle();
     lstBox->header()->hide();
-    const QIconSet *iconSet = Icon("webpress");
-    if (iconSet)
-        btnUpdate->setIconSet(*iconSet);
+    QIconSet iconSet = Icon("webpress");
+    if (!iconSet.pixmap(QIconSet::Small, QIconSet::Normal).isNull())
+        btnUpdate->setIconSet(iconSet);
     btnUpdate->hide();
     lstBox->setHScrollBarMode(QScrollView::AlwaysOff);
     fill(0);
@@ -307,16 +309,17 @@ ConfigureDialog::ConfigureDialog()
 ConfigureDialog::~ConfigureDialog()
 {
     lstBox->clear();
-    for (unsigned n = 0;; n++){
+    for (unsigned long n = 0;; n++){
         Event e(EventPluginGetInfo, (void*)n);
         pluginInfo *info = (pluginInfo*)e.process();
         if (info == NULL) break;
         if (info->plugin == NULL) continue;
         if (info->bDisabled){
-            Event eUnload(EventUnloadPlugin, info->name);
+            Event eUnload(EventUnloadPlugin, (char*)info->name.c_str());
             eUnload.process();
         }
     }
+    saveGeometry(this, CorePlugin::m_plugin->data.cfgGeo);
 }
 
 static unsigned itemWidth(QListViewItem *item, QFontMetrics &fm)
@@ -350,7 +353,7 @@ void ConfigureDialog::fill(unsigned id)
         }
     }
 
-    unsigned n;
+    unsigned long n;
     parentItem = NULL;
     list<unsigned> st;
     for (n = 0; n < getContacts()->nClients(); n++){
@@ -363,7 +366,8 @@ void ConfigureDialog::fill(unsigned id)
             parentItem->setOpen(true);
         }
         for (const CommandDef *d = protocol->statusList(); d->text; d++){
-            if ((d->id == STATUS_ONLINE) || (d->id == STATUS_OFFLINE))
+            if (((protocol->description()->flags & PROTOCOL_AR_OFFLINE) == 0) &&
+                    ((d->id == STATUS_ONLINE) || (d->id == STATUS_OFFLINE)))
                 continue;
             list<unsigned>::iterator it;
             for (it = st.begin(); it != st.end(); ++it)
@@ -378,7 +382,7 @@ void ConfigureDialog::fill(unsigned id)
 
     parentItem = new ConfigItem(lstBox, 0);
     parentItem->setText(0, i18n("Plugins"));
-    parentItem->setPixmap(0, Pict("run"));
+    parentItem->setPixmap(0, Pict("run", lstBox->colorGroup().base()));
     parentItem->setOpen(true);
 
     for ( n = 0;; n++){
@@ -386,7 +390,7 @@ void ConfigureDialog::fill(unsigned id)
         pluginInfo *info = (pluginInfo*)e.process();
         if (info == NULL) break;
         if (info->info == NULL){
-            Event e(EventLoadPlugin, info->name);
+            Event e(EventLoadPlugin, (char*)info->name.c_str());
             e.process();
         }
         if ((info->info == NULL) || (info->info->title == NULL)) continue;
@@ -465,10 +469,19 @@ void ConfigureDialog::apply()
             continue;
         size_t size = 0;
         for (const DataDef *d = def; d->name; ++d)
-            size += sizeof(unsigned) * d->n_values;
+            size += sizeof(Data) * d->n_values;
         void *data = malloc(size);
         string cfg = client->getConfig();
-        load_data(def, data, cfg.c_str());
+        if (cfg.empty()){
+            load_data(def, data, NULL);
+        }else{
+            Buffer config;
+            config << "[Title]\n";
+            config.pack(cfg.c_str(), cfg.length());
+            config.setWritePos(0);
+            config.getSection();
+            load_data(def, data, &config);
+        }
         emit applyChanges(client, data);
         client->setClientInfo(data);
         free_data(def, data);
@@ -646,7 +659,7 @@ QListViewItem *ConfigureDialog::findItem(QWidget *w, QListViewItem *parent)
     return NULL;
 }
 
-#ifndef WIN32
+#ifndef _MSC_VER
 #include "cfgdlg.moc"
 #endif
 
