@@ -87,10 +87,10 @@ void CMenu::processItem(CommandDef *s, bool &bSeparator, bool &bFirst, unsigned
         m_wrk->insertSeparator();
         bSeparator = false;
     }
-    const QIconSet *icons = NULL;
+    QIconSet icons;
     if ((s->flags & COMMAND_CHECKED) && s->icon_on)
         icons = Icon(s->icon_on);
-    if ((icons == NULL) && s->icon)
+    if (icons.pixmap(QIconSet::Small, QIconSet::Normal).isNull() && s->icon)
         icons = Icon(s->icon);
     QString title = i18n(s->text);
     if (s->text_wrk){
@@ -102,8 +102,8 @@ void CMenu::processItem(CommandDef *s, bool &bSeparator, bool &bFirst, unsigned
         title += i18n(s->accel);
     }
     if (s->flags & COMMAND_TITLE){
-        if (icons){
-            m_wrk->insertTitle(icons->pixmap(QIconSet::Automatic, QIconSet::Normal), title);
+        if (!icons.pixmap(QIconSet::Small, QIconSet::Normal).isNull()){
+            m_wrk->insertTitle(icons.pixmap(QIconSet::Automatic, QIconSet::Normal), title);
         }else{
             m_wrk->insertTitle(title);
         }
@@ -122,8 +122,8 @@ void CMenu::processItem(CommandDef *s, bool &bSeparator, bool &bFirst, unsigned
     }
     unsigned id = 0;
     if (popup){
-        if (icons){
-            m_wrk->insertItem(*icons, title, popup);
+        if (!icons.pixmap(QIconSet::Small, QIconSet::Normal).isNull()){
+            m_wrk->insertItem(icons, title, popup);
         }else{
             m_wrk->insertItem(title, popup);
         }
@@ -133,8 +133,8 @@ void CMenu::processItem(CommandDef *s, bool &bSeparator, bool &bFirst, unsigned
         c.base_id = base_id;
         m_cmds.push_back(c);
         id = m_cmds.size();
-        if (icons){
-            m_wrk->insertItem(*icons, title, id);
+        if (!icons.pixmap(QIconSet::Small, QIconSet::Normal).isNull()){
+            m_wrk->insertItem(icons, title, id);
         }else{
             m_wrk->insertItem(title, id);
         }
@@ -236,7 +236,7 @@ void CMenu::menuActivated(int n)
     }
 }
 
-#ifndef _WINDOWS
+#ifndef _MSC_VER
 #include "cmenu.moc"
 #endif
 
