@@ -19,6 +19,7 @@
 #include "userwnd.h"
 #include "core.h"
 #include "toolbtn.h"
+#include "buffer.h"
 
 #include <qmainwindow.h>
 #include <qframe.h>
@@ -111,32 +112,21 @@ Container::Container(unsigned id, const char *cfg)
     m_bReceived = false;
     m_bNoSwitch = false;
     m_bNoRead   = false;
-#ifdef WIN32
-    m_bInitWnd	= false;
-#endif
+    m_wnds		= NULL;
+    m_tabBar	= NULL;
 
     SET_WNDPROC("container")
     setWFlags(WDestructiveClose);
-    QFrame *frm = new QFrame(this, "container");
-    setCentralWidget(frm);
 
-    connect(CorePlugin::m_plugin, SIGNAL(modeChanged()), this, SLOT(modeChanged()));
-
-    QVBoxLayout *lay = new QVBoxLayout(frm);
-    m_wnds = new QWidgetStack(frm);
-    m_wnds->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding));
-    lay->addWidget(m_wnds);
-
-    m_tabSplitter = new Splitter(frm);
-    m_tabSplitter->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum));
-    m_tabBar = new UserTabBar(m_tabSplitter);
-    m_tabBar->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Expanding));
-    m_tabBar->hide();
-
-    m_status = new ContainerStatus(m_tabSplitter);
-    lay->addWidget(m_tabSplitter);
-
-    load_data(containerData, &data, cfg);
+    if (cfg && *cfg){
+        Buffer config;
+        config << "[Title]\n" << cfg;
+        config.setWritePos(0);
+        config.getSection();
+        load_data(containerData, &data, &config);
+    }else{
+        load_data(containerData, &data, NULL);
+    }
 
     bool bPos = true;
     if (cfg == NULL){
@@ -191,30 +181,85 @@ Container::Container(unsigned id, const char *cfg)
             delete list;
         }
         setStatusSize(CorePlugin::m_plugin->getContainerStatusSize());
-        showBar();
-        m_bInit = true;
     }
     m_bInSize = true;
     restoreGeometry(this, data.geometry, bPos, true);
     m_bInSize = false;
-    connect(m_tabBar, SIGNAL(selected(int)), this, SLOT(contactSelected(int)));
-    connect(this, SIGNAL(toolBarPositionChanged(QToolBar*)), this, SLOT(toolbarChanged(QToolBar*)));
-    connect(m_status, SIGNAL(sizeChanged(int)), this, SLOT(statusChanged(int)));
-
-    m_accel = new QAccel(this);
-    connect(m_accel, SIGNAL(activated(int)), this, SLOT(accelActivated(int)));
-    setupAccel();
 }
 
 Container::~Container()
 {
     list<UserWnd*> wnds = m_tabBar->windows();
-    for (list<UserWnd*>::iterator it = wnds.begin(); it != wnds.end(); ++it){
+    list<UserWnd*>::iterator it;
+    for (it = wnds.begin(); it != wnds.end(); ++it)
         disconnect(*it, SIGNAL(closed(UserWnd*)), this, SLOT(removeUserWnd(UserWnd*)));
-    }
+    for (it = m_childs.begin(); it != m_childs.end(); ++it)
+        delete (*it);
     free_data(containerData, &data);
 }
 
+void Container::init()
+{
+    if (m_bInit)
+        return;
+
+    QFrame *frm = new QFrame(this, "container");
+    setCentralWidget(frm);
+
+    connect(CorePlugin::m_plugin, SIGNAL(modeChanged()), this, SLOT(modeChanged()));
+
+    QVBoxLayout *lay = new QVBoxLayout(frm);
+    m_wnds = new QWidgetStack(frm);
+    m_wnds->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding));
+    lay->addWidget(m_wnds);
+
+    m_tabSplitter = new Splitter(frm);
+    m_tabSplitter->setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum));
+    m_tabBar = new UserTabBar(m_tabSplitter);
+    m_tabBar->setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Expanding));
+    m_tabBar->hide();
+
+    m_bInit = true;
+
+    m_status = new ContainerStatus(m_tabSplitter);
+    lay->addWidget(m_tabSplitter);
+    connect(m_tabBar, SIGNAL(selected(int)), this, SLOT(contactSelected(int)));
+    connect(this, SIGNAL(toolBarPositionChanged(QToolBar*)), this, SLOT(toolbarChanged(QToolBar*)));
+    connect(m_status, SIGNAL(sizeChanged(int)), this, SLOT(statusChanged(int)));
+    m_accel = new QAccel(this);
+    connect(m_accel, SIGNAL(activated(int)), this, SLOT(accelActivated(int)));
+    setupAccel();
+    showBar();
+
+    for (list<UserWnd*>::iterator it = m_childs.begin(); it != m_childs.end(); ++it)
+        addUserWnd((*it), false);
+    m_childs.clear();
+
+    string windows = getWindows();
+    while (!windows.empty()){
+        unsigned long id = strtoul(getToken(windows, ',').c_str(), NULL, 10);
+        Contact *contact = getContacts()->contact(id);
+        if (contact == NULL)
+            continue;
+        Buffer config;
+        const char *cfg = getWndConfig(id);
+        if (cfg && *cfg){
+            config << "[Title]\n" << cfg;
+            config.setWritePos(0);
+            config.getSection();
+        }
+        addUserWnd(new UserWnd(id, &config, false, true), true);
+    }
+
+    if (m_tabBar->count() == 0)
+        QTimer::singleShot(0, this, SLOT(close()));
+    setWindows(NULL);
+    clearWndConfig();
+    m_tabBar->raiseTab(getActiveWindow());
+
+    show();
+}
+
 void Container::setupAccel()
 {
     m_accel->clear();
@@ -258,6 +303,8 @@ string Container::getState()
 {
     clearWndConfig();
     string windows;
+    if (m_tabBar == NULL)
+        return save_data(containerData, &data);
     list<UserWnd*> userWnds = m_tabBar->windows();
     for (list<UserWnd*>::iterator it = userWnds.begin(); it != userWnds.end(); ++it){
         if (!windows.empty())
@@ -286,6 +333,14 @@ QString Container::name()
 
 void Container::addUserWnd(UserWnd *wnd, bool bRaise)
 {
+    if (m_wnds == NULL){
+        m_childs.push_back(wnd);
+        if (m_childs.size() == 1){
+            setIcon(Pict(wnd->getIcon()));
+            setCaption(wnd->getLongName());
+        }
+        return;
+    }
     connect(wnd, SIGNAL(closed(UserWnd*)), this, SLOT(removeUserWnd(UserWnd*)));
     connect(wnd, SIGNAL(statusChanged(UserWnd*)), this, SLOT(statusChanged(UserWnd*)));
     m_wnds->addWidget(wnd, -1);
@@ -320,6 +375,8 @@ void Container::addUserWnd(UserWnd *wnd, bool bRaise)
 
 void Container::raiseUserWnd(UserWnd *wnd)
 {
+    if (m_tabBar == NULL)
+        return;
     m_tabBar->raiseTab(wnd->id());
     contactSelected(0);
 }
@@ -339,11 +396,23 @@ void Container::removeUserWnd(UserWnd *wnd)
 
 UserWnd *Container::wnd(unsigned id)
 {
+    if (m_tabBar == NULL){
+        for (list<UserWnd*>::iterator it = m_childs.begin(); it != m_childs.end(); ++it){
+            if ((*it)->id() == id)
+                return (*it);
+        }
+        return NULL;
+    }
     return m_tabBar->wnd(id);
 }
 
 UserWnd *Container::wnd()
 {
+    if (m_tabBar == NULL){
+        if (m_childs.empty())
+            return NULL;
+        return m_childs.front();
+    }
     return m_tabBar->currentWnd();
 }
 
@@ -361,30 +430,6 @@ void Container::showBar()
     contactSelected(0);
 }
 
-void Container::init()
-{
-    if (m_bInit)
-        return;
-    m_bInit = true;
-
-    showBar();
-    string windows = getWindows();
-    while (!windows.empty()){
-        unsigned long id = strtoul(getToken(windows, ',').c_str(), NULL, 10);
-        Contact *contact = getContacts()->contact(id);
-        if (contact == NULL)
-            continue;
-        addUserWnd(new UserWnd(id, getWndConfig(id), false, true), true);
-    }
-    if (m_tabBar->count() == 0)
-        QTimer::singleShot(0, this, SLOT(close()));
-    setWindows(NULL);
-    clearWndConfig();
-    m_tabBar->raiseTab(getActiveWindow());
-
-    show();
-}
-
 void Container::contactSelected(int)
 {
     UserWnd *userWnd = m_tabBar->currentWnd();
@@ -523,6 +568,7 @@ static const char *accels[] =
 
 extern bool bFullScreen;
 
+#ifndef FLASHW_TRAY
 typedef struct FLASHWINFO
 {
     unsigned long cbSize;
@@ -532,12 +578,14 @@ typedef struct FLASHWINFO
     unsigned long dwTimeout;
 } FLASHWINFO;
 
-#ifndef FLASHW_TRAY
+
 #define FLASHW_TRAY         0x00000002
 #define FLASHW_TIMERNOFG    0x0000000C
-#endif
+
 
 static BOOL (WINAPI *FlashWindowEx)(FLASHWINFO*) = NULL;
+#endif
+static BOOL (WINAPI *fwe)(FLASHWINFO*) = NULL;
 static bool initFlash = false;
 #endif
 
@@ -552,17 +600,17 @@ void Container::flash()
     if (!initFlash){
         HINSTANCE hLib = GetModuleHandleA("user32");
         if (hLib != NULL)
-            (DWORD&)FlashWindowEx = (DWORD)GetProcAddress(hLib,"FlashWindowEx");
+			(DWORD&)fwe = (DWORD)GetProcAddress(hLib,"FlashWindowEx");
         initFlash = true;
     }
-    if (FlashWindowEx){
+    if (fwe){
         FLASHWINFO fInfo;
         fInfo.cbSize  = sizeof(fInfo);
         fInfo.dwFlags = FLASHW_TRAY | FLASHW_TIMERNOFG;
         fInfo.hwnd = winId();
         fInfo.uCount = 0;
         fInfo.dwTimeout = 1000;
-        FlashWindowEx(&fInfo);
+        fwe(&fInfo);
     }
 #else
 #if defined(USE_KDE)
@@ -575,6 +623,8 @@ void Container::flash()
 
 void *Container::processEvent(Event *e)
 {
+    if (m_tabBar == NULL)
+        return NULL;
     UserWnd *userWnd;
     Contact *contact;
     CommandDef *cmd;
@@ -680,7 +730,7 @@ void *Container::processEvent(Event *e)
     case EventCommandExec:
         cmd = (CommandDef*)(e->param());
         userWnd = m_tabBar->currentWnd();
-        if (userWnd && ((unsigned)(cmd->param) == userWnd->id())){
+        if (userWnd && ((unsigned long)(cmd->param) == userWnd->id())){
             if (cmd->menu_id == MenuContainerContact){
                 m_tabBar->raiseTab(cmd->id);
                 return e->param();
@@ -702,7 +752,7 @@ void *Container::processEvent(Event *e)
     case EventCheckState:
         cmd = (CommandDef*)(e->param());
         userWnd = m_tabBar->currentWnd();
-        if (userWnd && ((unsigned)(cmd->param) == userWnd->id()) &&
+        if (userWnd && ((unsigned long)(cmd->param) == userWnd->id()) &&
                 (cmd->menu_id == MenuContainerContact) &&
                 (cmd->id == CmdContainerContacts)){
             list<UserWnd*> userWnds = m_tabBar->windows();
@@ -756,18 +806,19 @@ void Container::wndClosed()
 bool Container::event(QEvent *e)
 {
 #ifdef WIN32
-    if ((e->type() == QEvent::WindowActivate) && !m_bInitWnd){
-        m_bInitWnd = true;
-        m_bNoRead  = false;
-        show();
-    }
+    if (e->type() == QEvent::WindowActivate)
+        init();
 #endif
     if ((e->type() == QEvent::WindowActivate) ||
-            (((e->type() == QEvent::ShowNormal) || (e->type() == QEvent::ShowMaximized)) &&
-             isActiveWindow())){
+            (((e->type() == QEvent::ShowNormal) ||
+              (e->type() == QEvent::ShowMaximized)) && isActiveWindow())){
         UserWnd *userWnd = m_tabBar->currentWnd();
-        if (userWnd && !m_bNoRead)
+        if (m_bNoRead){
+            m_bNoRead = false;
+        }
+        if (userWnd){
             userWnd->markAsRead();
+        }
         if (m_bNoSwitch){
             m_bNoSwitch = false;
         }else{
@@ -787,8 +838,13 @@ bool Container::event(QEvent *e)
 
 void Container::contactChanged(Contact *contact)
 {
-    UserWnd *userWnd = m_tabBar->currentWnd();
-    if (userWnd && (contact->id() == userWnd->id())){
+    UserWnd *userWnd = NULL;
+    if (m_tabBar){
+        userWnd = m_tabBar->currentWnd();
+    }else if (!m_childs.empty()){
+        userWnd = m_childs.front();
+    }
+    if (userWnd && contact && (contact->id() == userWnd->id())){
         QString name = userWnd->getName();
         Command cmd;
         cmd->id = CmdContainerContact;
@@ -804,6 +860,12 @@ void Container::contactChanged(Contact *contact)
     }
 }
 
+void Container::setReadMode()
+{
+    log(L_DEBUG, "Set read mode");
+    m_bNoRead = false;
+}
+
 UserTabBar::UserTabBar(QWidget *parent) : QTabBar(parent)
 {
     setShape(QTabBar::TriangularBelow);
@@ -1002,7 +1064,7 @@ bool UserTab::setBold(bool bBold)
     return true;
 }
 
-#ifndef WIN32
+#ifndef _MSC_VER
 #include "container.moc"
 #endif
 
