@@ -17,7 +17,7 @@
 
 #include "simapi.h"
 
-#if (COMPAT_QT_VERSION < 0x030000) || ((COMPAT_QT_VERSION >= 0x030000) && !(defined(HAVE_QSYNTAXHIGHLIGHTER_H)))
+#if QT_VERSION < 0x030000
 #include "qt3/qsyntaxhighlighter.h"
 #else
 #include <qsyntaxhighlighter.h>
@@ -186,13 +186,11 @@ HistoryConfig::HistoryConfig(QWidget *parent)
 {
     chkOwn->setChecked(CorePlugin::m_plugin->getOwnColors());
     chkSmile->setChecked(CorePlugin::m_plugin->getUseSmiles());
-    cmbPage->setEditable(true);
-    cmbPage->insertItem("100");
-    cmbPage->insertItem("50");
-    cmbPage->insertItem("25");
     m_cur = -1;
+    cmbPage->setEditable(true);
+    m_bDirty = false;
     QLineEdit *edit = cmbPage->lineEdit();
-    edit->setValidator(new QIntValidator(1, 500, edit));
+    edit->setValidator(new QIntValidator(1, 10000, edit));
     edit->setText(QString::number(CorePlugin::m_plugin->getHistoryPage()));
     QString str1 = i18n("Show %1 messages per page");
     QString str2;
@@ -205,7 +203,7 @@ HistoryConfig::HistoryConfig(QWidget *parent)
     lblPage2->setText(str2);
     edtStyle->setWordWrap(QTextEdit::NoWrap);
     edtStyle->setTextFormat(QTextEdit::RichText);
-    new XmlHighlighter(edtStyle);
+    highlighter = new XmlHighlighter(edtStyle);
     QStringList styles;
     addStyles(user_file(STYLES).c_str(), true);
 #ifdef USE_KDE
@@ -241,6 +239,7 @@ HistoryConfig::HistoryConfig(QWidget *parent)
 
 HistoryConfig::~HistoryConfig()
 {
+    delete highlighter;
 }
 
 static char BACKUP_SUFFIX[] = "~";
@@ -315,8 +314,8 @@ void HistoryConfig::apply()
     if (bChanged){
         Event e(EventHistoryConfig);
         e.process();
-        fillPreview();
     }
+    fillPreview();
     HistoryUserData *data = (HistoryUserData*)(getContacts()->getUserData(CorePlugin::m_plugin->history_data_id));
     data->CutDays.bValue = chkDays->isChecked();
     data->CutSize.bValue = chkSize->isChecked();
@@ -641,7 +640,7 @@ void HistoryConfig::fillPreview()
         xsl->setXSL(m_styles[cur].text);
     Contact *contact = getContacts()->contact(0, true);
     contact->setName("Buddy");
-    contact->setTemporary(true);
+    contact->setFlags(CONTACT_TEMP);
     edtPreview->clear();
     edtPreview->setXSL(xsl);
     time_t now;
@@ -706,7 +705,7 @@ void HistoryConfig::toggledSize(bool bState)
     edtSize->setEnabled(bState);
 }
 
-#ifndef WIN32
+#ifndef _MSC_VER
 #include "historycfg.moc"
 #endif
 
