@@ -29,6 +29,16 @@
 #include <qprogressbar.h>
 #include <qlayout.h>
 #include <qstringlist.h>
+#include <qmessagebox.h>
+
+#ifdef USE_KDE
+#include <kfiledialog.h>
+#define QFileDialog KFileDialog
+#else
+#include <qfiledialog.h>
+#endif
+
+#include <time.h>
 
 class HistoryProgressBar : public QWidget
 {
@@ -62,8 +72,10 @@ void HistoryProgressBar::setProgress(unsigned n)
     m_bar->setProgress(n);
 }
 
-HistoryWindow::HistoryWindow(unsigned id)
+HistoryWindow::HistoryWindow(unsigned long id)
 {
+    m_history_page_count=CorePlugin::m_plugin->getHistoryPage();
+
     setWFlags(WDestructiveClose);
     m_id = id;
     SET_WNDPROC("history")
@@ -131,7 +143,7 @@ void *HistoryWindow::processEvent(Event *e)
     }
     if (e->type() == EventCheckState){
         CommandDef *cmd = (CommandDef*)(e->param());
-        if ((cmd->id == CmdHistoryDirection) && ((unsigned)(cmd->param) == m_id)){
+        if ((cmd->id == CmdHistoryDirection) && ((unsigned long)(cmd->param) == m_id)){
             cmd->flags &= ~COMMAND_CHECKED;
             if (m_bDirection)
                 cmd->flags |= COMMAND_CHECKED;
@@ -146,7 +158,7 @@ void *HistoryWindow::processEvent(Event *e)
     }
     if (e->type() == EventCommandExec){
         CommandDef *cmd = (CommandDef*)(e->param());
-        if ((unsigned)(cmd->param) != m_id)
+        if ((unsigned long)(cmd->param) != m_id)
             return NULL;
         if (cmd->id == CmdHistoryDirection){
             bool bDirection = ((cmd->flags & COMMAND_CHECKED) != 0);
@@ -173,6 +185,35 @@ void *HistoryWindow::processEvent(Event *e)
             }
             return e->param();
         }
+        if (cmd->id == CmdHistorySave){
+            QString str = QFileDialog::getSaveFileName(QString::null, i18n("Textfile (*.txt)"), this);
+            if (str && !str.isEmpty()){
+                bool res = true;
+                if (QFile::exists(str)){
+                    QMessageBox mb(i18n("Error"), i18n("File already exists. Overwrite?"), 
+                            QMessageBox::Warning,
+                            QMessageBox::Yes | QMessageBox::Default,
+                            QMessageBox::No,
+                            QMessageBox::Cancel | QMessageBox::Escape);
+                    mb.setButtonText(QMessageBox::Yes, i18n("&Overwrite"));
+                    mb.setButtonText(QMessageBox::No, i18n("&Append"));
+                    switch (mb.exec()){
+                    case QMessageBox::Yes:
+                        res = History::save(m_id, str, false);
+                        break;
+                    case QMessageBox::No:
+                        res = History::save(m_id, str, true);
+                        break;
+                    case QMessageBox::Cancel:
+                        break;
+                    }
+                }else
+                    res = History::save(m_id, str);
+                if (!res)
+                    QMessageBox::critical(this, i18n("Error"), i18n("Save failed"), QMessageBox::Ok, QMessageBox::NoButton, QMessageBox::NoButton);
+            }
+            return e->param();
+        }
         if (cmd->id == CmdHistoryFind){
             m_filter = "";
             if (cmd->flags & COMMAND_CHECKED){
@@ -213,6 +254,7 @@ void HistoryWindow::toolbarChanged(QToolBar*)
 
 void HistoryWindow::fill()
 {
+    log(L_DEBUG, "Fill");
     if (m_it == NULL)
         m_it = new HistoryIterator(m_id);
     if (m_progress == NULL){
@@ -220,7 +262,7 @@ void HistoryWindow::fill()
         m_status->addWidget(m_progress, 1);
     }
     m_it->setFilter(m_filter);
-    m_progress->setTotalSteps(CorePlugin::m_plugin->getHistoryPage());
+    m_progress->setTotalSteps(m_history_page_count);
     m_progress->setProgress(0);
     m_progress->show();
     m_nMessages = 0;
@@ -250,37 +292,44 @@ void HistoryWindow::fill()
 
 void HistoryWindow::next()
 {
-    if (m_it == NULL)
+    if ( (m_it == NULL) )
         return;
-    string state = m_it->state();
-    Message *msg = NULL;
-    if (m_bDirection){
-        msg = --(*m_it);
-    }else{
-        msg = ++(*m_it);
-    }
-    if (++m_nMessages > CorePlugin::m_plugin->getHistoryPage()){
-        if (msg){
-            Command cmd;
-            cmd->id		= CmdHistoryNext;
-            cmd->flags  = 0;
-            cmd->param	= (void*)m_id;
-            Event eNext(EventCommandDisabled, cmd);
-            eNext.process();
-            msg = NULL;
-            m_states.push_back(state);
+
+    for (;;){
+        string state = m_it->state();
+        Message *msg = NULL;
+        if (m_bDirection){
+            msg = --(*m_it);
+        }else{
+            msg = ++(*m_it);
         }
-    }
-    if (msg){
-        m_view->addMessage(msg);
+
+        if (++m_nMessages > m_history_page_count){
+            if (msg){
+                Command cmd;
+                cmd->id		= CmdHistoryNext;
+                cmd->flags  = 0;
+                cmd->param	= (void*)m_id;
+                Event eNext(EventCommandDisabled, cmd);
+                eNext.process();
+                msg = NULL;
+                if (m_page+1>=m_states.size())
+                   m_states.push_back(state);
+            }
+        }
+
+        if (msg == NULL)
+            break;
+
+        m_view->addMessage(msg, false, false);
         m_progress->setProgress(m_nMessages);
-        QTimer::singleShot(0, this, SLOT(next()));
-        return;
     }
+
     delete m_progress;
     delete m_it;
     m_it = NULL;
     m_progress = NULL;
+    log(L_DEBUG, "Stop");
 }
 
 const unsigned MAX_HISTORY = 10;
@@ -312,7 +361,6 @@ void HistoryWindow::addHistory(const QString &str)
     CorePlugin::m_plugin->setHistorySearch(res);
 }
 
-#ifndef WIN32
+#ifndef _MSC_VER
 #include "historywnd.moc"
 #endif
-
