@@ -241,6 +241,14 @@ void LoginDialog::profileChanged(int)
         buttonOk->setEnabled(false);
         pswdChanged("");
     }
+    QTimer::singleShot(0, this, SLOT(adjust()));
+}
+
+void LoginDialog::adjust()
+{
+    int h = minimumSizeHint().height();
+    resize(width(), h);
+    move(x(), (qApp->desktop()->height() - h) / 2);
 }
 
 static void rmDir(const QString &path)
@@ -461,8 +469,10 @@ void *LoginDialog::processEvent(Event *e)
             QString msg;
             if (d->err_str && *d->err_str){
                 msg = i18n(d->err_str);
-                if (d->args)
+                if (d->args){
                     msg = msg.arg(QString::fromUtf8(d->args));
+                    free(d->args);
+                }
             }else{
                 msg = i18n("Login failed");
             }
@@ -477,7 +487,7 @@ void *LoginDialog::processEvent(Event *e)
     return NULL;
 }
 
-#ifndef WIN32
+#ifndef _MSC_VER
 #include "logindlg.moc"
 #endif
 
