@@ -16,6 +16,7 @@
  ***************************************************************************/
 
 #include "msgedit.h"
+#include "icons.h"
 #include "textshow.h"
 #include "toolbtn.h"
 #include "core.h"
@@ -31,6 +32,7 @@
 #include "userlist.h"
 #include "history.h"
 #include "container.h"
+#include "buffer.h"
 
 #include <qfontmetrics.h>
 #include <qtoolbar.h>
@@ -44,6 +46,7 @@
 #include <qtooltip.h>
 
 #include <time.h>
+#include <stdio.h>
 
 #ifdef USE_KDE
 #include "kdeisversion.h"
@@ -414,7 +417,7 @@ void MsgEdit::setInput()
     }
 }
 
-static Message *createGeneric(const char *cfg)
+static Message *createGeneric(Buffer *cfg)
 {
     return new Message(MessageGeneric, cfg);
 }
@@ -440,7 +443,7 @@ static MessageDef defGeneric =
         NULL
     };
 
-static Message *createSMS(const char *cfg)
+static Message *createSMS(Buffer *cfg)
 {
     return new SMSMessage(cfg);
 }
@@ -470,7 +473,7 @@ static MessageDef defSMS =
 i18n("URL", "%n URLs", 1);
 #endif
 
-static Message *createUrl(const char *cfg)
+static Message *createUrl(Buffer *cfg)
 {
     return new UrlMessage(MessageUrl, cfg);
 }
@@ -507,7 +510,7 @@ static MessageDef defUrl =
         dropUrl
     };
 
-static Message *createContacts(const char *cfg)
+static Message *createContacts(Buffer *cfg)
 {
     return new ContactsMessage(MessageContacts, cfg);
 }
@@ -545,7 +548,7 @@ static MessageDef defContacts =
         dropContacts
     };
 
-static Message *createFile(const char *cfg)
+static Message *createFile(Buffer *cfg)
 {
     return new FileMessage(MessageFile, cfg);
 }
@@ -641,7 +644,7 @@ static MessageDef defFile =
         dropFile
     };
 
-static Message *createAuthRequest(const char *cfg)
+static Message *createAuthRequest(Buffer *cfg)
 {
     return new AuthMessage(MessageAuthRequest, cfg);
 }
@@ -716,7 +719,7 @@ static MessageDef defAuthRequest =
         NULL
     };
 
-static Message *createAuthGranted(const char *cfg)
+static Message *createAuthGranted(Buffer *cfg)
 {
     return new AuthMessage(MessageAuthGranted, cfg);
 }
@@ -737,7 +740,7 @@ static MessageDef defAuthGranted =
         NULL
     };
 
-static Message *createAuthRefused(const char *cfg)
+static Message *createAuthRefused(Buffer *cfg)
 {
     return new AuthMessage(MessageAuthRefused, cfg);
 }
@@ -758,7 +761,7 @@ static MessageDef defAuthRefused =
         NULL
     };
 
-static Message *createAdded(const char *cfg)
+static Message *createAdded(Buffer *cfg)
 {
     return new AuthMessage(MessageAdded, cfg);
 }
@@ -779,7 +782,7 @@ static MessageDef defAdded =
         NULL
     };
 
-static Message *createRemoved(const char *cfg)
+static Message *createRemoved(Buffer *cfg)
 {
     return new AuthMessage(MessageRemoved, cfg);
 }
@@ -800,7 +803,7 @@ static MessageDef defRemoved =
         NULL
     };
 
-static Message *createStatus(const char *cfg)
+static Message *createStatus(Buffer *cfg)
 {
     return new StatusMessage(cfg);
 }
@@ -1200,7 +1203,7 @@ void *MsgEdit::processEvent(Event *e)
                 SmilePopup *popup = new SmilePopup(this);
                 QSize s = popup->minimumSizeHint();
                 popup->resize(s);
-                connect(popup, SIGNAL(insert(int)), this, SLOT(insertSmile(int)));
+                connect(popup, SIGNAL(insert(const char*)), this, SLOT(insertSmile(const char*)));
                 QPoint p = CToolButton::popupPos(btnSmile, popup);
                 popup->move(p);
                 popup->show();
@@ -1304,7 +1307,11 @@ void *MsgEdit::processEvent(Event *e)
                     if (def){
                         MessageDef *mdef = (MessageDef*)(def->param);
                         string cfg = m_msg->save();
-                        m_msg = (mdef->create)(cfg.c_str());
+                        Buffer config;
+                        config << "[Title]\n" << cfg.c_str();
+                        config.setWritePos(0);
+                        config.getSection();
+                        m_msg = (mdef->create)(&config);
                         m_msg->setContact(*multiply_it);
                         m_msg->setClient(NULL);
                         m_msg->setFlags(m_msg->getFlags() | MESSAGE_MULTIPLY);
@@ -1454,15 +1461,15 @@ void MsgEdit::colorsChanged()
     e.process();
 }
 
-void MsgEdit::insertSmile(int id)
+void MsgEdit::insertSmile(const char *id)
 {
     if (m_edit->textFormat() == QTextEdit::PlainText){
-        const smile *s = smiles(id);
-        if (s)
-            m_edit->insert(s->paste, false, true, true);
+        list<string> smiles = getIcons()->getSmile(id);
+        if (!smiles.empty())
+            m_edit->insert(QString::fromUtf8(smiles.front().c_str()), false, true, true);
         return;
     }
-    QString img_src = QString("<img src=icon:smile%1>").arg(QString::number(id, 16).upper());
+    QString img_src = QString("<img src=icon:%1>").arg(id);
     int para;
     int index;
     QFont saveFont = m_edit->font();
@@ -1546,29 +1553,34 @@ void MsgEdit::editEnterPressed()
     e.process();
 }
 
-SmileLabel::SmileLabel(int _id, const char *tip, QWidget *parent)
+SmileLabel::SmileLabel(const char *_id, QWidget *parent)
         : QLabel(parent)
 {
     id = _id;
-    char b[20];
-    sprintf(b, "smile%X", id);
-    const QIconSet *icon = Icon(b);
+    QIconSet icon = Icon(_id);
     QPixmap pict;
-    if (icon){
-        if (!icon->isGenerated(QIconSet::Large, QIconSet::Normal)){
-            pict = icon->pixmap(QIconSet::Large, QIconSet::Normal);
+    if (!icon.pixmap(QIconSet::Small, QIconSet::Normal).isNull()){
+        if (!icon.isGenerated(QIconSet::Large, QIconSet::Normal)){
+            pict = icon.pixmap(QIconSet::Large, QIconSet::Normal);
         }else{
-            pict = icon->pixmap(QIconSet::Small, QIconSet::Normal);
+            pict = icon.pixmap(QIconSet::Small, QIconSet::Normal);
         }
     }
     setPixmap(pict);
-    if (tip && *tip)
-        QToolTip::add(this, i18n(tip));
+    list<string> smiles = getIcons()->getSmile(_id);
+    QString tip = QString::fromUtf8(smiles.front().c_str());
+    string name = getIcons()->getSmileName(_id);
+    char c = name[0];
+    if ((c < '0') || (c > '9')){
+        tip += " ";
+        tip += i18n(name.c_str());
+    }
+    QToolTip::add(this, tip);
 }
 
 void SmileLabel::mouseReleaseEvent(QMouseEvent*)
 {
-    emit clicked(id);
+    emit clicked(id.c_str());
 }
 
 SmilePopup::SmilePopup(QWidget *popup)
@@ -1577,24 +1589,21 @@ SmilePopup::SmilePopup(QWidget *popup)
     setFrameShape(PopupPanel);
     setFrameShadow(Sunken);
     QSize s;
+    list<string> smiles;
+    getIcons()->getSmiles(smiles);
+    if (smiles.empty())
+        return;
     unsigned nSmiles = 0;
-    unsigned i;
-    for (i = 0; ; i++){
-        const smile *p = smiles(i);
-        if (p == NULL)
-            break;
-        if (*p->exp == 0)
-            continue;
-        char b[20];
-        sprintf(b, "smile%X", i);
-        const QIconSet *is = Icon(b);
-        if (is == NULL)
+    list<string>::iterator it;
+    for (it = smiles.begin(); it != smiles.end(); ++it){
+        QIconSet is = Icon(it->c_str());
+        if (is.pixmap(QIconSet::Small, QIconSet::Normal).isNull())
             continue;
         QPixmap pict;
-        if (!is->isGenerated(QIconSet::Large, QIconSet::Normal)){
-            pict = is->pixmap(QIconSet::Large, QIconSet::Normal);
+        if (!is.isGenerated(QIconSet::Large, QIconSet::Normal)){
+            pict = is.pixmap(QIconSet::Large, QIconSet::Normal);
         }else{
-            pict = is->pixmap(QIconSet::Small, QIconSet::Normal);
+            pict = is.pixmap(QIconSet::Small, QIconSet::Normal);
         }
         s = QSize(QMAX(s.width(), pict.width()), QMAX(s.height(), pict.height()));
         nSmiles++;
@@ -1610,17 +1619,15 @@ SmilePopup::SmilePopup(QWidget *popup)
     QGridLayout *lay = new QGridLayout(this, rows, cols);
     lay->setMargin(4);
     lay->setSpacing(2);
-    i = 0;
+    unsigned i = 0;
     unsigned j = 0;
-    for (unsigned id = 0; ; id++){
-        const smile *p = smiles(id);
-        if (p == NULL)
-            break;
-        if (*p->exp == 0)
+    for (it = smiles.begin(); it != smiles.end(); ++it){
+        QIconSet is = Icon(it->c_str());
+        if (is.pixmap(QIconSet::Small, QIconSet::Normal).isNull())
             continue;
-        QWidget *w = new SmileLabel(id, p->title, this);
+        QWidget *w = new SmileLabel(it->c_str(), this);
         w->setMinimumSize(s);
-        connect(w, SIGNAL(clicked(int)), this, SLOT(labelClicked(int)));
+        connect(w, SIGNAL(clicked(const char*)), this, SLOT(labelClicked(const char*)));
         lay->addWidget(w, i, j);
         if (++j >= cols){
             i++;
@@ -1630,7 +1637,7 @@ SmilePopup::SmilePopup(QWidget *popup)
     resize(minimumSizeHint());
 }
 
-void SmilePopup::labelClicked(int id)
+void SmilePopup::labelClicked(const char *id)
 {
     insert(id);
     close();
@@ -1688,7 +1695,7 @@ void MsgEdit::setupMessages()
     cmd->id			= MessageAuthRequest;
     cmd->text		= I18N_NOOP("&Authorization request");
     cmd->icon		= "auth";
-    cmd->accel		= "Ctrl+A";
+    cmd->accel		= "Ctrl+Q";
     cmd->menu_grp	= 0x3060;
     cmd->flags		= COMMAND_DEFAULT;
     cmd->param		= &defAuthRequest;
@@ -1713,7 +1720,7 @@ void MsgEdit::setupMessages()
     eMsg.process();
 
     cmd->id			= MessageAdded;
-    cmd->text		= "Added";
+    cmd->text		= I18N_NOOP("Added");
     cmd->icon		= "auth";
     cmd->menu_grp	= 0;
     cmd->flags		= COMMAND_DEFAULT;
@@ -1721,7 +1728,7 @@ void MsgEdit::setupMessages()
     eMsg.process();
 
     cmd->id			= MessageRemoved;
-    cmd->text		= "Removed";
+    cmd->text		= I18N_NOOP("Removed");
     cmd->icon		= "auth";
     cmd->menu_grp	= 0;
     cmd->flags		= COMMAND_DEFAULT;
@@ -1729,7 +1736,7 @@ void MsgEdit::setupMessages()
     eMsg.process();
 
     cmd->id			= MessageStatus;
-    cmd->text		= "Status";
+    cmd->text		= I18N_NOOP("Status");
     cmd->icon		= NULL;
     cmd->menu_grp	= 0;
     cmd->flags		= COMMAND_DEFAULT;
@@ -1737,7 +1744,7 @@ void MsgEdit::setupMessages()
     eMsg.process();
 }
 
-#ifndef WIN32
+#ifndef _MSC_VER
 #include "msgedit.moc"
 #endif
 
