@@ -213,7 +213,7 @@ void *MsgFile::processEvent(Event *e)
     return NULL;
 }
 
-#ifndef WIN32
+#ifndef _MSC_VER
 #include "msgfile.moc"
 #endif
 
