@@ -132,7 +132,7 @@ void *MsgGen::processEvent(Event *e)
     return NULL;
 }
 
-#ifndef WIN32
+#ifndef _MSC_VER
 #include "msggen.moc"
 #endif
 
