@@ -73,7 +73,7 @@ MsgSMS::MsgSMS(MsgEdit *parent, Message *msg)
     }
     textChanged();
     SMSUserData *data = (SMSUserData*)(contact->getUserData(CorePlugin::m_plugin->sms_data_id));
-    if (contact->getTemporary()){
+    if (contact->getFlags() & CONTACT_TEMP){
         m_panel = new SMSPanel(m_edit->m_frame);
         m_edit->m_layout->insertWidget(0, m_panel);
         connect(m_panel, SIGNAL(destroyed()), this, SLOT(panelDestroyed()));
@@ -243,10 +243,10 @@ void *MsgSMS::processEvent(Event *e)
                 if (m_edit->sendMessage(msg)){
                     Contact *contact = getContacts()->contact(m_edit->m_userWnd->id());
                     if (contact){
-                        if (contact->getTemporary()){
+                        if (contact->getFlags() & CONTACT_TEMP){
                             contact->setName(phone);
                             if (m_panel && m_panel->chkSave->isChecked()){
-                                contact->setTemporary(0);
+                                contact->setFlags(contact->getFlags() & ~CONTACT_TEMP);
                                 delete m_panel;
                             }
                             Event e(EventContactChanged, contact);
@@ -306,7 +306,7 @@ SMSPanel::SMSPanel(QWidget *parent)
 SMSPanel::~SMSPanel()
 {}
 
-#ifndef WIN32
+#ifndef _MSC_VER
 #include "msgsms.moc"
 #endif
 
