@@ -17,6 +17,7 @@
 
 #include "msgview.h"
 #include "core.h"
+#include "icons.h"
 #include "history.h"
 #include "html.h"
 #include "xsl.h"
@@ -42,25 +43,149 @@ class ViewParser : public HTMLParser
 public:
     ViewParser(bool bIgnoreColors, bool bUseSmiles);
     QString parse(const QString &str);
-    bool RTL;
-    bool m_bParaStart;
 protected:
     QString res;
     bool m_bIgnoreColors;
     bool m_bUseSmiles;
     bool m_bInLink;
     bool m_bInHead;
+    bool m_bInParagraph;
+    bool m_bParagraphEmpty;
     bool m_bFirst;
     bool m_bSpan;
-    bool m_bPara;
-    bool m_bRTL;
-    bool m_bParaEnd;
-    list<Smile> m_smiles;
+    // Marks the position in 'res' where " DIR=\"whatever\"" should be inserted,
+    // if the paragraph is DIR-less and we determine the DIR later on.
+    unsigned m_paraDirInsertionPos;
+    enum {
+        DirAuto, // Initial BiDi dir when not explicitly specified.
+        // Real dir will be determined from the first
+        // strong BiDi character.
+        DirLTR,
+        DirRTL,
+        DirUnknown
+    } m_paragraphDir;
     virtual void text(const QString &text);
     virtual void tag_start(const QString &tag, const list<QString> &options);
     virtual void tag_end(const QString &tag);
 };
 
+/*
+   This parser is run on the output of the 'history XSL'.
+   The text which the XSL process output should generally be HTML,
+   but may contain the following special tags:
+ 
+    <prepend>...</prepend>
+    Strips the PREPEND tags and prepends their contents to the beginning
+    of the next paragraph. Useful to make sure chat prefixes are prepended
+    to the first paragraph of a multi-paragraph message (instead of residing
+    on a new paragraph).
+*/
+
+class XslOutputParser : public HTMLParser
+{
+public:
+    XslOutputParser();
+    QString parse(const QString &str);
+
+protected:
+    virtual void text(const QString &text);
+    virtual void tag_start(const QString &tag, const list<QString> &options);
+    virtual void tag_end(const QString &tag);
+
+protected:
+    QString res;
+    bool m_bInPrepend;
+    QString m_sPrepend;
+};
+
+XslOutputParser::XslOutputParser()
+        : m_bInPrepend(false)
+{
+}
+
+QString XslOutputParser::parse(const QString &str)
+{
+    res = "";
+    HTMLParser::parse(str);
+    if (!m_sPrepend.isEmpty())
+        res = m_sPrepend + res;
+    return res;
+}
+
+void XslOutputParser::text(const QString& text)
+{
+    if (m_bInPrepend)
+        m_sPrepend += quoteString(text);
+    else
+        res += quoteString(text);
+}
+
+void XslOutputParser::tag_start(const QString &tag, const list<QString> &attrs)
+{
+    QString ltag = tag.lower();
+
+    if (ltag == "prepend")
+    {
+        m_bInPrepend = true;
+        return;
+    }
+
+    QString tagText;
+    tagText += "<";
+    tagText += tag;
+    for (list<QString>::const_iterator it = attrs.begin(); it != attrs.end(); ++it){
+        QString name = *it;
+        ++it;
+        QString value = *it;
+        tagText += " ";
+        tagText += name;
+        if (!value.isEmpty()){
+            tagText += "=\"";
+            tagText += value;
+            tagText += "\"";
+        }
+    }
+    tagText += ">";
+
+    if (m_bInPrepend)
+    {
+        m_sPrepend += tagText;
+    }
+    else
+    {
+        res += tagText;
+
+        // It's time to prepend whatever we've got in m_sPrepend
+        // to the start of a paragraph.
+        if ((ltag == "p") && !m_sPrepend.isEmpty())
+        {
+            res += m_sPrepend;
+            m_sPrepend = "";
+        }
+    }
+}
+
+void XslOutputParser::tag_end(const QString &tag)
+{
+    QString ltag = tag.lower();
+
+    if (ltag == "prepend")
+    {
+        m_bInPrepend = false;
+        return;
+    }
+
+    QString tagText;
+    tagText += "</";
+    tagText += tag;
+    tagText += ">";
+
+    if (m_bInPrepend)
+        m_sPrepend += tagText;
+    else
+        res += tagText;
+}
+
 MsgViewBase::MsgViewBase(QWidget *parent, const char *name, unsigned id)
         : TextShow(parent, name)
 {
@@ -128,18 +253,10 @@ void MsgViewBase::update()
     m_updated.clear();
     if (i >= (unsigned)paragraphs())
         return;
-    QPoint p = QPoint(0, 0);
-    p = mapToGlobal(p);
-    p = viewport()->mapFromGlobal(p);
-    int x, y;
-    viewportToContents(p.x(), p.y(), x, y);
-    int para;
-    int pos = charAt(QPoint(x, y), &para);
-    p = QPoint(0, viewport()->height());
-    p = viewport()->mapToGlobal(p);
-    p = mapFromGlobal(p);
-    if (p.y() + 2 == height())
-        pos = -1;
+    int x = contentsX();
+    int y = contentsY();
+    viewport()->setUpdatesEnabled(false);
+
     unsigned start = i;
     list<Msg_Id> msgs;
     for (; i < (unsigned)paragraphs(); i++){
@@ -169,7 +286,7 @@ void MsgViewBase::update()
     int paraTo, indexTo;
     getSelection(&paraFrom, &indexFrom, &paraTo, &indexTo);
     setReadOnly(false);
-    setSelection(start, 0, paragraphs() - 1, 0xFFFF);
+    setSelection(start, 0, paragraphs() - 1, 0xFFFF, 0);
     removeSelectedText();
     setReadOnly(true);
     QString text;
@@ -188,18 +305,17 @@ void MsgViewBase::update()
             }
         }
         text += messageText(msg, bUnread);
+        delete msg;
     }
+    viewport()->setUpdatesEnabled(true);
     append(text);
     if (!CorePlugin::m_plugin->getOwnColors())
-        setBackground(0);
+        setBackground(i);
     if ((paraFrom != paraTo) || (indexFrom != indexTo))
-        setSelection(paraFrom, indexFrom, paraTo, indexTo);
-    if (pos == -1){
-        scrollToBottom();
-    }else{
-        setCursorPosition(para, pos);
-        ensureCursorVisible();
-    }
+        setSelection(paraFrom, indexFrom, paraTo, indexTo, 0);
+    TextShow::sync();
+    setContentsPos(x, y);
+    viewport()->repaint();
 }
 
 QString MsgViewBase::messageText(Message *msg, bool bUnread)
@@ -307,7 +423,7 @@ QString MsgViewBase::messageText(Message *msg, bool bUnread)
     // in one chunk (since it's more efficient and causes less redraws), and there's
     // no way to set block's background color directly in Qt's HTML), so we make a note
     // of it in the HTML and set it retroactively in setBackground.
-    if ((msg->getBackground() != 0xFFFFFFFF) && (msg->getForeground() != msg->getBackground()))
+    if (!CorePlugin::m_plugin->getOwnColors() && (msg->getBackground() != 0xFFFFFFFF) && (msg->getForeground() != msg->getBackground()))
         id += QString::number(msg->getBackground());
     // </hack>
     string client_str;
@@ -368,14 +484,12 @@ QString MsgViewBase::messageText(Message *msg, bool bUnread)
     }else{
         msgText = status;
     }
-
     Event e(EventEncodeText, &msgText);
     e.process();
     ViewParser parser(CorePlugin::m_plugin->getOwnColors(), CorePlugin::m_plugin->getUseSmiles());
     msgText = parser.parse(msgText);
     s += "<body";
-
-    if ((msg->getForeground() != 0xFFFFFFFF) && (msg->getForeground() != msg->getBackground()))
+    if (!CorePlugin::m_plugin->getOwnColors() && (msg->getForeground() != 0xFFFFFFFF) && (msg->getForeground() != msg->getBackground()))
     {
         s += " fgcolor=\"#";
         s += QString::number(msg->getForeground(), 16).rightJustify(6, '0');
@@ -402,26 +516,14 @@ QString MsgViewBase::messageText(Message *msg, bool bUnread)
     if (p == NULL)
         p = CorePlugin::m_plugin->historyXSL;
     QString res = p->process(s);
+
+    XslOutputParser outParser;
+    res = outParser.parse(res);
+
     QString anchor = MSG_ANCHOR;
     anchor += id;
     anchor += "\">";
     res = anchor + res;
-    int startPos = res.find(MSG_BEGIN);
-    int paraPos;
-    if (parser.m_bParaStart){
-        paraPos = res.find("<p");
-        if (paraPos >= 0)
-            res = res.left(paraPos) + "</p>" + res.mid(paraPos);
-    }
-    paraPos = res.findRev("<p", startPos);
-    QString dir = " dir=\"";
-    dir += parser.RTL ? "rtl" : "ltr";
-    dir += "\"";
-    if (paraPos >= 0){
-        res = res.left(paraPos + 2) + dir + res.mid(paraPos + 2);
-    }else{
-        res = QString("<p") + dir + ">" + res;
-    }
     return res;
 }
 
@@ -462,7 +564,13 @@ void MsgViewBase::setBackground(unsigned n)
     QString sAnchor = QString::fromLatin1(MSG_ANCHOR),
                       sBegin = QString::fromLatin1(MSG_BEGIN);
 
-    for (unsigned i = n; i < (unsigned)paragraphs(); i++){
+    int i;
+    for (i = n; i >= 0; i--){
+        QString s = text(i);
+        if (s.find(sAnchor) >= 0)
+            break;
+    }
+    for (; i < paragraphs(); i++){
         QString s = text(i);
         int anchorPos = s.find(sAnchor);
         if (anchorPos >= 0)
@@ -505,14 +613,22 @@ void MsgViewBase::setBackground(unsigned n)
 }
 // </hack>
 
-void MsgViewBase::addMessage(Message *msg, bool bUnread)
+void MsgViewBase::addMessage(Message *msg, bool bUnread, bool bSync)
 {
-    unsigned n = paragraphs() - 1;
+    unsigned n = paragraphs();
+    if (n > 0)
+        n--;
     append(messageText(msg, bUnread));
-    if (!CorePlugin::m_plugin->getOwnColors()) {
-        // set all Backgrounds to the right colors
-        setBackground(0);
-    }
+    if (!CorePlugin::m_plugin->getOwnColors())
+        setBackground(n);
+    if (bSync)
+        sync(n);
+}
+
+#ifdef WIN32
+
+void MsgViewBase::sync(unsigned n)
+{
     if (!m_selectStr.isEmpty()){
         bool bStart = false;
         for (; n < (unsigned)paragraphs(); n++){
@@ -536,11 +652,21 @@ void MsgViewBase::addMessage(Message *msg, bool bUnread)
                 index += m_selectStr.length();
             }
             setCursorPosition(savePara, saveIndex);
+            repaintChanged();
         }
     }
-    sync();
+    TextShow::sync();
 }
 
+#else
+
+void MsgViewBase::sync(unsigned)
+{
+    TextShow::sync();
+}
+
+#endif
+
 bool MsgViewBase::findMessage(Message *msg)
 {
     bool bFound = false;
@@ -723,7 +849,7 @@ void *MsgViewBase::processEvent(Event *e)
             int pos = charAt(QPoint(x, y), &para);
             setReadOnly(false);
             for (unsigned i = 0; i < start_pos.size(); i++){
-                setSelection(start_pos[i], 0, end_pos[i], 0);
+                setSelection(start_pos[i], 0, end_pos[i], 0, 0);
                 removeSelectedText();
                 if ((unsigned)pos >= start_pos[i])
                     pos = end_pos[i] - start_pos[i];
@@ -736,7 +862,7 @@ void *MsgViewBase::processEvent(Event *e)
                     ensureCursorVisible();
                 }
             }else{
-                setSelection(paraFrom, indexFrom, paraTo, indexTo);
+                setSelection(paraFrom, indexFrom, paraTo, indexTo, 0);
             }
             setReadOnly(true);
             repaint();
@@ -762,13 +888,11 @@ void *MsgViewBase::processEvent(Event *e)
                 continue;
             string ss;
             ss = text(i).local8Bit();
-            log(L_DEBUG, "?: %s", ss.c_str());
 
             unsigned j;
             for (j = i + 1; j < (unsigned)paragraphs(); j++){
                 QString s = text(j);
                 ss = text(j).local8Bit();
-                log(L_DEBUG, ">: %s", ss.c_str());
                 int n = s.find(MSG_ANCHOR);
                 if (n < 0)
                     continue;
@@ -788,21 +912,21 @@ void *MsgViewBase::processEvent(Event *e)
                 j++;
                 pos = 0;
             }
-            setSelection(i, 0, j - 1, pos);
+            setSelection(i, 0, j - 1, pos, 0);
             setReadOnly(false);
             removeSelectedText();
             setReadOnly(true);
             if ((paraFrom == -1) && (paraTo == -1)){
                 scrollToBottom();
             }else{
-                setSelection(paraFrom, indexFrom, paraTo, indexTo);
+                setSelection(paraFrom, indexFrom, paraTo, indexTo, 0);
             }
             break;
         }
         return NULL;
     }
     if (e->type() == EventHistoryConfig){
-        unsigned id = (unsigned)(e->param());
+        unsigned long id = (unsigned long)(e->param());
         if (id && (id != m_id))
             return NULL;
         reload();
@@ -874,6 +998,7 @@ void *MsgViewBase::processEvent(Event *e)
                     for (const CommandDef *d = cmdsSpecial; d->text; d++){
                         cmds[n] = *d;
                         cmds[n].id = CmdMsgSpecial + n;
+                        cmds[n].flags = COMMAND_DEFAULT;
                         n++;
                     }
                 }
@@ -956,6 +1081,7 @@ void *MsgViewBase::processEvent(Event *e)
                             if (n-- == 0){
                                 CommandDef cmd = *d;
                                 cmd.param = msg;
+                                cmd.menu_id = 0;
                                 Event eCmd(EventCommandExec, &cmd);
                                 eCmd.process();
                                 return e->param();
@@ -1058,7 +1184,7 @@ MsgView::~MsgView()
 
 void MsgView::init()
 {
-    sync();
+    TextShow::sync();
     scrollToBottom();
 }
 
@@ -1103,62 +1229,16 @@ ViewParser::ViewParser(bool bIgnoreColors, bool bUseSmiles)
     m_bUseSmiles    = bUseSmiles;
     m_bInLink       = false;
     m_bInHead       = false;
+    m_bInParagraph  = false;
+    m_bParagraphEmpty = true;
     m_bFirst     	= true;
     m_bSpan			= false;
-    m_bPara			= false;
-    m_bParaStart	= false;
-    m_bRTL			= false;
-    m_bParaEnd		= false;
-    RTL				= false;
-    if (m_bUseSmiles){
-        for (unsigned i = 0; ;i++){
-            const smile *s = smiles(i);
-            if (s == NULL)
-                break;
-#if COMPAT_QT_VERSION < 0x030000
-            string str;
-            for (const char *p = s->exp;; p++){
-                if ((*p == 0) || (*p == '|')){
-                    if (!str.empty()){
-                        Smile ss;
-                        ss.nSmile = i;
-                        ss.re = QRegExp(str.c_str());
-                        if (ss.re.isValid())
-                            m_smiles.push_back(ss);
-                    }
-                    if (*p == 0)
-                        break;
-                    str = "";
-                    continue;
-                }
-                if (*p == '\\'){
-                    if (*(++p) == 0)
-                        break;
-                    str += '\\';
-                    str += *p;
-                    continue;
-                }
-                str += *p;
-            }
-#else
-            if (*(s->exp)){
-                Smile ss;
-                ss.nSmile = i;
-                ss.re = QRegExp(s->exp);
-                if (ss.re.isValid())
-                    m_smiles.push_back(ss);
-            }
-#endif
-        }
-    }
 }
 
 QString ViewParser::parse(const QString &str)
 {
     res = "";
     HTMLParser::parse(str);
-    if (m_bParaStart)
-        res += "</p>";
     return res;
 }
 
@@ -1166,63 +1246,76 @@ void ViewParser::text(const QString &text)
 {
     if (text.isEmpty())
         return;
-    if (m_bParaEnd){
-        m_bParaEnd = false;
-        res += "<br/>";
+        
+    if (m_bInParagraph)
+       m_bParagraphEmpty = false;
+
+    if (m_bInParagraph && (m_paragraphDir == DirAuto))
+    {
+        /* text isn't (unicode)-NULL terminated so we can't check for c->isNull! */
+        for(unsigned int i = 0; ((i < text.length()) && (m_paragraphDir == DirAuto)); i++)
+        {
+            const QChar c = text.unicode()[i];
+            // Note: Qt expects ltr/rtl to be lower-case.
+            switch(c.direction())
+            {
+            case QChar::DirL:
+                res.insert(m_paraDirInsertionPos, " dir=\"ltr\"");
+                m_paragraphDir = DirLTR;
+                break;
+            case QChar::DirR:
+                res.insert(m_paraDirInsertionPos, " dir=\"rtl\"");
+                m_paragraphDir = DirRTL;
+                break;
+            default: // avoid gcc warning
+                break;
+            }
+        }
     }
+
     if (!m_bUseSmiles || m_bInLink){
         res += quoteString(text);
         return;
     }
     m_bFirst = false;
-    QString str = text;
-    for (list<Smile>::iterator it = m_smiles.begin(); it != m_smiles.end(); ++it){
-        Smile &s = *it;
-        s.size = 0;
-        s.pos = s.re.match(str, 0, &s.size);
-        if (s.size == 0)
-            s.pos = -1;
+    if (m_bUseSmiles){
+        QString r = getIcons()->parseSmiles(text);
+        res += r;
+    }else{
+        res += quoteString(text);
     }
-    for (;;){
-        unsigned pos = (unsigned)(-1);
-        unsigned size = 0;
-        Smile *curSmile = NULL;
-        list<Smile>::iterator it;
-        for (it = m_smiles.begin(); it != m_smiles.end(); ++it){
-            Smile &s = *it;
-            if (s.pos < 0)
-                continue;
-            if (((unsigned)(s.pos) < pos) || (((unsigned)(s.pos) == pos) && ((unsigned)(s.size) > size) && (s.pos != -1))){
-                pos = s.pos;
-                size = s.size;
-                curSmile = &s;
-            }
-        }
-        if ((curSmile == NULL) || (size == 0))
-            break;
-        if (pos)
-            res += quoteString(str.left(pos));
-        res += "<img src=\"icon:smile";
-        res += QString::number(curSmile->nSmile, 16).upper();
-        res += "\"/>";
-        int len = pos + curSmile->size;
-        str = str.mid(len);
-        for (it = m_smiles.begin(); it != m_smiles.end(); ++it){
-            Smile &s = *it;
-            if (s.pos < 0)
-                continue;
-            s.pos -= len;
-            if (s.pos < 0){
-                s.size = 0;
-                s.pos = s.re.match(str, 0, &s.size);
-                if (s.size == 0)
-                    s.pos = -1;
-            }
-        }
-    }
-    res += quoteString(str);
 }
 
+static const char *def_smiles[] =
+    {
+        ":-)",
+        ":-0",
+        ":-|",
+        ":-/",
+        ":-(",
+        ":-{}",
+        ":*)",
+        ":'-(",
+        ";-)",
+        ":-@",
+        ":-\")",
+        ":-X",
+        ":-P",
+        "8-)",
+        "O:-)",
+        ":-D",
+        "*ANNOYED*",
+        "*DISGUSTED*",
+        "*DROOLING*",
+        "*GIGGLING*",
+        "*JOKINGLY*",
+        "*SHOCKED*",
+        "*WHINING*",
+        "*SURPRISED*",
+        "*SURPRISED*",
+        "*IN LOVE*"
+    };
+
 void ViewParser::tag_start(const QString &tag, const list<QString> &attrs)
 {
     // the tag that will be actually written out
@@ -1247,18 +1340,18 @@ void ViewParser::tag_start(const QString &tag, const list<QString> &attrs)
         if (src.left(10) == "icon:smile"){
             bool bOK;
             unsigned nSmile = src.mid(10).toUInt(&bOK, 16);
-            if (bOK){
-                const smile *s = smiles(nSmile);
-                if (s == NULL)
-                    return;
-                if (*s->exp == 0){
-                    res += quoteString(s->paste);
-                    return;
-                }
+            if (bOK && (nSmile < 26)){
+                QString s = def_smiles[nSmile];
+                res += getIcons()->parseSmiles(s);
+                return;
             }
         }
     }else if (tag == "a"){
         m_bInLink = true;
+    }else if (tag == "p"){
+        m_bInParagraph = true;
+        m_paragraphDir = DirAuto;
+        m_bParagraphEmpty = true;
     }else if (tag == "html"){ // we display as a part of a larger document
         return;
     }else if (tag == "head"){
@@ -1266,41 +1359,17 @@ void ViewParser::tag_start(const QString &tag, const list<QString> &attrs)
         return;
     }else if (tag == "body"){ // we display as a part of a larger document
         oTag = "span";
-    }else if (tag == "p"){
-        bool bRTL = false;
-        m_bParaEnd = false;
-        for (list<QString>::const_iterator it = attrs.begin(); it != attrs.end(); ++it){
-            QString name = (*it).lower();
-            ++it;
-            QString value = *it;
-            if ((name == "dir") && (value.lower() == "rtl"))
-                bRTL = true;
-        }
-        if (m_bPara){
-            if (bRTL == m_bRTL){
-                res += "<br/>";
-            }else{
-                if (m_bParaStart)
-                    res += "</p>";
-                res += "<p dir=\"";
-                res += bRTL ? "rtl" : "ltr";
-                res += "\">";
-                m_bParaStart = true;
-            }
-        }else{
-            RTL     = bRTL;
-            m_bRTL  = bRTL;
-            m_bPara = true;
-        }
-        return;
-    }
-    if (m_bParaEnd){
-        res += "<br/>";
-        m_bParaEnd = false;
     }
+
     QString tagText;
     tagText += "<";
     tagText += oTag;
+
+    if (tag == "p")
+    {
+        m_paraDirInsertionPos = res.length() + tagText.length();
+    }
+
     for (list<QString>::const_iterator it = attrs.begin(); it != attrs.end(); ++it){
         QString name = (*it).lower();
         ++it;
@@ -1312,6 +1381,16 @@ void ViewParser::tag_start(const QString &tag, const list<QString> &attrs)
                 style += "background-color:" + value + ";";
                 continue;
             }
+        }else if (tag == "p"){
+            if (name == "dir"){
+                QString dir = value.lower();
+                if (dir == "ltr")
+                    m_paragraphDir = DirLTR;
+                else if (dir == "rtl")
+                    m_paragraphDir = DirRTL;
+                else
+                    m_paragraphDir = DirUnknown;
+            }
         }else if (tag == "font"){
             if (name == "color" && m_bIgnoreColors)
                 continue;
@@ -1346,9 +1425,11 @@ void ViewParser::tag_start(const QString &tag, const list<QString> &attrs)
                     break;
                 QString value = *it;
                 if ((name == "color") ||
+						(name == "background") ||
                         (name == "background-color") ||
                         (name == "font-size") ||
                         (name == "font-style") ||
+						(name == "font-variant") ||
                         (name == "font-weight") ||
                         (name == "font-family"))
                     continue;
@@ -1369,6 +1450,16 @@ void ViewParser::tag_end(const QString &tag)
     QString oTag = tag;
     if (tag == "a"){
         m_bInLink = false;
+    }else if (tag == "p"){
+        if (m_bInParagraph && m_bParagraphEmpty)
+           // The user probably didn't intend to insert an empty paragraph.
+           // We are probably viewing faulty content produced by QTextEdit
+           // (which returns <p></p> but optimizes it away upon loading),
+           // so we fix it up.
+           res += "<br>";
+        res += "</p>";
+        m_bInParagraph = false;
+        return;
     }else if (tag == "head"){
         m_bInHead = false;
         return;
@@ -1376,9 +1467,6 @@ void ViewParser::tag_end(const QString &tag)
         return;
     }else if (tag == "body"){
         oTag = "span";
-    }else if (tag == "p"){
-        m_bParaEnd = true;
-        return;
     }
     if (m_bInHead)
         return;
@@ -1393,7 +1481,7 @@ QString MsgViewBase::parseText(const QString &text, bool bIgnoreColors, bool bUs
     return parser.parse(text);
 }
 
-#ifndef WIN32
+#ifndef _MSC_VER
 #include "msgview.moc"
 #endif
 
