@@ -36,8 +36,9 @@ typedef map<unsigned, unsigned> MAP_STATUS;
 CommonStatus::CommonStatus()
         : EventReceiver(LowPriority + 2)
 {
-    m_bBlink = false;
-    m_timer  = NULL;
+    m_bBlink  = false;
+    m_timer   = NULL;
+    m_balloon = NULL;
 
     Event eMenu(EventMenuCreate, (void*)MenuStatus);
     eMenu.process();
@@ -283,27 +284,61 @@ void *CommonStatus::processEvent(Event *e)
         checkInvisible();
         setBarStatus();
         break;
+    case EventShowError:{
+            clientErrorData *data = (clientErrorData*)(e->param());
+            for (list<BalloonItem>::iterator it = m_queue.begin(); it != m_queue.end(); ++it){
+                if ((*it).id == data->id)
+                    return e->param();
+            }
+            BalloonItem item;
+            item.id     = data->id;
+            item.client = data->client;
+            item.text   = i18n(data->err_str);
+            if (data->args){
+                if (item.text.find("%1") >= 0)
+                    item.text = item.text.arg(QString::fromUtf8(data->args));
+                free(data->args);
+            }
+            QString title = "SIM";
+            if (getContacts()->nClients() > 1){
+                for (unsigned i = 0; i < getContacts()->nClients(); i++){
+                    if (getContacts()->getClient(i) == data->client){
+                        title = data->client->name().c_str();
+                        int n = title.find(".");
+                        if (n > 0)
+                            title = title.left(n) + " " + title.mid(n + 1);
+                        break;
+                    }
+                }
+            }
+            item.text	= QString("<img src=\"icon:%1\">&nbsp;<b><nobr>%2</nobr></b><br><center>")
+                        .arg((data->flags & ERR_INFO) ? "info" : "error")
+                        .arg(title) + quoteString(item.text) + "</center>";
+            if (data->options){
+                for (const char *p = data->options; *p; p += strlen(p) + 1)
+                    item.buttons.append(i18n(p));
+            }else{
+                item.buttons.append(i18n("OK"));
+            }
+            m_queue.push_back(item);
+            if (m_balloon == NULL)
+                showBalloon();
+            break;
+        }
     case EventClientError:{
             clientErrorData *data = (clientErrorData*)(e->param());
-            QString msg;
-            if (data->err_str && *data->err_str){
-                msg = i18n(data->err_str);
-                if (data->args)
-                    msg = msg.arg(QString::fromUtf8(data->args));
-            }
             if (data->code == AuthError){
+                QString msg;
+                if (data->err_str && *data->err_str){
+                    msg = i18n(data->err_str);
+                    if (data->args)
+                        msg = msg.arg(QString::fromUtf8(data->args));
+                }
                 LoginDialog *loginDlg = new LoginDialog(false, data->client, msg, NULL);
                 raiseWindow(loginDlg);
             }else{
-                Command cmd;
-                cmd->id = CmdStatusBar;
-                Event eWidget(EventCommandWidget, cmd);
-                QWidget *widget = (QWidget*)(eWidget.process());
-                if (widget){
-                    raiseWindow(widget->topLevelWidget());
-                    if (!msg.isEmpty())
-                        BalloonMsg::message(msg, widget);
-                }
+                Event eShow(EventShowError, data);
+                eShow.process();
             }
             return e->param();
         }
@@ -388,7 +423,19 @@ void *CommonStatus::processEvent(Event *e)
                 }
                 if (curStatus == NULL)
                     return 0;
-                if ((def->id != STATUS_ONLINE) && (def->id != STATUS_OFFLINE)){
+                unsigned i;
+                bool bOfflineStatus = false;
+                for (i = 0; i < getContacts()->nClients(); i++){
+                    Client *client = getContacts()->getClient(i);
+                    if (client->getCommonStatus() &&
+                            (client->protocol()->description()->flags & PROTOCOL_AR_OFFLINE)){
+                        bOfflineStatus = true;
+                        break;
+                    }
+                }
+
+                if (bOfflineStatus ||
+                        ((def->id != STATUS_ONLINE) && (def->id != STATUS_OFFLINE))){
                     const char *noShow = CorePlugin::m_plugin->getNoShowAutoReply(def->id);
                     if ((noShow == NULL) || (*noShow == 0)){
                         AutoReplyDialog dlg(def->id);
@@ -397,7 +444,7 @@ void *CommonStatus::processEvent(Event *e)
                     }
                 }
                 CorePlugin::m_plugin->setManualStatus(def->id);
-                for (unsigned i = 0; i < getContacts()->nClients(); i++){
+                for (i = 0; i < getContacts()->nClients(); i++){
                     Client *client = getContacts()->getClient(i);
                     if (client->getCommonStatus())
                         client->setStatus(def->id, true);
@@ -410,7 +457,51 @@ void *CommonStatus::processEvent(Event *e)
     return NULL;
 }
 
-#ifndef WIN32
+void CommonStatus::showBalloon()
+{
+    if (m_balloon || m_queue.empty())
+        return;
+    Command cmd;
+    cmd->id = CmdStatusBar;
+    Event eWidget(EventCommandWidget, cmd);
+    QWidget *widget = (QWidget*)(eWidget.process());
+    if (widget == NULL){
+        m_queue.erase(m_queue.begin());
+        return;
+    }
+    BalloonItem &item = m_queue.front();
+    if (CorePlugin::m_plugin->m_statusWnd)
+        m_balloon = CorePlugin::m_plugin->m_statusWnd->showError(item.text, item.buttons, item.client);
+    if (m_balloon == NULL)
+        m_balloon = new BalloonMsg(NULL, item.text, item.buttons, widget);
+    connect(m_balloon, SIGNAL(yes_action(void*)), this, SLOT(yes_action(void*)));
+    connect(m_balloon, SIGNAL(finished()), this, SLOT(finished()));
+    raiseWindow(widget->topLevelWidget());
+    m_balloon->show();
+}
+
+void CommonStatus::yes_action(void*)
+{
+    if (!m_queue.empty() && m_balloon){
+        m_balloon->hide();
+        BalloonItem &item = m_queue.front();
+        Command cmd;
+        cmd->id    = item.id;
+        cmd->param = item.client;
+        Event e(EventCommandExec, cmd);
+        e.process();
+    }
+}
+
+void CommonStatus::finished()
+{
+    m_balloon = NULL;
+    if (!m_queue.empty())
+        m_queue.erase(m_queue.begin());
+    QTimer::singleShot(1000, this, SLOT(showBalloon()));
+}
+
+#ifndef _MSC_VER
 #include "status.moc"
 #endif
 
