@@ -35,6 +35,12 @@ UserViewItemBase::UserViewItemBase(UserViewItemBase *parent)
 {
 }
 
+void UserViewItemBase::setup()
+{
+    UserListBase *view = static_cast<UserListBase*>(listView());
+    setHeight(view->heightItem(this));
+}
+
 void UserViewItemBase::paintFocus(QPainter*, const QColorGroup&, const QRect & )
 {
 }
@@ -51,29 +57,28 @@ void UserViewItemBase::paintCell(QPainter *p, const QColorGroup &cg, int, int wi
     QPixmap bg(width, height());
     QPainter pp(&bg);
     int margin = 0;
+    pp.fillRect(QRect(0, 0, width, height()), cg.base());
+    PaintView pv;
+    pv.p        = &pp;
+    pv.pos      = view->viewport()->mapToParent(view->itemRect(this).topLeft());
+    pv.size		= QSize(width, height());
+    pv.win      = view;
+    pv.isStatic = false;
+    pv.height   = height();
+    pv.margin   = 0;
+    pv.isGroup  = (type() == GRP_ITEM);
+    if (CorePlugin::m_plugin->getUseSysColors()){
+        pp.setPen(cg.text());
+    }else{
+        pp.setPen(QColor(CorePlugin::m_plugin->getColorOnline()));
+    }
+    Event e(EventPaintView, &pv);
+    e.process();
+    view->setStaticBackground(pv.isStatic);
+    margin = pv.margin;
     if (isSelected() && view->hasFocus() && CorePlugin::m_plugin->getUseDblClick()){
         pp.fillRect(QRect(0, 0, width, height()), cg.highlight());
         pp.setPen(cg.highlightedText());
-    }else{
-        pp.fillRect(QRect(0, 0, width, height()), cg.base());
-        PaintView pv;
-        pv.p        = &pp;
-        pv.pos      = view->viewport()->mapToParent(view->itemRect(this).topLeft());
-        pv.size		= QSize(width, height());
-        pv.win      = view;
-        pv.isStatic = false;
-        pv.height   = height();
-        pv.margin   = 0;
-        pv.isGroup  = (type() == GRP_ITEM);
-        if (CorePlugin::m_plugin->getUseSysColors()){
-            pp.setPen(cg.text());
-        }else{
-            pp.setPen(QColor(CorePlugin::m_plugin->getColorOnline()));
-        }
-        Event e(EventPaintView, &pv);
-        e.process();
-        view->setStaticBackground(pv.isStatic);
-        margin = pv.margin;
     }
     view->drawItem(this, &pp, cg, width, margin);
     pp.end();
@@ -234,6 +239,7 @@ bool ContactItem::update(Contact *contact, unsigned status, unsigned style, cons
     setText(CONTACT_ICONS, icons_str);
     setText(CONTACT_ACTIVE, active);
     setText(CONTACT_STATUS, QString::number(9 - status));
+    setup();
     return true;
 }
 
@@ -295,6 +301,10 @@ void UserListBase::drawUpdates()
     m_bDirty = false;
     updTimer->stop();
     QListViewItem *item;
+    int x = contentsX();
+    int y = contentsY();
+    viewport()->setUpdatesEnabled(false);
+    bool bChanged = false;
     list<unsigned long>::iterator it;
     for (it = updGroups.begin(); it != updGroups.end(); ++it){
         Group *group = getContacts()->group(*it);
@@ -306,13 +316,16 @@ void UserListBase::drawUpdates()
             if (item){
                 if (!m_bShowEmpty && (item->firstChild() == NULL)){
                     delete item;
+                    bChanged = true;
                 }else{
                     static_cast<GroupItem*>(item)->update(group);
-                    repaintItem(item);
+                    addUpdatedItem(item);
                 }
             }else{
-                if (m_bShowEmpty)
+                if (m_bShowEmpty){
                     new GroupItem(this, group, true);
+                    bChanged = true;
+                }
             }
             break;
         case 2:
@@ -324,13 +337,16 @@ void UserListBase::drawUpdates()
                 if (grpItem){
                     if (!m_bShowEmpty && (item->firstChild() == NULL)){
                         delete grpItem;
+                        bChanged = true;
                     }else{
                         grpItem->update(group);
-                        repaintItem(grpItem);
+                        addUpdatedItem(grpItem);
                     }
                 }else{
-                    if (m_bShowEmpty)
+                    if (m_bShowEmpty){
                         new GroupItem(divItem, group, divItem->state() == DIV_OFFLINE);
+                        bChanged = true;
+                    }
                 }
             }
             break;
@@ -373,6 +389,7 @@ void UserListBase::drawUpdates()
                     contactItem = findContactItem(contact->id(), itemOnline);
                     if (contactItem){
                         deleteItem(contactItem);
+                        bChanged = true;
                         if (itemOnline->firstChild() == NULL){
                             deleteItem(itemOnline);
                             itemOnline = NULL;
@@ -384,6 +401,7 @@ void UserListBase::drawUpdates()
                         contactItem = findContactItem(contact->id(), itemOffline);
                         if (contactItem){
                             deleteItem(contactItem);
+                            bChanged = true;
                             if (itemOffline->firstChild() == NULL){
                                 deleteItem(itemOffline);
                                 itemOffline = NULL;
@@ -395,20 +413,23 @@ void UserListBase::drawUpdates()
                 if (itemOffline == NULL){
                     itemOffline = new DivItem(this, DIV_OFFLINE);
                     setOpen(itemOffline, true);
+                    bChanged = true;
                 }
                 contactItem = findContactItem(contact->id(), itemOffline);
                 if (contactItem){
                     if (contactItem->update(contact, status, style, icons.c_str(), unread))
                         addSortItem(itemOffline);
-                    repaintItem(contactItem);
+                    addUpdatedItem(contactItem);
                 }else{
                     contactItem = new ContactItem(itemOffline, contact, status, style, icons.c_str(), unread);
+                    bChanged = true;
                 }
             }else{
                 if (itemOffline){
                     contactItem = findContactItem(contact->id(), itemOffline);
                     if (contactItem){
                         deleteItem(contactItem);
+                        bChanged = true;
                         if (itemOffline->firstChild() == NULL){
                             deleteItem(itemOffline);
                             itemOffline = NULL;
@@ -418,14 +439,16 @@ void UserListBase::drawUpdates()
                 if (itemOnline == NULL){
                     itemOnline = new DivItem(this, DIV_ONLINE);
                     setOpen(itemOnline, true);
+                    bChanged = true;
                 }
                 contactItem = findContactItem(contact->id(), itemOnline);
                 if (contactItem){
                     if (contactItem->update(contact, status, style, icons.c_str(), unread))
                         addSortItem(itemOnline);
-                    repaintItem(contactItem);
+                    addUpdatedItem(contactItem);
                 }else{
                     contactItem = new ContactItem(itemOnline, contact, status, style, icons.c_str(), unread);
+                    bChanged = true;
                 }
             }
             break;
@@ -441,6 +464,7 @@ void UserListBase::drawUpdates()
                         grpItem->m_nContactsOnline--;
                     addGroupForUpdate(grpItem->id());
                     deleteItem(contactItem);
+                    bChanged = true;
                     if (!m_bShowEmpty && (grpItem->firstChild() == NULL))
                         delete grpItem;
                     contactItem = NULL;
@@ -452,15 +476,17 @@ void UserListBase::drawUpdates()
                     grpItem = findGroupItem(contact->getGroup());
                     if (grpItem == NULL){
                         Group *grp = getContacts()->group(contact->getGroup());
-                        if (grp)
+                        if (grp){
                             grpItem = new GroupItem(this, grp, true);
+                            bChanged = true;
+                        }
                     }
                 }
                 if (grpItem){
                     if (contactItem){
                         if (contactItem->update(contact, status, style, icons.c_str(), unread))
                             addSortItem(grpItem);
-                        repaintItem(contactItem);
+                        addUpdatedItem(contactItem);
                         if (!m_bShowOnline &&
                                 (contactItem->m_bOnline != (status > STATUS_OFFLINE))){
                             if (status <= STATUS_OFFLINE){
@@ -473,6 +499,7 @@ void UserListBase::drawUpdates()
                             addGroupForUpdate(grpItem->id());
                         }
                     }else{
+                        bChanged = true;
                         contactItem = new ContactItem(grpItem, contact, status, style, icons.c_str(), unread);
                         grpItem->m_nContacts++;
                         if (!m_bShowOnline && (status > STATUS_OFFLINE)){
@@ -493,6 +520,7 @@ void UserListBase::drawUpdates()
                     grpItem->m_nContacts--;
                     addGroupForUpdate(grpItem->id());
                     deleteItem(contactItem);
+                    bChanged = true;
                     if (!m_bShowEmpty && (grpItem->firstChild() == NULL))
                         delete grpItem;
                     grpItem = NULL;
@@ -509,6 +537,7 @@ void UserListBase::drawUpdates()
                         addGroupForUpdate(grpItem->id());
                         deleteItem(contactItem);
                         contactItem = NULL;
+                        bChanged = true;
                         if (m_bShowOnline && (grpItem->firstChild() == NULL)){
                             deleteItem(grpItem);
                             grpItem = NULL;
@@ -525,6 +554,7 @@ void UserListBase::drawUpdates()
             DivItem *divItem;
             if (status <= STATUS_OFFLINE){
                 if (itemOffline == NULL){
+                    bChanged = true;
                     itemOffline = new DivItem(this, DIV_OFFLINE);
                     setOpen(itemOffline, true);
                 }
@@ -537,6 +567,7 @@ void UserListBase::drawUpdates()
                 Group *grp = getContacts()->group(contact->getGroup());
                 if (grp == NULL)
                     break;
+                bChanged = true;
                 grpItem = new GroupItem(divItem, grp, true);
                 addSortItem(divItem);
             }
@@ -545,6 +576,7 @@ void UserListBase::drawUpdates()
                 if (contactItem->update(contact, status, style, icons.c_str(), unread))
                     addSortItem(grpItem);
             }else{
+                bChanged = true;
                 new ContactItem(grpItem, contact, status, style, icons.c_str(), unread);
                 grpItem->m_nContacts++;
                 addGroupForUpdate(grpItem->id());
@@ -553,9 +585,21 @@ void UserListBase::drawUpdates()
     }
     updContacts.clear();
     for (list<QListViewItem*>::iterator it_sort = sortItems.begin(); it_sort != sortItems.end(); ++it_sort){
+        if ((*it_sort)->firstChild() == NULL)
+            continue;
         (*it_sort)->sort();
+        bChanged = true;
     }
     sortItems.clear();
+    center(x, y, 0, 0);
+    viewport()->setUpdatesEnabled(true);
+    if (bChanged){
+        viewport()->repaint();
+    }else{
+        for (list<QListViewItem*>::iterator it = updatedItems.begin(); it != updatedItems.end(); ++it)
+            (*it)->repaint();
+    }
+    updatedItems.clear();
 }
 
 const unsigned UPDATE_TIME = 800;
@@ -622,6 +666,15 @@ void UserListBase::addSortItem(QListViewItem *item)
     sortItems.push_back(item);
 }
 
+void UserListBase::addUpdatedItem(QListViewItem *item)
+{
+    for (list<QListViewItem*>::iterator it = updatedItems.begin(); it != updatedItems.end(); ++it){
+        if ((*it) == item)
+            return;
+    }
+    updatedItems.push_back(item);
+}
+
 unsigned UserListBase::getUnread(unsigned)
 {
     return 0;
@@ -646,7 +699,7 @@ void UserListBase::fill()
         divItemOnline  = NULL;
         divItemOffline = NULL;
         while ((contact = ++contact_it) != NULL){
-            if (contact->getIgnore() || contact->getTemporary())
+            if (contact->getIgnore() || (contact->getFlags() & CONTACT_TEMPORARY))
                 continue;
             unsigned style;
             string icons;
@@ -683,7 +736,7 @@ void UserListBase::fill()
             grpItem = new GroupItem(this, list->group(0), true);
         }
         while ((contact = ++contact_it) != NULL){
-            if (contact->getIgnore() || contact->getTemporary())
+            if (contact->getIgnore() || (contact->getFlags() & CONTACT_TEMPORARY))
                 continue;
             unsigned style;
             string icons;
@@ -736,7 +789,7 @@ void UserListBase::fill()
             }
         }
         while ((contact = ++contact_it) != NULL){
-            if (contact->getIgnore() || contact->getTemporary())
+            if (contact->getIgnore() || (contact->getFlags() & CONTACT_TEMPORARY))
                 continue;
             unsigned style;
             string icons;
@@ -815,14 +868,14 @@ void *UserListBase::processEvent(Event *e)
             }
         case EventContactCreated:{
                 Contact *c = (Contact*)(e->param());
-                if (!c->getIgnore() && (c->getTemporary() == 0))
+                if (!c->getIgnore() && ((c->getFlags() & CONTACT_TEMPORARY) == 0))
                     addContactForUpdate(c->id());
                 break;
             }
         case EventContactStatus:
         case EventContactChanged:{
                 Contact *c = (Contact*)(e->param());
-                if (!c->getIgnore() && (c->getTemporary() == 0)){
+                if (!c->getIgnore() && ((c->getFlags() & CONTACT_TEMPORARY) == 0)){
                     addContactForUpdate(c->id());
                 }else{
                     Event e(EventContactDeleted, c);
@@ -1092,6 +1145,15 @@ int UserList::drawIndicator(QPainter *p, int x, QListViewItem *item, bool bState
     return x;
 }
 
+int UserListBase::heightItem(UserViewItemBase*)
+{
+    QFontMetrics fm(font());
+    int h = fm.height() + 4;
+    if (h < 22)
+        h = 22;
+    return h;
+}
+
 void UserList::contentsMouseReleaseEvent(QMouseEvent *e)
 {
     QListViewItem *list_item = itemAt(contentsToViewport(e->pos()));
@@ -1150,7 +1212,7 @@ void UserList::contentsMouseReleaseEvent(QMouseEvent *e)
     m_pressedItem = NULL;
 }
 
-#ifndef WIN32
+#ifndef _MSC_VER
 #include "userlist.moc"
 #endif
 
