@@ -68,6 +68,9 @@ UserView::UserView()
 
     setTreeStepSize(0);
 
+    setVScrollBarMode(CorePlugin::m_plugin->getNoScroller() ? QScrollView::AlwaysOff : QScrollView::Auto);
+    setHScrollBarMode(QScrollView::AlwaysOff);
+
     tipTimer = new QTimer(this);
     connect(tipTimer, SIGNAL(timeout()), this, SLOT(showTip()));
     blinkTimer = new QTimer(this);
@@ -149,11 +152,60 @@ void UserView::paintEmptyArea(QPainter *p, const QRect &r)
     setStaticBackground(pv.isStatic);
 }
 
-void UserView::drawItem(UserViewItemBase *base, QPainter *p, const QColorGroup &cg, int width, int margin)
+static void drawImage(QPainter *p, int x, int y, const QImage &img)
 {
+#if defined(WIN32) && (COMPAT_QT_VERSION < 0x030000)
+    if (p->device()->devType() != QInternal::Pixmap && pict->depth() < 32){
+        p->drawImage(x, y, img);
+        return;
+    }
+    QPixmap *pict = static_cast<QPixmap*>(p->device());
+    p->end();
+    QImage image = pict->convertToImage();
+    unsigned int *f = (unsigned int*)(img.bits());
+    unsigned int *t = (unsigned int*)(image.bits());
+    int w = img.width();
+    int h = img.height();
+    if (y < 0){
+        f -= img.width() * y;
+        h += y;
+        y = 0;
+    }
+    if (x < 0){
+        f -= x;
+        w += x;
+        x = 0;
+    }
+    if (h > image.height())
+        h = image.height();
+    if (w > image.width())
+        w = image.width();
+
+    t += (image.width() * y) + x;
+    for (int i = 0; i < h; i++){
+        for (int j = 0; j < w; j++){
+            unsigned char a = qAlpha(*f);
+            *t = qRgba((a * qRed(*f) + (0xFF - a) * qRed(*t)) >> 8,
+                       (a * qGreen(*f) + (0xFF - a) * qGreen(*t)) >> 8,
+                       (a * qBlue(*f) + (0xFF - a) * qBlue(*t)) >> 8, 0xFF);
+            f++;
+            t++;
+        }
+        t += (image.width() - w);
+        f += (img.width() - w);
+    }
+    pict->convertFromImage(image);
+    p->begin(pict);
+#else
+    p->drawImage(x, y, img);
+#endif
+}
+
+int UserView::heightItem(UserViewItemBase *base)
+{
+    QFont f(font());
+    int h = 0;
     if (base->type() == GRP_ITEM){
-        GroupItem *item = static_cast<GroupItem*>(base);
-        QFont f(font());
         if (CorePlugin::m_plugin->getSmallGroupFont()){
             int size = f.pixelSize();
             if (size <= 0){
@@ -163,8 +215,38 @@ void UserView::drawItem(UserViewItemBase *base, QPainter *p, const QColorGroup &
                 f.setPixelSize(size * 3 / 4);
             }
         }
-        f.setBold(true);
-        p->setFont(f);
+        h = 14;
+    }
+    if (base->type() == USR_ITEM){
+        ContactItem *item = static_cast<ContactItem*>(base);
+        string icons = item->text(CONTACT_ICONS).latin1();
+        while (!icons.empty()){
+            string icon = getToken(icons, ',');
+            const QImage *img = Image(icon.c_str());
+            if (img && (img->height() > h))
+                h = img->height();
+        }
+        if (item->m_unread){
+            CommandDef *def = CorePlugin::m_plugin->messageTypes.find(item->m_unread);
+            if (def){
+                const QImage *img = Image(def->icon);
+                if (img && (img->height() > h))
+                    h = img->height();
+            }
+        }
+    }
+    QFontMetrics fm(f);
+    int fh = fm.height();
+    if (fh > h)
+        h = fh;
+    return h + 2;
+}
+
+
+void UserView::drawItem(UserViewItemBase *base, QPainter *p, const QColorGroup &cg, int width, int margin)
+{
+    if (base->type() == GRP_ITEM){
+        GroupItem *item = static_cast<GroupItem*>(base);
         QString text;
         if (item->id()){
             Group *grp = getContacts()->group(item->id());
@@ -185,20 +267,35 @@ void UserView::drawItem(UserViewItemBase *base, QPainter *p, const QColorGroup &
             text += QString::number(item->m_nContacts);
             text += ")";
         }
-        const QPixmap &pict = Pict(item->isOpen() ? "expanded" : "collapsed");
-        p->drawPixmap(2 + margin, (item->height() - pict.height()) / 2, pict);
+        const QImage *img = Image(item->isOpen() ? "expanded" : "collapsed");
+        if (img)
+            drawImage(p, 2 + margin, (item->height() - img->height()) / 2, *img);
         int x = 24 + margin;
         if (!item->isOpen() && item->m_unread){
             CommandDef *def = CorePlugin::m_plugin->messageTypes.find(item->m_unread);
             if (def){
-                const QPixmap &pict = Pict(def->icon);
-                if (m_bUnreadBlink)
-                    p->drawPixmap(x, (item->height() - pict.height()) / 2, pict);
-                x += pict.width() + 2;
+                const QImage *img = Image(def->icon);
+                if (img){
+                    if (m_bUnreadBlink)
+                        drawImage(p, x, (item->height() - img->height()) / 2, *img);
+                    x += img->width() + 2;
+                }
             }
         }
         if (!CorePlugin::m_plugin->getUseSysColors())
             p->setPen(CorePlugin::m_plugin->getColorGroup());
+        QFont f(font());
+        if (CorePlugin::m_plugin->getSmallGroupFont()){
+            int size = f.pixelSize();
+            if (size <= 0){
+                size = f.pointSize();
+                f.setPointSize(size * 3 / 4);
+            }else{
+                f.setPixelSize(size * 3 / 4);
+            }
+        }
+        f.setBold(true);
+        p->setFont(f);
         x = item->drawText(p, x, width, text);
         if (CorePlugin::m_plugin->getGroupSeparator())
             item->drawSeparator(p, x, width, cg);
@@ -231,12 +328,6 @@ void UserView::drawItem(UserViewItemBase *base, QPainter *p, const QColorGroup &
             if (CorePlugin::m_plugin->getInvisibleStyle()  & STYLE_STRIKE)
                 f.setStrikeOut(true);
         }
-        if (item->m_bBlink){
-            f.setBold(true);
-        }else{
-            f.setBold(false);
-        }
-        p->setFont(f);
         string icons = item->text(CONTACT_ICONS).latin1();
         string icon = getToken(icons, ',');
         if (item->m_unread && m_bUnreadBlink){
@@ -246,10 +337,12 @@ void UserView::drawItem(UserViewItemBase *base, QPainter *p, const QColorGroup &
         }
         int x = margin;
         if (icon.length()){
-            const QPixmap &pict = Pict(icon.c_str());
-            x += 2;
-            p->drawPixmap(x, (item->height() - pict.height()) / 2, pict);
-            x += pict.width() + 2;
+            const QImage *img = Image(icon.c_str());
+            if (img){
+                x += 2;
+                drawImage(p, x, (item->height() - img->height()) / 2, *img);
+                x += img->width() + 2;
+            }
         }
         if (x < 24)
             x = 24;
@@ -260,6 +353,7 @@ void UserView::drawItem(UserViewItemBase *base, QPainter *p, const QColorGroup &
             }else{
                 switch (item->status()){
                 case STATUS_ONLINE:
+                    p->setPen(CorePlugin::m_plugin->getColorOnline());
                     break;
                 case STATUS_AWAY:
                     p->setPen(CorePlugin::m_plugin->getColorAway());
@@ -276,6 +370,12 @@ void UserView::drawItem(UserViewItemBase *base, QPainter *p, const QColorGroup &
                 }
             }
         }
+        if (item->m_bBlink){
+            f.setBold(true);
+        }else{
+            f.setBold(false);
+        }
+        p->setFont(f);
         QString highlight;
         QString text = item->text(CONTACT_TEXT);
         if (!m_search.isEmpty()){
@@ -309,11 +409,13 @@ void UserView::drawItem(UserViewItemBase *base, QPainter *p, const QColorGroup &
         unsigned xIcon = width;
         while (icons.length()){
             icon = getToken(icons, ',');
-            const QPixmap &pict = Pict(icon.c_str());
-            xIcon -= pict.width() + 2;
-            if (xIcon < (unsigned)x)
-                break;
-            p->drawPixmap(xIcon, (item->height() - pict.height()) / 2, pict);
+            const QImage *img = Image(icon.c_str());
+            if (img){
+                xIcon -= img->width() + 2;
+                if (xIcon < (unsigned)x)
+                    break;
+                drawImage(p, xIcon, (item->height() - img->height()) / 2, *img);
+            }
         }
         return;
     }
@@ -323,6 +425,9 @@ void UserView::drawItem(UserViewItemBase *base, QPainter *p, const QColorGroup &
 void *UserView::processEvent(Event *e)
 {
     switch (e->type()){
+    case EventRepaintView:
+        setVScrollBarMode(CorePlugin::m_plugin->getNoScroller() ? QScrollView::AlwaysOff : QScrollView::Auto);
+        break;
     case EventInit:
         m_bInit = true;
         fill();
@@ -359,7 +464,7 @@ void *UserView::processEvent(Event *e)
     case EventCommandExec:{
             CommandDef *cmd = (CommandDef*)(e->param());
             if (cmd->menu_id == MenuContact){
-                Contact *contact = getContacts()->contact((unsigned)(cmd->param));
+                Contact *contact = getContacts()->contact((unsigned long)(cmd->param));
                 if (contact){
                     if (cmd->id == CmdContactDelete){
                         QListViewItem *item = findContactItem(contact->id());
@@ -368,9 +473,11 @@ void *UserView::processEvent(Event *e)
                             QRect rc = itemRect(item);
                             QPoint p = viewport()->mapToGlobal(rc.topLeft());
                             rc = QRect(p.x(), p.y(), rc.width(), rc.height());
+                            m_bRemoveHistory = CorePlugin::m_plugin->getRemoveHistory();
                             BalloonMsg::ask((void*)contact->id(),
                                             i18n("Delete \"%1\"?") .arg(contact->getName()),
-                                            this, SLOT(deleteContact(void*)), NULL, &rc);
+                                            this, SLOT(deleteContact(void*)), NULL, &rc, NULL,
+                                            i18n("Remove history"), &m_bRemoveHistory);
                         }
                         return e->param();
                     }
@@ -404,7 +511,7 @@ void *UserView::processEvent(Event *e)
                         while ((w = it.current()) != NULL){
                             if (w->inherits("Container")){
                                 Container *c =  static_cast<Container*>(w);
-                                wnd = c->wnd((unsigned)(cmd->param));
+                                wnd = c->wnd((unsigned long)(cmd->param));
                                 if (wnd)
                                     break;
                             }
@@ -428,7 +535,7 @@ void *UserView::processEvent(Event *e)
                 }
             }
             if (cmd->menu_id == MenuContactGroup){
-                Contact *contact = getContacts()->contact((unsigned)(cmd->param));
+                Contact *contact = getContacts()->contact((unsigned long)(cmd->param));
                 if (contact){
                     Group *grp = getContacts()->group(cmd->id - CmdContactGroup);
                     if (grp && (grp->id() != contact->getGroup())){
@@ -440,7 +547,7 @@ void *UserView::processEvent(Event *e)
                 }
             }
             if (cmd->menu_id == MenuContainer){
-                Contact *contact = getContacts()->contact((unsigned)(cmd->param));
+                Contact *contact = getContacts()->contact((unsigned long)(cmd->param));
                 if (contact){
                     Container *from = NULL;
                     Container *to = NULL;
@@ -473,6 +580,7 @@ void *UserView::processEvent(Event *e)
                         userWnd = new UserWnd(contact->id(), NULL, true, true);
                     if (to == NULL)
                         to = new Container(max_id + 1);
+                    to->init();
                     to->addUserWnd(userWnd, true);
                     to->setNoSwitch(true);
                     raiseWindow(to);
@@ -505,6 +613,10 @@ void *UserView::processEvent(Event *e)
                 setGroupMode(2);
             if (cmd->id == CmdGrpCreate){
                 if (CorePlugin::m_plugin->getGroupMode()){
+                    /* Show empty groups because a new group is empty... */
+                    CorePlugin::m_plugin->setShowEmptyGroup(true);
+                    m_bShowEmpty = true;
+                    fill();
                     Group *g = getContacts()->group(0, true);
                     drawUpdates();
                     QListViewItem *item = findGroupItem(g->id());
@@ -516,7 +628,7 @@ void *UserView::processEvent(Event *e)
                 return e->param();
             }
             if (cmd->id == CmdGrpRename){
-                QListViewItem *item = findGroupItem((unsigned)(cmd->param));
+                QListViewItem *item = findGroupItem((unsigned long)(cmd->param));
                 if (item){
                     setCurrentItem(item);
                     renameGroup();
@@ -524,7 +636,7 @@ void *UserView::processEvent(Event *e)
                 return e->param();
             }
             if (cmd->id == CmdGrpUp){
-                unsigned grp_id = (unsigned)(cmd->param);
+                unsigned long grp_id = (unsigned long)(cmd->param);
                 getContacts()->moveGroup(grp_id, true);
                 QListViewItem *item = findGroupItem(grp_id);
                 if (item){
@@ -534,7 +646,7 @@ void *UserView::processEvent(Event *e)
                 return e->param();
             }
             if (cmd->id == CmdGrpDown){
-                unsigned grp_id = (unsigned)(cmd->param);
+                unsigned long grp_id = (unsigned long)(cmd->param);
                 getContacts()->moveGroup(grp_id, false);
                 QListViewItem *item = findGroupItem(grp_id);
                 if (item){
@@ -544,7 +656,7 @@ void *UserView::processEvent(Event *e)
                 return e->param();
             }
             if (cmd->id == CmdGrpDelete){
-                unsigned grp_id = (unsigned)(cmd->param);
+                unsigned long grp_id = (unsigned long)(cmd->param);
                 QListViewItem *item = findGroupItem(grp_id);
                 Group *g = getContacts()->group(grp_id);
                 if (item && g){
@@ -578,14 +690,14 @@ void *UserView::processEvent(Event *e)
             }
             if (cmd->menu_id == MenuContact){
                 if (cmd->id == CmdContactTitle){
-                    Contact *contact = getContacts()->contact((unsigned)(cmd->param));
+                    Contact *contact = getContacts()->contact((unsigned long)(cmd->param));
                     if (contact){
                         cmd->text_wrk = strdup(contact->getName().utf8());
                         return e->param();
                     }
                 }
                 if (cmd->id == CmdShowAlways){
-                    Contact *contact = getContacts()->contact((unsigned)(cmd->param));
+                    Contact *contact = getContacts()->contact((unsigned long)(cmd->param));
                     if (contact){
                         ListUserData *data = (ListUserData*)(contact->getUserData(CorePlugin::m_plugin->list_data_id, true));
                         cmd->flags &= ~COMMAND_CHECKED;
@@ -601,7 +713,7 @@ void *UserView::processEvent(Event *e)
                     QWidget * w;
                     while ((w = it.current()) != NULL){
                         if (w->inherits("Container")){
-                            wnd = static_cast<Container*>(w)->wnd((unsigned)(cmd->param));
+                            wnd = static_cast<Container*>(w)->wnd((unsigned long)(cmd->param));
                             if (wnd)
                                 break;
                         }
@@ -654,7 +766,7 @@ void *UserView::processEvent(Event *e)
             if (cmd->menu_id == MenuContactGroup){
                 if (cmd->id == CmdContactGroup){
                     unsigned grpId = 0;
-                    Contact *contact = getContacts()->contact((unsigned)(cmd->param));
+                    Contact *contact = getContacts()->contact((unsigned long)(cmd->param));
                     if (contact)
                         grpId = contact->getGroup();
                     unsigned nGroups = 0;
@@ -689,7 +801,7 @@ void *UserView::processEvent(Event *e)
                 }
             }
             if (cmd->menu_id == MenuGroup){
-                unsigned grp_id = (unsigned)(cmd->param);
+                unsigned long grp_id = (unsigned long)(cmd->param);
                 if (grp_id){
                     if (cmd->id == CmdGrpTitle){
                         Group *g = getContacts()->group(grp_id);
@@ -720,8 +832,10 @@ void *UserView::processEvent(Event *e)
                     }
                 }
             }
-            if (cmd->id == CmdGrpCreate)
+            if (cmd->id == CmdGrpCreate) {
+                cmd->flags &= ~COMMAND_CHECKED;
                 return CorePlugin::m_plugin->getGroupMode() ? e->param() : NULL;
+            }
             break;
         }
     case EventIconChanged:
@@ -739,19 +853,22 @@ void *UserView::processEvent(Event *e)
 
 void UserView::deleteGroup(void *p)
 {
-    Group *grp = getContacts()->group((unsigned)p);
+    Group *grp = getContacts()->group((unsigned long)p);
     if (grp)
         delete grp;
 }
 
 void UserView::deleteContact(void *p)
 {
-    Contact *contact = getContacts()->contact((unsigned)p);
+    Contact *contact = getContacts()->contact((unsigned long)p);
     if (contact == NULL)
         return;
     ContactItem *item = findContactItem(contact->id());
     if (item)
         setCurrentItem(item);
+    CorePlugin::m_plugin->setRemoveHistory(m_bRemoveHistory);
+    if (!m_bRemoveHistory)
+        contact->setFlags(contact->getFlags() | CONTACT_NOREMOVE_HISTORY);
     delete contact;
 }
 
@@ -895,8 +1012,23 @@ void UserView::doClick()
 void UserView::keyPressEvent(QKeyEvent *e)
 {
     if (CorePlugin::m_plugin->getUseDblClick() || m_searchItem){
-        if (m_searchItem)
-            setCurrentItem(m_searchItem);
+        if (m_searchItem) {
+	    int store = 0;
+	    list<QListViewItem*> items;
+	    list<QListViewItem*>::iterator it;
+            search(items);
+	    if (!items.empty()) {
+	        for (it = items.begin(); it != items.end(); ++it)
+  	            if (*it == m_searchItem) {
+		    store = 1;
+	        }
+	        if (!store) m_searchItem = items.front();
+	    } else {
+	        m_search = "";
+	        m_searchItem = NULL;
+            }
+	    setCurrentItem(m_searchItem);
+	}
         switch (e->key()){
         case Key_Return:
         case Key_Enter:
@@ -987,6 +1119,10 @@ void UserView::keyPressEvent(QKeyEvent *e)
                 return;
             }
         }
+    case Key_Delete:
+        // e->text() is not empty, but we don't need to specially handle Del 
+        UserListBase::keyPressEvent(e);
+        return;
     default:
         QString t = e->text();
         if (t.isEmpty()){
@@ -1317,22 +1453,13 @@ DWORD __stdcall DragScrollThread(LPVOID)
 UserViewContactDragObject::UserViewContactDragObject(UserView *view, Contact *contact)
         : ContactDragObject(view, contact)
 {
-#ifdef WIN32
-    dragView = view;
-    DWORD threadId;
-    CreateThread(NULL, 0, DragScrollThread, NULL, 0, &threadId);
-#else
-    QTimer *dragTimer = new QTimer(this);
-    connect(dragTimer, SIGNAL(timeout()), view, SLOT(dragScroll()));
-    dragTimer->start(200);
-#endif
+	QTimer *dragTimer = new QTimer(this);
+	connect(dragTimer, SIGNAL(timeout()), view, SLOT(dragScroll()));
+	dragTimer->start(200);
 }
 
 UserViewContactDragObject::~UserViewContactDragObject()
 {
-#ifdef WIN32
-    dragView = NULL;
-#endif
 }
 
 QDragObject *UserView::dragObject()
@@ -1379,7 +1506,7 @@ void UserView::dragEvent(QDropEvent *e, bool isDrop)
                 Contact *contact = ContactDragObject::decode(e);
                 m_dropItem = item;
                 m_dropContactId = contact->id();
-                contact->setTemporary(contact->getTemporary() & ~CONTACT_DRAG);
+                contact->setFlags(contact->getFlags() & ~CONTACT_DRAG);
                 QTimer::singleShot(0, this, SLOT(doDrop()));
             }
             e->accept();
@@ -1396,7 +1523,7 @@ void UserView::dragEvent(QDropEvent *e, bool isDrop)
                 if (isDrop){
                     m_dropItem = item;
                     m_dropContactId = contact->id();
-                    contact->setTemporary(contact->getTemporary() & ~CONTACT_DRAG);
+                    contact->setFlags(contact->getFlags() & ~CONTACT_DRAG);
                     QTimer::singleShot(0, this, SLOT(doDrop()));
                 }
                 e->accept();
@@ -1463,7 +1590,7 @@ void UserView::doDrop()
             GroupItem *grp_item = static_cast<GroupItem*>(m_dropItem);
             contact->setGroup(grp_item->id());
             contact->setIgnore(false);
-            contact->setTemporary(0);
+            contact->setFlags(contact->getFlags() & ~CONTACT_TEMPORARY);
             Event eContact(EventContactChanged, contact);
             eContact.process();
             break;
@@ -1523,7 +1650,7 @@ void UserView::joinContacts(void*)
 void UserView::cancelJoinContacts(void*)
 {
     Contact *contact2 = getContacts()->contact(joinContactsData.contact2);
-    if (contact2 && contact2->getTemporary())
+    if (contact2 && (contact2->getFlags() & CONTACT_TEMPORARY))
         delete contact2;
 }
 
@@ -1584,7 +1711,7 @@ void UserView::dragScroll()
         ensureItemVisible(item);
 }
 
-#ifndef WIN32
+#ifndef _MSC_VER
 #include "userview.moc"
 #endif
 
