@@ -35,7 +35,7 @@ static DataDef userWndData[] =
         { NULL, 0, 0, 0 }
     };
 
-UserWnd::UserWnd(unsigned id, const char *cfg, bool bReceived, bool bAdjust)
+UserWnd::UserWnd(unsigned long id, Buffer *cfg, bool bReceived, bool bAdjust)
         : QSplitter(Horizontal, NULL)
 {
     load_data(userWndData, &data, cfg);
@@ -83,7 +83,7 @@ UserWnd::~UserWnd()
     emit closed(this);
     free_data(userWndData, &data);
     Contact *contact = getContacts()->contact(id());
-    if (contact && contact->getTemporary()){
+    if (contact && (contact->getFlags() & CONTACT_TEMPORARY)){
         m_id = 0;
         delete contact;
     }
@@ -294,7 +294,7 @@ void UserWnd::markAsRead()
     }
 }
 
-#ifndef WIN32
+#ifndef _MSC_VER
 #include "userwnd.moc"
 #endif
 
