@@ -188,7 +188,7 @@ void ActionConfig::help()
     BalloonMsg::message(helpString, btnHelp, false, 400);
 }
 
-#ifndef WIN32
+#ifndef _MSC_VER
 #include "actioncfg.moc"
 #endif
 
