@@ -40,7 +40,7 @@ class DockPlugin : public QObject, public Plugin, public EventReceiver
 {
     Q_OBJECT
 public:
-    DockPlugin(unsigned, const char*);
+    DockPlugin(unsigned, Buffer*);
     virtual ~DockPlugin();
 protected slots:
     void showPopup(QPoint);
@@ -55,10 +55,10 @@ protected:
     DockWnd *dock;
     QWidget *getMainWindow();
     bool isMainShow();
-    unsigned DockMenu;
-    unsigned CmdTitle;
-    unsigned CmdToggle;
-    unsigned CmdCustomize;
+    unsigned long DockMenu;
+    unsigned long CmdTitle;
+    unsigned long CmdToggle;
+    unsigned long CmdCustomize;
     QPopupMenu *m_popup;
     bool bQuit;
     PROP_BOOL(AutoHide);
@@ -70,7 +70,7 @@ protected:
 #endif
     PROP_ULONG(Desktop);
     CorePlugin *core;
-    unsigned inactiveTime;
+    time_t inactiveTime;
     void init();
     DockData data;
     friend class DockCfg;
