@@ -23,6 +23,7 @@
 typedef struct FilterData
 {
     Data	FromList;
+    Data	AuthFromList;
 } FilterData;
 
 typedef struct FilterUserData
@@ -36,16 +37,17 @@ class FilterPlugin : public QObject, public Plugin, public EventReceiver
 {
     Q_OBJECT
 public:
-    FilterPlugin(unsigned, const char *cfg);
+    FilterPlugin(unsigned, Buffer *cfg);
     virtual ~FilterPlugin();
     PROP_BOOL(FromList);
+    PROP_BOOL(AuthFromList);
 protected slots:
     void addToIgnore(void*);
 protected:
-    unsigned user_data_id;
-    unsigned CmdIgnoreList;
-    unsigned CmdIgnore;
-    unsigned CmdIgnoreText;
+    unsigned long user_data_id;
+    unsigned long CmdIgnoreList;
+    unsigned long CmdIgnore;
+    unsigned long CmdIgnoreText;
     virtual void *processEvent(Event*);
     virtual QWidget *createConfigWindow(QWidget *parent);
     virtual string getConfig();
