@@ -32,6 +32,7 @@ FilterConfig::FilterConfig(QWidget *parent, FilterUserData *data, FilterPlugin *
     m_ignore = NULL;
     if (bMain){
         chkFromList->setChecked(m_plugin->getFromList());
+        chkAuthFromList->setChecked(m_plugin->getAuthFromList());
         for (QObject *p = parent; p != NULL; p = p->parent()){
             if (!p->inherits("QTabWidget"))
                 continue;
@@ -42,6 +43,7 @@ FilterConfig::FilterConfig(QWidget *parent, FilterUserData *data, FilterPlugin *
         }
     }else{
         chkFromList->hide();
+        chkAuthFromList->hide();
         lblFilter->hide();
     }
     if (data->SpamList.ptr)
@@ -57,6 +59,7 @@ FilterConfig::~FilterConfig()
 void FilterConfig::apply()
 {
     m_plugin->setFromList(chkFromList->isChecked());
+    m_plugin->setAuthFromList(chkAuthFromList->isChecked());
     apply(m_data);
 }
 
@@ -66,7 +69,7 @@ void FilterConfig::apply(void *_data)
     set_str(&data->SpamList.ptr, edtFilter->text().utf8());
 }
 
-#ifndef WIN32
+#ifndef _MSC_VER
 #include "filtercfg.moc"
 #endif
 
