@@ -23,6 +23,7 @@
 #include "textshow.h"
 #include "userwnd.h"
 #include "exec.h"
+#include "passphrase.h"
 
 #include <qtimer.h>
 #include <qfile.h>
@@ -37,7 +38,7 @@
 static string GPGpath;
 #endif
 
-Plugin *createGpgPlugin(unsigned base, bool, const char *cfg)
+Plugin *createGpgPlugin(unsigned base, bool, Buffer *cfg)
 {
 #ifndef WIN32
     if (GPGpath.empty())
@@ -98,6 +99,10 @@ static DataDef gpgData[] =
         { "Encrypt", DATA_STRING, 1, "--batch --yes --armor --comment \"\" --no-version --recipient \"%userid%\" --trusted-key \"%userid%\" --output \"%cipherfile%\" --encrypt \"%plainfile%\"" },
         { "Decrypt", DATA_STRING, 1, "--yes --passphrase-fd 0 --output \"%plainfile%\" --decrypt \"%cipherfile%\"" },
         { "Key", DATA_STRING, 1, 0 },
+        { "Passphrases", DATA_UTFLIST, 1, 0 },
+        { "Keys", DATA_STRLIST, 1, 0 },
+        { "NPassphrases", DATA_ULONG, 1, 0 },
+        { "", DATA_BOOL, 1, 0 },
         { NULL, 0, 0, 0 }
     };
 
@@ -110,11 +115,12 @@ static DataDef gpgUserData[] =
 
 GpgPlugin *GpgPlugin::plugin = NULL;
 
-GpgPlugin::GpgPlugin(unsigned base, const char *cfg)
+GpgPlugin::GpgPlugin(unsigned base, Buffer *cfg)
         : Plugin(base), EventReceiver(HighestPriority - 0x100)
 {
     load_data(gpgData, &data, cfg);
     m_bMessage = false;
+    m_passphraseDlg = NULL;
     user_data_id = getContacts()->registerUserData(info.title, gpgUserData);
     reset();
     plugin = this;
@@ -122,6 +128,8 @@ GpgPlugin::GpgPlugin(unsigned base, const char *cfg)
 
 GpgPlugin::~GpgPlugin()
 {
+    if (m_passphraseDlg)
+        delete m_passphraseDlg;
     unregisterMessage();
     free_data(gpgData, &data);
     list<DecryptMsg>::iterator it;
@@ -137,6 +145,8 @@ GpgPlugin::~GpgPlugin()
     }
     for (it = m_public.begin(); it != m_public.end(); ++it)
         delete (*it).exec;
+    for (it = m_wait.begin(); it != m_wait.end(); ++it)
+        delete (*it).msg;
     getContacts()->unregisterUserData(user_data_id);
 }
 
@@ -158,6 +168,8 @@ void GpgPlugin::clear()
             continue;
         }
         delete (*it).exec;
+        QFile::remove((*it).infile);
+        QFile::remove((*it).outfile);
         m_decrypt.erase(it);
         it = m_decrypt.begin();
     }
@@ -167,6 +179,8 @@ void GpgPlugin::clear()
             continue;
         }
         delete (*it).exec;
+        QFile::remove((*it).infile);
+        QFile::remove((*it).outfile);
         m_import.erase(it);
         it = m_import.begin();
     }
@@ -176,6 +190,8 @@ void GpgPlugin::clear()
             continue;
         }
         delete (*it).exec;
+        QFile::remove((*it).infile);
+        QFile::remove((*it).outfile);
         m_public.erase(it);
         it = m_public.begin();
     }
@@ -185,26 +201,102 @@ void GpgPlugin::decryptReady(Exec *exec, int res, const char*)
 {
     for (list<DecryptMsg>::iterator it = m_decrypt.begin(); it != m_decrypt.end(); ++it){
         if ((*it).exec == exec){
+            Message *msg = (*it).msg;
+            (*it).msg = NULL;
+            QTimer::singleShot(0, this, SLOT(clear()));
             if (res == 0){
                 QFile f((*it).outfile);
                 if (f.open(IO_ReadOnly)){
                     string text;
                     text.append(f.size(), '\x00');
                     f.readBlock((char*)(text.c_str()), f.size());
-                    (*it).msg->setText(text.c_str());
-                    (*it).msg->setFlags((*it).msg->getFlags() | MESSAGE_SECURE);
+                    msg->setText(text.c_str());
+                    msg->setFlags(msg->getFlags() | MESSAGE_SECURE);
                 }else{
                     string s;
                     s = (*it).outfile.local8Bit();
                     log(L_WARN, "Can't open output decrypt file %s", s.c_str());
                     res = -1;
                 }
+                if (!(*it).key.empty()){
+                    unsigned i = 1;
+                    for (i = 1; i <= getnPassphrases(); i++){
+                        if ((*it).key == getKeys(i))
+                            break;
+                    }
+                    if (i > getnPassphrases()){
+                        setnPassphrases(i);
+                        setKeys(i, (*it).key.c_str());
+                    }
+                    setPassphrases(i, (*it).passphrase);
+                    for (;;){
+                        list<DecryptMsg>::iterator itw;
+                        bool bDecode = false;
+                        for (itw = m_wait.begin(); itw != m_wait.end(); ++itw){
+                            if ((*itw).key == (*it).key){
+                                decode((*itw).msg, (*it).passphrase.utf8(), (*it).key.c_str());
+                                m_wait.erase(itw);
+                                bDecode = true;
+                                break;
+                            }
+                        }
+                        if (!bDecode)
+                            break;
+                    }
+                    if (m_passphraseDlg && ((*it).key == m_passphraseDlg->m_key)){
+                        delete m_passphraseDlg;
+                        m_passphraseDlg = NULL;
+                        askPassphrase();
+                    }
+                }
+            }else{
+                string key;
+                string res;
+                QString passphrase;
+                exec->bErr.scan("\n", key);
+                if (exec->bErr.scan("bad passphrase", res)){
+                    int n = key.find("ID ");
+                    if (n > 0)
+                        key = key.substr(n + 3);
+                    key = getToken(key, ' ');
+                    key = getToken(key, ',');
+                    if (m_passphraseDlg && ((*it).key == m_passphraseDlg->m_key)){
+                        DecryptMsg m;
+                        m.msg    = msg;
+                        m.key    = key;
+                        m_wait.push_back(m);
+                        m_passphraseDlg->error();
+                        return;
+                    }
+                    if ((*it).passphrase.isEmpty()){
+                        for (unsigned i = 1; i <= getnPassphrases(); i++){
+                            if (key == getKeys(i)){
+                                passphrase = getPassphrases(i);
+                                break;
+                            }
+                        }
+                    }
+                    if ((*it).passphrase.isEmpty() && !passphrase.isEmpty()){
+                        if (decode(msg, passphrase.utf8(), key.c_str()))
+                            return;
+                    }else{
+                        DecryptMsg m;
+                        m.msg    = msg;
+                        m.key    = key;
+                        m_wait.push_back(m);
+                        (*it).msg = NULL;
+                        QTimer::singleShot(0, this, SLOT(clear()));
+                        askPassphrase();
+                        return;
+                    }
+                }else{
+                    if (m_passphraseDlg && ((*it).key == m_passphraseDlg->m_key)){
+                        delete m_passphraseDlg;
+                        m_passphraseDlg = NULL;
+                        askPassphrase();
+                    }
+                }
             }
-            Message *msg = (*it).msg;
-            (*it).msg = NULL;
-            QFile::remove((*it).infile);
-            QFile::remove((*it).outfile);
-            QTimer::singleShot(0, this, SLOT(clear()));
             Event e(EventMessageReceived, msg);
             if ((res == 0) && processEvent(&e))
                 return;
@@ -247,7 +339,7 @@ void GpgPlugin::importReady(Exec *exec, int res, const char*)
                     QString gpg;
                     gpg += "\"";
                     gpg += QFile::decodeName(GPG());
-                    gpg += "\" --homedir \"";
+                    gpg += "\" --no-tty --homedir \"";
                     gpg += home;
                     gpg += "\" ";
                     gpg += getPublicList();
@@ -264,7 +356,6 @@ void GpgPlugin::importReady(Exec *exec, int res, const char*)
             if (!e.process(this))
                 delete (*it).msg;
             (*it).msg = NULL;
-            QFile::remove((*it).infile);
             QTimer::singleShot(0, this, SLOT(clear()));
             return;
         }
@@ -274,7 +365,23 @@ void GpgPlugin::importReady(Exec *exec, int res, const char*)
 
 string GpgPlugin::getConfig()
 {
-    return save_data(gpgData, &data);
+    QStringList keys;
+    QStringList passphrases;
+    unsigned i;
+    for (i = 1; i <= getnPassphrases(); i++){
+        keys.append(getKeys(i));
+        passphrases.append(getPassphrases(i));
+    }
+    if (!getSavePassphrase()){
+        clearKeys();
+        clearPassphrases();
+    }
+    string res = save_data(gpgData, &data);
+    for (i = 0; i < getnPassphrases(); i++){
+        setKeys(i + 1, keys[i].latin1());
+        setPassphrases(i + 1, passphrases[i]);
+    }
+    return res;
 }
 
 void *GpgPlugin::processEvent(Event *e)
@@ -292,7 +399,7 @@ void *GpgPlugin::processEvent(Event *e)
                 }
                 if (cmd->id == MessageGPGUse){
                     cmd->flags &= ~COMMAND_CHECKED;
-                    Contact *contact = getContacts()->contact((unsigned)(cmd->param));
+                    Contact *contact = getContacts()->contact((unsigned long)(cmd->param));
                     if (contact == NULL)
                         return NULL;
                     GpgUserData *data = (GpgUserData*)(contact->userData.getUserData(user_data_id, false));
@@ -308,7 +415,7 @@ void *GpgPlugin::processEvent(Event *e)
     case EventCommandExec:{
             CommandDef *cmd = (CommandDef*)(e->param());
             if ((cmd->menu_id == MenuMessage) && (cmd->id == MessageGPGUse)){
-                Contact *contact = getContacts()->contact((unsigned)(cmd->param));
+                Contact *contact = getContacts()->contact((unsigned long)(cmd->param));
                 if (contact == NULL)
                     return NULL;
                 GpgUserData *data = (GpgUserData*)(contact->userData.getUserData(user_data_id, false));
@@ -369,7 +476,7 @@ void *GpgPlugin::processEvent(Event *e)
                     GpgUserData *data = (GpgUserData*)(contact->userData.getUserData(user_data_id, false));
                     if (data && data->Key.ptr && data->Use.bValue){
                         QString output = QFile::decodeName(user_file("m.").c_str());
-                        output += QString::number((unsigned)ms->msg);
+                        output += QString::number((unsigned long)ms->msg);
                         QString input = output + ".in";
                         QFile in(input);
                         if (!in.open(IO_WriteOnly | IO_Truncate)){
@@ -386,7 +493,7 @@ void *GpgPlugin::processEvent(Event *e)
                         QString gpg;
                         gpg += "\"";
                         gpg += QFile::decodeName(GPG());
-                        gpg += "\" --homedir \"";
+                        gpg += "\" --no-tty --homedir \"";
                         gpg += home;
                         gpg += "\" ";
                         gpg += getEncrypt();
@@ -397,16 +504,22 @@ void *GpgPlugin::processEvent(Event *e)
                         exec.execute(gpg.local8Bit(), "\n", true);
                         if (exec.result){
                             ms->msg->setError(I18N_NOOP("Encrypt failed"));
+                            QFile::remove(input);
+                            QFile::remove(output);
                             return ms->msg;
                         }
+                        QFile::remove(input);
                         QFile out(output);
                         if (!out.open(IO_ReadOnly)){
+                            QFile::remove(output);
                             ms->msg->setError(I18N_NOOP("Encrypt failed"));
                             return ms->msg;
                         }
                         *ms->text = "";
                         ms->text->append(out.size(), '\x00');
                         out.readBlock((char*)(ms->text->c_str()), out.size());
+                        out.close();
+                        QFile::remove(output);
                         return NULL;
                     }
                 }
@@ -422,45 +535,13 @@ void *GpgPlugin::processEvent(Event *e)
                 char SIGN_MSG[] = "-----BEGIN PGP MESSAGE-----";
                 char SIGN_KEY[] = "-----BEGIN PGP PUBLIC KEY BLOCK-----";
                 if (text.left(strlen(SIGN_MSG)) == SIGN_MSG){
-                    QString output = QFile::decodeName(user_file("m.").c_str());
-                    output += QString::number((unsigned)msg);
-                    QString input = output + ".in";
-                    QFile in(input);
-                    if (!in.open(IO_WriteOnly | IO_Truncate)){
-                        string s;
-                        s = input.local8Bit();
-                        log(L_WARN, "Can't create %s", s.c_str());
-                        return NULL;
-                    }
-                    string t;
-                    t = text.latin1();
-                    in.writeBlock(t.c_str(), t.length());
-                    in.close();
-                    QString home = QFile::decodeName(user_file(GpgPlugin::plugin->getHome()).c_str());
-                    if (home[(int)(home.length() - 1)] == '\\')
-                        home = home.left(home.length() - 1);
-                    QString gpg;
-                    gpg += "\"";
-                    gpg += QFile::decodeName(GPG());
-                    gpg += "\" --homedir \"";
-                    gpg += home;
-                    gpg += "\" ";
-                    gpg += getDecrypt();
-                    gpg = gpg.replace(QRegExp("\\%plainfile\\%"), output);
-                    gpg = gpg.replace(QRegExp("\\%cipherfile\\%"), input);
-                    DecryptMsg dm;
-                    dm.exec = new Exec;
-                    dm.msg  = msg;
-                    dm.infile  = input;
-                    dm.outfile = output;
-                    m_decrypt.push_back(dm);
-                    connect(dm.exec, SIGNAL(ready(Exec*,int,const char*)), this, SLOT(decryptReady(Exec*,int,const char*)));
-                    dm.exec->execute(gpg.local8Bit(), "\n");
-                    return msg;
+                    if (decode(msg, "", ""))
+                        return msg;
+                    return NULL;
                 }
                 if (text.left(strlen(SIGN_KEY)) == SIGN_KEY){
                     QString input = QFile::decodeName(user_file("m.").c_str());
-                    input  += QString::number((unsigned)msg);
+                    input  += QString::number((unsigned long)msg);
                     input += ".in";
                     QFile in(input);
                     if (!in.open(IO_WriteOnly | IO_Truncate)){
@@ -479,7 +560,7 @@ void *GpgPlugin::processEvent(Event *e)
                     QString gpg;
                     gpg += "\"";
                     gpg += QFile::decodeName(GPG());
-                    gpg += "\" --homedir \"";
+                    gpg += "\" --no-tty --homedir \"";
                     gpg += home;
                     gpg += "\" ";
                     gpg += getImport();
@@ -500,6 +581,51 @@ void *GpgPlugin::processEvent(Event *e)
     return NULL;
 }
 
+static unsigned decode_index = 0;
+
+bool GpgPlugin::decode(Message *msg, const char *aPassphrase, const char *key)
+{
+    QString output = QFile::decodeName(user_file("md.").c_str());
+    output += QString::number(decode_index++);
+    QString input = output + ".in";
+    QFile in(input);
+    if (!in.open(IO_WriteOnly | IO_Truncate)){
+        string s;
+        s = input.local8Bit();
+        log(L_WARN, "Can't create %s", s.c_str());
+        return false;
+    }
+    string t = msg->getPlainText().latin1();
+    in.writeBlock(t.c_str(), t.length());
+    in.close();
+    QString home = QFile::decodeName(user_file(GpgPlugin::plugin->getHome()).c_str());
+    if (home[(int)(home.length() - 1)] == '\\')
+        home = home.left(home.length() - 1);
+    QString gpg;
+    gpg += "\"";
+    gpg += QFile::decodeName(GPG());
+    gpg += "\" --no-tty --homedir \"";
+    gpg += home;
+    gpg += "\" ";
+    gpg += getDecrypt();
+    gpg = gpg.replace(QRegExp("\\%plainfile\\%"), output);
+    gpg = gpg.replace(QRegExp("\\%cipherfile\\%"), input);
+    DecryptMsg dm;
+    dm.exec = new Exec;
+    dm.exec->setCLocale(true);
+    dm.msg  = msg;
+    dm.infile  = input;
+    dm.outfile = output;
+    dm.passphrase = QString::fromUtf8(aPassphrase);
+    dm.key = key;
+    m_decrypt.push_back(dm);
+    string passphrase = aPassphrase;
+    passphrase += "\n";
+    connect(dm.exec, SIGNAL(ready(Exec*,int,const char*)), this, SLOT(decryptReady(Exec*,int,const char*)));
+    dm.exec->execute(gpg.local8Bit(), passphrase.c_str());
+    return true;
+}
+
 void GpgPlugin::publicReady(Exec *exec, int res, const char*)
 {
     for (list<DecryptMsg>::iterator it = m_public.begin(); it != m_public.end(); ++it){
@@ -540,6 +666,21 @@ void GpgPlugin::publicReady(Exec *exec, int res, const char*)
     }
 }
 
+void GpgPlugin::passphraseApply(const QString &passphrase)
+{
+    for (list<DecryptMsg>::iterator it = m_wait.begin(); it != m_wait.end(); ++it){
+        if ((*it).key == m_passphraseDlg->m_key){
+            Message *msg = (*it).msg;
+            m_wait.erase(it);
+            decode(msg, passphrase.utf8(), m_passphraseDlg->m_key.c_str());
+            return;
+        }
+    }
+    delete m_passphraseDlg;
+    m_passphraseDlg = NULL;
+    askPassphrase();
+}
+
 QWidget *GpgPlugin::createConfigWindow(QWidget *parent)
 {
     return new GpgCfg(parent, this);
@@ -561,7 +702,7 @@ void GpgPlugin::reset()
 i18n("%n GPG key", "%n GPG keys", 1);
 #endif
 
-static Message *createGPGKey(const char *cfg)
+static Message *createGPGKey(Buffer *cfg)
 {
     return new Message(MessageGPGKey, cfg);
 }
@@ -642,6 +783,35 @@ void GpgPlugin::unregisterMessage()
     eUser.process();
 }
 
+void GpgPlugin::askPassphrase()
+{
+    if (m_passphraseDlg || m_wait.empty())
+        return;
+    m_passphraseDlg = new PassphraseDlg(this, m_wait.front().key.c_str());
+    connect(m_passphraseDlg, SIGNAL(finished()), this, SLOT(passphraseFinished()));
+    connect(m_passphraseDlg, SIGNAL(apply(const QString&)), this, SLOT(passphraseApply(const QString&)));
+    raiseWindow(m_passphraseDlg);
+}
+
+void GpgPlugin::passphraseFinished()
+{
+    if (m_passphraseDlg){
+        for (list<DecryptMsg>::iterator it = m_wait.begin(); it != m_wait.end();){
+            if ((*it).key != m_passphraseDlg->m_key){
+                ++it;
+                continue;
+            }
+            Event e(EventMessageReceived, (*it).msg);
+            if (!e.process(this))
+                delete (*it).msg;
+            m_wait.erase(it);
+            it = m_wait.begin();
+        }
+    }
+    m_passphraseDlg = NULL;
+    askPassphrase();
+}
+
 MsgGPGKey::MsgGPGKey(MsgEdit *parent, Message *msg)
         : QObject(parent)
 {
@@ -666,7 +836,7 @@ MsgGPGKey::MsgGPGKey(MsgEdit *parent, Message *msg)
         home = home.left(home.length() - 1);
 
     gpg = QString("\"") + gpg + "\"";
-    gpg += " --homedir \"";
+    gpg += " --no-tty --homedir \"";
     gpg += home;
     gpg += "\" ";
     gpg += GpgPlugin::plugin->getExport();
@@ -781,7 +951,7 @@ extern "C" BOOL __stdcall _DllMainCRTStartup( HINSTANCE hinstDLL, DWORD fdwReaso
 }
 #endif
 
-#ifndef WIN32
+#ifndef _MSC_VER
 #include "gpg.moc"
 #endif
 
