@@ -19,10 +19,12 @@
 #define _GPG_H
 
 #include "simapi.h"
-#include "stl.h"
 
-const unsigned MessageGPGKey	= 0x5000;
-const unsigned MessageGPGUse	= 0x5001;
+#include <list>
+#include <string>
+
+const unsigned long MessageGPGKey	= 0x5000;
+const unsigned long MessageGPGUse	= 0x5001;
 
 typedef struct GpgData
 {
@@ -36,6 +38,10 @@ typedef struct GpgData
     Data	Encrypt;
     Data	Decrypt;
     Data	Key;
+    Data	Passphrases;
+    Data	Keys;
+    Data	nPassphrases;
+    Data	SavePassphrase;
 } GpgData;
 
 typedef struct GpgUserData
@@ -53,6 +59,8 @@ typedef struct DecryptMsg
     QString		infile;
     QString		outfile;
     unsigned	contact;
+    QString		passphrase;
+    string		key;
 } DecryptMsg;
 
 typedef struct KeyMsg
@@ -61,11 +69,13 @@ typedef struct KeyMsg
     Message	*msg;
 } KeyMsg;
 
+class PassphraseDlg;
+
 class GpgPlugin : public QObject, public Plugin, public EventReceiver
 {
     Q_OBJECT
 public:
-    GpgPlugin(unsigned, const char*);
+    GpgPlugin(unsigned, Buffer*);
     virtual ~GpgPlugin();
     PROP_STR(GPG);
     PROP_STR(Home);
@@ -77,26 +87,36 @@ public:
     PROP_STR(Encrypt);
     PROP_STR(Decrypt);
     PROP_STR(Key);
+    PROP_UTFLIST(Passphrases);
+    PROP_STRLIST(Keys);
+    PROP_ULONG(nPassphrases);
+    PROP_BOOL(SavePassphrase);
     const char *GPG();
     void reset();
     static GpgPlugin *plugin;
     list<KeyMsg>	 m_sendKeys;
-    unsigned user_data_id;
+    unsigned long user_data_id;
 protected slots:
     void decryptReady(Exec*,int,const char*);
     void importReady(Exec*,int,const char*);
     void publicReady(Exec*,int,const char*);
     void clear();
+    void passphraseFinished();
+    void passphraseApply(const QString&);
 protected:
     virtual QWidget *createConfigWindow(QWidget *parent);
     virtual string getConfig();
     void *processEvent(Event*);
     void registerMessage();
     void unregisterMessage();
+    void askPassphrase();
+    bool decode(Message *msg, const char *pass, const char *key);
     bool m_bMessage;
     list<DecryptMsg> m_decrypt;
     list<DecryptMsg> m_import;
     list<DecryptMsg> m_public;
+    list<DecryptMsg> m_wait;
+    PassphraseDlg	 *m_passphraseDlg;
     GpgData data;
 };
 
