@@ -48,6 +48,7 @@ GpgCfg::GpgCfg(QWidget *parent, GpgPlugin *plugin)
 #endif
     edtHome->setText(QFile::decodeName(user_file(m_plugin->getHome()).c_str()));
     edtHome->setDirMode(true);
+    edtHome->setShowHidden(true);
     edtHome->setTitle(i18n("Select home directory"));
     lnkGPG->setUrl("http://www.gnupg.org/(en)/download/index.html");
     lnkGPG->setText(i18n("Download GPG"));
@@ -191,13 +192,13 @@ void GpgCfg::refresh()
     if (home[(int)(home.length() - 1)] == '\\')
         home = home.left(home.length() - 1);
     gpg = QString("\"") + gpg + "\"";
-    gpg += " --homedir \"";
+    gpg += " --no-tty --homedir \"";
     gpg += home;
     gpg += "\" ";
     gpg += m_plugin->getSecretList();
     m_exec = new Exec;
     connect(m_exec, SIGNAL(ready(Exec*,int,const char*)), this, SLOT(secretReady(Exec*,int,const char*)));
-    m_exec->execute(gpg.local8Bit(), "");
+    m_exec->execute(gpg.local8Bit(), 0, true);
 }
 
 void GpgCfg::secretReady(Exec *exec, int res, const char*)
@@ -226,7 +227,7 @@ void GpgCfg::selectKey(int n)
     }
 }
 
-#ifndef WIN32
+#ifndef _MSC_VER
 #include "gpgcfg.moc"
 #endif
 
