@@ -66,7 +66,7 @@ void GpgUser::refresh()
     if (home[(int)(home.length() - 1)] == '\\')
         home = home.left(home.length() - 1);
     gpg = QString("\"") + gpg + "\"";
-    gpg += " --homedir \"";
+    gpg += " --no-tty --homedir \"";
     gpg += home;
     gpg += "\" ";
     gpg += GpgPlugin::plugin->getPublicList();
@@ -121,7 +121,7 @@ void GpgUser::clearExec()
     }
 }
 
-#ifndef WIN32
+#ifndef _MSC_VER
 #include "gpguser.moc"
 #endif
 
