@@ -27,7 +27,7 @@ class AIMInfo : public AIMInfoBase, public EventReceiver
 {
     Q_OBJECT
 public:
-    AIMInfo(QWidget *parent, struct ICQUserData*, ICQClient *client);
+    AIMInfo(QWidget *parent, struct ICQUserData*, unsigned contact, ICQClient *client);
 public slots:
     void apply();
     void apply(Client*, void*);
@@ -35,7 +35,7 @@ protected:
     void *processEvent(Event *e);
     void fill();
     struct ICQUserData *m_data;
-    bool m_bInit;
+    unsigned  m_contact;
     ICQClient *m_client;
 };
 
