@@ -17,7 +17,6 @@
 
 #include "encodingdlg.h"
 #include "icqclient.h"
-#include "core.h"
 
 #include <qcombobox.h>
 #include <qpixmap.h>
@@ -36,13 +35,13 @@ EncodingDlg::EncodingDlg(QWidget *parent, ICQClient *client)
     connect(buttonOk, SIGNAL(clicked()), this, SLOT(apply()));
     connect(cmbEncoding, SIGNAL(activated(int)), this, SLOT(changed(int)));
     cmbEncoding->insertItem("");
-    const ENCODING *e = ICQPlugin::core->encodings;
+    const ENCODING *e = getContacts()->getEncodings();
     for (e++; e->language; e++){
         if (!e->bMain)
             continue;
         cmbEncoding->insertItem(i18n(e->language) + " (" + e->codec + ")");
     }
-    for (e = ICQPlugin::core->encodings; e->language; e++){
+    for (e = getContacts()->getEncodings(); e->language; e++){
         if (e->bMain)
             continue;
         cmbEncoding->insertItem(i18n(e->language) + " (" + e->codec + ")");
@@ -52,25 +51,29 @@ EncodingDlg::EncodingDlg(QWidget *parent, ICQClient *client)
 
 void EncodingDlg::apply()
 {
+    // Subtract 1 to account for the first menu item which is empty
     int n = cmbEncoding->currentItem();
-    if (n-- == 0)
-        return;
-    const ENCODING *e = ICQPlugin::core->encodings;
-    for (e++; e->language; e++){
+
+    if (n == 0)
+        return; // User selected the empty menu item
+
+    const ENCODING *e;
+    for (e = getContacts()->getEncodings() + 1; e->language; e++){
         if (!e->bMain)
             continue;
-        if (n--){
-            set_str(&m_client->data.owner.Encoding.ptr, e->codec);
-            ICQPlugin::core->setDefaultEncoding(e->codec);
+        --n;
+        if (n == 0){
+            getContacts()->owner()->setEncoding(e->codec);
             return;
         }
     }
-    for (e = ICQPlugin::core->encodings; e->language; e++){
-        if (e->bMain)
+
+    for (e = getContacts()->getEncodings(); e->language; e++){
+        if (!e->bMain)
             continue;
-        if (n--){
-            set_str(&m_client->data.owner.Encoding.ptr, e->codec);
-            ICQPlugin::core->setDefaultEncoding(e->codec);
+        --n;
+        if (n == 0){
+            getContacts()->owner()->setEncoding(e->codec);
             return;
         }
     }
@@ -81,7 +84,7 @@ void EncodingDlg::changed(int n)
     buttonOk->setEnabled(n > 0);
 }
 
-#ifndef WIN32
+#ifndef _MSC_VER
 #include "encodingdlg.moc"
 #endif
 
