@@ -33,26 +33,25 @@ const unsigned EventRandomChat		= IcqCmdBase + 3;
 const unsigned EventRandomChatInfo	= IcqCmdBase + 4;
 const unsigned EventServiceReady	= IcqCmdBase + 5;
 
-const unsigned CmdVisibleList		= IcqCmdBase;
-const unsigned CmdInvisibleList		= IcqCmdBase + 1;
-const unsigned CmdChangeEncoding	= IcqCmdBase + 2;
-const unsigned CmdAllEncodings		= IcqCmdBase + 3;
-const unsigned CmdGroups			= IcqCmdBase + 4;
-const unsigned CmdCheckInvisibleAll	= IcqCmdBase + 5;
-const unsigned CmdCheckInvisible	= IcqCmdBase + 6;
-const unsigned CmdIcqSendMessage	= IcqCmdBase + 7;
+const unsigned long CmdVisibleList		= IcqCmdBase;
+const unsigned long CmdInvisibleList		= IcqCmdBase + 1;
+const unsigned long CmdGroups			= IcqCmdBase + 2;
+//const unsigned long CmdCheckInvisibleAll	= IcqCmdBase + 3;
+//const unsigned long CmdCheckInvisible	= IcqCmdBase + 4;
+const unsigned long CmdIcqSendMessage	= IcqCmdBase + 5;
+const unsigned long CmdShowWarning		= IcqCmdBase + 6;
+const unsigned long CmdPasswordFail		= IcqCmdBase + 7;
 
-const unsigned MenuEncoding			= IcqCmdBase;
-const unsigned MenuSearchResult		= IcqCmdBase + 1;
-const unsigned MenuCheckInvisible	= IcqCmdBase + 2;
-const unsigned MenuIcqGroups		= IcqCmdBase + 2;
+const unsigned long MenuSearchResult		= IcqCmdBase;
+//const unsigned long MenuCheckInvisible	= IcqCmdBase + 1;
+const unsigned long MenuIcqGroups		= IcqCmdBase + 2;
 
 class ICQProtocol : public Protocol
 {
 public:
     ICQProtocol(Plugin *plugin);
     ~ICQProtocol();
-    Client	*createClient(const char *cfg);
+    Client	*createClient(Buffer *cfg);
     const CommandDef *description();
     const CommandDef *statusList();
     static const CommandDef *_statusList();
@@ -65,7 +64,7 @@ class AIMProtocol : public Protocol
 public:
     AIMProtocol(Plugin *plugin);
     ~AIMProtocol();
-    Client	*createClient(const char *cfg);
+    Client	*createClient(Buffer *cfg);
     const CommandDef *description();
     const CommandDef *statusList();
     virtual const DataDef *userDataDef();
