@@ -31,6 +31,7 @@
 const unsigned short ICQ_SNACxBDY_REQUESTxRIGHTS   = 0x0002;
 const unsigned short ICQ_SNACxBDY_RIGHTSxGRANTED   = 0x0003;
 const unsigned short ICQ_SNACxBDY_ADDxTOxLIST      = 0x0004;
+const unsigned short ICQ_SNACxBDY_REMOVExFROMxLIST = 0x0005;
 const unsigned short ICQ_SNACxBDY_USERONLINE	   = 0x000B;
 const unsigned short ICQ_SNACxBDY_USEROFFLINE	   = 0x000C;
 
@@ -144,14 +145,14 @@ void ICQClient::snac_buddy(unsigned short type, unsigned short)
                         if (*capabilities[i] == 0) break;
                         unsigned size = sizeof(capability);
                         if (i == CAP_SIMOLD) size--;
-                        if ((i == CAP_MICQ) || (i == CAP_LICQ) || (i == CAP_SIM)) size -= 4;
+                        if ((i == CAP_MICQ) || (i == CAP_LICQ) || (i == CAP_SIM) || (i == CAP_KOPETE)) size -= 4;
                         if (!memcmp(cap, capabilities[i], size)){
                             if (i == CAP_SIMOLD){
                                 unsigned char build = cap[sizeof(capability)-1];
                                 if (build && ((build == 0x92) || (build < (1 << 6)))) continue;
                                 data->Build.value = build;
                             }
-                            if ((i == CAP_MICQ) || (i == CAP_LICQ) || (i == CAP_SIM)){
+                            if ((i == CAP_MICQ) || (i == CAP_LICQ) || (i == CAP_SIM) || (i == CAP_KOPETE)){
                                 unsigned char *p = (unsigned char*)cap;
                                 p += 12;
                                 data->Build.value = (p[0] << 24) + (p[1] << 16) + (p[2] << 8) + p[3];
@@ -211,28 +212,32 @@ void ICQClient::snac_buddy(unsigned short type, unsigned short)
                     addFullInfoRequest(data->Uin.value);
                     break;
                 case 2:
-                    info.incReadPos(6);
-                    info.unpack((char*)p, sizeof(p));
-                    data->PluginInfoTime.value = time;
-                    for (plugin_index = 0; plugin_index < PLUGIN_NULL; plugin_index++)
-                        if (!memcmp(p, plugins[plugin_index], sizeof(p)))
+                    if		((getInvisible() && data->VisibleId.value) ||
+                            (!getInvisible() && (data->InvisibleId.value == 0))){
+                        info.incReadPos(6);
+                        info.unpack((char*)p, sizeof(p));
+                        data->PluginInfoTime.value = time;
+                        for (plugin_index = 0; plugin_index < PLUGIN_NULL; plugin_index++)
+                            if (!memcmp(p, plugins[plugin_index], sizeof(p)))
+                                break;
+                        switch (plugin_index){
+                        case PLUGIN_PHONEBOOK:
+                            log(L_DEBUG, "Updated phonebook");
+                            addPluginInfoRequest(data->Uin.value, plugin_index);
                             break;
-                    switch (plugin_index){
-                    case PLUGIN_PHONEBOOK:
-                        log(L_DEBUG, "Updated phonebook");
-                        addPluginInfoRequest(data->Uin.value, plugin_index);
-                        break;
-                    case PLUGIN_PICTURE:
-                        log(L_DEBUG, "Updated picture");
-                        addPluginInfoRequest(data->Uin.value, plugin_index);
-                        break;
-                    case PLUGIN_QUERYxINFO:
-                        log(L_DEBUG, "Updated info plugin list");
-                        addPluginInfoRequest(data->Uin.value, plugin_index);
-                        break;
-                    default:
-                        if (plugin_index >= PLUGIN_NULL)
-                            log(L_WARN, "Unknown plugin sign");
+                        case PLUGIN_PICTURE:
+                            log(L_DEBUG, "Updated picture");
+                            addPluginInfoRequest(data->Uin.value, plugin_index);
+                            break;
+                        case PLUGIN_QUERYxINFO:
+                            log(L_DEBUG, "Updated info plugin list");
+                            addPluginInfoRequest(data->Uin.value, plugin_index);
+                            break;
+                        default:
+                            if (plugin_index >= PLUGIN_NULL)
+                                log(L_WARN, "Unknown plugin sign (%04X %04X)",
+                                    type, plugin_index);
+                        }
                     }
                     break;
                 case 3:
@@ -265,7 +270,8 @@ void ICQClient::snac_buddy(unsigned short type, unsigned short)
                         break;
                     default:
                         if (plugin_index >= PLUGIN_NULL)
-                            log(L_WARN, "Unknown plugin sign");
+                            log(L_WARN, "Unknown plugin sign (%04X %04X)",
+                                type, plugin_index);
                     }
                     break;
 
@@ -274,7 +280,9 @@ void ICQClient::snac_buddy(unsigned short type, unsigned short)
                 data->InfoUpdateTime.value   = infoUpdateTime;
                 data->PluginInfoTime.value   = pluginInfoTime;
                 data->PluginStatusTime.value = pluginStatusTime;
-                if (getAutoUpdate()){
+                if (!getDisableAutoUpdate() &&
+                        ((getInvisible() && data->VisibleId.value) ||
+                         (!getInvisible() && (data->InvisibleId.value == 0)))){
                     if (infoUpdateTime == 0)
                         infoUpdateTime = 1;
                     if (infoUpdateTime != data->InfoFetchTime.value)
@@ -344,7 +352,7 @@ void ICQClient::snac_buddy(unsigned short type, unsigned short)
                     Event e(EventContactOnline, contact);
                     e.process();
                 }
-                if (getAutoReplyUpdate() && ((data->Status.value & 0xFF) != ICQ_STATUS_ONLINE)){
+                if (!getDisableAutoReplyUpdate() && ((data->Status.value & 0xFF) != ICQ_STATUS_ONLINE)){
                     if ((getInvisible() && data->VisibleId.value) ||
                             (!getInvisible() && (data->InvisibleId.value == 0)))
                         addPluginInfoRequest(data->Uin.value, PLUGIN_AR);
@@ -360,7 +368,7 @@ void ICQClient::snac_buddy(unsigned short type, unsigned short)
 void ICQClient::buddyRequest()
 {
     snac(ICQ_SNACxFAM_BUDDY, ICQ_SNACxBDY_REQUESTxRIGHTS);
-    sendPacket();
+    sendPacket(true);
 }
 
 void ICQClient::sendContactList()
@@ -387,7 +395,7 @@ void ICQClient::sendContactList()
                 m_socket->writeBuffer.packScreen(screen(data).c_str());
         }
     }
-    sendPacket();
+    sendPacket(true);
 }
 
 void ICQClient::addBuddy(Contact *contact)
@@ -409,9 +417,32 @@ void ICQClient::addBuddy(Contact *contact)
         if ((data->IgnoreId.value == 0)  && (data->WaitAuth.bValue || (data->GrpId.value == 0))){
             snac(ICQ_SNACxFAM_BUDDY, ICQ_SNACxBDY_ADDxTOxLIST);
             m_socket->writeBuffer.packScreen(screen(data).c_str());
-            sendPacket();
+            sendPacket(true);
             buddies.push_back(screen(data));
         }
     }
 }
 
+void ICQClient::removeBuddy(Contact *contact)
+{
+    if (getState() != Connected)
+        return;
+    if (contact->id() == 0)
+        return;
+    ICQUserData *data;
+    ClientDataIterator it_data(contact->clientData, this);
+    while ((data = (ICQUserData*)(++it_data)) != NULL){
+        list<string>::iterator it;
+        for (it = buddies.begin(); it != buddies.end(); ++it){
+            if (screen(data) == *it)
+                break;
+        }
+        if (it == buddies.end())
+            continue;
+        snac(ICQ_SNACxFAM_BUDDY, ICQ_SNACxBDY_REMOVExFROMxLIST);
+        m_socket->writeBuffer.packScreen(screen(data).c_str());
+        sendPacket(true);
+        buddies.erase(it);
+    }
+}
+
