@@ -24,6 +24,8 @@
 #include "socket.h"
 #include "icq.h"
 
+#include <qdatetime.h>
+
 #include <map>
 using namespace std;
 
@@ -58,6 +60,8 @@ const unsigned short ICQ_SNACxFAM_LISTS            = 0x0013;
 const unsigned short ICQ_SNACxFAM_VARIOUS          = 0x0015;
 const unsigned short ICQ_SNACxFAM_LOGIN            = 0x0017;
 
+#define SNAC(A, B)	((A << 16) + B)
+
 // Status
 const unsigned short ICQ_STATUS_OFFLINE            = 0xFFFF;
 const unsigned short ICQ_STATUS_ONLINE             = 0x0000;
@@ -77,8 +81,8 @@ const unsigned long ICQ_STATUS_FxPFM               = 0x00040000;  // not impleme
 const unsigned long ICQ_STATUS_FxBIRTHDAY          = 0x00080000;
 const unsigned long ICQ_STATUS_FxDIRECTxDISABLED   = 0x00100000;
 const unsigned long ICQ_STATUS_FxICQxHOMEPAGE      = 0x00200000;  // not implemented
-const unsigned long ICQ_STATUS_FxDIRECTxLISTED     = 0x20000000;  // will accept connectio only when listed
-const unsigned long ICQ_STATUS_FxDIRECTxAUTH       = 0x10000000;  // will accept connectio only when authorized
+const unsigned long ICQ_STATUS_FxDIRECTxAUTH       = 0x10000000;  // will accept connections only when authorized
+const unsigned long ICQ_STATUS_FxDIRECTxLISTED     = 0x20000000;  // will accept connections only when listed
 
 const unsigned CLASS_UNCONFIRMED	= 0x0001;	// AOL unconfirmed user flsg
 const unsigned CLASS_ADMINISTRATOR	= 0x0002;   // AOL administrator flag
@@ -140,6 +144,9 @@ const unsigned MAX_TYPE2_MESSAGE_SIZE = 4096;
 
 const unsigned PING_TIMEOUT = 60;
 
+const unsigned RATE_PAUSE = 3;
+const unsigned RATE_LIMIT = 5;
+
 const unsigned short SEARCH_DONE = (unsigned short)(-1);
 
 class DirectClient;
@@ -225,7 +232,6 @@ typedef struct ICQUserData
     Data		FollowMe;
     Data		SharedFiles;
     Data		ICQPhone;
-    Data        Encoding;
     Data        Picture;
     Data		PictureWidth;
     Data		PictureHeight;
@@ -256,9 +262,12 @@ typedef struct ICQClientData
     Data		RandomChatGroup;
     Data		RandomChatGroupCurrent;
     Data		SendFormat;
-    Data		AutoUpdate;
-    Data		AutoReplyUpdate;
-    Data		TypingNotification;
+    Data		DisablePlugins;
+    Data		DisableAutoUpdate;
+    Data		DisableAutoReplyUpdate;
+    Data		DisableTypingNotification;
+//    Data		AutoCheckInvisible;
+//    Data		CheckInvisibleInterval;
     Data		AcceptInDND;
     Data		AcceptInOccupied;
     Data		MinPort;
@@ -275,9 +284,9 @@ class ICQClient;
 
 typedef struct SearchResult
 {
-    ICQUserData        data;
+    ICQUserData       data;
     unsigned short    id;
-    ICQClient        *client;
+    ICQClient         *client;
 } SearchResult;
 
 class QTimer;
@@ -305,7 +314,10 @@ const unsigned CAP_AIM_BUDDYLIST	= 16;
 const unsigned CAP_MICQ				= 17;
 const unsigned CAP_LICQ				= 18;
 const unsigned CAP_SIMOLD			= 19;
-const unsigned CAP_NULL				= 20;
+const unsigned CAP_KOPETE			= 20;
+const unsigned CAP_XTRAZ			= 21;
+const unsigned CAP_IS_2002			= 21;
+const unsigned CAP_NULL				= 22;
 
 const unsigned PLUGIN_PHONEBOOK          = 0;
 const unsigned PLUGIN_PICTURE            = 1;
@@ -317,14 +329,15 @@ const unsigned PLUGIN_QUERYxSTATUS       = 6;
 const unsigned PLUGIN_INFOxMANAGER       = 7;
 const unsigned PLUGIN_STATUSxMANAGER     = 8;
 const unsigned PLUGIN_RANDOMxCHAT        = 9;
-const unsigned PLUGIN_NULL               = 10;
-const unsigned PLUGIN_FILE               = 11;
-const unsigned PLUGIN_CHAT               = 12;
-const unsigned PLUGIN_AR                 = 13;
-const unsigned PLUGIN_INVISIBLE          = 14;
-const unsigned PLUGIN_REVERSE            = 15;
-const unsigned PLUGIN_AIM_FT			 = 16;
-const unsigned PLUGIN_AIM_FT_ACK		 = 17;
+const unsigned PLUGIN_VIDEO_CHAT         = 10;
+const unsigned PLUGIN_NULL               = 11;
+const unsigned PLUGIN_FILE               = 12;
+const unsigned PLUGIN_CHAT               = 13;
+const unsigned PLUGIN_AR                 = 14;
+const unsigned PLUGIN_INVISIBLE          = 15;
+const unsigned PLUGIN_REVERSE            = 16;
+const unsigned PLUGIN_AIM_FT			 = 17;
+const unsigned PLUGIN_AIM_FT_ACK		 = 18;
 
 class ICQClient;
 
@@ -419,20 +432,16 @@ public:
     OscarSocket();
     virtual ~OscarSocket();
 protected:
+    void sendPacket(bool bSend=true);
     virtual ClientSocket *socket() = 0;
     virtual void packet() = 0;
-    void sendPacket();
     void flap(char channel);
     void snac(unsigned short fam, unsigned short type, bool msgId=false, bool bType=true);
     void connect_ready();
     void packet_ready();
-    void write_ready();
     bool m_bHeader;
     char m_nChannel;
-    Buffer   delayed;
-    unsigned m_time;
-    unsigned m_packets;
-    unsigned short m_nSequence;
+    unsigned short m_nFlapSequence;
     unsigned short m_nMsgSequence;
 };
 
@@ -442,18 +451,37 @@ typedef struct alias_group
     unsigned	grp;
 } alias_group;
 
+typedef struct RateInfo
+{
+    Buffer				delayed;
+    QDateTime			m_lastSend;
+    unsigned			m_curLevel;
+    unsigned			m_maxLevel;
+    unsigned			m_minLevel;
+    unsigned			m_winSize;
+} RateInfo;
+
+typedef struct InfoRequest
+{
+    unsigned	uin;
+    unsigned	request_id;
+    unsigned	start_time;
+} InfoRequest;
+
 typedef map<my_string, alias_group> CONTACTS_MAP;
+typedef map<unsigned, unsigned>	    RATE_MAP;
 
 class ICQClient : public TCPClient, public OscarSocket
 {
     Q_OBJECT
 public:
-    ICQClient(Protocol*, const char *cfg, bool bAIM);
+    ICQClient(Protocol*, Buffer *cfg, bool bAIM);
     ~ICQClient();
     virtual string name();
     virtual QWidget    *setupWnd();
     virtual string getConfig();
     virtual unsigned getStatus();
+    virtual void contactsLoaded();
     void setUin(unsigned long);
     void setScreen(const char*);
     unsigned long getUin();
@@ -472,9 +500,12 @@ public:
     PROP_USHORT(RandomChatGroup);
     PROP_ULONG(RandomChatGroupCurrent);
     PROP_ULONG(SendFormat);
-    PROP_BOOL(AutoUpdate);
-    PROP_BOOL(AutoReplyUpdate);
-    PROP_BOOL(TypingNotification);
+    PROP_BOOL(DisablePlugins);
+    PROP_BOOL(DisableAutoUpdate);
+    PROP_BOOL(DisableAutoReplyUpdate);
+    PROP_BOOL(DisableTypingNotification);
+//    PROP_BOOL(AutoCheckInvisible);
+//    PROP_ULONG(CheckInvisibleInterval);
     PROP_BOOL(AcceptInDND);
     PROP_BOOL(AcceptInOccupied);
     PROP_USHORT(MinPort);
@@ -485,23 +516,23 @@ public:
     PROP_BOOL(AutoHTTP);
     PROP_BOOL(KeepAlive);
     ICQClientData    data;
-    QString toUnicode(const char *str, ICQUserData *client_data);
-    string fromUnicode(const QString &str, ICQUserData *client_data);
     unsigned short findByUin(unsigned long uin);
+    unsigned short findByMail(const char *mail);
     unsigned short findWP(const char *first, const char *last, const char *nick,
                           const char *email, char age, char nGender,
-                          char nLanguage, const char *city, const char *szState,
+                          unsigned short nLanguage, const char *city, const char *szState,
                           unsigned short nCountryCode,
                           const char *cCoName, const char *szCoDept, const char *szCoPos,
-                          char nOccupation,
+                          unsigned short nOccupation,
                           unsigned short nPast, const char *szPast,
                           unsigned short nInterests, const char *szInterests,
                           unsigned short nAffiliation, const char *szAffiliation,
                           unsigned short nHomePoge, const char *szHomePage,
-                          bool bOnlineOnly);
+                          const char *sKeyWord, bool bOnlineOnly);
+    Contact *getContact(ICQUserData*);
     ICQUserData *findContact(const char *screen, const char *alias, bool bCreate, Contact *&contact, Group *grp=NULL, bool bJoin=true);
     ICQUserData *findGroup(unsigned id, const char *name, Group *&group);
-    void addFullInfoRequest(unsigned long uin, bool bInLast = true);
+    void addFullInfoRequest(unsigned long uin);
     ListRequest *findContactListRequest(const char *screen);
     ListRequest *findGroupListRequest(unsigned short id);
     void removeListRequest(ListRequest *lr);
@@ -523,10 +554,7 @@ public:
     Message *parseMessage(unsigned short type, const char *screen,
                           string &p, Buffer &packet, MessageId &id, unsigned cookie);
     bool messageReceived(Message*, const char *screen);
-    QTextCodec *getCodec(const char *encoding);
-    static QTextCodec *_getCodec(const char *encoding);
-    static QString toUnicode(const char *serverText, const char *clientName, unsigned contactId);
-    static bool parseRTF(const char *str, const char *encoding, QString &result);
+    static bool parseRTF(const char *str, Contact *contact, QString &result);
     static QString pictureFile(ICQUserData *data);
     static const capability *capabilities;
     static const plugin *plugins;
@@ -534,18 +562,16 @@ public:
     static QString convert(const char *text, unsigned size, TlvList &tlvs, unsigned n);
     string screen(ICQUserData*);
     static unsigned warnLevel(unsigned short);
-    static QString addCRLF(const QString &str);
     static unsigned clearTags(QString &text);
     bool m_bAIM;
-    void fillEncoding(QComboBox *cmb, ICQUserData *data);
-    void getEncoding(QComboBox *cmb, ICQUserData *data, bool bDefault);
+    static QString addCRLF(const QString &str);
 protected slots:
     void ping();
-    void infoRequest();
-    void infoRequestFail();
     void processSendQueue();
+    void sendTimeout();
     void retry(int n, void*);
 protected:
+    void sendPacket(bool bSend);
     virtual void setInvisible(bool bState);
     virtual void setStatus(unsigned status, bool bCommon);
     virtual void setStatus(unsigned status);
@@ -562,14 +588,14 @@ protected:
     virtual QWidget *infoWindow(QWidget *parent, Contact *contact, void *_data, unsigned id);
     virtual CommandDef *configWindows();
     virtual QWidget *configWindow(QWidget *parent, unsigned id);
-    virtual QWidget *searchWindow();
+    virtual QWidget *searchWindow(QWidget *parent);
     virtual void updateInfo(Contact *contact, void *_data);
     virtual void setClientInfo(void *data);
     virtual Socket  *createSocket();
     virtual QString ownerName();
     virtual QString contactName(void *clientData);
     string  dataName(const char *screen);
-    Buffer            m_cookie;
+    Buffer  m_cookie;
     virtual ClientSocket *socket();
     virtual void packet();
     void snac_service(unsigned short, unsigned short);
@@ -581,6 +607,8 @@ protected:
     void snac_lists(unsigned short, unsigned short);
     void snac_various(unsigned short, unsigned short);
     void snac_login(unsigned short, unsigned short);
+    void parseRosterItem(unsigned short type, string str,unsigned short grp_id,
+                         unsigned short id, TlvList *inf, bool &bIgnoreTime);
     void chn_login();
     void chn_close();
     void listsRequest();
@@ -596,52 +624,53 @@ protected:
     void setServiceSocket(Tlv *tlv_addr, Tlv *tlv_cookie, unsigned short service);
     void serverRequest(unsigned short cmd, unsigned short seq=0);
     void sendServerRequest();
-    void sendVisibleList();
-    void sendInvisibleList();
+    void sendInvisible(bool bState);
     void sendContactList();
     void setInvisible();
     void setOffline(ICQUserData*);
     void fillDirectInfo(Buffer &directInfo);
     void removeFullInfoRequest(unsigned long uin);
-    void send(bool bTimer);
     void requestService(ServiceSocket*);
     unsigned long fullStatus(unsigned status);
     string cryptPassword();
     virtual void connect_ready();
     virtual void packet_ready();
-    virtual void write_ready();
     const char* error_message(unsigned short error);
-    ICQListener		*m_listener;
+    bool m_bVerifying;
+    ICQListener			*m_listener;
     list<ServiceSocket*> m_services;
+    QTimer *m_processTimer;
     QTimer *m_sendTimer;
-    QTimer *m_infoTimer;
-    unsigned short m_infoRequestId;
     unsigned short m_sendSmsId;
     unsigned short m_offlineMessagesRequestId;
     ListServerRequest *m_listRequest;
-    unsigned m_listRequestTime;
+    time_t m_listRequestTime;
     bool m_bRosters;
     bool m_bBirthday;
+    bool m_bNoSend;
     list<ServerRequest*> varRequests;
-    list<unsigned long>  infoRequests;
+    list<InfoRequest>    infoRequests;
     list<string>		 buddies;
     list<ListRequest>    listRequests;
     list<SendMsg>        smsQueue;
-    list<SendMsg>        sendQueue;
+    list<SendMsg>        sendFgQueue;
+    list<SendMsg>		 sendBgQueue;
     list<SendMsg>        replyQueue;
     list<ar_request>    arRequests;
     void addGroupRequest(Group *group);
     void addContactRequest(Contact *contact);
     void addBuddy(Contact *contact);
+    void removeBuddy(Contact *contact);
     void checkListRequest();
+    void checkInfoRequest();
     ServerRequest *findServerRequest(unsigned short id);
     void clearServerRequests();
     void clearListServerRequest();
     void clearSMSQueue();
     void clearMsgQueue();
-    void processListRequest();
-    void processSMSQueue();
-    void infoRequestPause();
+    unsigned processListRequest();
+    unsigned processSMSQueue();
+    unsigned processInfoRequest();
     void sendIdleTime();
     void sendPluginInfoUpdate(unsigned plugin_id);
     void sendPluginStatusUpdate(unsigned plugin_id, unsigned long status);
@@ -664,8 +693,8 @@ protected:
     bool isOwnData(const char *screen);
     void packInfoList(char *str);
     QString packContacts(ContactsMessage *msg, ICQUserData *data, CONTACTS_MAP &c);
-    string createRTF(const QString &text, unsigned long foreColor, const char *encoding);
-    QString removeImages(const QString &text, unsigned maxSmile);
+    string createRTF(QString &text, QString &part, unsigned long foreColor, Contact *contact, unsigned max_size);
+    QString removeImages(const QString &text, bool icqSmiles);
     void ackMessage(SendMsg &s);
     void accept(Message *msg, const char *dir, OverwriteMode overwrite);
     void decline(Message *msg, const char *reason);
@@ -687,7 +716,7 @@ protected:
     Message *parseExtendedMessage(const char *screen, Buffer &packet, MessageId &id, unsigned cookie);
     void parsePluginPacket(Buffer &b, unsigned plugin_index, ICQUserData *data, unsigned uin, bool bDirect);
     void pluginAnswer(unsigned plugin_type, unsigned long uin, Buffer &b);
-    void packMessage(Buffer &b, Message *msg, ICQUserData *data, unsigned short &type, unsigned short flags=ICQ_TCPxMSG_NORMAL);
+    void packMessage(Buffer &b, Message *msg, ICQUserData *data, unsigned short &type, bool bDirect, unsigned short flags=ICQ_TCPxMSG_NORMAL);
     void packExtendedMessage(Message *msg, Buffer &buf, Buffer &msgBuf, ICQUserData *data);
     bool ackMessage(Message *msg, unsigned short ackFlags, const char *str);
     void fetchProfile(ICQUserData *data);
@@ -696,16 +725,24 @@ protected:
     void setAwayMessage(const char *msg);
     void encodeString(const QString &text, const char *type, unsigned short charsetTlv, unsigned short infoTlv);
     void encodeString(const char *_str, unsigned short nTlv, bool bWide);
+    bool processMsg();
+    void packTlv(unsigned short tlv, unsigned short code, const char *keywords);
     ICQUserData *findInfoRequest(unsigned short seq, Contact *&contact);
     INFO_REQ_MAP m_info_req;
     unsigned short msgStatus();
     unsigned short m_advCounter;
     unsigned m_nUpdates;
-    unsigned m_nSendTimeout;
     bool     m_bJoin;
     bool	 m_bFirstTry;
     bool	 m_bHTTP;
+    bool	 m_bReady;
     SendMsg  m_send;
+    vector<RateInfo>	m_rates;
+    RATE_MAP			m_rate_grp;
+    void				setNewLevel(RateInfo &r);
+    unsigned			delayTime(unsigned snac);
+    unsigned			delayTime(RateInfo &r);
+    RateInfo			*rateInfo(unsigned snac);
     list<Message*>     	m_processMsg;
     list<DirectSocket*>	m_sockets;
     list<Message*>		m_acceptMsg;
@@ -731,7 +768,6 @@ public:
 protected:
     virtual void connect_ready();
     virtual void packet_ready();
-    virtual void write_ready();
     virtual ClientSocket *socket();
     virtual void packet();
     virtual void data(unsigned short fam, unsigned short type, unsigned short seq) = 0;
@@ -823,7 +859,7 @@ protected:
         Logged,
         SSLconnect
     };
-    State        m_state;
+    State       m_state;
     unsigned    m_channel;
     void processPacket();
     void connect_ready();
