@@ -17,13 +17,6 @@
 
 #include "simapi.h"
 
-#ifdef USE_OPENSSL
-#include <openssl/ssl.h>
-#include <openssl/err.h>
-#include <openssl/bio.h>
-#include <openssl/rand.h>
-#endif
-
 #include "icqclient.h"
 #include "icqmessage.h"
 
@@ -39,6 +32,8 @@
 #include <arpa/inet.h>
 #endif
 
+#include <time.h>
+
 #include <qfile.h>
 #include <qtimer.h>
 #include <qregexp.h>
@@ -277,7 +272,7 @@ void DirectSocket::packet_ready()
     }
     if (m_state != Logged){
         ICQPlugin *plugin = static_cast<ICQPlugin*>(m_client->protocol()->plugin());
-        log_packet(m_socket->readBuffer, false, plugin->ICQDirectPacket, number((unsigned)this).c_str());
+        log_packet(m_socket->readBuffer, false, plugin->ICQDirectPacket, number((unsigned long)this).c_str());
     }
     switch (m_state){
     case Logged:{
@@ -402,7 +397,7 @@ void DirectSocket::sendInit()
     if (m_version >= 7)
         m_socket->writeBuffer.pack(0x00000000L);
     ICQPlugin *plugin = static_cast<ICQPlugin*>(m_client->protocol()->plugin());
-    log_packet(m_socket->writeBuffer, true, plugin->ICQDirectPacket, number((unsigned)this).c_str());
+    log_packet(m_socket->writeBuffer, true, plugin->ICQDirectPacket, number((unsigned long)this).c_str());
     m_socket->write();
 }
 
@@ -413,7 +408,7 @@ void DirectSocket::sendInitAck()
     m_socket->writeBuffer.pack((unsigned short)0x0001);
     m_socket->writeBuffer.pack((unsigned short)0x0000);
     ICQPlugin *plugin = static_cast<ICQPlugin*>(m_client->protocol()->plugin());
-    log_packet(m_socket->writeBuffer, true, plugin->ICQDirectPacket, number((unsigned)this).c_str());
+    log_packet(m_socket->writeBuffer, true, plugin->ICQDirectPacket, number((unsigned long)this).c_str());
     m_socket->write();
 }
 
@@ -506,7 +501,7 @@ void DirectClient::processPacket()
     case WaitInit2:
         if (m_bIncoming){
             ICQPlugin *plugin = static_cast<ICQPlugin*>(m_client->protocol()->plugin());
-            log_packet(m_socket->readBuffer, false, plugin->ICQDirectPacket, number((unsigned)this).c_str());
+            log_packet(m_socket->readBuffer, false, plugin->ICQDirectPacket, number((unsigned long)this).c_str());
             if (m_version < 8){
                 if (m_data->Direct.ptr){
                     m_socket->error_state("Direct connection already established");
@@ -517,7 +512,7 @@ void DirectClient::processPacket()
                 break;
             }
             plugin = static_cast<ICQPlugin*>(m_client->protocol()->plugin());
-            log_packet(m_socket->readBuffer, false, plugin->ICQDirectPacket, number((unsigned)this).c_str());
+            log_packet(m_socket->readBuffer, false, plugin->ICQDirectPacket, number((unsigned long)this).c_str());
             m_socket->readBuffer.incReadPos(13);
             char p[16];
             m_socket->readBuffer.unpack(p, 16);
@@ -787,8 +782,9 @@ void DirectClient::processPacket()
         }
         break;
     case TCP_CANCEL:
-    case TCP_ACK:
+	case TCP_ACK: {
         log(L_DEBUG, "Ack %X %X", ackFlags, msgFlags);
+		bool itDeleted = false;
         for (it = m_queue.begin(); it != m_queue.end(); ++it){
             if ((*it).seq != seq)
                 continue;
@@ -796,10 +792,10 @@ void DirectClient::processPacket()
                 if ((*it).type == PLUGIN_AR){
                     set_str(&m_data->AutoReply.ptr, msg_str.c_str());
                     m_queue.erase(it);
+					itDeleted = true;
                     break;
                 }
                 unsigned plugin_index = (*it).type;
-                m_queue.erase(it);
                 switch (plugin_index){
                 case PLUGIN_FILESERVER:
                 case PLUGIN_FOLLOWME:
@@ -811,7 +807,9 @@ void DirectClient::processPacket()
                     break;
                 }
                 m_client->parsePluginPacket(m_socket->readBuffer, plugin_index, m_data, m_data->Uin.value, true);
-                break;
+                m_queue.erase(it);
+				itDeleted = true;
+				break;
             }
             Message *msg = (*it).msg;
             if (command == TCP_CANCEL){
@@ -845,7 +843,7 @@ void DirectClient::processPacket()
                     if (msg_str.empty()){
                         msg->setError(I18N_NOOP("Send message fail"));
                     }else{
-                        QString err = m_client->toUnicode(msg_str.c_str(), m_data);
+                        QString err = getContacts()->toUnicode(m_client->getContact(m_data), msg_str.c_str());
                         msg->setError(err.utf8());
                     }
                     Event e(EventMessageSent, msg);
@@ -876,7 +874,7 @@ void DirectClient::processPacket()
                         m.setContact(msg->contact());
                         m.setClient(msg->client());
                         if ((*it).type == CAP_RTF){
-                            m.setText(m_client->removeImages(msg->getRichText(), 16));
+                            m.setText(m_client->removeImages(msg->getRichText(), true));
                             flags |= MESSAGE_RICHTEXT;
                         }else{
                             m.setText(msg->getPlainText());
@@ -901,7 +899,7 @@ void DirectClient::processPacket()
             delete msg;
             break;
         }
-        if (it == m_queue.end()){
+        if (!itDeleted && it == m_queue.end()){
             list<Message*>::iterator it;
             for (it = m_client->m_acceptMsg.begin(); it != m_client->m_acceptMsg.end(); ++it){
                 string name = m_client->dataName(m_data);
@@ -928,6 +926,7 @@ void DirectClient::processPacket()
                 log(L_WARN, "Message for ACK not found");
         }
         break;
+	}
     default:
         m_socket->error_state("Unknown TCP command");
     }
@@ -1250,7 +1249,7 @@ void DirectClient::acceptMessage(Message *msg)
 void DirectClient::declineMessage(Message *msg, const char *reason)
 {
     string r;
-    r = m_client->fromUnicode(QString::fromUtf8(reason), m_data);
+    r = getContacts()->fromUnicode(m_client->getContact(m_data), QString::fromUtf8(reason));
     unsigned short seq = 0;
     switch (msg->type()){
     case MessageICQFile:
@@ -1302,7 +1301,9 @@ void DirectClient::processMsgQueue()
                 if ((sm.msg->getFlags() & MESSAGE_RICHTEXT) &&
                         (m_client->getSendFormat() == 0) &&
                         (m_client->hasCap(m_data, CAP_RTF))){
-                    message = m_client->createRTF(sm.msg->getRichText(), sm.msg->getForeground(), m_data->Encoding.ptr);
+                    QString text = sm.msg->getRichText();
+                    QString part;
+                    message = m_client->createRTF(text, part, sm.msg->getForeground(), m_client->getContact(m_data), 0xFFFFFFFF);
                     sm.type = CAP_RTF;
                 }else if (m_client->hasCap(m_data, CAP_UTF) &&
                           (m_client->getSendFormat() <= 1) &&
@@ -1310,7 +1311,7 @@ void DirectClient::processMsgQueue()
                     message = ICQClient::addCRLF(sm.msg->getPlainText()).utf8();
                     sm.type = CAP_UTF;
                 }else{
-                    message = m_client->fromUnicode(ICQClient::addCRLF(sm.msg->getPlainText()), m_data);
+                    message = getContacts()->fromUnicode(m_client->getContact(m_data), sm.msg->getPlainText());
                     messageSend ms;
                     ms.msg  = sm.msg;
                     ms.text = &message;
@@ -1337,7 +1338,7 @@ void DirectClient::processMsgQueue()
             case MessageOpenSecure:
             case MessageCloseSecure:
                 startPacket(TCP_START, 0);
-                m_client->packMessage(mb, sm.msg, m_data, sm.icq_type);
+                m_client->packMessage(mb, sm.msg, m_data, sm.icq_type, true);
                 sendPacket();
                 sm.seq = m_nSequence;
                 break;
@@ -1443,51 +1444,9 @@ class ICQ_SSLClient : public SSLClient
 {
 public:
 ICQ_SSLClient(Socket *s) : SSLClient(s) {}
-    virtual bool initSSL();
+    virtual bool initSSL() { return initTLS1(true); }
 };
 
-// AUTOGENERATED by dhparam
-static DH *get_dh512()
-{
-    static unsigned char dh512_p[]={
-                                       0xFF,0xD3,0xF9,0x7C,0xEB,0xFE,0x45,0x2E,0x47,0x41,0xC1,0x8B,
-                                       0xF7,0xB9,0xC6,0xF2,0x40,0xCF,0x10,0x8B,0xF3,0xD7,0x08,0xC7,
-                                       0xF0,0x3F,0x46,0x7A,0xAD,0x71,0x6A,0x70,0xE1,0x76,0x8F,0xD9,
-                                       0xD4,0x46,0x70,0xFB,0x31,0x9B,0xD8,0x86,0x58,0x03,0xE6,0x6F,
-                                       0x08,0x9B,0x16,0xA0,0x78,0x70,0x6C,0xB1,0x78,0x73,0x52,0x3F,
-                                       0xD2,0x74,0xED,0x9B,
-                                   };
-    static unsigned char dh512_g[]={
-                                       0x02,
-                                   };
-    DH *dh;
-
-    if ((dh=DH_new()) == NULL) return(NULL);
-    dh->p=BN_bin2bn(dh512_p,sizeof(dh512_p),NULL);
-    dh->g=BN_bin2bn(dh512_g,sizeof(dh512_g),NULL);
-    if ((dh->p == NULL) || (dh->g == NULL))
-    { DH_free(dh); return(NULL); }
-    return(dh);
-}
-
-bool ICQ_SSLClient::initSSL()
-{
-    mpCTX = SSL_CTX_new(TLSv1_method());
-    if (mpCTX == NULL)
-        return false;
-#if OPENSSL_VERSION_NUMBER >= 0x00905000L
-    SSL_CTX_set_cipher_list(pCTX, "ADH:@STRENGTH");
-#else
-SSL_CTX_set_cipher_list(pCTX, "ADH");
-#endif
-    DH *dh = get_dh512();
-    SSL_CTX_set_tmp_dh(pCTX, dh);
-    DH_free(dh);
-    mpSSL = SSL_new(pCTX);
-    if(!mpSSL)
-        return false;
-    return true;
-}
 
 void DirectClient::secureConnect()
 {
@@ -1559,7 +1518,7 @@ const char *DirectClient::name()
     }
     m_name += number(m_data->Uin.value);
     m_name += ".";
-    m_name += number((unsigned)this);
+    m_name += number((unsigned long)this);
     return m_name.c_str();
 }
 
@@ -1733,7 +1692,7 @@ void ICQFileTransfer::initReceive(char cmd)
     string fileName;
     char isDir;
     m_socket->readBuffer >> isDir >> fileName;
-    QString fName = m_client->toUnicode(fileName.c_str(), m_data);
+    QString fName = getContacts()->toUnicode(m_client->getContact(m_data), fileName.c_str());
     string dir;
     unsigned long n;
     m_socket->readBuffer >> dir;
@@ -1741,7 +1700,7 @@ void ICQFileTransfer::initReceive(char cmd)
     if (m_notify)
         m_notify->transfer(false);
     if (!dir.empty())
-        fName = m_client->toUnicode(dir.c_str(), m_data) + "/" + fName;
+        fName = getContacts()->toUnicode(m_client->getContact(m_data), dir.c_str()) + "/" + fName;
     if (isDir)
         fName += "/";
     m_state = Wait;
@@ -1969,10 +1928,10 @@ void ICQFileTransfer::sendFileInfo()
         dir = dir.replace(QRegExp("/"), "\\");
         fn  = fn.mid(n);
     }
-    string s1 = m_client->fromUnicode(fn, m_data);
+    string s1 = getContacts()->fromUnicode(m_client->getContact(m_data), fn);
     string s2;
     if (!dir.isEmpty())
-        s2 = m_client->fromUnicode(dir, m_data);
+        s2 = getContacts()->fromUnicode(m_client->getContact(m_data), dir);
     m_socket->writeBuffer << s1 << s2;
     m_socket->writeBuffer.pack((unsigned long)m_fileSize);
     m_socket->writeBuffer.pack((unsigned long)0);
@@ -2105,8 +2064,8 @@ void AIMFileTransfer::bind_ready(unsigned short port)
     s.socket = this;
     s.screen = m_client->screen(m_data);
     s.msg	 = m_msg;
-    m_client->sendQueue.push_front(s);
-    m_client->send(false);
+    m_client->sendFgQueue.push_front(s);
+    m_client->processSendQueue();
 }
 
 bool AIMFileTransfer::accept(Socket *s, unsigned long)
