@@ -98,7 +98,7 @@ QString ICQClient::convert(const char *text, unsigned size, TlvList &tlvs, unsig
             res = codec->toUnicode(text, size);
         }else{
             res = QString::fromUtf8(text, size);
-            log(L_WARN, "Unknown encdoing %s", charset.c_str());
+            log(L_WARN, "Unknown encoding %s", charset.c_str());
         }
     }
     return res;
@@ -143,6 +143,8 @@ void ICQClient::snac_location(unsigned short type, unsigned short seq)
                         e.process();
                     }
                 }
+                break;	/* Because we won't find tlv(0x03) which is
+                           "since online" instead of encoding... */                            
             }
             Tlv *tlvAway = tlvs(0x04);
             if (tlvAway){
@@ -210,7 +212,7 @@ void ICQClient::snac_location(unsigned short type, unsigned short seq)
 void ICQClient::locationRequest()
 {
     snac(ICQ_SNACxFAM_LOCATION, ICQ_SNACxLOC_REQUESTxRIGHTS);
-    sendPacket();
+    sendPacket(true);
 }
 
 #define cap_id   0x82, 0x22, 0x44, 0x45, 0x53, 0x54, 0x00, 0x00
@@ -238,7 +240,7 @@ const capability arrCapabilities[] =
         // CAP_SIM
         { 'S', 'I', 'M', ' ', 'c', 'l', 'i', 'e',
           'n', 't', ' ', ' ', 0, 0, 0, 0 },
-        // CAP_STR_2001
+        // CAP_STR_2001_2002
         { 0xa0, 0xe9, 0x3f, 0x37, cap_mstr, cap_str },
         // CAP_IS_2001
         { 0x2e, 0x7a, 0x64, 0x75, 0xfa, 0xdf, 0x4d, 0xc8,
@@ -271,6 +273,14 @@ const capability arrCapabilities[] =
         // CAP_SIMOLD
         { 0x97, 0xb1, 0x27, 0x51, 0x24, 0x3c, 0x43, 0x34,
           0xad, 0x22, 0xd6, 0xab, 0xf7, 0x3f, 0x14, 0x00 },
+        // CAP_KOPETE
+        { 'K', 'o', 'p', 'e', 't', 'e', ' ', 'I',
+          'C', 'Q', ' ', ' ',   0,   0,   0,  0 },
+        // CAP_XTRAZ
+        { 0x1A, 0x09, 0x3C, 0x6C, 0xD7, 0xFD, 0x4E, 0xC5,
+          0x9D, 0x51, 0xA6, 0x47, 0x4E, 0x34, 0xF5, 0xA0  },
+        // CAP_IS_2002
+        { 0x10, 0xcf, 0x40, 0xd1, cap_mid, cap_id },
         // CAP_NULL
         { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
           0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
@@ -366,9 +376,9 @@ void ICQClient::sendCapability(const char *away_msg)
     os_ver = 0x80;
 #else
 #ifdef QT_MACOSX_VERSION
-    os_ver = 0x40;
+os_ver = 0x40;
 #else
-    os_ver = 0;
+os_ver = 0;
 #endif
 #endif
     *(pack_ver++) = os_ver;
@@ -381,12 +391,13 @@ void ICQClient::sendCapability(const char *away_msg)
     }else{
         cap.pack((char*)capabilities[CAP_DIRECT], sizeof(capability));
         cap.pack((char*)capabilities[CAP_SRV_RELAY], sizeof(capability));
+        cap.pack((char*)capabilities[CAP_XTRAZ], sizeof(capability));
         if (getSendFormat() <= 1)
             cap.pack((char*)capabilities[CAP_UTF], sizeof(capability));
         if (getSendFormat() == 0)
             cap.pack((char*)capabilities[CAP_RTF], sizeof(capability));
     }
-    if (getTypingNotification())
+    if (!getDisableTypingNotification())
         cap.pack((char*)capabilities[CAP_TYPING], sizeof(capability));
     cap.pack((char*)capabilities[CAP_AIM_SUPPORT], sizeof(capability));
     cap.pack((char*)c, sizeof(c));
@@ -405,7 +416,7 @@ void ICQClient::sendCapability(const char *away_msg)
     m_socket->writeBuffer.tlv(0x0005, cap);
     if (m_bAIM)
         m_socket->writeBuffer.tlv(0x0006, "\x00\x04\x00\x02\x00\x02", 6);
-    sendPacket();
+    sendPacket(true);
 }
 
 void ICQClient::setAwayMessage(const char *msg)
@@ -416,7 +427,7 @@ void ICQClient::setAwayMessage(const char *msg)
     }else{
         m_socket->writeBuffer.tlv(0x0004);
     }
-    sendPacket();
+    sendPacket(true);
 }
 
 void ICQClient::fetchProfile(ICQUserData *data)
@@ -424,10 +435,10 @@ void ICQClient::fetchProfile(ICQUserData *data)
     snac(ICQ_SNACxFAM_LOCATION, ICQ_SNACxLOC_REQUESTxUSERxINFO, true);
     m_socket->writeBuffer << (unsigned short)0x0001;
     m_socket->writeBuffer.packScreen(screen(data).c_str());
-    sendPacket();
+    sendPacket(false);
     snac(ICQ_SNACxFAM_LOCATION, ICQ_SNACxLOC_REQUESTxDIRxINFO, true);
     m_socket->writeBuffer.packScreen(screen(data).c_str());
-    sendPacket();
+    sendPacket(false);
     m_info_req.insert(INFO_REQ_MAP::value_type(m_nMsgSequence, screen(data)));
 }
 
@@ -436,7 +447,7 @@ void ICQClient::fetchAwayMessage(ICQUserData *data)
     snac(ICQ_SNACxFAM_LOCATION, ICQ_SNACxLOC_REQUESTxUSERxINFO, true);
     m_socket->writeBuffer << (unsigned short)0x0003;
     m_socket->writeBuffer.packScreen(screen(data).c_str());
-    sendPacket();
+    sendPacket(false);
 }
 
 void ICQClient::fetchProfiles()
@@ -501,7 +512,7 @@ void ICQClient::setAIMInfo(ICQUserData *data)
     encodeString(data->Nick.ptr, 0x0C, bWide);
     encodeString(data->Zip.ptr, 0x0D, bWide);
     encodeString(data->State.ptr, 0x21, bWide);
-    sendPacket();
+    sendPacket(false);
 
     ICQUserData *ownerData = &this->data.owner;
     set_str(&ownerData->FirstName.ptr, data->FirstName.ptr);
@@ -513,6 +524,7 @@ void ICQClient::setAIMInfo(ICQUserData *data)
     set_str(&ownerData->Nick.ptr, data->Nick.ptr);
     set_str(&ownerData->Zip.ptr, data->Zip.ptr);
     set_str(&ownerData->State.ptr, data->State.ptr);
+    ownerData->Country.value = data->Country.value;
 }
 
 void ICQClient::setProfile(ICQUserData *data)
@@ -523,6 +535,6 @@ void ICQClient::setProfile(ICQUserData *data)
         profile = QString::fromUtf8(data->About.ptr);
     profile = QString("<HTML>") + profile + "</HTML>";
     encodeString(profile, "text/aolrtf", 1, 2);
-    sendPacket();
+    sendPacket(false);
 }
 
