@@ -35,6 +35,7 @@ const unsigned short ICQ_SNACxSRV_SERVICExRESP	= 0x0005;
 const unsigned short ICQ_SNACxSRV_REQxRATExINFO = 0x0006;
 const unsigned short ICQ_SNACxSRV_RATExINFO     = 0x0007;
 const unsigned short ICQ_SNACxSRV_RATExACK      = 0x0008;
+const unsigned short ICQ_SNACxSRV_RATExDEL      = 0x0009;   /* Not implemented */
 const unsigned short ICQ_SNACxSRV_RATExCHANGE   = 0x000A;
 const unsigned short ICQ_SNACxSRV_PAUSE         = 0x000B;
 const unsigned short ICQ_SNACxSRV_PAUSExACK     = 0x000C;
@@ -45,6 +46,7 @@ const unsigned short ICQ_SNACxSRV_EVIL			= 0x0010;
 const unsigned short ICQ_SNACxSRV_SETxIDLE      = 0x0011;
 const unsigned short ICQ_SNACxSRV_MIGRATE       = 0x0012;
 const unsigned short ICQ_SNACxSRV_MOTD          = 0x0013;
+const unsigned short ICQ_SNACxSRV_PRIVATY_FLAGS = 0x0014;   /* Not implemented */
 const unsigned short ICQ_SNACxSRV_IMxICQ        = 0x0017;
 const unsigned short ICQ_SNACxSRV_ACKxIMxICQ    = 0x0018;
 const unsigned short ICQ_SNACxSRV_SETxSTATUS    = 0x001E;
@@ -54,10 +56,7 @@ void ICQClient::snac_service(unsigned short type, unsigned short)
     switch (type){
     case ICQ_SNACxSRV_PAUSE:
         log(L_DEBUG, "Server pause");
-        /* we now shouldn't send any packets to the server ...
-        but I don't know how to solve this. Valdimir do you
-        have an idea? */
-        /*        m_bDontSendPakets = true; */
+        m_bNoSend = true;
         snac(ICQ_SNACxFAM_SERVICE, ICQ_SNACxSRV_PAUSExACK);
         m_socket->writeBuffer << ICQ_SNACxFAM_SERVICE
         << ICQ_SNACxFAM_LOCATION
@@ -68,13 +67,16 @@ void ICQClient::snac_service(unsigned short type, unsigned short)
         << ICQ_SNACxFAM_LISTS
         << ICQ_SNACxFAM_VARIOUS
         << ICQ_SNACxFAM_LOGIN;
-        sendPacket();
+        sendPacket(true);
         break;
     case ICQ_SNACxSRV_RESUME:
-        /*        m_bDontSendPakets = true;
-        		emit canSendPakets(); */
+        log(L_DEBUG, "Server resume");
+        m_bNoSend = false;
+        processSendQueue();
         break;
     case ICQ_SNACxSRV_MIGRATE:{
+            log(L_DEBUG, "Server migrate");
+            m_bNoSend = true;
             int i;
             unsigned short cnt;
             unsigned short fam[0x17];
@@ -89,37 +91,138 @@ void ICQClient::snac_service(unsigned short type, unsigned short)
             for (; i >= 0; i--) {
                 setServiceSocket(tlv_adr,tlv_cookie,fam[i]);
             }
-            /*            m_bDontSendPakets = true;
-                        emit canSendPakets(); */
             break;
         }
-    case ICQ_SNACxSRV_RATExCHANGE:
-        log(L_DEBUG, "Rate change");
-        if (m_nSendTimeout < 200){
-            m_nSendTimeout = m_nSendTimeout + 2;
-            if (m_sendTimer->isActive()){
-                m_sendTimer->stop();
-                m_sendTimer->start(m_nSendTimeout * 500);
+    case ICQ_SNACxSRV_RATExCHANGE:{
+            const char *msg_text = NULL;
+            unsigned short msg_code;
+            unsigned short class_id;
+            unsigned long  window_size;
+            unsigned long  clear_level;
+            unsigned long  alert_level;
+            unsigned long  limit_level;
+            unsigned long  discon_level;
+            unsigned long  current_level;
+            unsigned long  max_level;
+            unsigned long  last_send;
+            char current_state;
+            m_socket->readBuffer >> msg_code
+            >> class_id
+            >> window_size
+            >> clear_level
+            >> alert_level
+            >> limit_level
+            >> discon_level
+            >> current_level
+            >> max_level
+            >> last_send
+            >> current_state;
+            if (type == ICQ_SNACxSRV_RATExCHANGE){
+                switch (msg_code) {
+                case 0x0001:
+                    msg_text = "Rate limits parameters changed";
+                    break;
+                case 0x0002:
+                    msg_text = "Rate limits warning";
+                    break;
+                case 0x0003:
+                    msg_text = "Rate limit hit";
+                    break;
+                case 0x0004:
+                    msg_text = "Rate limit clear";
+                    break;
+                default:
+                    msg_text = "Unknown";
+                }
+                log(L_DEBUG, "%s", msg_text);
             }
+            log(L_DEBUG, "grp: %02X, ws: %04lX, cl %04lX, al %04lX, ll %04lX, dl: %04lX, cur %04lX, ml %04lX",
+                class_id,window_size,clear_level,alert_level,limit_level,discon_level,
+                current_level,max_level);
+            if (--class_id < m_rates.size()){
+                RateInfo &r = m_rates[class_id];
+                r.m_winSize  = window_size;
+                r.m_maxLevel = max_level;
+                r.m_minLevel = alert_level;
+                r.m_curLevel = current_level;
+                r.m_lastSend = QDateTime::currentDateTime();
+                processSendQueue();
+            }
+            break;
         }
-        break;
     case ICQ_SNACxSRV_RATExINFO:
-        snac(ICQ_SNACxFAM_SERVICE, ICQ_SNACxSRV_RATExACK);
-        m_socket->writeBuffer << 0x00010002L << 0x00030004L << 0x0005;
-        sendPacket();
-        snac(ICQ_SNACxFAM_SERVICE, ICQ_SNACxSRV_GETxUSERxINFO);
-        sendPacket();
-        listsRequest();
-        locationRequest();
-        buddyRequest();
-        icmbRequest();
-        bosRequest();
+        if (m_rates.size() == 0){
+            unsigned short n_rates;
+            m_socket->readBuffer >> n_rates;
+            unsigned n;
+            for (n = 0; n < n_rates; n++){
+                unsigned short class_id;
+                unsigned long  window_size;
+                unsigned long  clear_level;
+                unsigned long  alert_level;
+                unsigned long  limit_level;
+                unsigned long  discon_level;
+                unsigned long  current_level;
+                unsigned long  max_level;
+                unsigned long  last_send;
+                char current_state;
+                m_socket->readBuffer
+                >> class_id
+                >> window_size
+                >> clear_level
+                >> alert_level
+                >> limit_level
+                >> discon_level
+                >> current_level
+                >> max_level
+                >> last_send
+                >> current_state;
+                log(L_DEBUG, "grp: %02X, ws: %04lX, cl %04lX, al %04lX, ll %04lX, dl: %04lX, cur %04lX, ml %04lX",
+                    class_id,window_size,clear_level,alert_level,limit_level,discon_level,
+                    current_level,max_level);
+                RateInfo r;
+                r.m_winSize		= window_size;
+                r.m_minLevel	= alert_level;
+                r.m_maxLevel	= max_level;
+                r.m_curLevel	= current_level;
+                r.m_lastSend	= QDateTime::currentDateTime();
+                m_rates.push_back(r);
+            }
+            for (n = 0; n < n_rates; n++){
+                unsigned short class_id;
+                unsigned short pairs;
+                m_socket->readBuffer
+                >> class_id
+                >> pairs;
+                class_id--;
+                for (unsigned i = 0; i < pairs; i++){
+                    unsigned long snac;
+                    m_socket->readBuffer >> snac;
+                    if (class_id >= m_rates.size())
+                        continue;
+                    RATE_MAP::iterator it = m_rate_grp.find(snac);
+                    if (it != m_rate_grp.end())
+                        continue;
+                    m_rate_grp.insert(RATE_MAP::value_type(snac, class_id));
+                }
+            }
+            snac(ICQ_SNACxFAM_SERVICE, ICQ_SNACxSRV_RATExACK);
+            m_socket->writeBuffer << 0x00010002L << 0x00030004L << 0x0005;
+            sendPacket(true);
+            snac(ICQ_SNACxFAM_SERVICE, ICQ_SNACxSRV_GETxUSERxINFO);
+            sendPacket(true);
+            listsRequest();
+            locationRequest();
+            buddyRequest();
+            icmbRequest();
+            bosRequest();
+        }
         break;
     case ICQ_SNACxSRV_MOTD:
         break;
     case ICQ_SNACxSRV_ACKxIMxICQ:
         snac(ICQ_SNACxFAM_SERVICE, ICQ_SNACxSRV_REQxRATExINFO);
-        sendPacket();
+        sendPacket(true);
         break;
     case ICQ_SNACxSRV_NAMExINFO:{
             string screen = m_socket->readBuffer.unpackScreen();
@@ -138,7 +241,6 @@ void ICQClient::snac_service(unsigned short type, unsigned short)
             Tlv *tlvIP = tlv(0x000A);
             if (tlvIP)
                 set_ip(&data.owner.IP, htonl((unsigned long)(*tlvIP)));
-            log(L_DEBUG, "Name info");
             break;
         }
     case ICQ_SNACxSRV_SERVICExRESP:{
@@ -154,7 +256,6 @@ void ICQClient::snac_service(unsigned short type, unsigned short)
             break;
         }
     case ICQ_SNACxSRV_READYxSERVER:
-        log(L_DEBUG, "Server ready");
         snac(ICQ_SNACxFAM_SERVICE, ICQ_SNACxSRV_IMxICQ);
         if (m_bAIM){
             m_socket->writeBuffer
@@ -181,7 +282,7 @@ void ICQClient::snac_service(unsigned short type, unsigned short)
             << 0x000A0001L
             << 0x000B0001L;
         }
-        sendPacket();
+        sendPacket(true);
         break;
     case ICQ_SNACxSRV_ERROR:
         break;
@@ -199,6 +300,9 @@ void ICQClient::snac_service(unsigned short type, unsigned short)
             d.code    = 0;
             d.err_str = I18N_NOOP("You've been warned by %1");
             d.args    = strdup(f.utf8());
+            d.flags   = ERR_INFO;
+            d.options = NULL;
+            d.id	  = CmdShowWarning;
             Event e(EventClientError, &d);
             e.process();
             free(d.args);
@@ -215,7 +319,7 @@ void ICQClient::setServiceSocket(Tlv *tlv_addr, Tlv *tlv_cookie, unsigned short
     for (list<ServiceSocket*>::iterator it = m_services.begin(); it != m_services.end(); ++it){
         if ((*it)->id() == service){
             s = *it;
-            return;
+            break;
         }
     }
     if (!s){
@@ -257,18 +361,17 @@ void ICQClient::sendClientReady()
     << 0x000A0001L << 0x0110047BL
     << 0x000B0001L << 0x0110047BL;
 
-    sendPacket();
+    sendPacket(true);
 }
 
 void ICQClient::sendLogonStatus()
 {
-    log(L_DEBUG, "Logon status %u", m_logonStatus);
     if (getInvisible())
-        sendVisibleList();
+        sendInvisible(false);
     sendContactList();
 
-    unsigned long now;
-    time((time_t*)&now);
+    time_t now;
+    time(&now);
     if (data.owner.PluginInfoTime.value == 0)
         data.owner.PluginInfoTime.value = now;
     if (data.owner.PluginStatusTime.value == 0)
@@ -297,9 +400,9 @@ void ICQClient::sendLogonStatus()
     m_socket->writeBuffer.tlv(0x0008, (unsigned short)0);
     m_socket->writeBuffer.tlv(0x000C, directInfo);
 
-    sendPacket();
+    sendPacket(true);
     if (!getInvisible())
-        sendInvisibleList();
+        sendInvisible(true);
     sendIdleTime();
     m_status = m_logonStatus;
 }
@@ -307,19 +410,19 @@ void ICQClient::sendLogonStatus()
 void ICQClient::setInvisible()
 {
     if (getInvisible())
-        sendVisibleList();
+        sendInvisible(false);
     snac(ICQ_SNACxFAM_SERVICE, ICQ_SNACxSRV_SETxSTATUS);
     m_socket->writeBuffer.tlv(0x0006, fullStatus(m_status));
-    sendPacket();
+    sendPacket(true);
     if (!getInvisible())
-        sendInvisibleList();
+        sendInvisible(true);
 }
 
 void ICQClient::sendStatus()
 {
     snac(ICQ_SNACxFAM_SERVICE, ICQ_SNACxSRV_SETxSTATUS);
     m_socket->writeBuffer.tlv(0x0006, fullStatus(m_status));
-    sendPacket();
+    sendPacket(true);
     sendIdleTime();
 }
 
@@ -341,7 +444,7 @@ void ICQClient::sendPluginInfoUpdate(unsigned plugin_id)
     b << (char)0;
     m_socket->writeBuffer.tlv(0x0011, b);
     m_socket->writeBuffer.tlv(0x0012, (unsigned short)0);
-    sendPacket();
+    sendPacket(false);
 }
 
 void ICQClient::sendPluginStatusUpdate(unsigned plugin_id, unsigned long status)
@@ -366,7 +469,7 @@ void ICQClient::sendPluginStatusUpdate(unsigned plugin_id, unsigned long status)
     b.pack((unsigned short)1);
     m_socket->writeBuffer.tlv(0x0011, b);
     m_socket->writeBuffer.tlv(0x0012, (unsigned short)0);
-    sendPacket();
+    sendPacket(false);
 }
 
 void ICQClient::sendUpdate()
@@ -383,7 +486,7 @@ void ICQClient::sendUpdate()
     Buffer directInfo(25);
     fillDirectInfo(directInfo);
     m_socket->writeBuffer.tlv(0x000C, directInfo);
-    sendPacket();
+    sendPacket(false);
 }
 
 void ICQClient::fillDirectInfo(Buffer &directInfo)
@@ -443,7 +546,7 @@ void ICQClient::sendIdleTime()
         idle = 1;
     snac(ICQ_SNACxFAM_SERVICE, ICQ_SNACxSRV_SETxIDLE);
     m_socket->writeBuffer << idle;
-    sendPacket();
+    sendPacket(false);
     m_bIdleTime = true;
 }
 
@@ -451,5 +554,5 @@ void ICQClient::requestService(ServiceSocket *s)
 {
     snac(ICQ_SNACxFAM_SERVICE, ICQ_SNACxSRV_SERVICExREQ, true);
     m_socket->writeBuffer << s->id();
-    sendPacket();
+    sendPacket(true);
 }
