@@ -18,6 +18,7 @@
 #include "icqclient.h"
 
 #include <time.h>
+#include <stdio.h>
 #ifdef WIN32
 #include <winsock.h>
 #else
@@ -43,9 +44,9 @@ const unsigned short ICQ_SRVxREQ_FULL_INFO          = 0xB204;
 const unsigned short ICQ_SRVxREQ_SHORT_INFO         = 0xBA04;
 const unsigned short ICQ_SRVxREQ_OWN_INFO           = 0xD004;
 const unsigned short ICQ_SRVxREQ_SEND_SMS           = 0x8214;
-const unsigned short ICQ_SRVxREQ_WP_INFO_UIN        = 0x6905;
-const unsigned short ICQ_SRVxREQ_WP_SHORT           = 0x1505;
-const unsigned short ICQ_SRVxREQ_WP_FULL            = 0x3305;
+const unsigned short ICQ_SRVxREQ_WP_UIN				= 0x6905;
+const unsigned short ICQ_SRVxREQ_WP_MAIL            = 0x7305;
+const unsigned short ICQ_SRVxREQ_WP_FULL            = 0x5F05;
 const unsigned short ICQ_SRVxREQ_CHANGE_PASSWD      = 0x2E04;
 const unsigned short ICQ_SRVxREQ_PERMISSIONS        = 0x2404;
 const unsigned short ICQ_SRVxREQ_XML_KEY            = 0x9808;
@@ -72,16 +73,56 @@ const unsigned short ICQ_SRVxREQ_PHONE_UPDATE       = 0x5406;
 const unsigned short ICQ_SRVxREQ_SET_CHAT_GROUP     = 0x5807;
 const unsigned short ICQ_SRVxREQ_RANDOM_CHAT        = 0x4E07;
 
+const unsigned short TLV_UIN						= 0x0136;
+const unsigned short TLV_FIRST_NAME					= 0x0140;
+const unsigned short TLV_LAST_NAME					= 0x014A;
+const unsigned short TLV_NICK						= 0x0154;
+const unsigned short TLV_EMAIL						= 0x015E;
+const unsigned short TLV_AGE_RANGE					= 0x0168;
+const unsigned short TLV_AGE						= 0x0172;
+const unsigned short TLV_GENDER						= 0x017C;
+const unsigned short TLV_LANGUAGE					= 0x0186;
+const unsigned short TLV_CITY						= 0x0190;
+const unsigned short TLV_STATE						= 0x019A;
+const unsigned short TLV_COUNTRY					= 0x01A4;
+const unsigned short TLV_WORK_COMPANY				= 0x01AE;
+const unsigned short TLV_WORK_DEPARTMENT			= 0x01B8;
+const unsigned short TLV_WORK_POSITION				= 0x01C2;
+const unsigned short TLV_WORK_OCCUPATION			= 0x01CC;
+const unsigned short TLV_AFFILATIONS				= 0x01D6;
+const unsigned short TLV_INTERESTS					= 0x01EA;
+const unsigned short TLV_PAST						= 0x01FE;
+const unsigned short TLV_HOMEPAGE_CATEGORY			= 0x0212;
+const unsigned short TLV_HOMEPAGE					= 0x0213;
+const unsigned short TLV_KEYWORDS					= 0x0226;
+const unsigned short TLV_SEARCH_ONLINE				= 0x0230;
+const unsigned short TLV_BIRTHDAY					= 0x023A;
+const unsigned short TLV_NOTES						= 0x0258;
+const unsigned short TLV_STREET						= 0x0262;
+const unsigned short TLV_ZIP						= 0x026C;
+const unsigned short TLV_PHONE						= 0x0276;
+const unsigned short TLV_FAX						= 0x0280;
+const unsigned short TLV_CELLULAR					= 0x028A;
+const unsigned short TLV_WORK_STREET				= 0x0294;
+const unsigned short TLV_WORK_CITY					= 0x029E;
+const unsigned short TLV_WORK_STATE					= 0x02A8;
+const unsigned short TLV_WORK_CIUNTRY				= 0x02B2;
+const unsigned short TLV_WORK_ZIP					= 0x02BC;
+const unsigned short TLV_WORK_PHONE					= 0x02C6;
+const unsigned short TLV_WORK_FAX					= 0x02D0;
+const unsigned short TLV_WORK_HOMEPAGE				= 0x02DA;
+const unsigned short TLV_SHOW_WEB					= 0x02F8;
+const unsigned short TLV_NEED_AUTH					= 0x030C;
+const unsigned short TLV_TIMEZONE					= 0x0316;
+const unsigned short TLV_ORIGINALLY_CITY			= 0x0320;
+const unsigned short TLV_ORIGINALLY_STATE			= 0x032A;
+const unsigned short TLV_ORIGINALLY_COUNTRY			= 0x0334;
+
 const char SEARCH_STATE_OFFLINE  = 0;
 const char SEARCH_STATE_ONLINE   = 1;
 const char SEARCH_STATE_DISABLED = 2;
 
-const unsigned ERROR_RATE_LIMIT		= 0x0002;
-
 const unsigned INFO_REQUEST_TIMEOUT = 60;
-const unsigned INFO_PAUSE_TIMEOUT   = 300;
-
-const unsigned short PAUSE_ID		= 0xFFFF;
 
 class ServerRequest
 {
@@ -120,9 +161,9 @@ void ICQClient::clearServerRequests()
         delete *it_req;
     }
     varRequests.clear();
-    list<unsigned long>::iterator it;
+    list<InfoRequest>::iterator it;
     for (it = infoRequests.begin(); it != infoRequests.end(); ++it){
-        Contact *contact = getContacts()->contact(*it);
+        Contact *contact = getContacts()->contact((*it).uin);
         if (contact == NULL)
             continue;
         Event e(EventFetchInfoFail, contact);
@@ -169,9 +210,9 @@ void ICQClient::snac_various(unsigned short type, unsigned short id)
                 serverRequest(ICQ_SRVxREQ_ACK_OFFLINE_MSG);
                 sendServerRequest();
                 setChatGroup();
-                addFullInfoRequest(data.owner.Uin.value, false);
-                infoRequest();
-                processListRequest();
+                addFullInfoRequest(data.owner.Uin.value);
+                m_bReady = true;
+                processSendQueue();
                 break;
             case ICQ_SRVxOFFLINE_MSG:{
                     unsigned long uin;
@@ -195,7 +236,7 @@ void ICQClient::snac_various(unsigned short type, unsigned short id)
                 #else
                     time_t now = time (NULL);
                     sendTM = *localtime (&now);
-                    sendTM.tm_sec  = -sendTM.tm_gmtoff;
+                    sendTM.tm_sec  = sendTM.tm_gmtoff - (sendTM.tm_isdst == 1 ? 3600 : 0);
                 #endif
                     sendTM.tm_year = year-1900;
                     sendTM.tm_mon  = month-1;
@@ -269,7 +310,7 @@ void ICQClient::sendServerRequest()
     size = (unsigned short)(packet_size - 0x16);
     packet[0x14] = (char)(size & 0xFF);
     packet[0x15] = (char)((size >> 8) & 0xFF);
-    sendPacket();
+    sendPacket(true);
 }
 
 void ICQClient::sendMessageRequest()
@@ -302,7 +343,7 @@ FullInfoRequest::FullInfoRequest(ICQClient *client, unsigned short id, unsigned
     m_uin = uin;
 }
 
-void FullInfoRequest::fail(unsigned short error_code)
+void FullInfoRequest::fail(unsigned short)
 {
     Contact *contact = NULL;
     if (m_nParts){
@@ -321,11 +362,7 @@ void FullInfoRequest::fail(unsigned short error_code)
         Event e(EventFetchInfoFail, contact);
         e.process();
     }
-    if (error_code == ERROR_RATE_LIMIT){
-        m_client->infoRequestPause();
-    }else{
-        m_client->removeFullInfoRequest(m_uin);
-    }
+    m_client->removeFullInfoRequest(m_uin);
 }
 
 string FullInfoRequest::unpack_list(Buffer &b)
@@ -357,7 +394,7 @@ bool FullInfoRequest::answer(Buffer &b, unsigned short nSubtype)
     }else{
         data = m_client->findContact(number(m_uin).c_str(), NULL, false, contact);
         if (data == NULL){
-            log(L_DEBUG, "Info request %u not found", m_uin);
+            log(L_DEBUG, "Info request %lu not found", m_uin);
             m_client->removeFullInfoRequest(m_uin);
             return true;
         }
@@ -493,11 +530,11 @@ bool FullInfoRequest::answer(Buffer &b, unsigned short nSubtype)
             if (tm->tm_isdst) tz -= (60 * 60);
 #endif
             tz = - tz / (30 * 60);
+            m_client->setupContact(getContacts()->owner(), data);
             if (data->TimeZone.value != (unsigned)tz){
                 data->TimeZone.value = tz;
                 m_client->setMainInfo(data);
             }
-            m_client->setupContact(getContacts()->owner(), data);
             Event eContact(EventContactChanged, getContacts()->owner());
             eContact.process();
             Event e(EventClientChanged, m_client);
@@ -509,82 +546,71 @@ bool FullInfoRequest::answer(Buffer &b, unsigned short nSubtype)
     return false;
 }
 
-void ICQClient::infoRequest()
+unsigned ICQClient::processInfoRequest()
 {
-    m_infoTimer->stop();
     if ((getState() != Connected) || infoRequests.empty())
-        return;
-    if (m_infoRequestId == PAUSE_ID)
-        return;
-    unsigned long uin = infoRequests.front();
-    serverRequest(ICQ_SRVxREQ_MORE);
-    m_socket->writeBuffer << ((uin == data.owner.Uin.value) ? ICQ_SRVxREQ_OWN_INFO : ICQ_SRVxREQ_FULL_INFO);
-    m_socket->writeBuffer.pack(uin);
-    sendServerRequest();
-    m_infoTimer->start(INFO_REQUEST_TIMEOUT * 1000);
-    m_infoRequestId = m_nMsgSequence;
-    varRequests.push_back(new FullInfoRequest(this, m_infoRequestId, uin));
-}
-
-void ICQClient::infoRequestFail()
-{
-    m_infoTimer->stop();
-    if (m_infoRequestId == PAUSE_ID){
-        m_infoRequestId = 0;
-        infoRequest();
-        return;
+        return 0;
+    for (list<InfoRequest>::iterator it = infoRequests.begin(); it != infoRequests.end(); ++it){
+        if ((*it).request_id)
+            continue;
+        unsigned delay = delayTime(SNAC(ICQ_SNACxFAM_VARIOUS, ICQ_SNACxVAR_REQxSRV));
+        if (delay)
+            return delay;
+        unsigned long uin = (*it).uin;
+        serverRequest(ICQ_SRVxREQ_MORE);
+        m_socket->writeBuffer << ((uin == data.owner.Uin.value) ? ICQ_SRVxREQ_OWN_INFO : ICQ_SRVxREQ_FULL_INFO);
+        m_socket->writeBuffer.pack(uin);
+        sendServerRequest();
+        (*it).request_id = m_nMsgSequence;
+        time_t now;
+        time(&now);
+        (*it).start_time = now;
+        varRequests.push_back(new FullInfoRequest(this, m_nMsgSequence, uin));
     }
-    ServerRequest *req = findServerRequest(m_infoRequestId);
-    if (req)
-        req->fail();
+    return 0;
 }
 
-void ICQClient::infoRequestPause()
+void ICQClient::checkInfoRequest()
 {
-    m_infoRequestId = PAUSE_ID;
-    m_infoTimer->stop();
-    m_infoTimer->start(INFO_PAUSE_TIMEOUT * 1000);
-}
-
-void ICQClient::addFullInfoRequest(unsigned long uin, bool bInLast)
-{
-    bool bCanStart = infoRequests.empty();
-    if (bInLast){
-        list<unsigned long>::iterator it;
-        for (it = infoRequests.begin(); it != infoRequests.end(); ++it){
-            if ((*it) == uin)
-                return;
+    time_t now;
+    time(&now);
+    for (list<InfoRequest>::iterator it = infoRequests.begin(); it != infoRequests.end(); ){
+        if (((*it).request_id == 0) || ((time_t)((*it).start_time + INFO_REQUEST_TIMEOUT) < now)){
+            ++it;
+            continue;
         }
-        infoRequests.push_back(uin);
-    }else{
-        if (!infoRequests.empty() && (infoRequests.front() == uin))
+        ServerRequest *req = findServerRequest((*it).request_id);
+        if (req){
+            req->fail();
+        }else{
+            infoRequests.erase(it);
+        }
+        it = infoRequests.begin();
+    }
+}
+
+void ICQClient::addFullInfoRequest(unsigned long uin)
+{
+    for (list<InfoRequest>::iterator it = infoRequests.begin(); it != infoRequests.end(); ++it){
+        if ((*it).uin == uin)
             return;
-        list<unsigned long>::iterator it;
-        for (it = infoRequests.begin(); it != infoRequests.end(); ++it){
-            if ((*it) == uin){
-                infoRequests.erase(it);
-                break;
-            }
-        }
-        infoRequests.push_front(uin);
     }
-    if (bCanStart)
-        infoRequest();
+    InfoRequest r;
+    r.uin = uin;
+    r.request_id = 0;
+    infoRequests.push_back(r);
+    processSendQueue();
 }
 
 void ICQClient::removeFullInfoRequest(unsigned long uin)
 {
-    m_infoTimer->stop();
-    list<unsigned long>::iterator it;
+    list<InfoRequest>::iterator it;
     for (it = infoRequests.begin(); it != infoRequests.end(); ++it){
-        if ((*it) == uin){
+        if ((*it).uin == uin){
             infoRequests.erase(it);
             break;
         }
     }
-    if (infoRequests.empty())
-        return;
-    QTimer::singleShot(1000, this, SLOT(infoRequest()));
 }
 
 // _________________________________________________________________________________________
@@ -627,14 +653,18 @@ bool SearchWPRequest::answer(Buffer &b, unsigned short nSubType)
     b >> n;
     b.unpack(res.data.Uin.value);
     char waitAuth;
-    char state;
+    unsigned short state;
+    char gender;
+    unsigned short age;
     b
     >> &res.data.Nick.ptr
     >> &res.data.FirstName.ptr
     >> &res.data.LastName.ptr
     >> &res.data.EMail.ptr
-    >> waitAuth
-    >> state;
+    >> waitAuth;
+    b.unpack(state);
+    b >> gender;
+    b.unpack(age);
 
     if (waitAuth)
         res.data.WaitAuth.bValue = true;
@@ -649,13 +679,20 @@ bool SearchWPRequest::answer(Buffer &b, unsigned short nSubType)
         res.data.Status.value = STATUS_UNKNOWN;
         break;
     }
+    res.data.Gender.value = gender;
+    res.data.Age.value    = age;
 
-    Event e(EventSearch, &res);
-    e.process();
+    if (res.data.Uin.value != m_client->data.owner.Uin.value){
+        Event e(EventSearch, &res);
+        e.process();
+    }
     free_data(ICQProtocol::icqUserData, &res.data);
 
     if (nSubType == 0xAE01){
+        unsigned long all;
+        b >> all;
         load_data(ICQProtocol::icqUserData, &res.data, NULL);
+        res.data.Uin.value = all;
         Event e(EventSearchDone, &res);
         e.process();
         free_data(ICQProtocol::icqUserData, &res.data);
@@ -670,47 +707,58 @@ unsigned short ICQClient::findByUin(unsigned long uin)
         return (unsigned short)(-1);
     serverRequest(ICQ_SRVxREQ_MORE);
     m_socket->writeBuffer
-    << ICQ_SRVxREQ_WP_INFO_UIN
-    << 0x36010400L;
-    m_socket->writeBuffer.pack(uin);
+    << ICQ_SRVxREQ_WP_UIN;
+    m_socket->writeBuffer.tlvLE(TLV_UIN, uin);
     sendServerRequest();
     varRequests.push_back(new SearchWPRequest(this, m_nMsgSequence));
     return m_nMsgSequence;
 }
 
+unsigned short ICQClient::findByMail(const char *mail)
+{
+    if (getState() != Connected)
+        return (unsigned short)(-1);
+    serverRequest(ICQ_SRVxREQ_MORE);
+    m_socket->writeBuffer
+    << ICQ_SRVxREQ_WP_MAIL;
+    m_socket->writeBuffer.tlvLE(TLV_EMAIL, mail);
+    sendServerRequest();
+    varRequests.push_back(new SearchWPRequest(this, m_nMsgSequence));
+    return m_nMsgSequence;
+}
+
+void ICQClient::packTlv(unsigned short tlv, unsigned short code, const char *keywords)
+{
+    string k;
+    if (keywords)
+        k = keywords;
+    if ((code == 0) && k.empty())
+        return;
+    Buffer b;
+    b.pack(code);
+    b << k;
+    m_socket->writeBuffer.tlvLE(tlv, b);
+}
+
 unsigned short ICQClient::findWP(const char *szFirst, const char *szLast, const char *szNick,
                                  const char *szEmail, char age, char nGender,
-                                 char nLanguage, const char *szCity, const char *szState,
+                                 unsigned short nLanguage, const char *szCity, const char *szState,
                                  unsigned short nCountryCode,
                                  const char *szCoName, const char *szCoDept, const char *szCoPos,
-                                 char nOccupation,
+                                 unsigned short nOccupation,
                                  unsigned short nPast, const char *szPast,
                                  unsigned short nInterests, const char *szInterests,
-                                 unsigned short nAffiliation, const char *szAffiliation,
+                                 unsigned short nAffilation, const char *szAffilation,
                                  unsigned short nHomePage, const char *szHomePage,
-                                 bool bOnlineOnly)
+                                 const char *szKeyWords, bool bOnlineOnly)
 {
     if (getState() != Connected)
         return (unsigned short)(-1);
     serverRequest(ICQ_SRVxREQ_MORE);
     m_socket->writeBuffer << ICQ_SRVxREQ_WP_FULL;
 
-    string sFirst = szFirst ? szFirst : "";
-    string sLast = szLast ? szLast : "";
-    string sNick = szNick ? szNick : "";
-    string sEmail = szEmail ? szEmail : "";
-    string sCity = szCity ? szCity : "";
-    string sState = szState ? szState : "";
-    string sCoName = szCoName ? szCoName : "";
-    string sCoDept = szCoDept ? szCoDept : "";
-    string sCoPos = szCoPos ? szCoPos : "";
-    string sPast = szPast ? szPast : "";
-    string sInterests = szInterests ? szInterests : "";
-    string sAffiliation = szAffiliation ? szAffiliation : "";
-    string sHomePage = szHomePage ? szHomePage : "";
-
-    unsigned short nMinAge = 0;
-    unsigned short nMaxAge = 0;
+    unsigned long nMinAge = 0;
+    unsigned long nMaxAge = 0;
     switch (age){
     case 1:
         nMinAge = 18;
@@ -738,38 +786,43 @@ unsigned short ICQClient::findWP(const char *szFirst, const char *szLast, const
         break;
     }
 
-    m_socket->writeBuffer
-    << sFirst
-    << sLast
-    << sNick
-    << sEmail;
-    m_socket->writeBuffer.pack(nMinAge);
-    m_socket->writeBuffer.pack(nMaxAge);
-    m_socket->writeBuffer
-    << nGender
-    << nLanguage
-    << sCity
-    << sState;
-    m_socket->writeBuffer.pack(nCountryCode);
-    m_socket->writeBuffer
-    << sCoName
-    << sCoDept
-    << sCoPos
-
-    << nOccupation
-    << nPast
-    << sPast
-    << nInterests
-    << sInterests
-    << nAffiliation
-    << sAffiliation
-    << nHomePage
-    << sHomePage;
-
-    m_socket->writeBuffer << (char)(bOnlineOnly ? 1 : 0);
-
+    if (szCity && *szCity)
+        m_socket->writeBuffer.tlvLE(TLV_CITY, szCity);
+    if (szState && *szState)
+        m_socket->writeBuffer.tlvLE(TLV_STATE, szState);
+    if (szCoName && *szCoName)
+        m_socket->writeBuffer.tlvLE(TLV_WORK_COMPANY, szCoName);
+    if (szCoDept && *szCoDept)
+        m_socket->writeBuffer.tlvLE(TLV_WORK_DEPARTMENT, szCoDept);
+    if (szCoPos && *szCoPos)
+        m_socket->writeBuffer.tlvLE(TLV_WORK_POSITION, szCoPos);
+    if (nMinAge || nMaxAge)
+        m_socket->writeBuffer.tlvLE(TLV_AGE_RANGE, (nMaxAge << 16) + nMinAge);
+    if (nGender)
+        m_socket->writeBuffer.tlvLE(TLV_GENDER, nGender);
+    if (nLanguage)
+        m_socket->writeBuffer.tlvLE(TLV_LANGUAGE, nLanguage);
+    if (nCountryCode)
+        m_socket->writeBuffer.tlvLE(TLV_COUNTRY, nCountryCode);
+    if (nOccupation)
+        m_socket->writeBuffer.tlvLE(TLV_WORK_OCCUPATION, nOccupation);
+    packTlv(TLV_PAST, nPast, szPast);
+    packTlv(TLV_INTERESTS, nInterests, szInterests);
+    packTlv(TLV_AFFILATIONS, nAffilation, szAffilation);
+    packTlv(TLV_HOMEPAGE, nHomePage, szHomePage);
+    if (szFirst && *szFirst)
+        m_socket->writeBuffer.tlvLE(TLV_FIRST_NAME, szFirst);
+    if (szLast && *szLast)
+        m_socket->writeBuffer.tlvLE(TLV_LAST_NAME, szLast);
+    if (szNick && *szNick)
+        m_socket->writeBuffer.tlvLE(TLV_NICK, szNick);
+    if (szKeyWords && *szKeyWords)
+        m_socket->writeBuffer.tlvLE(TLV_KEYWORDS, szKeyWords);
+    if (szEmail && *szEmail)
+        m_socket->writeBuffer.tlvLE(TLV_EMAIL, szEmail);
+    if (bOnlineOnly)
+        m_socket->writeBuffer.tlvLE(TLV_SEARCH_ONLINE, (char)1);
     sendServerRequest();
-
     varRequests.push_back(new SearchWPRequest(this, m_nMsgSequence));
     return m_nMsgSequence;
 }
@@ -1192,16 +1245,16 @@ void ICQClient::setClientInfo(void *_data)
         QString number = getToken(phoneValue, ',');
         QString type = getToken(phoneValue, ',');
         if (type == "Home Phone"){
-            set_str(&d->HomePhone.ptr, fromUnicode(number, NULL).c_str());
+            set_str(&d->HomePhone.ptr, getContacts()->fromUnicode(NULL, number).c_str());
         }else if (type == "Home Fax"){
-            set_str(&d->HomeFax.ptr, fromUnicode(number, NULL).c_str());
+            set_str(&d->HomeFax.ptr, getContacts()->fromUnicode(NULL, number).c_str());
         }else if (type == "Work Phone"){
-            set_str(&d->WorkPhone.ptr, fromUnicode(number, NULL).c_str());
+            set_str(&d->WorkPhone.ptr, getContacts()->fromUnicode(NULL, number).c_str());
         }else if (type == "Work Fax"){
-            set_str(&d->WorkFax.ptr, fromUnicode(number, NULL).c_str());
+            set_str(&d->WorkFax.ptr, getContacts()->fromUnicode(NULL, number).c_str());
         }else if (type == "Private Cellular"){
             number += " SMS";
-            set_str(&d->PrivateCellular.ptr, fromUnicode(number, NULL).c_str());
+            set_str(&d->PrivateCellular.ptr, getContacts()->fromUnicode(NULL, number).c_str());
         }
     }
     d->HiddenEMail.bValue = false;
@@ -1212,12 +1265,12 @@ void ICQClient::setClientInfo(void *_data)
         QString mail = getToken(mailItem, '/');
         if (!s.empty())
             s += ';';
-        s += fromUnicode(mail, NULL);
+        s += getContacts()->fromUnicode(NULL, mail);
         s += '/';
         if (mailItem.length())
             s += '-';
         if (d->EMail.ptr == NULL){
-            set_str(&d->EMail.ptr, fromUnicode(mail, NULL).c_str());
+            set_str(&d->EMail.ptr, getContacts()->fromUnicode(NULL, mail).c_str());
             d->HiddenEMail.bValue = !mailItem.isEmpty();
         }
     }
@@ -1404,6 +1457,9 @@ void SetPasswordRequest::fail(unsigned short error_code)
     d.code    = 0;
     d.err_str = I18N_NOOP("Change password fail");
     d.args    = NULL;
+    d.flags   = ERR_ERROR;
+    d.options = NULL;
+    d.id	  = CmdPasswordFail;
     Event e(EventClientError, &d);
     e.process();
 }
@@ -1414,7 +1470,7 @@ void ICQClient::changePassword(const char *new_pswd)
     serverRequest(ICQ_SRVxREQ_MORE);
     m_socket->writeBuffer
     << ICQ_SRVxREQ_CHANGE_PASSWD
-    << fromUnicode(pwd, NULL);
+    << getContacts()->fromUnicode(NULL, pwd);
     sendServerRequest();
     varRequests.push_back(new SetPasswordRequest(this, m_nMsgSequence, new_pswd));
 }
@@ -1516,7 +1572,7 @@ bool SMSRequest::answer(Buffer &b, unsigned short code)
             }
         }
     }
-    m_client->processSMSQueue();
+    m_client->processSendQueue();
     return true;
 }
 
@@ -1532,7 +1588,7 @@ void SMSRequest::fail(unsigned short)
     e.process();
     delete sms;
     m_client->m_sendSmsId = 0;
-    m_client->processSMSQueue();
+    m_client->processSendQueue();
 }
 
 const unsigned MAX_SMS_LEN_LATIN1	= 160;
@@ -1565,13 +1621,16 @@ static const char *months[] =
         I18N_NOOP("Dec")
     };
 
-void ICQClient::processSMSQueue()
+unsigned ICQClient::processSMSQueue()
 {
     if (m_sendSmsId)
-        return;
+        return 0;
     for (;;){
         if (smsQueue.empty())
             break;
+        unsigned delay = delayTime(SNAC(ICQ_SNACxFAM_VARIOUS, ICQ_SNACxVAR_REQxSRV));
+        if (delay)
+            return delay;
         SendMsg &s = smsQueue.front();
         if (s.text.isEmpty() || (!(s.flags & SEND_1STPART) && (s.msg->getFlags() & MESSAGE_1ST_PART))){
             Event e(EventMessageSent, s.msg);
@@ -1625,8 +1684,9 @@ void ICQClient::processSMSQueue()
         sendServerRequest();
         varRequests.push_back(new SMSRequest(this, m_nMsgSequence));
         m_sendSmsId = m_nMsgSequence;
-        return;
+        break;
     }
+    return 0;
 }
 
 void ICQClient::clearSMSQueue()
