@@ -19,6 +19,7 @@
 #include "icqclient.h"
 #include "html.h"
 #include "core.h"
+#include "icons.h"
 
 #include <qtextcodec.h>
 #include <qregexp.h>
@@ -28,7 +29,7 @@
 #endif
 
 #include <ctype.h>
-
+#include <stdio.h>
 #include <stack>
 #include <cctype>
 
@@ -215,21 +216,26 @@ int htmlFontSizeToPt(int fontSize, int baseSize = 12)
 class RTFGenParser : public HTMLParser
 {
 public:
-    RTFGenParser(ICQClient *client, const QColor& foreColor, const char *encoding);
+    RTFGenParser(ICQClient *client, const QColor& foreColor, Contact *contact, unsigned max_size);
     string parse(const QString &text);
     // Returns the color's index in the colors table, adding the color if necessary.
     int getColorIdx(const QColor &color);
     // Returns the font face's index in the fonts table, adding the font face if necessary.
     int getFontFaceIdx(const QString &fontFace);
+    unsigned textPos;
+    stack<QString> tags;
+    stack<QString> options;
+    unsigned	m_res_size;
 protected:
     virtual void text(const QString &text);
     virtual void tag_start(const QString &tag, const list<QString> &attrs);
     virtual void tag_end(const QString &tag);
     string res;
     ICQClient  *m_client;
-    const char *m_encoding;
+    Contact    *m_contact;
     QTextCodec *m_codec;
     bool		m_bSpace;
+    unsigned	m_max_size;
 
     TagStack m_tags;
 
@@ -250,11 +256,12 @@ protected:
     } m_paragraphDir;
 };
 
-RTFGenParser::RTFGenParser(ICQClient *client, const QColor& foreColor, const char *encoding)
+RTFGenParser::RTFGenParser(ICQClient *client, const QColor& foreColor, Contact *contact, unsigned max_size)
 {
     m_client    = client;
-    m_encoding  = encoding;
+    m_contact   = contact;
     m_foreColor = foreColor;
+    m_max_size	= max_size;
     m_lastParagraphPos = 0;
     m_paragraphDir = DirUnknown;
 }
@@ -314,17 +321,17 @@ int RTFGenParser::getFontFaceIdx(const QString& fontFace)
 string RTFGenParser::parse(const QString &text)
 {
     res = "";
-    m_codec = m_client->getCodec(m_encoding);
-    m_encoding = m_codec->name();
+    m_res_size = 0;
+    m_codec = getContacts()->getCodec(m_contact);
     int charset = 0;
-    for (const ENCODING *c = ICQPlugin::core->encodings; c->language; c++){
-        if (!strcasecmp(c->codec, m_encoding)){
+    for (const ENCODING *c = getContacts()->getEncodings(); c->language; c++){
+        if (!strcasecmp(c->codec, m_codec->name())){
             charset = c->rtf_code;
             break;
         }
     }
 #ifdef WIN32
-    if ((charset == 0) && !strcasecmp(m_encoding, "system")){
+    if ((charset == 0) && !strcasecmp(m_codec->name(), "system")){
         char buff[256];
         int res = GetLocaleInfoA(LOCALE_USER_DEFAULT, LOCALE_IDEFAULTANSICODEPAGE, (char*)&buff, sizeof(buff));
         if (res){
@@ -342,10 +349,10 @@ string RTFGenParser::parse(const QString &text)
     const char *send_encoding = 0;
     m_codec = NULL;
     if (charset){
-        for (const ENCODING *c = ICQPlugin::core->encodings; c->language; c++){
+        for (const ENCODING *c = getContacts()->getEncodings(); c->language; c++){
             if ((c->rtf_code == charset) && c->bMain){
                 send_encoding = c->codec;
-                m_codec = m_client->getCodec(send_encoding);
+                m_codec = getContacts()->getCodecByName(send_encoding);
                 ansicpg = c->cp_code;
                 break;
             }
@@ -423,8 +430,24 @@ string RTFGenParser::parse(const QString &text)
 
 void RTFGenParser::text(const QString &text)
 {
+    if (m_res_size)
+        return;
+    unsigned size = res.length();
+    if (size > m_max_size){
+        textPos = start_pos;
+        m_res_size = size;
+        return;
+    }
     for (int i = 0; i < (int)(text.length()); i++){
         QChar c = text[i];
+        if (c.isSpace()){
+            unsigned size = res.length();
+            if (size > m_max_size){
+                textPos = start_pos + i;
+                m_res_size = size;
+                return;
+            }
+        }
         // In Qt, unless you force the paragraph direction with (Left/Right)
         // Ctrl-Shift (also known as Key_Direction_L and Key_Direction_R),
         // the P tag won't have a DIR attribute at all. In such cases, unlike
@@ -486,8 +509,40 @@ void RTFGenParser::text(const QString &text)
     }
 }
 
+static const char *def_smiles[] =
+    {
+        ":-)",
+        ":-0",
+        ":-|",
+        ":-/",
+        ":-(",
+        ":-{}",
+        ":*)",
+        ":'-(",
+        ";-)",
+        ":-@",
+        ":-\")",
+        ":-X",
+        ":-P",
+        "8-)",
+        "O:-)",
+        ":-D",
+        "*ANNOYED*",
+        "*DISGUSTED*",
+        "*DROOLING*",
+        "*GIGGLING*",
+        "*JOKINGLY*",
+        "*SHOCKED*",
+        "*WHINING*",
+        "*SURPRISED*",
+        "*SURPRISED*",
+        "*IN LOVE*"
+    };
+
 void RTFGenParser::tag_start(const QString &tagName, const list<QString> &attrs)
 {
+    if (m_res_size)
+        return;
     CharStyle parentStyle, style;
     {
         Tag* pParentTag = m_tags.getTopTagWithCharStyle();
@@ -497,6 +552,25 @@ void RTFGenParser::tag_start(const QString &tagName, const list<QString> &attrs)
         }
     }
     style = parentStyle;
+    if ((tagName == "b") || (tagName == "i") || (tagName == "u") ||
+            (tagName == "font") || (tagName == "p") || (tagName == "span")){
+        QString tag = tagName;
+        QString option;
+        for (list<QString>::const_iterator it = attrs.begin(); it != attrs.end(); ++it){
+            QString key = *it;
+            ++it;
+            QString value = *it;
+            option += " ";
+            option += key;
+            if (!value.isEmpty()){
+                option += "=\"";
+                option += value;
+                option += "\"";
+            }
+        }
+        tags.push(tag);
+        options.push(option);
+    }
 
     if (tagName == "b"){
         style.bold = true;
@@ -564,6 +638,7 @@ void RTFGenParser::tag_start(const QString &tagName, const list<QString> &attrs)
     }
     else if (tagName == "img"){
         QString src;
+        QString alt;
         for (list<QString>::const_iterator it = attrs.begin(); it != attrs.end(); ++it){
             QString name = (*it);
             ++it;
@@ -572,26 +647,32 @@ void RTFGenParser::tag_start(const QString &tagName, const list<QString> &attrs)
                 src = value;
                 break;
             }
-        }
-        if (src.left(10) != "icon:smile")
-            return;
-        bool bOK;
-        unsigned nSmile = src.mid(10).toUInt(&bOK, 16);
-        if (!bOK)
-            return;
-        if (nSmile < 16){
-            res += "<##icqimage000";
-            if (nSmile < 10){
-                res += (char)(nSmile + '0');
-            }else{
-                res += (char)(nSmile - 10 + 'A');
+            if (name == "alt"){
+                alt = value;
+                break;
             }
-            res += ">";
-            return;
         }
-        const smile *p = smiles(nSmile);
-        if (p)
-            res += p->paste;
+        if (src.left(5) == "icon:"){
+            list<string> smiles = getIcons()->getSmile(src.mid(5).latin1());
+            for (list<string>::iterator its = smiles.begin(); its != smiles.end(); ++its){
+                string s = *its;
+                for (unsigned nSmile = 0; nSmile < 26; nSmile++){
+                    if (s != def_smiles[nSmile])
+                        continue;
+                    res += "<##icqimage00";
+                    char buf[4];
+                    sprintf(buf, "%02X", nSmile);
+                    res += buf;
+                    res += ">";
+                    return;
+                }
+            }
+            if (!smiles.empty()){
+                text(QString::fromUtf8(smiles.front().c_str()));
+                return;
+            }
+        }
+        text(alt);
         return;
     }
 
@@ -696,6 +777,19 @@ void RTFGenParser::tag_start(const QString &tagName, const list<QString> &attrs)
 
 void RTFGenParser::tag_end(const QString &tagName)
 {
+    if (m_res_size)
+        return;
+    if ((tagName == "b") || (tagName == "i") || (tagName == "u") ||
+            (tagName == "font") || (tagName == "p") || (tagName == "span")){
+        while (!tags.empty()){
+            QString tag = tags.top();
+            tags.pop();
+            options.pop();
+            if (tag == tagName)
+                break;
+        }
+    }
+
     // Roll back until we find our tag.
     bool found = false;
     for(Tag* pTag = m_tags.peek(); pTag != NULL && !found; pTag = m_tags.peek())
@@ -748,16 +842,34 @@ void RTFGenParser::tag_end(const QString &tagName)
     }
 }
 
-string ICQClient::createRTF(const QString &text, unsigned long foreColor, const char *encoding)
+string ICQClient::createRTF(QString &text, QString &part, unsigned long foreColor, Contact *contact, unsigned max_size)
 {
-    RTFGenParser p(this, foreColor, encoding);
-    return p.parse(text);
+    RTFGenParser p(this, foreColor, contact, max_size);
+    string res = p.parse(text);
+    if (p.m_res_size == 0){
+        part = text;
+        text = "";
+        return res;
+    }
+    QString endTags;
+    QString startTags;
+    while (!p.tags.empty()){
+        QString tag = p.tags.top();
+        QString option = p.options.top();
+        p.tags.pop();
+        p.options.pop();
+        endTags   += "</" + tag + ">";
+        startTags = QString("<") + tag + option + ">" + startTags;
+    }
+    part = text.left(p.textPos) + endTags;
+    text = startTags + text.mid(p.textPos);
+    return res;
 }
 
 class ImageParser : public HTMLParser
 {
 public:
-    ImageParser(unsigned maxSmile);
+    ImageParser(bool bIcq);
     QString parse(const QString &text);
 protected:
     virtual void text(const QString &text);
@@ -767,12 +879,12 @@ protected:
     void endBody();
     QString res;
     bool	 m_bBody;
-    unsigned m_maxSmile;
+    bool	 m_bIcq;
 };
 
-ImageParser::ImageParser(unsigned maxSmile)
+ImageParser::ImageParser(bool bIcq)
 {
-    m_maxSmile = maxSmile;
+    m_bIcq = bIcq;
 }
 
 QString ImageParser::parse(const QString &text)
@@ -821,6 +933,7 @@ void ImageParser::tag_start(const QString &tag, const list<QString> &attrs)
         return;
     if (tag == "img"){
         QString src;
+        QString alt;
         for (list<QString>::const_iterator it = attrs.begin(); it != attrs.end(); ++it){
             QString name = *it;
             ++it;
@@ -829,20 +942,36 @@ void ImageParser::tag_start(const QString &tag, const list<QString> &attrs)
                 src = value;
                 break;
             }
-        }
-        if (src.left(10) != "icon:smile")
-            return;
-        bool bOK;
-        unsigned nIcon = src.mid(10).toUInt(&bOK, 16);
-        if (!bOK)
-            return;
-        if (nIcon >= m_maxSmile){
-            const smile *p = smiles(nIcon);
-            if (p){
-                res += p->paste;
-                return;
+            if (name == "alt"){
+                alt = value;
+                break;
             }
         }
+        if (src.left(5) != "icon:"){
+            text(alt);
+            return;
+        }
+        list<string> smiles = getIcons()->getSmile(src.mid(5).latin1());
+        if (smiles.empty()){
+            text(alt);
+            return;
+        }
+        if (m_bIcq){
+            for (list<string>::iterator its = smiles.begin(); its != smiles.end(); ++its){
+                for (unsigned nSmile = 0; nSmile < 26; nSmile++){
+                    if ((*its) != def_smiles[nSmile])
+                        continue;
+                    res += "<img src=\"icon:smile";
+                    char b[4];
+                    sprintf(b, "%X", nSmile);
+                    res += b;
+                    res += "\">";
+                    return;
+                }
+            }
+        }
+        text(QString::fromUtf8(smiles.front().c_str()));
+        return;
     }
     res += "<";
     res += oTag;
@@ -875,9 +1004,9 @@ void ImageParser::tag_end(const QString &tag)
     res += ">";
 }
 
-QString ICQClient::removeImages(const QString &text, unsigned maxSmile)
+QString ICQClient::removeImages(const QString &text, bool bIcq)
 {
-    ImageParser p(maxSmile);
+    ImageParser p(bIcq);
     return p.parse(text);
 }
 
