@@ -24,11 +24,12 @@
 #include <qmultilineedit.h>
 #include <qcombobox.h>
 
-WorkInfo::WorkInfo(QWidget *parent, struct ICQUserData *data, ICQClient *client)
+WorkInfo::WorkInfo(QWidget *parent, struct ICQUserData *data, unsigned contact, ICQClient *client)
         : WorkInfoBase(parent)
 {
-    m_data   = data;
-    m_client = client;
+    m_data    = data;
+    m_client  = client;
+    m_contact = contact;
     btnSite->setPixmap(Pict("home"));
     connect(btnSite, SIGNAL(clicked()), this, SLOT(goUrl()));
     if (m_data){
@@ -88,32 +89,35 @@ static const ext_info occupations[] =
         { I18N_NOOP("Medical/Health"), 17  },
         { I18N_NOOP("Military"), 18  },
         { I18N_NOOP("Non-Goverment Organisation"), 19  },
-        { I18N_NOOP("Other services"), 20  },
-        { I18N_NOOP("Professional"), 21  },
-        { I18N_NOOP("Retail"), 22  },
-        { I18N_NOOP("Retired"), 23  },
-        { I18N_NOOP("Science & Research"), 24  },
-        { I18N_NOOP("Sports"), 25  },
-        { I18N_NOOP("Technical"), 26  },
-        { I18N_NOOP("University student"), 27  },
-        { I18N_NOOP("Web building"), 28  },
+        { I18N_NOOP("Professional"), 20  },
+        { I18N_NOOP("Retail"), 21  },
+        { I18N_NOOP("Retired"), 22  },
+        { I18N_NOOP("Science & Research"), 23  },
+        { I18N_NOOP("Sports"), 24  },
+        { I18N_NOOP("Technical"), 25  },
+        { I18N_NOOP("University student"), 26  },
+        { I18N_NOOP("Web building"), 27  },
+        { I18N_NOOP("Other services"), 99  },
         { "", 0  }
     };
 
+const ext_info *p_occupations = occupations;
+
 void WorkInfo::fill()
 {
     ICQUserData *data = m_data;
     if (data == NULL) data = &m_client->data.owner;
-    edtAddress->setText(m_client->toUnicode(data->WorkAddress.ptr, data));
-    edtCity->setText(m_client->toUnicode(data->WorkCity.ptr, data));
-    edtState->setText(m_client->toUnicode(data->WorkState.ptr, data));
-    edtZip->setText(m_client->toUnicode(data->WorkZip.ptr, data));
+    Contact *contact = getContacts()->contact(m_contact);
+    edtAddress->setText(getContacts()->toUnicode(contact, data->WorkAddress.ptr));
+    edtCity->setText(getContacts()->toUnicode(contact, data->WorkCity.ptr));
+    edtState->setText(getContacts()->toUnicode(contact, data->WorkState.ptr));
+    edtZip->setText(getContacts()->toUnicode(contact, data->WorkZip.ptr));
     initCombo(cmbCountry, (unsigned short)(data->WorkCountry.value), getCountries());
     initCombo(cmbOccupation, (unsigned short)(data->Occupation.value), occupations);
-    edtName->setText(m_client->toUnicode(data->WorkName.ptr, data));
-    edtDept->setText(m_client->toUnicode(data->WorkDepartment.ptr, data));
-    edtPosition->setText(m_client->toUnicode(data->WorkPosition.ptr, data));
-    edtSite->setText(m_client->toUnicode(data->WorkHomepage.ptr, data));
+    edtName->setText(getContacts()->toUnicode(contact, data->WorkName.ptr));
+    edtDept->setText(getContacts()->toUnicode(contact, data->WorkDepartment.ptr));
+    edtPosition->setText(getContacts()->toUnicode(contact, data->WorkPosition.ptr));
+    edtSite->setText(getContacts()->toUnicode(contact, data->WorkHomepage.ptr));
     urlChanged(edtSite->text());
 }
 
@@ -122,7 +126,9 @@ void WorkInfo::goUrl()
     QString url = edtSite->text();
     if (url.isEmpty())
         return;
-    Event e(EventGoURL, (void*)(const char*)(url.local8Bit()));
+    string url_str;
+    url_str = url.local8Bit();
+    Event e(EventGoURL, (void*)url_str.c_str());
     e.process();
 }
 
@@ -136,19 +142,19 @@ void WorkInfo::apply(Client *client, void *_data)
     if (client != m_client)
         return;
     ICQUserData *data = (ICQUserData*)_data;
-    set_str(&data->WorkAddress.ptr, m_client->fromUnicode(edtAddress->text(), NULL).c_str());
-    set_str(&data->WorkCity.ptr, m_client->fromUnicode(edtCity->text(), NULL).c_str());
-    set_str(&data->WorkState.ptr, m_client->fromUnicode(edtState->text(), NULL).c_str());
-    set_str(&data->WorkZip.ptr, m_client->fromUnicode(edtZip->text(), NULL).c_str());
+    set_str(&data->WorkAddress.ptr, getContacts()->fromUnicode(NULL, edtAddress->text()).c_str());
+    set_str(&data->WorkCity.ptr, getContacts()->fromUnicode(NULL, edtCity->text()).c_str());
+    set_str(&data->WorkState.ptr, getContacts()->fromUnicode(NULL, edtState->text()).c_str());
+    set_str(&data->WorkZip.ptr, getContacts()->fromUnicode(NULL, edtZip->text()).c_str());
     data->WorkCountry.value = getComboValue(cmbCountry, getCountries());
     data->Occupation.value = getComboValue(cmbOccupation, occupations);
-    set_str(&data->WorkName.ptr, m_client->fromUnicode(edtName->text(), NULL).c_str());
-    set_str(&data->WorkDepartment.ptr, m_client->fromUnicode(edtDept->text(), NULL).c_str());
-    set_str(&data->WorkPosition.ptr, m_client->fromUnicode(edtPosition->text(), NULL).c_str());
-    set_str(&data->WorkHomepage.ptr, m_client->fromUnicode(edtSite->text(), NULL).c_str());
+    set_str(&data->WorkName.ptr, getContacts()->fromUnicode(NULL, edtName->text()).c_str());
+    set_str(&data->WorkDepartment.ptr, getContacts()->fromUnicode(NULL, edtDept->text()).c_str());
+    set_str(&data->WorkPosition.ptr, getContacts()->fromUnicode(NULL, edtPosition->text()).c_str());
+    set_str(&data->WorkHomepage.ptr, getContacts()->fromUnicode(NULL, edtSite->text()).c_str());
 }
 
-#ifndef WIN32
+#ifndef _MSC_VER
 #include "workinfo.moc"
 #endif
 
