@@ -162,7 +162,7 @@ void RostersRequest::element_end(const char *el)
                 string resource;
                 data = m_client->findContact(m_jid.c_str(), m_name.c_str(), true, contact, resource);
                 if (m_bSubscription){
-                    contact->setTemporary(CONTACT_TEMP);
+                    contact->setFlags(CONTACT_TEMP);
                     Event eContact(EventContactChanged, contact);
                     eContact.process();
                     m_client->auth_request(m_jid.c_str(), MessageAuthRequest, m_subscription.c_str(), true);
@@ -340,7 +340,7 @@ InfoRequest::~InfoRequest()
         bChanged |= set_str(&data->Phone.ptr, m_phone.c_str());
 
         if (m_bVCard){
-            Event e(static_cast<JabberPlugin*>(m_client->protocol()->plugin())->EventVCard, data);
+            Event e(EventVCard, data);
             e.process();
             free_data(jabberUserData, &u_data);
             return;
@@ -445,7 +445,7 @@ void InfoRequest::element_start(const char *el, const char**)
         m_bStarted = true;
         return;
     }
-    if (!strcmp(el, "nick")){
+    if (!strcmp(el, "nickname")){
         m_data = &m_nick;
         return;
     }
@@ -576,90 +576,46 @@ public:
     SetInfoRequest(JabberClient *client, JabberUserData *data);
 protected:
     virtual void element_start(const char *el, const char **attr);
-    string	m_jid;
-    string  m_firstName;
-    string	m_nick;
-    string	m_desc;
-    string	m_bday;
-    string	m_url;
-    string	m_orgName;
-    string	m_orgUnit;
-    string	m_title;
-    string	m_role;
-    string	m_street;
-    string	m_ext;
-    string	m_city;
-    string	m_region;
-    string	m_pcode;
-    string	m_country;
 };
 
 SetInfoRequest::SetInfoRequest(JabberClient *client, JabberUserData *data)
         : JabberClient::ServerRequest(client, _SET, NULL, client->buildId(data).c_str())
 {
-    m_jid	= data->ID.ptr;
-    if (data->FirstName.ptr)
-        m_firstName = data->FirstName.ptr;
-    if (data->Nick.ptr)
-        m_nick = data->Nick.ptr;
-    if (data->Desc.ptr)
-        m_desc = data->Desc.ptr;
-    if (data->Bday.ptr)
-        m_bday = data->Bday.ptr;
-    if (data->Url.ptr)
-        m_url = data->Url.ptr;
-    if (data->OrgName.ptr)
-        m_orgName = data->OrgName.ptr;
-    if (data->OrgUnit.ptr)
-        m_orgUnit = data->OrgUnit.ptr;
-    if (data->Title.ptr)
-        m_title = data->Title.ptr;
-    if (data->Role.ptr)
-        m_role = data->Role.ptr;
-    if (data->Street.ptr)
-        m_street = data->Street.ptr;
-    if (data->ExtAddr.ptr)
-        m_ext = data->ExtAddr.ptr;
-    if (data->City.ptr)
-        m_city = data->City.ptr;
-    if (data->Region.ptr)
-        m_region = data->Region.ptr;
-    if (data->PCode.ptr)
-        m_pcode = data->PCode.ptr;
-    if (data->Country.ptr)
-        m_country = data->Country.ptr;
 }
 
 void SetInfoRequest::element_start(const char *el, const char **attr)
 {
     if (!strcmp(el, "iq")){
         string type = JabberClient::get_attr("type", attr);
-        if (type == "result"){
-            set_str(&m_client->data.owner.FirstName.ptr, m_firstName.c_str());
-            set_str(&m_client->data.owner.Nick.ptr, m_nick.c_str());
-            set_str(&m_client->data.owner.Desc.ptr, m_desc.c_str());
-            set_str(&m_client->data.owner.Bday.ptr, m_bday.c_str());
-            set_str(&m_client->data.owner.Url.ptr, m_url.c_str());
-            set_str(&m_client->data.owner.OrgName.ptr, m_orgName.c_str());
-            set_str(&m_client->data.owner.OrgUnit.ptr, m_orgUnit.c_str());
-            set_str(&m_client->data.owner.Title.ptr, m_title.c_str());
-            set_str(&m_client->data.owner.Role.ptr, m_role.c_str());
-            set_str(&m_client->data.owner.Street.ptr, m_street.c_str());
-            set_str(&m_client->data.owner.ExtAddr.ptr, m_ext.c_str());
-            set_str(&m_client->data.owner.City.ptr, m_city.c_str());
-            set_str(&m_client->data.owner.Region.ptr, m_region.c_str());
-            set_str(&m_client->data.owner.PCode.ptr, m_pcode.c_str());
-            set_str(&m_client->data.owner.Country.ptr, m_country.c_str());
-        }
+        if (type == "result")
+            m_client->setInfoUpdated(false);
     }
 }
 
 void JabberClient::setClientInfo(void *_data)
 {
     JabberUserData *data = (JabberUserData*)_data;
+    if (data != &this->data.owner){
+        set_str(&this->data.owner.FirstName.ptr, data->FirstName.ptr);
+        set_str(&this->data.owner.Nick.ptr, data->Nick.ptr);
+        set_str(&this->data.owner.Desc.ptr, data->Desc.ptr);
+        set_str(&this->data.owner.Bday.ptr, data->Bday.ptr);
+        set_str(&this->data.owner.Url.ptr, data->Url.ptr);
+        set_str(&this->data.owner.OrgName.ptr, data->OrgName.ptr);
+        set_str(&this->data.owner.OrgUnit.ptr, data->OrgUnit.ptr);
+        set_str(&this->data.owner.Title.ptr, data->Title.ptr);
+        set_str(&this->data.owner.Role.ptr, data->Role.ptr);
+        set_str(&this->data.owner.Street.ptr, data->Street.ptr);
+        set_str(&this->data.owner.ExtAddr.ptr, data->ExtAddr.ptr);
+        set_str(&this->data.owner.City.ptr, data->City.ptr);
+        set_str(&this->data.owner.Region.ptr, data->Region.ptr);
+        set_str(&this->data.owner.PCode.ptr, data->PCode.ptr);
+        set_str(&this->data.owner.Country.ptr, data->Country.ptr);
+    }
+    setInfoUpdated(true);
     if (getState() != Connected)
         return;
-    SetInfoRequest *req = new SetInfoRequest(this, data);
+    SetInfoRequest *req = new SetInfoRequest(this, &this->data.owner);
     req->start_element("vCard");
     req->add_attribute("prodid", "-//HandGen//NONSGML vGen v1.0//EN");
     req->add_attribute("xmlns", "vcard-temp");
@@ -812,6 +768,35 @@ JabberClient::PresenceRequest::PresenceRequest(JabberClient *client)
 {
 }
 
+static unsigned get_number(string &s, unsigned digits)
+{
+    if (s.length() < digits){
+        s = "";
+        return 0;
+    }
+    string p = s.substr(0, digits);
+    s = s.substr(digits);
+    return atol(p.c_str());
+}
+
+static time_t fromDelay(const char *t)
+{
+    string s = t;
+    time_t now;
+    time(&now);
+    struct tm _tm = *localtime(&now);
+    _tm.tm_year = get_number(s, 4) - 1900;
+    _tm.tm_mon  = get_number(s, 2) - 1;
+    _tm.tm_mday = get_number(s, 2);
+    get_number(s, 1);
+    _tm.tm_hour = get_number(s, 2);
+    get_number(s, 1);
+    _tm.tm_min  = get_number(s, 2);
+    get_number(s, 1);
+    _tm.tm_sec  = get_number(s, 2);
+    return mktime(&_tm);
+}
+
 JabberClient::PresenceRequest::~PresenceRequest()
 {
     unsigned status = STATUS_UNKNOWN;
@@ -859,7 +844,7 @@ JabberClient::PresenceRequest::~PresenceRequest()
                 status = STATUS_ONLINE;
                 bInvisible = true;
             }else if (!m_status.empty()){
-                log(L_DEBUG, "Unsupported status %s", m_status.c_str());
+                status = STATUS_ONLINE;
             }
         }else{
             log(L_DEBUG, "Unsupported available status %s", m_show.c_str());
@@ -867,6 +852,21 @@ JabberClient::PresenceRequest::~PresenceRequest()
     }else{
         log(L_DEBUG, "Unsupported presence type %s", m_type.c_str());
     }
+    time_t now;
+    time(&now);
+    time_t time1 = now;
+    time_t time2 = 0;
+    if (!m_stamp1.empty())
+        time1 = fromDelay(m_stamp1.c_str());
+    if (!m_stamp2.empty()){
+        time2 = fromDelay(m_stamp2.c_str());
+        if (time2 > time1){
+            time_t t = time1;
+            time1 = time2;
+            time2 = t;
+        }
+    }
+
     if (status != STATUS_UNKNOWN){
         Contact *contact;
         string resource;
@@ -878,10 +878,8 @@ JabberClient::PresenceRequest::~PresenceRequest()
                     break;
             }
             bool bChanged = false;
-            time_t now;
-            time(&now);
             if (status == STATUS_OFFLINE){
-                if (i < data->nResources.value){
+                if (i <= data->nResources.value){
                     bChanged = true;
                     vector<string> resources;
                     vector<string> resourceReply;
@@ -911,17 +909,19 @@ JabberClient::PresenceRequest::~PresenceRequest()
                     }
                     data->nResources.value = resources.size();
                 }
+                if (data->nResources.value == 0)
+                    set_str(&data->AutoReply.ptr, m_status.c_str());
             }else{
                 if (i > data->nResources.value){
                     bChanged = true;
                     data->nResources.value = i;
                     set_str(&data->Resources, i, resource.c_str());
-                    set_str(&data->ResourceOnlineTime, i, number(now).c_str());
+                    set_str(&data->ResourceOnlineTime, i, number(time2 ? time2 : time1).c_str());
                 }
                 if (number(status) != get_str(data->ResourceStatus, i)){
                     bChanged = true;
                     set_str(&data->ResourceStatus, i, number(status).c_str());
-                    set_str(&data->ResourceStatusTime, i, number(now).c_str());
+                    set_str(&data->ResourceStatusTime, i, number(time1).c_str());
                 }
                 if (m_status != get_str(data->ResourceReply, i)){
                     bChanged = true;
@@ -938,15 +938,15 @@ JabberClient::PresenceRequest::~PresenceRequest()
             if (data->Status.value != status){
                 bChanged = true;
                 if ((status == STATUS_ONLINE) &&
-                        ((now - m_client->data.owner.OnlineTime.value > 60) ||
+                        (((int)(time1 - m_client->data.owner.OnlineTime.value) > 60) ||
                          (data->Status.value != STATUS_OFFLINE)))
                     bOnLine = true;
                 if (data->Status.value == STATUS_OFFLINE){
-                    data->OnlineTime.value = now;
+                    data->OnlineTime.value = time1;
                     data->richText.bValue = true;
                 }
                 data->Status.value = status;
-                data->StatusTime.value = now;
+                data->StatusTime.value = time1;
             }
             if (data->invisible.bValue != bInvisible){
                 data->invisible.bValue = bInvisible;
@@ -975,6 +975,18 @@ void JabberClient::PresenceRequest::element_start(const char *el, const char **a
         m_from = JabberClient::get_attr("from", attr);
         m_type = JabberClient::get_attr("type", attr);
     }
+    if (!strcmp(el, "x")){
+        if (JabberClient::get_attr("xmlns", attr) == "jabber:x:delay"){
+            string stamp = JabberClient::get_attr("stamp", attr);
+            if (!stamp.empty()){
+                if (m_stamp1.empty()){
+                    m_stamp1 = stamp;
+                }else if (m_stamp2.empty()){
+                    m_stamp2 = stamp;
+                }
+            }
+        }
+    }
     m_data = "";
 }
 
@@ -996,10 +1008,12 @@ JabberClient::IqRequest::IqRequest(JabberClient *client)
         : ServerRequest(client, NULL, NULL, NULL)
 {
     m_data = NULL;
+    m_file_size = 0;
 }
 
 JabberClient::IqRequest::~IqRequest()
 {
+    JabberFileMessage *msg = NULL;
     if (m_query == "jabber:iq:oob"){
         string proto = m_url.substr(0, 7);
         if (proto != "http://"){
@@ -1020,6 +1034,17 @@ JabberClient::IqRequest::~IqRequest()
             return;
         }
         string file = m_url.substr(n + 1);
+        msg = new JabberFileMessage;
+        msg->setDescription(QString::fromUtf8(file.c_str()));
+        msg->setText(QString::fromUtf8(m_descr.c_str()));
+        msg->setHost(host.c_str());
+        msg->setPort(port);
+    }else if (!m_file_name.empty()){
+        msg = new JabberFileMessage;
+        msg->setDescription(QString::fromUtf8(m_file_name.c_str()));
+        msg->setSize(m_file_size);
+    }
+    if (msg){
         Contact *contact;
         string resource;
         JabberUserData *data = m_client->findContact(m_from.c_str(), NULL, false, contact, resource);
@@ -1028,13 +1053,8 @@ JabberClient::IqRequest::~IqRequest()
             data = m_client->findContact(m_from.c_str(), NULL, true, contact, resource);
             if (data == NULL)
                 return;
-            contact->setTemporary(CONTACT_TEMP);
+            contact->setFlags(CONTACT_TEMP);
         }
-        JabberFileMessage *msg = new JabberFileMessage;
-        msg->setDescription(QString::fromUtf8(file.c_str()));
-        msg->setText(QString::fromUtf8(m_descr.c_str()));
-        msg->setHost(host.c_str());
-        msg->setPort(port);
         msg->setFrom(m_from.c_str());
         msg->setID(m_id.c_str());
         msg->setFlags(MESSAGE_RECEIVED | MESSAGE_TEMP);
@@ -1107,6 +1127,10 @@ void JabberClient::IqRequest::element_start(const char *el, const char **attr)
         m_data = &m_url;
     if (!strcmp(el, "desc"))
         m_data = &m_descr;
+    if (!strcmp(el, "file")){
+        m_file_name = JabberClient::get_attr("name", attr);
+        m_file_size = atol(JabberClient::get_attr("size", attr).c_str());
+    }
 }
 
 void JabberClient::IqRequest::element_end(const char*)
@@ -1228,7 +1252,7 @@ JabberClient::MessageRequest::~MessageRequest()
         data = m_client->findContact(m_from.c_str(), NULL, true, contact, resource);
         if (data == NULL)
             return;
-        contact->setTemporary(CONTACT_TEMP);
+        contact->setFlags(CONTACT_TEMP);
     }
     Message *msg = NULL;
     if (!m_id.empty()){
@@ -1449,11 +1473,12 @@ void JabberClient::MessageRequest::char_data(const char *str, int len)
 class AgentRequest : public JabberClient::ServerRequest
 {
 public:
-    AgentRequest(JabberClient *client);
+    AgentRequest(JabberClient *client, const char *jid);
     ~AgentRequest();
 protected:
     JabberAgentsInfo	data;
     string m_data;
+    string m_jid;
     bool   m_bError;
     virtual void element_start(const char *el, const char **attr);
     virtual void element_end(const char *el);
@@ -1528,7 +1553,7 @@ AgentDiscoRequest::~AgentDiscoRequest()
     if (data.Name.ptr){
         set_str(&data.VHost.ptr, m_client->VHost().c_str());
         data.Client = m_client;
-        Event e(static_cast<JabberPlugin*>(m_client->protocol()->plugin())->EventAgentFound, &data);
+        Event e(EventAgentFound, &data);
         e.process();
     }
     free_data(jabberAgentsInfo, &data);
@@ -1572,11 +1597,12 @@ void AgentsDiscoRequest::element_start(const char *el, const char **attr)
     }
 }
 
-AgentRequest::AgentRequest(JabberClient *client)
-        : ServerRequest(client, _GET, NULL, client->VHost().c_str())
+AgentRequest::AgentRequest(JabberClient *client, const char *jid)
+        : ServerRequest(client, _GET, NULL, jid)
 {
     load_data(jabberAgentsInfo, &data, NULL);
     m_bError = false;
+    m_jid    = jid;
 }
 
 AgentRequest::~AgentRequest()
@@ -1614,7 +1640,7 @@ void AgentRequest::element_end(const char *el)
         if (data.ID.ptr && *data.ID.ptr){
             set_str(&data.VHost.ptr, m_client->VHost().c_str());
             data.Client = m_client;
-            Event e(static_cast<JabberPlugin*>(m_client->protocol()->plugin())->EventAgentFound, &data);
+            Event e(EventAgentFound, &data);
             e.process();
         }
     }else if (!strcmp(el, "name")){
@@ -1627,14 +1653,15 @@ void AgentRequest::char_data(const char *el, int len)
     m_data.append(el, len);
 }
 
-void JabberClient::get_agents()
+string JabberClient::get_agents(const char *jid)
 {
-    AgentRequest *req = new AgentRequest(this);
+    AgentRequest *req = new AgentRequest(this, jid);
     req->start_element("query");
     req->add_attribute("xmlns", "jabber:iq:agents");
     addLang(req);
     req->send();
     m_requests.push_back(req);
+    return req->m_id;
 }
 
 class AgentInfoRequest : public JabberClient::ServerRequest
@@ -1655,7 +1682,6 @@ protected:
     virtual void char_data(const char *el, int len);
 };
 
-
 /*
 
 typedef struct JabberAgentInfo
@@ -1682,6 +1708,7 @@ static DataDef jabberAgentInfo[] =
         { "", DATA_STRING, 1, 0 },
         { "", DATA_STRING, 1, 0 },
         { "", DATA_STRING, 1, 0 },
+        { "", DATA_STRING, 1, 0 },
         { "", DATA_STRLIST, 1, 0 },
         { "", DATA_STRLIST, 1, 0 },
         { "", DATA_ULONG, 1, 0 },
@@ -1708,7 +1735,7 @@ AgentInfoRequest::~AgentInfoRequest()
     set_str(&data.ReqID.ptr, m_id.c_str());
     data.nOptions.value = m_error_code;
     set_str(&data.Label.ptr, m_error.c_str());
-    Event e(static_cast<JabberPlugin*>(m_client->protocol()->plugin())->EventAgentInfo, &data);
+    Event e(EventAgentInfo, &data);
     e.process();
     free_data(jabberAgentInfo, &data);
 }
@@ -1742,7 +1769,7 @@ void AgentInfoRequest::element_start(const char *el, const char **attr)
         set_str(&data.Type.ptr, "x");
         set_str(&data.ReqID.ptr, m_id.c_str());
         set_str(&data.ID.ptr, m_jid.c_str());
-        Event e(static_cast<JabberPlugin*>(m_client->protocol()->plugin())->EventAgentInfo, &data);
+        Event e(EventAgentInfo, &data);
         e.process();
         free_data(jabberAgentInfo, &data);
         load_data(jabberAgentInfo, &data, NULL);
@@ -1760,12 +1787,16 @@ void AgentInfoRequest::element_end(const char *el)
     }
     if (m_bError)
         return;
+    if (!strcmp(el, "desc")){
+        set_str(&data.Desc.ptr, m_data.c_str());
+        return;
+    }
     if (!strcmp(el, "field")){
         if (data.Field.ptr && *data.Field.ptr){
             set_str(&data.VHost.ptr, m_client->VHost().c_str());
             set_str(&data.ReqID.ptr, m_id.c_str());
             set_str(&data.ID.ptr, m_jid.c_str());
-            Event e(static_cast<JabberPlugin*>(m_client->protocol()->plugin())->EventAgentInfo, &data);
+            Event e(EventAgentInfo, &data);
             e.process();
             free_data(jabberAgentInfo, &data);
             load_data(jabberAgentInfo, &data, NULL);
@@ -1788,7 +1819,7 @@ void AgentInfoRequest::element_end(const char *el)
         set_str(&data.ID.ptr, m_jid.c_str());
         set_str(&data.ReqID.ptr, m_id.c_str());
         set_str(&data.Type.ptr, el);
-        Event e(static_cast<JabberPlugin*>(m_client->protocol()->plugin())->EventAgentInfo, &data);
+        Event e(EventAgentInfo, &data);
         e.process();
         free_data(jabberAgentInfo, &data);
         load_data(jabberAgentInfo, &data, NULL);
@@ -1797,7 +1828,7 @@ void AgentInfoRequest::element_end(const char *el)
         set_str(&data.ID.ptr, m_jid.c_str());
         set_str(&data.ReqID.ptr, m_id.c_str());
         set_str(&data.Type.ptr, el);
-        Event e(static_cast<JabberPlugin*>(m_client->protocol()->plugin())->EventAgentInfo, &data);
+        Event e(EventAgentInfo, &data);
         e.process();
         free_data(jabberAgentInfo, &data);
         load_data(jabberAgentInfo, &data, NULL);
@@ -1865,6 +1896,7 @@ static DataDef jabberSearchData[] =
         { "", DATA_STRING, 1, 0 },
         { "", DATA_STRING, 1, 0 },
         { "", DATA_STRING, 1, 0 },
+        { "", DATA_STRING, 1, 0 },
         { "", DATA_STRLIST, 1, 0 },
         { "", DATA_ULONG, 1, 0 },
         { NULL, 0, 0, 0 }
@@ -1880,7 +1912,7 @@ SearchRequest::SearchRequest(JabberClient *client, const char *jid)
 
 SearchRequest::~SearchRequest()
 {
-    Event e(static_cast<JabberPlugin*>(m_client->protocol()->plugin())->EventSearchDone, (void*)m_id.c_str());
+    Event e(EventSearchDone, (void*)m_id.c_str());
     e.process();
     free_data(jabberSearchData, &data);
 }
@@ -1902,11 +1934,7 @@ void SearchRequest::element_start(const char *el, const char **attr)
                 if (label.empty())
                     label = var;
                 m_values.insert(VALUE_MAP::value_type(var.c_str(), label));
-                if (var == "nickname"){
-                    m_fields.push_front(var);
-                }else{
-                    m_fields.push_back(var);
-                }
+                m_fields.push_back(var);
             }
         }else{
             m_attr = var;
@@ -1926,23 +1954,28 @@ void SearchRequest::element_end(const char *el)
             VALUE_MAP::iterator itv = m_values.find((*it).c_str());
             if (itv != m_values.end())
                 value = (*itv).second;
-            set_str(&data.Fields, data.nFields.value++, value.c_str());
+            set_str(&data.Fields, data.nFields.value * 2, value.c_str());
+            set_str(&data.Fields, data.nFields.value * 2 + 1, value.c_str());
+            data.nFields.value++;
         }
         set_str(&data.ID.ptr, m_id.c_str());
-        Event e(static_cast<JabberPlugin*>(m_client->protocol()->plugin())->EventSearch, &data);
+        Event e(EventSearch, &data);
         e.process();
-        m_fields.clear();
+        m_values.clear();
     }else if (!strcmp(el, "item")){
         if (data.JID.ptr && *data.JID.ptr){
             for (list<string>::iterator it = m_fields.begin(); it != m_fields.end(); ++it){
                 VALUE_MAP::iterator itv = m_values.find((*it).c_str());
-                if (itv != m_values.end())
-                    set_str(&data.Fields, data.nFields.value, (*itv).second.c_str());
+                if (itv != m_values.end()){
+                    string val = (*itv).second.c_str();
+                    set_str(&data.Fields, data.nFields.value, val.c_str());
+                }
                 data.nFields.value++;
             }
             set_str(&data.ID.ptr, m_id.c_str());
-            Event e(static_cast<JabberPlugin*>(m_client->protocol()->plugin())->EventSearch, &data);
+            Event e(EventSearch, &data);
             e.process();
+            m_values.clear();
         }
     }else if (!strcmp(el, "value") || !strcmp(el, "field")){
         if (!m_attr.empty() && !m_data.empty()){
@@ -1961,6 +1994,8 @@ void SearchRequest::element_end(const char *el)
         set_str(&data.Nick.ptr, m_data.c_str());
     }else if (!strcmp(el, "email")){
         set_str(&data.EMail.ptr, m_data.c_str());
+    }else if (!strcmp(el, "status")){
+        set_str(&data.Status.ptr, m_data.c_str());
     }
 }
 
@@ -2014,7 +2049,7 @@ RegisterRequest::~RegisterRequest()
     ai.id = m_id.c_str();
     ai.err_code = m_error_code;
     ai.error = m_error.c_str();
-    Event e(static_cast<JabberPlugin*>(m_client->protocol()->plugin())->EventAgentRegister, &ai);
+    Event e(EventAgentRegister, &ai);
     e.process();
 }
 
@@ -2087,6 +2122,8 @@ void JabberClient::processList()
     m_listRequests.clear();
 }
 
+#if 0
+
 class SendFileRequest : public JabberClient::ServerRequest
 {
 public:
@@ -2144,14 +2181,78 @@ void SendFileRequest::char_data(const char *str, int len)
         m_data->append(str, len);
 }
 
-void JabberClient::sendFileRequest(FileMessage *msg, unsigned short port, JabberUserData *data, const char *fname)
+#endif
+
+class SendFileRequest : public JabberClient::ServerRequest
+{
+public:
+    SendFileRequest(JabberClient *client, const char *jid, FileMessage *msg);
+    ~SendFileRequest();
+protected:
+    virtual void	element_start(const char *el, const char **attr);
+    virtual void	element_end(const char *el);
+    virtual	void	char_data(const char *str, int len);
+    FileMessage *m_msg;
+};
+
+SendFileRequest::SendFileRequest(JabberClient *client, const char *jid, FileMessage *msg)
+        : JabberClient::ServerRequest(client, _SET, NULL, jid)
+{
+    m_msg = msg;
+}
+
+SendFileRequest::~SendFileRequest()
+{
+}
+
+void SendFileRequest::element_start(const char*, const char**)
+{
+}
+
+void SendFileRequest::element_end(const char*)
+{
+}
+
+void SendFileRequest::char_data(const char*, int)
+{
+}
+
+void JabberClient::sendFileRequest(FileMessage *msg, unsigned short, JabberUserData *data, const char *fname, unsigned size)
 {
     string jid = data->ID.ptr;
-    if (data->Resource.ptr){
+    string resource;
+    if (msg->getResource().isEmpty()){
+        if (data->Resource.ptr){
+            jid += "/";
+            jid += data->Resource.ptr;
+        }
+    }else{
         jid += "/";
-        jid += data->Resource.ptr;
+        jid += msg->getResource().utf8();
     }
     SendFileRequest *req = new SendFileRequest(this, jid.c_str(), msg);
+    req->start_element("si");
+    req->add_attribute("xmlns", "http://jabber.org/protocol/si");
+    req->add_attribute("profile", "http://jabber.org/protocol/si/profile/file-transfer");
+    req->add_attribute("id", get_unique_id().c_str());
+    req->start_element("file");
+    req->add_attribute("xmns", "http://jabber.org/protocol/si/profile/file-transfer");
+    req->add_attribute("size", number(size).c_str());
+    req->add_attribute("name", fname);
+    req->start_element("range");
+    req->end_element();
+    req->end_element();
+    req->start_element("feature");
+    req->add_attribute("xmlns", "http://jabber.org/protocol/feature-neg");
+    req->start_element("x");
+    req->add_attribute("xmlns", "jabber:x:data");
+    req->add_attribute("type", "form");
+    req->start_element("field");
+    req->add_attribute("type", "list-single");
+    req->add_attribute("var", "stream-method");
+    req->start_element("option");
+    req->text_tag("value", "http://jabber.org/protocol/bytestreams");
+#if 0
     req->start_element("query");
     req->add_attribute("xmlns", "jabber:iq:oob");
     string url  = "http://";
@@ -2166,10 +2267,39 @@ void JabberClient::sendFileRequest(FileMessage *msg, unsigned short port, Jabber
     desc = msg->getText().utf8();
     req->text_tag("url", url.c_str());
     req->text_tag("desc", desc.c_str());
+#endif
     req->send();
     m_requests.push_back(req);
 }
 
+void JabberClient::sendFileAccept(FileMessage *msg, JabberUserData *data)
+{
+    JabberFileMessage *m = static_cast<JabberFileMessage*>(msg);
+    string jid = data->ID.ptr;
+    string resource;
+    if (msg->getResource().isEmpty()){
+        if (data->Resource.ptr){
+            jid += "/";
+            jid += data->Resource.ptr;
+        }
+    }else{
+        jid += "/";
+        jid += msg->getResource().utf8();
+    }
+    ServerRequest req(this, ServerRequest::_RESULT, NULL, jid.c_str(), m->getID());
+    req.start_element("si");
+    req.add_attribute("xmlns", "http://jabber.org/protocol/si");
+    req.start_element("feature");
+    req.add_attribute("xmlns", "http://jabber.org/protocol/feature-neg");
+    req.start_element("x");
+    req.add_attribute("xmlns", "jabber:x:data");
+    req.add_attribute("type", "submit");
+    req.start_element("field");
+    req.add_attribute("var", "stream-method");
+    req.text_tag("value", "http://jabber.org/protocol/bytestreams");
+    req.send();
+}
+
 class DiscoItemsRequest : public JabberClient::ServerRequest
 {
 public:
@@ -2193,26 +2323,26 @@ DiscoItemsRequest::DiscoItemsRequest(JabberClient *client, const char *jid)
 
 DiscoItemsRequest::~DiscoItemsRequest()
 {
-    JabberDiscoItem item;
+    DiscoItem item;
     item.id		= m_id;
     if (m_code){
         item.name	= m_error;
         item.node	= number(m_code);
     }
-    Event e(static_cast<JabberPlugin*>(m_client->protocol()->plugin())->EventDiscoItem, &item);
+    Event e(EventDiscoItem, &item);
     e.process();
 }
 
 void DiscoItemsRequest::element_start(const char *el, const char **attr)
 {
     if (!strcmp(el, "item")){
-        JabberDiscoItem item;
+        DiscoItem item;
         item.id		= m_id;
         item.jid	= JabberClient::get_attr("jid", attr);
         item.name	= JabberClient::get_attr("name", attr);
         item.node	= JabberClient::get_attr("node", attr);
         if (!item.jid.empty()){
-            Event e(static_cast<JabberPlugin*>(m_client->protocol()->plugin())->EventDiscoItem, &item);
+            Event e(EventDiscoItem, &item);
             e.process();
         }
     }
@@ -2260,6 +2390,10 @@ protected:
     virtual	void	char_data(const char *str, int len);
     string			*m_data;
     string			m_error;
+    string			m_features;
+    string			m_name;
+    string			m_type;
+    string			m_category;
     unsigned		m_code;
 };
 
@@ -2272,37 +2406,40 @@ DiscoInfoRequest::DiscoInfoRequest(JabberClient *client, const char *jid)
 
 DiscoInfoRequest::~DiscoInfoRequest()
 {
-    JabberDiscoItem item;
+    if (m_code == 0){
+        DiscoItem item;
+        item.id			= m_id;
+        item.jid		= "info";
+        item.name		= m_name;
+        item.category	= m_category;
+        item.type		= m_type;
+        item.features	= m_features;
+        Event e(EventDiscoItem, &item);
+        e.process();
+    }
+    DiscoItem item;
     item.id		= m_id;
     if (m_code){
         item.name	= m_error;
         item.node	= number(m_code);
     }
-    Event e(static_cast<JabberPlugin*>(m_client->protocol()->plugin())->EventDiscoItem, &item);
+    Event e(EventDiscoItem, &item);
     e.process();
 }
 
 void DiscoInfoRequest::element_start(const char *el, const char **attr)
 {
     if (!strcmp(el, "identity")){
-        JabberDiscoItem item;
-        item.id		= m_id;
-        item.jid	= JabberClient::get_attr("category", attr);
-        item.name	= JabberClient::get_attr("name", attr);
-        item.node	= JabberClient::get_attr("type", attr);
-        if (!item.jid.empty()){
-            Event e(static_cast<JabberPlugin*>(m_client->protocol()->plugin())->EventDiscoItem, &item);
-            e.process();
-        }
+        m_category	= JabberClient::get_attr("category", attr);
+        m_name		= JabberClient::get_attr("name", attr);
+        m_type		= JabberClient::get_attr("type", attr);
     }
     if (!strcmp(el, "feature")){
-        JabberDiscoItem item;
-        item.id		= m_id;
-        item.jid	= "feature";
-        item.name	= JabberClient::get_attr("var", attr);
-        if (!item.jid.empty()){
-            Event e(static_cast<JabberPlugin*>(m_client->protocol()->plugin())->EventDiscoItem, &item);
-            e.process();
+        string feature = JabberClient::get_attr("var", attr);
+        if (!feature.empty()){
+            if (!m_features.empty())
+                m_features += "\n";
+            m_features += feature;
         }
     }
     if (!strcmp(el, "error")){
@@ -2337,6 +2474,137 @@ string JabberClient::discoInfo(const char *jid, const char *node)
     return req->m_id;
 }
 
+class BrowseRequest : public JabberClient::ServerRequest
+{
+public:
+    BrowseRequest(JabberClient *client, const char *jid);
+    ~BrowseRequest();
+protected:
+    virtual void	element_start(const char *el, const char **attr);
+    virtual void	element_end(const char *el);
+    virtual	void	char_data(const char *str, int len);
+    string			*m_data;
+    string			m_jid;
+    string			m_error;
+    string			m_name;
+    string			m_type;
+    string			m_category;
+    string			m_features;
+    string			m_ns;
+    unsigned		m_code;
+};
+
+BrowseRequest::BrowseRequest(JabberClient *client, const char *jid)
+        : JabberClient::ServerRequest(client, _GET, NULL, jid)
+{
+    m_data	= NULL;
+    m_code	= 0;
+    m_jid	= jid;
+}
+
+BrowseRequest::~BrowseRequest()
+{
+    if (!m_jid.empty() && !m_name.empty() && (m_code == 0)){
+        DiscoItem item;
+        item.id			= m_id;
+        item.jid		= m_jid;
+        item.name		= m_name;
+        item.type		= m_type;
+        item.category	= m_category;
+        item.features	= m_features;
+        Event e(EventDiscoItem, &item);
+        e.process();
+    }
+    DiscoItem item;
+    item.id		= m_id;
+    if (m_code){
+        item.name	= m_error;
+        item.node	= number(m_code);
+    }
+    Event e(EventDiscoItem, &item);
+    e.process();
+}
+
+void BrowseRequest::element_start(const char *el, const char **attr)
+{
+    if (!strcmp(el, "error")){
+        m_code = atol(JabberClient::get_attr("code", attr).c_str());
+        m_data = &m_error;
+    }
+    if (!strcmp(el, "item") || !strcmp(el, "service") || !strcmp(el, "agent") || !strcmp(el, "headline")){
+        if (!m_jid.empty() && !m_name.empty()){
+            DiscoItem item;
+            item.id			= m_id;
+            item.jid		= m_jid;
+            item.name		= m_name;
+            item.type		= m_type;
+            item.category	= m_category;
+            item.features	= m_features;
+            Event e(EventDiscoItem, &item);
+            e.process();
+        }
+        m_jid		= JabberClient::get_attr("jid", attr).c_str();
+        m_name		= JabberClient::get_attr("name", attr).c_str();
+        m_type		= JabberClient::get_attr("type", attr).c_str();
+        m_category	= JabberClient::get_attr("category", attr).c_str();
+        if (!strcmp(el, "headline"))
+            m_category = "headline";
+        m_features	= "";
+    }
+    if (!strcmp(el, "query")){
+        m_name		= JabberClient::get_attr("name", attr).c_str();
+        m_type		= JabberClient::get_attr("type", attr).c_str();
+        m_category	= JabberClient::get_attr("category", attr).c_str();
+    }
+    if (!strcmp(el, "ns"))
+        m_data  = &m_ns;
+}
+
+void BrowseRequest::element_end(const char *el)
+{
+    if (!strcmp(el, "error"))
+        m_data = NULL;
+    if (!strcmp(el, "ns") && !m_ns.empty()){
+        if (!m_features.empty())
+            m_features += "\n";
+        m_features += m_ns;
+        m_ns = "";
+        m_data = NULL;
+    }
+    if ((!strcmp(el, "item") || !strcmp(el, "service") ||
+            !strcmp(el, "agent") || !strcmp(el, "headline"))
+            && !m_jid.empty()){
+        DiscoItem item;
+        item.id			= m_id;
+        item.jid		= m_jid;
+        item.name		= m_name;
+        item.type		= m_type;
+        item.category	= m_category;
+        item.features	= m_features;
+        Event e(EventDiscoItem, &item);
+        e.process();
+        m_jid = "";
+    }
+}
+
+void BrowseRequest::char_data(const char *buf, int len)
+{
+    if (m_data)
+        m_data->append(buf, len);
+}
+
+string JabberClient::browse(const char *jid)
+{
+    if (getState() != Connected)
+        return "";
+    BrowseRequest *req = new BrowseRequest(this, jid);
+    req->start_element("query");
+    req->add_attribute("xmlns", "jabber:iq:browse");
+    req->send();
+    m_requests.push_back(req);
+    return req->m_id;
+}
+
 class VersionInfoRequest : public JabberClient::ServerRequest
 {
 public:
@@ -2360,12 +2628,12 @@ VersionInfoRequest::VersionInfoRequest(JabberClient *client, const char *jid)
 
 VersionInfoRequest::~VersionInfoRequest()
 {
-    JabberDiscoItem item;
+    DiscoItem item;
     item.id		= m_id;
     item.jid	= m_version;
     item.name	= m_name;
     item.node	= m_os;
-    Event e(static_cast<JabberPlugin*>(m_client->protocol()->plugin())->EventDiscoItem, &item);
+    Event e(EventDiscoItem, &item);
     e.process();
 }
 
@@ -2425,10 +2693,10 @@ TimeInfoRequest::TimeInfoRequest(JabberClient *client, const char *jid)
 
 TimeInfoRequest::~TimeInfoRequest()
 {
-    JabberDiscoItem item;
+    DiscoItem item;
     item.id		= m_id;
     item.jid	= m_time;
-    Event e(static_cast<JabberPlugin*>(m_client->protocol()->plugin())->EventDiscoItem, &item);
+    Event e(EventDiscoItem, &item);
     e.process();
 }
 
@@ -2479,10 +2747,10 @@ LastInfoRequest::LastInfoRequest(JabberClient *client, const char *jid)
 void LastInfoRequest::element_start(const char *el, const char **attr)
 {
     if (!strcmp(el, "query")){
-        JabberDiscoItem item;
+        DiscoItem item;
         item.id		= m_id;
         item.jid	= JabberClient::get_attr("seconds", attr);
-        Event e(static_cast<JabberPlugin*>(m_client->protocol()->plugin())->EventDiscoItem, &item);
+        Event e(EventDiscoItem, &item);
         e.process();
     }
 }
@@ -2519,22 +2787,22 @@ StatRequest::StatRequest(JabberClient *client, const char *jid, const char *id)
 
 StatRequest::~StatRequest()
 {
-    JabberDiscoItem item;
+    DiscoItem item;
     item.id		= m_id;
     item.jid	= "";
-    Event e(static_cast<JabberPlugin*>(m_client->protocol()->plugin())->EventDiscoItem, &item);
+    Event e(EventDiscoItem, &item);
     e.process();
 }
 
 void StatRequest::element_start(const char *el, const char **attr)
 {
     if (!strcmp(el, "stat")){
-        JabberDiscoItem item;
+        DiscoItem item;
         item.id		= m_id;
         item.jid	= JabberClient::get_attr("name", attr);;
         item.name	= JabberClient::get_attr("units", attr);;
         item.node	= JabberClient::get_attr("value", attr);;
-        Event e(static_cast<JabberPlugin*>(m_client->protocol()->plugin())->EventDiscoItem, &item);
+        Event e(EventDiscoItem, &item);
         e.process();
     }
 }
@@ -2562,10 +2830,10 @@ StatItemsRequest::StatItemsRequest(JabberClient *client, const char *jid, const
 StatItemsRequest::~StatItemsRequest()
 {
     if (m_stats.empty()){
-        JabberDiscoItem item;
+        DiscoItem item;
         item.id		= m_id;
         item.jid	= "";
-        Event e(static_cast<JabberPlugin*>(m_client->protocol()->plugin())->EventDiscoItem, &item);
+        Event e(EventDiscoItem, &item);
         e.process();
         return;
     }
@@ -2618,4 +2886,37 @@ void JabberClient::addLang(ServerRequest *req)
     req->add_attribute("xml:lang", s.utf8());
 }
 
+class ChangePasswordRequest : public JabberClient::ServerRequest
+{
+public:
+    ChangePasswordRequest(JabberClient *client, const char *password);
+    ~ChangePasswordRequest();
+protected:
+    string	m_password;
+};
+
+ChangePasswordRequest::ChangePasswordRequest(JabberClient *client, const char *password)
+        : JabberClient::ServerRequest(client, _SET, NULL, NULL)
+{
+    m_password = password;
+}
+
+ChangePasswordRequest::~ChangePasswordRequest()
+{
+    m_client->setPassword(m_password.c_str());
+}
+
+void JabberClient::changePassword(const char *password)
+{
+    if (getState() != Connected)
+        return;
+    ChangePasswordRequest *req = new ChangePasswordRequest(this, password);
+    req->start_element("query");
+    req->add_attribute("xmlns", "jabber:iq:register");
+    req->text_tag("username", data.owner.ID.ptr);
+    req->text_tag("password", password);
+    m_requests.push_back(req);
+    req->send();
+}
+
 
