@@ -15,9 +15,9 @@
  *                                                                         *
  ***************************************************************************/
 
+#include "jabberclient.h"
 #include "simapi.h"
 #include "jabberaboutinfo.h"
-#include "jabberclient.h"
 #include "jabber.h"
 
 #include <qmultilineedit.h>
@@ -50,7 +50,7 @@ void *JabberAboutInfo::processEvent(Event *e)
         if (client == m_client)
             fill(m_data);
     }
-    if (m_data && (e->type() == static_cast<JabberPlugin*>(m_client->protocol()->plugin())->EventVCard)){
+    if (m_data && (e->type() == EventVCard)){
         JabberUserData *data = (JabberUserData*)(e->param());
         if (!str_cmp(m_data->ID.ptr, data->ID.ptr) && !str_cmp(m_data->Node.ptr, data->Node.ptr))
             fill(data);
@@ -72,7 +72,7 @@ void JabberAboutInfo::apply(Client *client, void *_data)
     set_str(&data->Desc.ptr, edtAbout->text().utf8());
 }
 
-#ifndef WIN32
+#ifndef _MSC_VER
 #include "jabberaboutinfo.moc"
 #endif
 
