@@ -15,8 +15,8 @@
  *                                                                         *
  ***************************************************************************/
 
-#include "jabber.h"
 #include "jabberclient.h"
+#include "jabber.h"
 #include "jabberconfig.h"
 #include "jabber_ssl.h"
 #include "jabberadd.h"
@@ -27,9 +27,9 @@
 #include "jabberpicture.h"
 #include "jabbermessage.h"
 #include "jabberbrowser.h"
-#include "services.h"
 #include "infoproxy.h"
 #include "html.h"
+#include "icons.h"
 
 #include "core.h"
 
@@ -47,6 +47,8 @@
 #include <ctype.h>
 #endif
 
+#include <algorithm>
+
 #ifndef XML_STATUS_OK
 #define XML_STATUS_OK    1
 #define XML_STATUS_ERROR 0
@@ -122,6 +124,7 @@ DataDef jabberUserData[] =
         { "", DATA_STRLIST, 1, 0 },			// ResourceReply
         { "", DATA_STRLIST, 1, 0 },			// ResourceStatusTime
         { "", DATA_STRLIST, 1, 0 },			// ResourceOnlineTime
+        { "AutoReply", DATA_UTF, 1, 0 },
         { NULL, 0, 0, 0 }
     };
 
@@ -156,18 +159,16 @@ static DataDef jabberClientData[] =
         { "MaxPort", DATA_ULONG, 1, DATA(0xFFFF) },
         { "Photo", DATA_UTF, 1, 0 },
         { "Logo", DATA_UTF, 1, 0 },
-        { "Browser", DATA_ULONG, 5, 0 },
-        { "BrowserBar", DATA_LONG, 7, 0 },
-        { "BrowserHistory", DATA_UTF, 1, 0 },
         { "AutoSubscribe", DATA_BOOL, 1, DATA(1) },
         { "AutoAccept", DATA_BOOL, 1, DATA(1) },
         { "UseHTTP", DATA_BOOL, 1, 0 },
         { "URL", DATA_STRING, 1, 0 },
+        { "InfoUpdated", DATA_BOOL, 1, 0 },
         { "", DATA_STRUCT, sizeof(JabberUserData) / sizeof(Data), DATA(jabberUserData) },
         { NULL, 0, 0, 0 }
     };
 
-JabberClient::JabberClient(JabberProtocol *protocol, const char *cfg)
+JabberClient::JabberClient(JabberProtocol *protocol, Buffer *cfg)
         : TCPClient(protocol, cfg)
 {
     load_data(jabberClientData, &data, cfg);
@@ -200,19 +201,15 @@ JabberClient::JabberClient(JabberProtocol *protocol, const char *cfg)
     }
     setListRequest(NULL);
 
-    m_bXML		 = false;
     m_bSSL		 = false;
     m_curRequest = NULL;
     m_msg_id	 = 0;
-    m_browser	 = NULL;
     m_bJoin		 = false;
     init();
 }
 
 JabberClient::~JabberClient()
 {
-    if (m_browser)
-        delete m_browser;
     TCPClient::setStatus(STATUS_OFFLINE, false);
     free_data(jabberClientData, &data);
     freeData();
@@ -251,8 +248,6 @@ string JabberClient::getConfig()
     string res = Client::getConfig();
     if (res.length())
         res += "\n";
-    if (m_browser)
-        m_browser->save();
     return res += save_data(jabberClientData, &data);
 }
 
@@ -328,14 +323,7 @@ void JabberClient::connected()
     log(L_DEBUG, "Connect ready");
     startHandshake();
     TCPClient::connect_ready();
-    if (!m_bXML){
-        memset(&m_handler, 0, sizeof(m_handler));
-        m_handler.startElement = p_element_start;
-        m_handler.endElement   = p_element_end;
-        m_handler.characters   = p_char_data;
-        m_context = xmlCreatePushParserCtxt(&m_handler, this, "", 0, "");
-        m_bXML = true;
-    }
+    reset();
 }
 
 void JabberClient::packet_ready()
@@ -344,12 +332,7 @@ void JabberClient::packet_ready()
         return;
     JabberPlugin *plugin = static_cast<JabberPlugin*>(protocol()->plugin());
     log_packet(m_socket->readBuffer, false, plugin->JabberPacket);
-#if LIBXML_VERSION > 20604
-    // libxml2 parser sux since 2.6.5
-    if (m_socket->readBuffer.data()[m_socket->readBuffer.size() - 1] == '>')
-        m_socket->readBuffer << "<a/>";
-#endif
-    if (xmlParseChunk(m_context, m_socket->readBuffer.data(), m_socket->readBuffer.size(), 0))
+    if (!parse(m_socket->readBuffer.data(), m_socket->readBuffer.size(), true))
         m_socket->error_state("XML parse error");
     m_socket->readBuffer.init(0);
     m_socket->readBuffer.packetStart();
@@ -399,138 +382,6 @@ void *JabberClient::processEvent(Event *e)
         }
         return NULL;
     }
-    if (e->type() == EventCheckState){
-        CommandDef *cmd = (CommandDef*)(e->param());
-        if (cmd->id == static_cast<JabberPlugin*>(protocol()->plugin())->CmdBrowser){
-            cmd->flags &= ~COMMAND_CHECKED;
-            if (getState() != Connected)
-                return NULL;
-            if (cmd->menu_id == MenuMain){
-                unsigned n = 0;
-                for (unsigned i = 0; i < getContacts()->nClients(); i++){
-                    Client *client = getContacts()->getClient(i);
-                    if (client->protocol() != protocol())
-                        continue;
-                    if (client->getState() != Connected)
-                        continue;
-                    n++;
-                }
-                if (n > 1){
-                    cmd->popup_id = static_cast<JabberPlugin*>(protocol()->plugin())->MenuClients;
-                }else{
-                    cmd->popup_id = 0;
-                }
-                return e->param();
-            }
-            if (cmd->menu_id == static_cast<JabberPlugin*>(protocol()->plugin())->MenuClients){
-                unsigned n = getContacts()->nClients() + 1;
-                CommandDef *cmds = new CommandDef[n];
-                memset(cmds, 0, sizeof(CommandDef) * n);
-                n = 0;
-                for (unsigned i = 0; i < getContacts()->nClients(); i++){
-                    Client *client = getContacts()->getClient(i);
-                    if (client->protocol() != protocol())
-                        continue;
-                    if (client->getState() != Connected)
-                        continue;
-                    JabberClient *jc = static_cast<JabberClient*>(client);
-                    QString url;
-                    if (jc->getUseVHost())
-                        url = QString::fromUtf8(jc->getVHost());
-                    if (url.isEmpty())
-                        url = QString::fromUtf8(jc->getServer());
-                    cmds[n].id       = static_cast<JabberPlugin*>(protocol()->plugin())->CmdBrowser + i;
-                    cmds[n].text     = "_";
-                    cmds[n].text_wrk = strdup(url.utf8());
-                    n++;
-                }
-                cmd->param = cmds;
-                cmd->flags |= COMMAND_RECURSIVE;
-                return e->param();
-            }
-            Contact *contact = getContacts()->contact((unsigned)(cmd->param));
-            if (contact == NULL)
-                return NULL;
-            clientData *data;
-            ClientDataIterator it(contact->clientData, this);
-            while ((data = ++it) != NULL)
-                return e->param();
-            ClientDataIterator it1(contact->clientData);
-            while ((data = ++it1) != NULL){
-                if (isMyData(data, contact))
-                    return e->param();
-            }
-            return NULL;
-        }
-    }
-    if (e->type() == EventCommandExec){
-        CommandDef *cmd = (CommandDef*)(e->param());
-        if (cmd->menu_id == static_cast<JabberPlugin*>(protocol()->plugin())->MenuClients){
-            unsigned n = cmd->id - static_cast<JabberPlugin*>(protocol()->plugin())->CmdBrowser;
-            if (n >= getContacts()->nClients())
-                return NULL;
-            Client *client = getContacts()->getClient(n);
-            if (client->protocol() != protocol())
-                return NULL;
-            if (getState() != Connected)
-                return NULL;
-            JabberClient *jc = static_cast<JabberClient*>(client);
-            if (jc->m_browser == NULL){
-                jc->m_browser = new JabberBrowser(jc);
-                bool bSize = (jc->data.browser_geo[WIDTH].value && jc->data.browser_geo[HEIGHT].value);
-                restoreGeometry(m_browser, jc->data.browser_geo, bSize, bSize);
-            }
-            QString url;
-            if (jc->getUseVHost())
-                url = QString::fromUtf8(jc->getVHost());
-            if (url.isEmpty())
-                url = QString::fromUtf8(jc->getServer());
-            jc->m_browser->goUrl(url, "");
-            raiseWindow(jc->m_browser);
-            return e->param();
-        }
-        if (cmd->id == static_cast<JabberPlugin*>(protocol()->plugin())->CmdBrowser){
-            if (getState() != Connected)
-                return NULL;
-            if (m_browser == NULL){
-                m_browser = new JabberBrowser(this);
-                bool bSize = (data.browser_geo[WIDTH].value && data.browser_geo[HEIGHT].value);
-                restoreGeometry(m_browser, data.browser_geo, bSize, bSize);
-            }
-            QString url;
-            if (cmd->menu_id == MenuMain){
-                if (getUseVHost())
-                    url = QString::fromUtf8(getVHost());
-                if (url.isEmpty())
-                    url = QString::fromUtf8(getServer());
-            }else{
-                Contact *contact = getContacts()->contact((unsigned)(cmd->param));
-                if (contact){
-                    clientData *data;
-                    ClientDataIterator it(contact->clientData, this);
-                    while ((data = ++it) != NULL){
-                        JabberUserData *d = (JabberUserData*)data;
-                        url = QString::fromUtf8(d->ID.ptr);
-                    }
-                    if (url.isEmpty()){
-                        ClientDataIterator it(contact->clientData);
-                        while ((data = ++it) != NULL){
-                            if (!isMyData(data, contact))
-                                continue;
-                            JabberUserData *d = (JabberUserData*)data;
-                            url = QString::fromUtf8(d->ID.ptr);
-                        }
-                    }
-                }
-            }
-            int n = url.find("/");
-            if (n > 0)
-                url = url.left(n);
-            m_browser->goUrl(url, "");
-            raiseWindow(m_browser);
-            return e->param();
-        }
-    }
     if (e->type() == EventGoURL){
         string url = (const char*)(e->param());
         string proto;
@@ -560,7 +411,7 @@ void *JabberClient::processEvent(Event *e)
     }
     if (e->type() == EventTemplateExpanded){
         TemplateExpand *t = (TemplateExpand*)(e->param());
-        setStatus((unsigned)(t->param), quoteString(t->tmpl, quoteNOBR).utf8());
+        setStatus((unsigned long)(t->param), quoteString(t->tmpl, quoteNOBR).utf8());
     }
     if (e->type() == EventContactChanged){
         Contact *contact = (Contact*)(e->param());
@@ -663,7 +514,7 @@ void *JabberClient::processEvent(Event *e)
                     reason = md->reason;
                 ServerRequest req(this, "error", NULL, msg->getFrom(), msg->getID());
                 req.start_element("error");
-                req.add_attribute("code", "406");
+                req.add_attribute("code", "403");
                 req.add_text(reason.c_str());
                 req.send();
                 Event e(EventMessageDeleted, msg);
@@ -679,11 +530,7 @@ void *JabberClient::processEvent(Event *e)
 
 void JabberClient::setStatus(unsigned status)
 {
-    if (status == STATUS_OFFLINE){
-        setStatus(status, NULL);
-        return;
-    }
-    if (getInvisible()){
+    if (getInvisible() && (status != STATUS_OFFLINE)){
         if (m_status != status){
             m_status = status;
             Event e(EventClientChanged, static_cast<Client*>(this));
@@ -691,15 +538,11 @@ void JabberClient::setStatus(unsigned status)
         }
         return;
     }
-    if (status == STATUS_ONLINE){
-        setStatus(status, NULL);
-        return;
-    }
     ARRequest ar;
     ar.contact  = NULL;
     ar.status   = status;
     ar.receiver = this;
-    ar.param	= (void*)status;
+    ar.param	= (void*)(long)status;
     Event e(EventARRequest, &ar);
     e.process();
 }
@@ -789,14 +632,6 @@ void JabberClient::setStatus(unsigned status, const char *ar)
 
 void JabberClient::disconnected()
 {
-    if (m_browser){
-        delete m_browser;
-        m_browser = NULL;
-    }
-    if (m_bXML){
-        xmlFreeParserCtxt(m_context);
-        m_bXML = false;
-    }
     for (list<ServerRequest*>::iterator it = m_requests.begin(); it != m_requests.end(); ++it)
         delete *it;
     m_requests.clear();
@@ -837,21 +672,6 @@ void JabberClient::sendPacket()
     m_socket->write();
 }
 
-void JabberClient::p_element_start(void *data, const xmlChar *el, const xmlChar **attr)
-{
-    ((JabberClient*)data)->element_start((char*)el, (const char**)attr);
-}
-
-void JabberClient::p_element_end(void *data, const xmlChar *el)
-{
-    ((JabberClient*)data)->element_end((char*)el);
-}
-
-void JabberClient::p_char_data(void *data, const xmlChar *str, int len)
-{
-    ((JabberClient*)data)->char_data((char*)str, len);
-}
-
 string JabberClient::get_attr(const char *name, const char **attr)
 {
     if (attr == NULL)
@@ -957,6 +777,8 @@ JabberClient::ServerRequest::ServerRequest(JabberClient *client, const char *typ
     }else{
         m_id  = m_client->get_unique_id();
     }
+    if (m_client->m_socket == NULL)
+        return;
     m_client->m_socket->writeBuffer.packetStart();
     m_client->m_socket->writeBuffer
     << "<iq type=\'" << type << "\' id=\'"
@@ -1007,7 +829,9 @@ void JabberClient::ServerRequest::start_element(const char *name)
 void JabberClient::ServerRequest::add_attribute(const char *name, const char *value)
 {
     m_client->m_socket->writeBuffer
-    << " " << name << "=\'" << JabberClient::encodeXML(QString::fromUtf8(value)) << "\'";
+    << " " << name << "=\'" 
+    << JabberClient::encodeXML(QString::fromUtf8(value)).replace('\'', "&apos;")
+    << "\'";
 }
 
 void JabberClient::ServerRequest::end_element(bool bNewLevel)
@@ -1076,6 +900,7 @@ void JabberClient::ServerRequest::add_condition(const char *condition, bool bXDa
 
 const char *JabberClient::ServerRequest::_GET = "get";
 const char *JabberClient::ServerRequest::_SET = "set";
+const char *JabberClient::ServerRequest::_RESULT = "result";
 
 void JabberClient::startHandshake()
 {
@@ -1121,7 +946,11 @@ void JabberClient::auth_ok()
     setState(Connected);
     setPreviousPassword(NULL);
     rosters_request();
-    info_request(NULL, false);
+    if (getInfoUpdated()){
+        setClientInfo(&data.owner);
+    }else{
+        info_request(NULL, false);
+    }
     setStatus(m_logonStatus);
     QTimer::singleShot(PING_TIMEOUT * 1000, this, SLOT(ping()));
 }
@@ -1147,12 +976,12 @@ QCString JabberClient::encodeXML(const QString &str)
     return quoteString(str, quoteNOBR).utf8();
 }
 
-JabberUserData *JabberClient::findContact(const char *_jid, const char *name, bool bCreate, Contact *&contact, string &resource)
+JabberUserData *JabberClient::findContact(const char *_jid, const char *name, bool bCreate, Contact *&contact, string &resource, bool bJoin)
 {
     resource = "";
     string jid = _jid;
     int n = jid.find('/');
-    if ((n >= 0) && (jid.substr(n + 1) != "registered")){
+    if (n >= 0){
         resource = jid.substr(n + 1);
         jid = jid.substr(0, n);
     }
@@ -1182,19 +1011,21 @@ JabberUserData *JabberClient::findContact(const char *_jid, const char *name, bo
         if (pos > 0)
             sname = sname.left(pos);
     }
-    while ((contact = ++it) != NULL){
-        if (contact->getName().lower() == sname.lower()){
-            JabberUserData *data = (JabberUserData*)(contact->clientData.createData(this));
-            set_str(&data->ID.ptr, jid.c_str());
-            if (!resource.empty())
-                set_str(&data->Resource.ptr, resource.c_str());
-            if (name)
-                set_str(&data->Name.ptr, name);
-            info_request(data, false);
-            Event e(EventContactChanged, contact);
-            e.process();
-            m_bJoin = true;
-            return data;
+    if (bJoin){
+        while ((contact = ++it) != NULL){
+            if (contact->getName().lower() == sname.lower()){
+                JabberUserData *data = (JabberUserData*)(contact->clientData.createData(this));
+                set_str(&data->ID.ptr, jid.c_str());
+                if (!resource.empty())
+                    set_str(&data->Resource.ptr, resource.c_str());
+                if (name)
+                    set_str(&data->Name.ptr, name);
+                info_request(data, false);
+                Event e(EventContactChanged, contact);
+                e.process();
+                m_bJoin = true;
+                return data;
+            }
         }
     }
     contact = getContacts()->contact(0, true);
@@ -1366,12 +1197,24 @@ void JabberClient::contactInfo(void *_data, unsigned long &curStatus, unsigned &
 string JabberClient::buildId(JabberUserData *data)
 {
     string res = data->ID.ptr;
+    int n = res.find('@');
+    if (n < 0){
+        res += "@";
+        string server;
+        if (getUseVHost())
+            server = getVHost();
+        if (server.empty())
+            server = getServer();
+        res += server;
+    }
     return res;
 }
 
-QWidget *JabberClient::searchWindow()
+QWidget *JabberClient::searchWindow(QWidget *parent)
 {
-    return new JabberAdd(this);
+    if (getState() != Connected)
+        return NULL;
+    return new JabberAdd(this, parent);
 }
 
 void JabberClient::ping()
@@ -1427,6 +1270,13 @@ QString JabberClient::contactTip(void *_data)
             res += ": </font>";
             res += formatDateTime(data->StatusTime.value);
         }
+        const char *reply = data->AutoReply.ptr;
+        if (reply && *reply){
+            res += "<br/>";
+            QString r = QString::fromUtf8(reply);
+            r = r.replace(QRegExp("\n"), "<br/>");
+            res += r;
+        }
     }else{
         for (unsigned i = 1; i <= data->nResources.value; i++){
             unsigned status = atol(get_str(data->ResourceStatus, i));
@@ -1460,6 +1310,13 @@ QString JabberClient::contactTip(void *_data)
                 res += ": </font>";
                 res += formatDateTime(statusTime);
             }
+            const char *reply = get_str(data->ResourceReply, i);
+            if (reply && *reply){
+                res += "<br/>";
+                QString r = QString::fromUtf8(reply);
+                r = r.replace(QRegExp("\n"), "<br/>");
+                res += r;
+            }
             if (i < data->nResources.value)
                 res += "<br>_________<br>";
         }
@@ -1547,8 +1404,7 @@ const unsigned WORK_INFO  = 3;
 const unsigned ABOUT_INFO = 4;
 const unsigned PHOTO_INFO = 5;
 const unsigned LOGO_INFO  = 6;
-const unsigned SERVICES	  = 7;
-const unsigned NETWORK	  = 8;
+const unsigned NETWORK	  = 7;
 
 static CommandDef jabberWnd[] =
     {
@@ -1751,21 +1607,6 @@ static CommandDef cfgJabberWnd[] =
             NULL,
             NULL
         },
-        {
-            SERVICES,
-            I18N_NOOP("Agents"),
-            "configure",
-            NULL,
-            NULL,
-            0,
-            0,
-            0,
-            0,
-            0,
-            0,
-            NULL,
-            NULL
-        },
         {
             NETWORK,
             I18N_NOOP("Network"),
@@ -1855,8 +1696,6 @@ QWidget *JabberClient::configWindow(QWidget *parent, unsigned id)
         return new JabberPicture(parent, NULL, this, false);
     case NETWORK:
         return new JabberConfig(parent, this, true);
-    case SERVICES:
-        return new Services(parent, this);
     }
     return NULL;
 }
@@ -1881,7 +1720,7 @@ string JabberClient::resources(void *_data)
             if (!resource.empty())
                 resource += ";";
             const char *dicon = get_icon(data, atol(get_str(data->ResourceStatus, i)), false);
-            resource += number((unsigned)dicon);
+            resource += number((unsigned long)dicon);
             resource += ",";
             resource += quoteChars(get_str(data->Resources, i), ";");
         }
@@ -2069,26 +1908,29 @@ void JabberImageParser::tag_start(const QString &tag, const list<QString> &attrs
         return;
     if (tag == "img"){
         QString src;
+        QString alt;
         for (list<QString>::const_iterator it = attrs.begin(); it != attrs.end(); ++it){
             QString name = *it;
             ++it;
             QString value = *it;
-            if (name == "src"){
+            if (name == "src")
                 src = value;
-                break;
+            if (name == "alt")
+                alt = value;
+        }
+        if (!alt.isEmpty()){
+            res += unquoteString(alt);
+            return;
+        }
+        if (src.left(5) == "icon:"){
+            list<string> smiles = getIcons()->getSmile(src.mid(5).latin1());
+            if (!smiles.empty()){
+                res += QString::fromUtf8(smiles.front().c_str());
+                return;
             }
         }
-        if (src.left(10) != "icon:smile")
-            return;
-        bool bOK;
-        unsigned nIcon = src.mid(10).toUInt(&bOK, 16);
-        if (!bOK)
-            return;
-        const smile *p = smiles(nIcon);
-        if (p){
-            res += p->paste;
-            return;
-        }
+        text(alt);
+        return;
     }
     if (tag == "p"){
         if (m_bPara){
@@ -2268,12 +2110,12 @@ bool JabberClient::send(Message *msg, void *_data)
             }
             m_socket->writeBuffer
             << "\'><body>"
-            << (const char*)(quoteString(QString::fromUtf8(text.c_str()), quoteNOBR).utf8())
+            << (const char*)(quote_nbsp(quoteString(QString::fromUtf8(text.c_str()), quoteNOBR)).utf8())
             << "</body>";
             if (data->richText.bValue && getRichText() && (msg->getFlags() & MESSAGE_RICHTEXT)){
                 m_socket->writeBuffer
                 << "<html xmlns='http://jabber.org/protocol/xhtml-im'><body>"
-                << removeImages(msg->getRichText(), msg->getBackground()).utf8()
+                << quote_nbsp(removeImages(msg->getRichText(), msg->getBackground())).utf8()
                 << "</body></html>";
             }
             m_socket->writeBuffer
@@ -2642,6 +2484,34 @@ bool JabberClient::isAgent(const char *jid)
     return false;
 }
 
+class JabberClient::JabberAuthMessage : public AuthMessage
+{
+public:
+    JabberAuthMessage(std::vector<JabberAuthMessage*> &tempMessages, unsigned type, Buffer *cfg=NULL)
+        : AuthMessage(type, cfg)
+        , tempMessages(tempMessages)
+    {
+        tempMessages.push_back(this);
+    }
+    virtual ~JabberAuthMessage()
+    {
+        remove(tempMessages, this);
+    }
+
+    static bool remove(std::vector<JabberAuthMessage*>&messages, JabberAuthMessage *value)
+    {
+        std::vector<JabberAuthMessage*>::iterator it = find(messages.begin(), messages.end(), value);
+        if (it != messages.end())
+        {
+            messages.erase(it);
+            return true;
+        }
+        return false;
+    }
+
+private:
+    std::vector<JabberAuthMessage*> &tempMessages;
+};
 
 void JabberClient::auth_request(const char *jid, unsigned type, const char *text, bool bCreate)
 {
@@ -2682,18 +2552,34 @@ void JabberClient::auth_request(const char *jid, unsigned type, const char *text
     }
     if ((data == NULL) && bCreate){
         data = findContact(jid, NULL, true, contact, resource);
-        contact->setTemporary(CONTACT_TEMP);
+        contact->setFlags(CONTACT_TEMP);
     }
     if (data == NULL)
         return;
-    AuthMessage msg(type);
-    msg.setContact(contact->id());
-    msg.setClient(dataName(data).c_str());
-    msg.setFlags(MESSAGE_RECEIVED);
+    if (((type == MessageAuthGranted) || (type ==MessageAuthRefused)) &&
+            (contact->getFlags() & CONTACT_TEMP)){
+        contact->setFlags(contact->getFlags() & ~CONTACT_TEMP);
+        Event e(EventContactChanged, contact);
+        e.process();
+        return;
+    }
+    //FIXME: In EventMessageReceived handler from filter plugin, there is conditional delete(msg)
+    //that causes crash when filter plugin configured to block Authorisation messages.
+    //So parameter must be pointer to object from heap, but then I don't know how it can be safely 
+    //feed if it wasn't in EventMessageReceived. serzh.
+    //12.june.2006 zowers: FIXED.
+    JabberAuthMessage *msg = new JabberAuthMessage(tempAuthMessages, type);
+    msg->setContact(contact->id());
+    msg->setClient(dataName(data).c_str());
+    msg->setFlags(MESSAGE_RECEIVED);
     if (text)
-        msg.setText(unquoteString(QString::fromUtf8(text)));
-    Event e(EventMessageReceived, &msg);
+        msg->setText(unquoteString(QString::fromUtf8(text)));
+    Event e(EventMessageReceived, msg);
     e.process();
+    if (JabberAuthMessage::remove(tempAuthMessages, msg))
+    {
+        delete msg;
+    }
 }
 
 void JabberClient::setInvisible(bool bState)
@@ -2754,7 +2640,6 @@ void JabberFileTransfer::listen()
             if (!isDirectory())
                 break;
         }
-        return;
     }
     bind(m_client->getMinPort(), m_client->getMaxPort(), m_client);
 }
@@ -2786,7 +2671,7 @@ void JabberFileTransfer::bind_ready(unsigned short port)
     if (n >= 0)
         fname = fname.mid(n + 1);
     m_url = fname.utf8();
-    m_client->sendFileRequest(m_msg, port, m_data, m_url.c_str());
+    m_client->sendFileRequest(m_msg, port, m_data, m_url.c_str(), m_fileSize);
 }
 
 bool JabberFileTransfer::error(const char *err)
@@ -3142,6 +3027,8 @@ void JabberFileTransfer::send_line(const char *line)
 void JabberFileTransfer::connect()
 {
     m_nFiles = 1;
+    if (static_cast<JabberFileMessage*>(m_msg)->getPort() == 0)
+        m_client->sendFileAccept(m_msg, m_data);
     if (m_notify)
         m_notify->createFile(m_msg->getDescription(), 0xFFFFFFFF, false);
 }
@@ -3170,7 +3057,7 @@ QString JabberClient::logoFile(JabberUserData *data)
     return QFile::decodeName(f.c_str());
 }
 
-#ifndef WIN32
+#ifndef _MSC_VER
 #include "jabberclient.moc"
 #endif
 
