@@ -15,8 +15,8 @@
  *                                                                         *
  ***************************************************************************/
 
-#include "jabberconfig.h"
 #include "jabberclient.h"
+#include "jabberconfig.h"
 #include "jabber.h"
 #include "linklabel.h"
 
@@ -81,6 +81,10 @@ JabberConfig::JabberConfig(QWidget *parent, JabberClient *client, bool bConfig)
     connect(chkVHost, SIGNAL(toggled(bool)), this, SLOT(toggledVHost(bool)));
     chkHTTP->setChecked(m_client->getUseHTTP());
     edtUrl->setText(m_client->getURL());
+    lnkRich->setText("(JEP-0071)");
+    lnkRich->setUrl("http://www.jabber.org/jeps/jep-0071.html");
+    lnkTyping->setText("(JEP-0085)");
+    lnkTyping->setUrl("http://www.jabber.org/jeps/jep-0085.html");
 }
 
 void JabberConfig::apply(Client*, void*)
@@ -104,19 +108,9 @@ void JabberConfig::apply()
     }
     QString jid = edtID->text();
     int n = jid.find('@');
-    if (n < 0){
-        QString host;
-        if (chkVHost->isChecked() && !edtVHost->text().isEmpty()){
-            host = edtVHost->text();
-        }else{
-            host = edtServer->text();
-        }
-        if (!host.isEmpty()){
-            jid += "@";
-            jid += host;
-        }
-    }else{
+    if (n >= 0){
         QString host = jid.mid(n + 1);
+        jid = jid.left(n);
         set_str(&m_client->data.VHost.ptr, host.utf8());
         m_client->setUseVHost(true);
     }
@@ -192,7 +186,7 @@ void JabberConfig::changed()
     emit okEnabled(bOK);
 }
 
-#ifndef WIN32
+#ifndef _MSC_VER
 #include "jabberconfig.moc"
 #endif
 
