@@ -15,9 +15,9 @@
  *                                                                         *
  ***************************************************************************/
 
+#include "jabberclient.h"
 #include "simapi.h"
 #include "jabberpicture.h"
-#include "jabberclient.h"
 #include "editfile.h"
 #include "preview.h"
 #include "ballonmsg.h"
@@ -159,7 +159,7 @@ void JabberPicture::setPict(QImage &img)
     lblPict->setMinimumSize(pict.size());
 }
 
-#ifndef WIN32
+#ifndef _MSC_VER
 #include "jabberpicture.moc"
 #endif
 
