@@ -18,6 +18,8 @@
 #include "jabberclient.h"
 #include "jabbersearch.h"
 #include "jabber.h"
+#include "jidadvsearch.h"
+#include "ballonmsg.h"
 
 #include <qlayout.h>
 #include <qlabel.h>
@@ -30,6 +32,7 @@
 #include <qobjectlist.h>
 #include <qregexp.h>
 #include <qcheckbox.h>
+#include <qgroupbox.h>
 
 #include <vector>
 
@@ -48,6 +51,7 @@ protected:
 CComboBox::CComboBox(QWidget *parent, const char *name)
         : QComboBox(parent, name)
 {
+    setSizePolicy(QSizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed));
 }
 
 void CComboBox::addItem(const QString &label, const char *value)
@@ -66,8 +70,12 @@ QString CComboBox::value()
 
 const unsigned MAX_ELEMENTS = 8;
 
-JabberSearch::JabberSearch(QWidget *receiver, JabberClient *client, const char *jid, const char *node, const QString &name, bool bRegister)
-        : QChildWidget(NULL)
+JabberSearch::JabberSearch(QWidget *parent, const char *name)
+        : QChildWidget(parent, name)
+{
+}
+
+void JabberSearch::init(QWidget *receiver, JabberClient *client, const char *jid, const char *node, const QString &name, bool bRegister)
 {
     m_client    = client;
     m_jid       = jid;
@@ -78,11 +86,6 @@ JabberSearch::JabberSearch(QWidget *receiver, JabberClient *client, const char *
     m_bXData	= false;
     m_bFirst	= true;
     m_bRegister = bRegister;
-    QVBoxLayout *vlay = new QVBoxLayout(this);
-    vlay->setMargin(6);
-    lay = new QGridLayout(vlay);
-    lay->setSpacing(4);
-    vlay->addStretch();
     m_bDirty = false;
 }
 
@@ -98,8 +101,8 @@ static defFlds fields[] =
         { "username", I18N_NOOP("Username"), true },
         { "nick", I18N_NOOP("Nick"), false },
         { "email", I18N_NOOP("EMail"), false },
-        { "first", I18N_NOOP("First name"), false },
-        { "last", I18N_NOOP("Last name"), false },
+        { "first", I18N_NOOP("First Name"), false },
+        { "last", I18N_NOOP("Last Name"), false },
         { "age_min", I18N_NOOP("Age min"), false },
         { "age_max", I18N_NOOP("Age max"), false },
         { "city", I18N_NOOP("City"), false },
@@ -113,6 +116,20 @@ void JabberSearch::addWidget(JabberAgentInfo *data)
     if (data->Type.ptr){
         if (!strcmp(data->Type.ptr, "x")){
             m_bXData = true;
+            vector<QWidget*>::iterator it;
+            for (it = m_widgets.begin(); it != m_widgets.end(); ++it)
+                if (*it)
+                    delete (*it);
+            m_widgets.clear();
+            for (it = m_labels.begin(); it != m_labels.end(); ++it)
+                if (*it)
+                    delete (*it);
+            m_labels.clear();
+            for (it = m_descs.begin(); it != m_descs.end(); ++it)
+                if (*it)
+                    delete (*it);
+            m_descs.clear();
+            m_instruction = "";
         }else if (!strcmp(data->Type.ptr, "title")){
             if (data->Value.ptr && *data->Value.ptr)
                 m_title = QString::fromUtf8(data->Value.ptr);
@@ -222,42 +239,10 @@ void JabberSearch::addWidget(JabberAgentInfo *data)
             }
         }
     }else{
-        if (!m_widgets.empty()){
-            unsigned nCols = (m_widgets.size() + MAX_ELEMENTS - 1) / MAX_ELEMENTS;
-            unsigned nRows = (m_widgets.size() + nCols - 1) / nCols;
-            unsigned start = 0;
-            if (!m_label.isEmpty()){
-                QLabel *label = new QLabel(m_label, this);
-                label->setAlignment(WordBreak);
-                lay->addMultiCellWidget(label, 0, 0, 0, nCols * 2 + 1);
-                m_label = "";
-                start = 1;
-            }
-            unsigned row = start;
-            unsigned col = 0;
-            for (unsigned i = 0; i < m_widgets.size(); i++, row++){
-                if (row >= nRows + start){
-                    row  = 0;
-                    col += 2;
-                }
-                if (m_labels[i]){
-                    lay->addWidget(m_labels[i], row, col);
-                    lay->addWidget(m_widgets[i], row, col + 1);
-                    m_labels[i]->show();
-                }else{
-                    lay->addMultiCellWidget(m_widgets[i], row, row, col, col + 1);
-                }
-                m_widgets[i]->show();
-            }
-            if (!m_instruction.isEmpty()){
-                QLabel *label = new QLabel(m_instruction, this);
-                label->setAlignment(WordBreak);
-                lay->addMultiCellWidget(label, nRows + start, nRows + start, 0, nCols * 2 + 1);
-                m_instruction = "";
-            }
-        }
+        createLayout();
         m_widgets.clear();
         m_labels.clear();
+        m_descs.clear();
         m_bDirty = true;
         QTimer::singleShot(0, this, SLOT(setSize()));
         return;
@@ -268,11 +253,18 @@ void JabberSearch::addWidget(JabberAgentInfo *data)
             m_required.push_back(widget);
         QLabel *label = NULL;
         if (!bJoin && data->Label.ptr){
-            label = new QLabel(i18(data->Label.ptr), this);
+            QString text = i18(data->Label.ptr);
+            if (!text.isEmpty() && (text[(int)(text.length() - 1)] != ':'))
+                text += ":";
+            label = new QLabel(text, this);
             label->setAlignment(AlignRight);
         }
+        QWidget *help = NULL;
+        if (data->Desc.ptr && *data->Desc.ptr)
+            help = new HelpButton(QString::fromUtf8(data->Desc.ptr), this);
         m_labels.push_back(label);
         m_widgets.push_back(widget);
+        m_descs.push_back(help);
     }
 }
 
@@ -309,8 +301,8 @@ static const char *any_data[] =
         I18N_NOOP("Fast / Less accurate"),
         I18N_NOOP("Slower / More extensive")
         I18N_NOOP("Full name")
-        I18N_NOOP("First name")
-        I18N_NOOP("Last name")
+        I18N_NOOP("First Name")
+        I18N_NOOP("Last Name")
         I18N_NOOP("Nickname")
         I18N_NOOP("E-mail")
         I18N_NOOP("Username")
@@ -382,14 +374,16 @@ bool JabberSearch::canSearch()
     return bRes;
 }
 
-QString JabberSearch::condition(bool &bXSearch)
+QString JabberSearch::condition(QWidget *w)
 {
-    bXSearch = m_bXData;
     QString res;
-    if (m_bXData)
+    if (m_bXData && (w == NULL))
         res += "x:data";
 
-    QObjectList *l = queryList("QLineEdit");
+    if (w == NULL)
+        w = this;
+
+    QObjectList *l = w->queryList("QLineEdit");
     QObjectListIt it( *l );
     QObject *obj;
     while ((obj = it.current()) != 0 ){
@@ -405,7 +399,7 @@ QString JabberSearch::condition(bool &bXSearch)
     }
     delete l;
 
-    l = queryList("QComboBox");
+    l = w->queryList("QComboBox");
     QObjectListIt it1( *l );
     while ((obj = it1.current()) != 0 ){
         CComboBox *box = static_cast<CComboBox*>(obj);
@@ -422,23 +416,19 @@ QString JabberSearch::condition(bool &bXSearch)
     }
     delete l;
 
-    l = queryList("QCheckBox");
+    l = w->queryList("QCheckBox");
     QObjectListIt it2( *l );
     while ((obj = it2.current()) != 0 ){
         QCheckBox *box = static_cast<QCheckBox*>(obj);
-        if (!box->isChecked()){
-            ++it2;
-            continue;
-        }
         if (!res.isEmpty())
             res += ";";
         res += box->name();
-        res += "=1";
+        res += box->isChecked() ? "=1" : "=0";
         ++it2;
     }
     delete l;
 
-    l = queryList("QMultiLineEdit");
+    l = w->queryList("QMultiLineEdit");
     QObjectListIt it3( *l );
     while ((obj = it3.current()) != 0 ){
         QMultiLineEdit *edit = static_cast<QMultiLineEdit*>(obj);
@@ -453,17 +443,224 @@ QString JabberSearch::condition(bool &bXSearch)
     }
     delete l;
 
-    if (!m_key.empty()){
+    if (!m_key.empty() && (w == NULL)){
         if (!res.isEmpty())
             res += ";";
         res += "key=";
         res += quoteChars(QString::fromUtf8(m_key.c_str()), ";");
     }
-
     return res;
 }
 
-#ifndef WIN32
+void JabberSearch::createLayout()
+{
+    unsigned start = 0;
+    unsigned nCols = 0;
+    unsigned nRows = 0;
+    QVBoxLayout *vlay = new QVBoxLayout(this);
+    QGridLayout *lay = new QGridLayout(vlay);
+    vlay->setMargin(11);
+    lay->setSpacing(6);
+    vlay->addStretch();
+    if (!m_widgets.empty()){
+        nCols = (m_widgets.size() + MAX_ELEMENTS - 1) / MAX_ELEMENTS;
+        nRows = (m_widgets.size() + nCols - 1) / nCols;
+        start = 0;
+        if (!m_label.isEmpty()){
+            QLabel *label = new QLabel(m_label, this);
+            label->setAlignment(WordBreak);
+            lay->addMultiCellWidget(label, 0, 0, 0, nCols * 3 + 1);
+            m_label = "";
+            start = 1;
+        }
+        unsigned row = start;
+        unsigned col = 0;
+        for (unsigned i = 0; i < m_widgets.size(); i++, row++){
+            if (row >= nRows + start){
+                row  = 0;
+                col += 3;
+            }
+            if (m_labels[i]){
+                static_cast<QLabel*>(m_labels[i])->setAlignment( AlignVCenter | AlignRight);
+                lay->addWidget(m_labels[i], row, col);
+                if (m_descs[i]){
+                    lay->addWidget(m_widgets[i], row, col + 1, AlignVCenter);
+                    lay->addWidget(m_descs[i], row, col + 2, AlignVCenter);
+                    m_descs[i]->show();
+                }else{
+                    lay->addMultiCellWidget(m_widgets[i], row, row, col + 1, col + 2, AlignVCenter);
+                }
+                m_labels[i]->show();
+            }else{
+                if (m_descs[i]){
+                    lay->addMultiCellWidget(m_widgets[i], row, row, col, col + 1, AlignVCenter);
+                    lay->addWidget(m_descs[i], row, col + 2, AlignBottom);
+                    m_descs[i]->show();
+                }else{
+                    lay->addMultiCellWidget(m_widgets[i], row, row, col, col + 2, AlignVCenter);
+                }
+            }
+            m_widgets[i]->show();
+        }
+    }
+    if (!m_instruction.isEmpty()){
+        QLabel *label = new QLabel(m_instruction, this);
+        label->setAlignment(WordBreak);
+        lay->addMultiCellWidget(label, nRows + start, nRows + start, 0, nCols * 3 - 1);
+        m_instruction = "";
+    }
+}
+
+HelpButton::HelpButton(const QString &help, QWidget *parent)
+        : QPushButton(parent)
+{
+    const QPixmap p = Pict("help");
+    setPixmap(p);
+    m_help = help;
+    connect(this, SIGNAL(clicked()), this, SLOT(click()));
+    setMinimumSize(p.width() + 2, p.height() + 2);
+    setMaximumSize(p.width() + 2, p.height() + 2);
+}
+
+void HelpButton::click()
+{
+    BalloonMsg::message(m_help, this);
+}
+
+JIDJabberSearch::JIDJabberSearch(QWidget *parent, const char *name)
+        : JabberSearch(parent, name)
+{
+    m_adv = NULL;
+}
+
+void JIDJabberSearch::setAdvanced(JIDAdvSearch *adv)
+{
+    m_adv = adv;
+}
+
+static const char *names[] =
+    {
+        "username",
+        "email",
+        "nick",
+        "first",
+        "last",
+        "user",
+        "fn",
+        "given",
+        "online",
+        NULL
+    };
+
+const unsigned MAX_MAIN	= 6;
+
+void JIDJabberSearch::createLayout()
+{
+    unsigned rowMain = 0;
+    QGridLayout *lay = new QGridLayout(this);
+    QGridLayout *alay = NULL;
+    lay->setSpacing(6);
+    unsigned nAdv = 0;
+    unsigned nMain = 0;
+    if (m_widgets.size() > MAX_MAIN){
+        alay = new QGridLayout(m_adv->grpSearch);
+        alay->setMargin(11);
+        alay->setSpacing(6);
+        for (unsigned i = 0; i < m_widgets.size(); i++){
+            if (nMain > MAX_MAIN){
+                nAdv++;
+                continue;
+            }
+            const char **p;
+            for (p = names; *p; p++)
+                if (!strcmp(*p, m_widgets[i]->name()))
+                    break;
+            if (*p == NULL){
+                nAdv++;
+                continue;
+            }
+            nMain++;
+        }
+    }
+    unsigned nCols = (nAdv + MAX_ELEMENTS - 1) / MAX_ELEMENTS;
+    unsigned nRows = nCols ? (nAdv + nCols - 1) / nCols : 0;
+    unsigned start = 0;
+    unsigned row   = 0;
+    unsigned col   = 0;
+    nMain = 0;
+    for (unsigned i = 0; i < m_widgets.size(); i++){
+        bool bMain = false;
+        if (alay){
+            if (nMain < MAX_MAIN){
+                const char **p;
+                for (p = names; *p; p++)
+                    if (!strcmp(*p, m_widgets[i]->name()))
+                        break;
+                if (*p){
+                    nMain++;
+                    bMain = true;
+                }
+            }
+        }else{
+            bMain = true;
+        }
+        if (bMain){
+            if (m_labels[i]){
+                static_cast<QLabel*>(m_labels[i])->setAlignment(AlignVCenter);
+                lay->addMultiCellWidget(m_labels[i], rowMain, rowMain, 0, 1, AlignVCenter);
+                m_labels[i]->show();
+                rowMain++;
+            }
+            if (m_descs[i]){
+                lay->addWidget(m_widgets[i], rowMain, 0, AlignVCenter);
+                lay->addWidget(m_descs[i], rowMain, 1, AlignVCenter);
+                m_descs[i]->show();
+            }else{
+                lay->addMultiCellWidget(m_widgets[i], rowMain, rowMain, 0, 1, AlignVCenter);
+            }
+            m_widgets[i]->show();
+            rowMain++;
+        }else{
+            if (row >= nRows + start){
+                row  = 0;
+                col += 3;
+            }
+            m_widgets[i]->reparent(m_adv->grpSearch, QPoint(0, 0));
+            if (m_descs[i])
+                m_descs[i]->reparent(m_adv->grpSearch, QPoint(0, 0));
+            if (m_labels[i]){
+                m_labels[i]->reparent(m_adv->grpSearch, QPoint(0, 0));
+                static_cast<QLabel*>(m_labels[i])->setAlignment(AlignVCenter | AlignRight);
+                alay->addWidget(m_labels[i], row, col);
+                if (m_descs[i]){
+                    alay->addWidget(m_widgets[i], row, col + 1, AlignVCenter);
+                    alay->addWidget(m_descs[i], row, col + 2, AlignVCenter);
+                    m_descs[i]->show();
+                }else{
+                    alay->addMultiCellWidget(m_widgets[i], row, row, col + 1, col + 2, AlignVCenter);
+                }
+                m_labels[i]->show();
+            }else{
+                if (m_descs[i]){
+                    alay->addMultiCellWidget(m_widgets[i], row, row, col, col + 1, AlignVCenter);
+                    alay->addWidget(m_descs[i], row, col + 2, AlignBottom);
+                    m_descs[i]->show();
+                }else{
+                    alay->addMultiCellWidget(m_widgets[i], row, row, col, col + 2, AlignVCenter);
+                }
+            }
+            m_widgets[i]->show();
+            row++;
+        }
+    }
+    if (alay){
+        m_adv->lblTitle->setText(m_title);
+        m_adv->lblInstruction->setText(m_instruction);
+    }
+    m_instruction = "";
+}
+
+#ifndef _MSC_VER
 #include "jabbersearch.moc"
 #endif
 
