@@ -15,9 +15,9 @@
  *                                                                         *
  ***************************************************************************/
 
+#include "jabberclient.h"
 #include "simapi.h"
 #include "jabberworkinfo.h"
-#include "jabberclient.h"
 #include "jabber.h"
 
 #include <qlineedit.h>
@@ -55,7 +55,7 @@ void *JabberWorkInfo::processEvent(Event *e)
         if (client == m_client)
             fill(m_data);
     }
-    if (m_data && (e->type() == static_cast<JabberPlugin*>(m_client->protocol()->plugin())->EventVCard)){
+    if (m_data && (e->type() == EventVCard)){
         JabberUserData *data = (JabberUserData*)(e->param());
         if (!str_cmp(m_data->ID.ptr, data->ID.ptr) && !str_cmp(m_data->Node.ptr, data->Node.ptr))
             fill(data);
@@ -83,7 +83,7 @@ void JabberWorkInfo::apply(Client *client, void *_data)
     set_str(&data->Role.ptr, edtRole->text().utf8());
 }
 
-#ifndef WIN32
+#ifndef _MSC_VER
 #include "jabberworkinfo.moc"
 #endif
 
