@@ -23,14 +23,6 @@
 #include "html.h"
 #include "core.h"
 
-#include "xpm/livejournal.xpm"
-
-#ifdef USE_OPENSSL
-#include <openssl/md5.h>
-#else
-#include "md5.h"
-#endif
-
 #ifdef TM_IN_SYS_TIME
 #include <sys/time.h>
 #else
@@ -39,8 +31,9 @@
 
 #include <qtimer.h>
 #include <qregexp.h>
+#include <stdio.h>
 
-Plugin *createLiveJournalPlugin(unsigned base, bool, const char*)
+Plugin *createLiveJournalPlugin(unsigned base, bool, Buffer*)
 {
     Plugin *plugin = new LiveJournalPlugin(base);
     return plugin;
@@ -72,7 +65,7 @@ static DataDef journalMessageData[] =
         { NULL, 0, 0, 0 }
     };
 
-JournalMessage::JournalMessage(const char *cfg)
+JournalMessage::JournalMessage(Buffer *cfg)
         : Message(MessageJournal, cfg)
 {
     load_data(journalMessageData, &data, cfg);
@@ -110,7 +103,7 @@ i18n("LiveJournal post", "%n LiveJournal posts", 1);
 i18n("Friends updated", "Friends updated %n", 1);
 #endif
 
-static Message *createJournalMessage(const char *cfg)
+static Message *createJournalMessage(Buffer *cfg)
 {
     return new JournalMessage(cfg);
 }
@@ -178,7 +171,7 @@ static MessageDef defWWWJournalMessage =
         NULL
     };
 
-static Message *createUpdatedMessage(const char *cfg)
+static Message *createUpdatedMessage(Buffer *cfg)
 {
     return new Message(MessageUpdated, cfg);
 }
@@ -201,18 +194,6 @@ unsigned LiveJournalPlugin::MenuCount = 0;
 LiveJournalPlugin::LiveJournalPlugin(unsigned base)
         : Plugin(base)
 {
-    IconDef icon;
-    icon.name  = "LiveJournal";
-    icon.xpm   = livejournal;
-
-    Event eIcon(EventAddIcon, &icon);
-    eIcon.process();
-
-    icon.name	= "upd";
-    icon.xpm	= NULL;
-    icon.flags	= 160;
-    eIcon.process();
-
     m_protocol = new LiveJournalProtocol(this);
 
     Event eMenu(EventMenuCreate, (void*)MenuWeb);
@@ -284,7 +265,7 @@ LiveJournalProtocol::~LiveJournalProtocol()
 {
 }
 
-Client *LiveJournalProtocol::createClient(const char *cfg)
+Client *LiveJournalProtocol::createClient(Buffer *cfg)
 {
     return new LiveJournalClient(this, cfg);
 }
@@ -301,7 +282,7 @@ static CommandDef livejournal_descr =
         0,
         0,
         0,
-        PROTOCOL_NOSMS | PROTOCOL_SEARCH | PROTOCOL_NOPROXY,
+        PROTOCOL_NOSMS | PROTOCOL_NOPROXY,
         NULL,
         NULL
     };
@@ -393,6 +374,9 @@ static DataDef liveJournalClientData[] =
         { "Menu", DATA_STRLIST, 1, 0 },
         { "MenuURL", DATA_STRLIST, 1, 0 },
         { "FastServer", DATA_BOOL, 1, 0 },
+        { "UseFormatting", DATA_BOOL, 1, 0 },
+        { "UseSignature", DATA_BOOL, 1, DATA(1) },
+        { "Signature", DATA_UTF, 1, 0 },
         { "", DATA_STRING, 1, 0 },			// LastUpdate
         { "", DATA_STRUCT, sizeof(LiveJournalUserData) / sizeof(Data), DATA(liveJournalUserData) },
         { NULL, 0, 0, 0 }
@@ -403,7 +387,7 @@ const DataDef *LiveJournalProtocol::userDataDef()
     return liveJournalUserData;
 }
 
-LiveJournalClient::LiveJournalClient(Protocol *proto, const char *cfg)
+LiveJournalClient::LiveJournalClient(Protocol *proto, Buffer *cfg)
         : TCPClient(proto, cfg)
 {
     load_data(liveJournalClientData, &data, cfg);
@@ -547,14 +531,23 @@ MessageRequest::MessageRequest(LiveJournalClient *client, JournalMessage *msg, c
     m_msg = msg;
     m_bEdit   = (msg->getID() != 0);
     m_bResult = false;
+    QString text;
     if (msg->getRichText().isEmpty()){
-        addParam("event", "");
+        text = QString::null;
     }else{
-        BRParser parser(msg->getBackground());
-        parser.parse(msg->getRichText());
-        addParam("event", parser.m_str.utf8());
+        // if (msg->getFlags() & MESSAGE_RICHTEXT){
+        if (client->getUseFormatting()){
+            BRParser parser(msg->getBackground());
+            parser.parse(msg->getRichText());
+            text = parser.m_str;
+        }else{
+            text = msg->getPlainText();
+        }
         addParam("subject", msg->getSubject().utf8());
     }
+    if (!m_bEdit && client->getUseSignature())
+        text += "\n" + client->getSignatureText();
+    addParam("event", text.utf8());
     addParam("lineendings", "unix");
     if (msg->getID())
         addParam("itemid", number(msg->getID()).c_str());
@@ -571,8 +564,20 @@ MessageRequest::MessageRequest(LiveJournalClient *client, JournalMessage *msg, c
     addParam("day", number(tm->tm_mday).c_str());
     addParam("hour", number(tm->tm_hour).c_str());
     addParam("min", number(tm->tm_min).c_str());
-    if (msg->getPrivate())
-        addParam("security", "private");
+    if (msg->getPrivate()){
+        switch (msg->getPrivate()){
+        case 0:
+            addParam("security", "public");
+            break;
+        case 1:
+            addParam("security", "usemask");
+            addParam("allowmask", "0");
+            break;
+        case 2:
+            addParam("security", "private");
+            break;
+        }
+    }
     if (msg->getMood())
         addParam("prop_current_moodid", number(msg->getMood()).c_str());
     if (journal)
@@ -762,7 +767,7 @@ bool LiveJournalClient::add(const char *name)
     return true;
 }
 
-LiveJournalUserData *LiveJournalClient::findContact(const char *user, Contact *&contact, bool bCreate)
+LiveJournalUserData *LiveJournalClient::findContact(const char *user, Contact *&contact, bool bCreate, bool bJoin)
 {
     ContactList::ContactIterator it;
     while ((contact = ++it) != NULL){
@@ -773,15 +778,17 @@ LiveJournalUserData *LiveJournalClient::findContact(const char *user, Contact *&
                 return data;
         }
     }
+    if (!bCreate)
+        return NULL;
     QString sname = QString::fromUtf8(user);
-    it.reset();
-    while ((contact = ++it) != NULL){
-        if (contact->getName().lower() == sname.lower())
-            break;;
+    if (bJoin){
+        it.reset();
+        while ((contact = ++it) != NULL){
+            if (contact->getName().lower() == sname.lower())
+                break;;
+        }
     }
     if (contact == NULL){
-        if (!bCreate)
-            return NULL;
         contact = getContacts()->contact(0, true);
         contact->setName(sname);
     }
@@ -834,6 +841,14 @@ void LiveJournalClient::statusChanged()
     findContact(data.owner.User.ptr, contact);
 }
 
+QString LiveJournalClient::getSignatureText()
+{
+    QString res = getSignature();
+    if (res.isEmpty())
+        res = i18n("<div style=\"text-align:right;font-size:0.7em;font-style:italic;width:100%\">Powered by <a style=\"font-size:1em;font-style:italic;\" href=\"http://sim-im.org\">SIM Instant Messenger</a></div>");
+    return res;
+}
+
 static void addIcon(string *s, const char *icon, const char *statusIcon)
 {
     if (s == NULL)
@@ -897,7 +912,7 @@ protected:
 };
 
 LoginRequest::LoginRequest(LiveJournalClient *client)
-: LiveJournalRequest(client, "login")
+        : LiveJournalRequest(client, "login")
 {
     m_bOK     = false;
     m_bResult = false;
@@ -1032,9 +1047,9 @@ void LiveJournalClient::auth_fail(const char *err)
     error_state(err, AuthError);
 }
 
-QWidget *LiveJournalClient::searchWindow()
+QWidget *LiveJournalClient::searchWindow(QWidget *parent)
 {
-    return new JournalSearch(this);
+    return new JournalSearch(this, parent);
 }
 
 bool LiveJournalClient::done(unsigned code, Buffer &data, const char*)
@@ -1088,7 +1103,11 @@ void *LiveJournalClient::processEvent(Event *e)
             ClientDataIterator it(contact->clientData, this);
             while ((data = (LiveJournalUserData*)(++it)) != NULL){
                 if (dataName(data) == msg->client()){
-                    JournalMessage *m = new JournalMessage(msg->save().c_str());
+                    Buffer cfg;
+                    cfg << "[Title]\n" << msg->save().c_str();
+                    cfg.setWritePos(0);
+                    cfg.getSection();
+                    JournalMessage *m = new JournalMessage(&cfg);
                     m->setContact(msg->contact());
                     m->setOldID(msg->id());
                     m->setText("");
@@ -1146,7 +1165,7 @@ void *LiveJournalClient::processEvent(Event *e)
                     if (url && (*url == '@')){
                         unsigned nSub = atol(url + 1);
                         while (nSub > LiveJournalPlugin::MenuCount){
-                            unsigned menu_id = MenuWeb + (++LiveJournalPlugin::MenuCount);
+                            unsigned long menu_id = MenuWeb + (++LiveJournalPlugin::MenuCount);
                             Event eMenu(EventMenuCreate, (void*)menu_id);
                             eMenu.process();
                             Command cmd;
@@ -1288,17 +1307,14 @@ LiveJournalRequest::LiveJournalRequest(LiveJournalClient *client, const char *mo
     m_client = client;
     m_buffer = new Buffer;
     addParam("mode", mode);
+    addParam("ver", "1");
     if (client->data.owner.User.ptr)
         addParam("user", client->data.owner.User.ptr);
-    MD5_CTX md5;
-    MD5_Init(&md5);
-    MD5_Update(&md5, client->getPassword().utf8(), strlen(client->getPassword().utf8()));
-    unsigned char pass[MD5_DIGEST_LENGTH];
-    MD5_Final(pass, &md5);
+    string pass = md5(client->getPassword().utf8());
     string hpass;
-    for (unsigned i = 0; i < MD5_DIGEST_LENGTH; i++){
+    for (unsigned i = 0; i < pass.length(); i++){
         char b[5];
-        sprintf(b, "%02x", pass[i]);
+        sprintf(b, "%02x", pass[i] & 0xFF);
         hpass += b;
     }
     addParam("hpassword", hpass.c_str());
@@ -1445,7 +1461,6 @@ I18N_NOOP("working")
 I18N_NOOP("productive")
 I18N_NOOP("accomplished")
 I18N_NOOP("busy")
-I18N_NOOP("blah")
 I18N_NOOP("full")
 I18N_NOOP("grumpy")
 I18N_NOOP("weird")
@@ -1521,7 +1536,7 @@ extern "C" BOOL __stdcall _DllMainCRTStartup( HINSTANCE hinstDLL, DWORD fdwReaso
 
 #endif
 
-#ifndef WIN32
+#ifndef _MSC_VER
 #include "livejournal.moc"
 #endif
 
