@@ -22,7 +22,7 @@
 #include "stl.h"
 
 const unsigned short L_PACKETS = 0x08;
-const unsigned short L_EVENTS  = 0x10;
+// const unsigned short L_EVENTS  = 0x10;
 
 typedef struct LoggerData
 {
@@ -37,7 +37,7 @@ class LoggerPlugin : public QObject, public Plugin, public EventReceiver
 {
     Q_OBJECT
 public:
-    LoggerPlugin(unsigned, const char*);
+    LoggerPlugin(unsigned, Buffer*);
     virtual ~LoggerPlugin();
     PROP_ULONG(LogLevel);
     PROP_STR(LogPackets);
@@ -45,7 +45,7 @@ public:
     bool isLogType(unsigned id);
     void setLogType(unsigned id, bool bLog);
 protected:
-    bool eventFilter(QObject *o, QEvent *e);
+//    bool eventFilter(QObject *o, QEvent *e);
     list<unsigned> m_packets;
     virtual QWidget *createConfigWindow(QWidget *parent);
     virtual string getConfig();
