@@ -119,7 +119,7 @@ static DataDef msnClientData[] =
         { NULL, 0, 0, 0 }
     };
 
-MSNClient::MSNClient(Protocol *protocol, const char *cfg)
+MSNClient::MSNClient(Protocol *protocol, Buffer *cfg)
         : TCPClient(protocol, cfg)
 {
     load_data(msnClientData, &data, cfg);
@@ -410,7 +410,7 @@ void MSNClient::processLST(const char *mail, const char *name, unsigned state, u
     }
     if (lr == NULL){
         bool bChanged = ((data->Flags.value & MSN_FLAGS) != (data->sFlags.value & MSN_FLAGS));
-        if (getAutoAuth() && (data->Flags.value & MSN_FORWARD) && (data->Flags.value & MSN_ACCEPT == 0) && (data->Flags.value & MSN_BLOCKED == 0))
+        if (getAutoAuth() && (data->Flags.value & MSN_FORWARD) && ((data->Flags.value & MSN_ACCEPT) == 0) && ((data->Flags.value & MSN_BLOCKED) == 0))
             bChanged = true;
         unsigned grp = 0;
         if (group)
@@ -690,7 +690,7 @@ void MSNClient::getLine(const char *line)
         MSNUserData *data = findContact(email.c_str(), contact);
         if (data == NULL){
             data = findContact(email.c_str(), nick.c_str(), contact);
-            contact->setTemporary(CONTACT_TEMP);
+            contact->setFlags(CONTACT_TEMP);
             Event e(EventContactChanged, contact);
             e.process();
         }
@@ -704,7 +704,7 @@ void MSNClient::getLine(const char *line)
     }
     if (cmd == "OUT"){
         m_reconnect = NO_RECONNECT;
-        m_socket->error_state(I18N_NOOP("Youur account is being used from another location"));
+        m_socket->error_state(I18N_NOOP("Your account is being used from another location"));
         return;
     }
     if (cmd == "GTC")
@@ -1201,7 +1201,7 @@ QString MSNClient::contactName(void *clientData)
     return res;
 }
 
-MSNUserData *MSNClient::findContact(const char *mail, const char *name, Contact *&contact)
+MSNUserData *MSNClient::findContact(const char *mail, const char *name, Contact *&contact, bool bJoin)
 {
     unsigned i;
     for (i = 1; i <= getNDeleted(); i++){
@@ -1228,34 +1228,19 @@ MSNUserData *MSNClient::findContact(const char *mail, const char *name, Contact
         setupContact(contact, data);
         return data;
     }
-    ContactList::ContactIterator it;
-    while ((contact = ++it) != NULL){
-        if (contact->getName() == name_str){
-            data = (MSNUserData*)(contact->clientData.createData(this));
-            set_str(&data->EMail.ptr, mail);
-            set_str(&data->ScreenName.ptr, name);
-            setupContact(contact, data);
-            Event e(EventContactChanged, contact);
-            e.process();
-            return data;
+    if (bJoin){
+        ContactList::ContactIterator it;
+        while ((contact = ++it) != NULL){
+            if (contact->getName() == name_str){
+                data = (MSNUserData*)(contact->clientData.createData(this));
+                set_str(&data->EMail.ptr, mail);
+                set_str(&data->ScreenName.ptr, name);
+                setupContact(contact, data);
+                Event e(EventContactChanged, contact);
+                e.process();
+                return data;
+            }
         }
-    }
-    it.reset();
-    while ((contact = ++it) != NULL){
-        if (contact->getName().lower() == name_str.lower()){
-            data = (MSNUserData*)(contact->clientData.createData(this));
-            set_str(&data->EMail.ptr, mail);
-            set_str(&data->ScreenName.ptr, name);
-            setupContact(contact, data);
-            Event e(EventContactChanged, contact);
-            e.process();
-            m_bJoin = true;
-            return data;
-        }
-    }
-    int n = name_str.find('@');
-    if (n > 0){
-        name_str = name_str.left(n);
         it.reset();
         while ((contact = ++it) != NULL){
             if (contact->getName().lower() == name_str.lower()){
@@ -1269,6 +1254,23 @@ MSNUserData *MSNClient::findContact(const char *mail, const char *name, Contact
                 return data;
             }
         }
+        int n = name_str.find('@');
+        if (n > 0){
+            name_str = name_str.left(n);
+            it.reset();
+            while ((contact = ++it) != NULL){
+                if (contact->getName().lower() == name_str.lower()){
+                    data = (MSNUserData*)(contact->clientData.createData(this));
+                    set_str(&data->EMail.ptr, mail);
+                    set_str(&data->ScreenName.ptr, name);
+                    setupContact(contact, data);
+                    Event e(EventContactChanged, contact);
+                    e.process();
+                    m_bJoin = true;
+                    return data;
+                }
+            }
+        }
     }
     contact = getContacts()->contact(0, true);
     data = (MSNUserData*)(contact->clientData.createData(this));
@@ -1379,6 +1381,19 @@ bool MSNClient::done(unsigned code, Buffer&, const char *headers)
 void *MSNClient::processEvent(Event *e)
 {
     TCPClient::processEvent(e);
+    if (e->type() == EventCommandExec){
+        CommandDef *cmd = (CommandDef*)(e->param());
+        if (cmd->id == static_cast<MSNPlugin*>(protocol()->plugin())->MSNInitMail){
+            Event eGo(EventGoURL, (void*)m_init_mail.c_str());
+            eGo.process();
+            return e->param();
+        }
+        if (cmd->id == static_cast<MSNPlugin*>(protocol()->plugin())->MSNNewMail){
+            Event eGo(EventGoURL, (void*)m_new_mail.c_str());
+            eGo.process();
+            return e->param();
+        }
+    }
     if (e->type() == EventAddContact){
         addContact *ac = (addContact*)(e->param());
         if (ac->proto && !strcmp(protocol()->description()->text, ac->proto)){
@@ -1853,9 +1868,11 @@ QString MSNClient::contactTip(void *_data)
     return res;
 }
 
-QWidget *MSNClient::searchWindow()
+QWidget *MSNClient::searchWindow(QWidget *parent)
 {
-    return new MSNSearch(this);
+    if (getState() != Connected)
+        return NULL;
+    return new MSNSearch(this, parent);
 }
 
 SBSocket::SBSocket(MSNClient *client, Contact *contact, MSNUserData *data)
@@ -3103,6 +3120,6 @@ bool SBSocket::declineMessage(Message *msg, const char *reason)
     return false;
 }
 
-#ifndef WIN32
+#ifndef _MSC_VER
 #include "msnclient.moc"
 #endif
