@@ -168,7 +168,7 @@ class MSNClient : public TCPClient, public FetchClient
 {
     Q_OBJECT
 public:
-    MSNClient(Protocol*, const char *cfg);
+    MSNClient(Protocol*, Buffer *cfg);
     ~MSNClient();
     virtual string name();
     virtual QWidget	*setupWnd();
@@ -192,7 +192,7 @@ public:
     void setLogin(const QString&);
     MSNClientData	data;
     string dataName(void*);
-    MSNUserData *findContact(const char *mail, const char *name, Contact *&contact);
+    MSNUserData *findContact(const char *mail, const char *name, Contact *&contact, bool nJoin=true);
     MSNUserData *findContact(const char *mail, Contact *&contact);
     MSNUserData *findGroup(unsigned long id, const char *name, Group *&grp);
     void auth_message(Contact *contact, unsigned type, MSNUserData *data);
@@ -224,7 +224,7 @@ protected:
     virtual bool send(Message*, void*);
     virtual bool canSend(unsigned, void*);
     virtual void *processEvent(Event*);
-    virtual QWidget *searchWindow();
+    virtual QWidget *searchWindow(QWidget *parent);
     virtual bool isMyData(clientData*&, Contact*&);
     virtual bool createData(clientData*&, Contact*);
     Socket *createSocket();
@@ -255,6 +255,8 @@ protected:
     };
     AuthState	 m_state;
     string		 m_authChallenge;
+    string		 m_init_mail;
+    string		 m_new_mail;
     bool		 m_bFirstTry;
     bool		 m_bHTTP;
     bool		 m_bFirst;
