@@ -25,27 +25,34 @@ typedef struct NavigateData
 #ifdef WIN32
     Data NewWindow;
 #else
-Data Browser;
+    Data Browser;
     Data Mailer;
 #endif
+#ifdef USE_KDE
+    SIM::Data UseKDE;
+#endif
 } NavigateData;
 
 class NavigatePlugin : public Plugin, public EventReceiver
 {
 public:
-    NavigatePlugin(unsigned, const char *name);
+    NavigatePlugin(unsigned, Buffer *name);
     virtual ~NavigatePlugin();
 protected:
     virtual void *processEvent(Event*);
     QString parseUrl(const QString &text);
-    unsigned CmdMail;
-    unsigned CmdMailList;
-    unsigned MenuMail;
+    unsigned long CmdMail;
+    unsigned long CmdMailList;
+    unsigned long CmdCopyLocation;
+    unsigned long MenuMail;
 #ifdef WIN32
     PROP_BOOL(NewWindow);
 #else
     PROP_STR(Browser);
     PROP_STR(Mailer);
+#endif
+#ifdef USE_KDE
+    PROP_BOOL(UseKDE);
 #endif
     virtual string getConfig();
     virtual QWidget *createConfigWindow(QWidget *parent);
