@@ -40,24 +40,29 @@ const int mnuDebug = 6;
 const int mnuWarning = 7;
 const int mnuError = 8;
 const int mnuPause = 9;
+const int mnuAutoscroll = 10;
 
 MonitorWindow *monitor = NULL;
 
 MonitorWindow::MonitorWindow(NetmonitorPlugin *plugin)
         : QMainWindow(NULL, "monitor", WType_TopLevel)
 {
+    bPause = true;  // no debug output during creation
     m_plugin = plugin;
     SET_WNDPROC("monitor")
     setCaption(i18n("Network monitor"));
     setIcon(Pict("network"));
-    bPause = false;
+
     edit = new TextShow(this);
     edit->setWordWrap(QTextEdit::NoWrap);
     setCentralWidget(edit);
     QMenuBar *menu = menuBar();
     menuFile = new QPopupMenu(this);
+    menuFile->setCheckable(true);
     connect(menuFile, SIGNAL(aboutToShow()), this, SLOT(adjustFile()));
     menuFile->insertItem(Pict("filesave"), i18n("&Save"), this, SLOT(save()), 0, mnuSave);
+    menuFile->insertSeparator();
+    menuFile->insertItem(i18n("&Autoscroll"), this, SLOT(toggleAutoscroll()), 0, mnuAutoscroll);
     menuFile->insertItem(i18n("&Pause"), this, SLOT(pause()), 0, mnuPause);
     menuFile->insertSeparator();
     menuFile->insertItem(Pict("exit"), i18n("E&xit"), this, SLOT(exit()), 0, mnuExit);
@@ -72,6 +77,8 @@ MonitorWindow::MonitorWindow(NetmonitorPlugin *plugin)
     connect(menuLog, SIGNAL(aboutToShow()), this, SLOT(adjustLog()));
     connect(menuLog, SIGNAL(activated(int)), this, SLOT(toggleType(int)));
     menu->insertItem(i18n("&Log"), menuLog);
+    bPause = false;
+    bAutoscroll = true;
 }
 
 void MonitorWindow::closeEvent(QCloseEvent *e)
@@ -110,6 +117,7 @@ void MonitorWindow::adjustFile()
 {
     menuFile->setItemEnabled(mnuSave, !edit->hasSelectedText());
     menuFile->changeItem(mnuPause, bPause ? i18n("&Resume") : i18n("&Pause"));
+    menuFile->setItemChecked(mnuAutoscroll, bAutoscroll);
 }
 
 void MonitorWindow::copy()
@@ -141,6 +149,11 @@ void MonitorWindow::toggleType(int id)
     m_plugin->setLogType(id, !m_plugin->isLogType(id));
 }
 
+void MonitorWindow::toggleAutoscroll()
+{
+    bAutoscroll = !bAutoscroll;
+}
+
 void MonitorWindow::pause()
 {
     bPause = !bPause;
@@ -195,6 +208,9 @@ static LevelColorDef levelColors[] =
 
 void *MonitorWindow::processEvent(Event *e)
 {
+    if (!e) {
+        return 0;
+    }
     if ((e->type() == EventLog) && !bPause){
         LogInfo *li = (LogInfo*)e->param();
         if (((li->packet_id == 0) && (li->log_level & m_plugin->getLogLevel())) ||
@@ -216,14 +232,15 @@ void *MonitorWindow::processEvent(Event *e)
             logString += "</pre></p>";
             setLogEnable(false);
             edit->append(logString);
-            edit->scrollToBottom();
+            if (bAutoscroll)
+                edit->scrollToBottom();
             setLogEnable(true);
         }
     }
     return NULL;
 }
 
-#ifndef _WINDOWS
+#ifndef _MSC_VER
 #include "monitor.moc"
 #endif
 
