@@ -35,7 +35,7 @@ class NetmonitorPlugin : public QObject, public Plugin, public EventReceiver
 {
     Q_OBJECT
 public:
-    NetmonitorPlugin(unsigned, const char *name);
+    NetmonitorPlugin(unsigned, Buffer *name);
     virtual ~NetmonitorPlugin();
     PROP_ULONG(LogLevel);
     PROP_STR(LogPackets);
@@ -50,7 +50,7 @@ protected:
     virtual string getConfig();
     void showMonitor();
     void saveState();
-    unsigned CmdNetMonitor;
+    unsigned long CmdNetMonitor;
     list<unsigned> m_packets;
     NetMonitorData data;
     MonitorWindow *monitor;
