@@ -27,8 +27,6 @@
 
 #include "fontedit.h"
 
-#include "xpm/alert.xpm"
-
 #include <qtimer.h>
 #include <qpainter.h>
 #include <qapplication.h>
@@ -37,10 +35,13 @@
 #include <qregion.h>
 #include <qstyle.h>
 #include <qregexp.h>
+#include <qpushbutton.h>
 
 #ifdef WIN32
 #include <windows.h>
-
+#ifndef CS_DROPSHADOW
+#define CS_DROPSHADOW   0x00020000
+#endif
 #ifndef SPI_GETSCREENSAVERRUNNING 
 #define SPI_GETSCREENSAVERRUNNING 114
 #endif
@@ -53,10 +54,10 @@
 #endif
 #endif
 
-const unsigned SHADOW_OFFS	= 2;
+const unsigned SHADOW_DEF	= 2;
 const unsigned XOSD_MARGIN	= 5;
 
-Plugin *createOSDPlugin(unsigned base, bool, const char*)
+Plugin *createOSDPlugin(unsigned base, bool, Buffer*)
 {
     Plugin *plugin = new OSDPlugin(base);
     return plugin;
@@ -82,6 +83,14 @@ typedef struct OSDUserData
 {
 	unsigned	EnableMessage;
 	unsigned	EnableAlert;
+	unsigned	EnableAlertOnline;,
+	unsigned	EnableAlertAway; 
+	unsigned	EnableAlertNA;
+	unsigned	EnableAlertDND;
+	unsigned	EnableAlertOccupied;
+	unsigned	EnableAlertFFC;
+	unsigned	EnableAlertOffline;
+	unsigned	EnableTyping;
 	unsigned	Position;
 	unsigned	Offset;
 	unsigned	Color;
@@ -100,6 +109,13 @@ static DataDef osdUserData[] =
         { "EnableMessageShowContent", DATA_BOOL, 1, DATA(0) },
         { "ContentTypes", DATA_ULONG, 1, DATA(3) },
         { "EnableAlert", DATA_BOOL, 1, DATA(1) },
+        { "EnableAlertOnline", DATA_BOOL, 1, DATA(1) },
+        { "EnableAlertAway", DATA_BOOL, 1, DATA(0) },
+        { "EnableAlertNA", DATA_BOOL, 1, DATA(0) },
+        { "EnableAlertDND", DATA_BOOL, 1, DATA(0) },
+        { "EnableAlertOccupied", DATA_BOOL, 1, DATA(0) },
+        { "EnableAlertFFC", DATA_BOOL, 1, DATA(0) },
+        { "EnableAlertOffline", DATA_BOOL, 1, DATA(0) },
         { "EnableTyping", DATA_BOOL, 1, 0 },
         { "Position", DATA_ULONG, 1, 0 },
         { "Offset", DATA_ULONG, 1, DATA(30) },
@@ -125,12 +141,6 @@ OSDPlugin::OSDPlugin(unsigned base)
 {
     osdPlugin    = this;
 
-    IconDef icon;
-    icon.name = "alert";
-    icon.xpm = alert;
-    Event eIcon(EventAddIcon, &icon);
-    eIcon.process();
-
     user_data_id = getContacts()->registerUserData(info.title, osdUserData);
     Command cmd;
     cmd->id		 = user_data_id + 1;
@@ -173,12 +183,26 @@ void OSDPlugin::timeout()
     processQueue();
 }
 
+static const char * const arrow_h_xpm[] = {
+            "9 7 3 1",
+            "	    c None",
+            ".	    c #000000",
+            "+	    c none",
+            "..++..+++",
+            "+..++..++",
+            "++..++..+",
+            "+++..++..",
+            "++..++..+",
+            "+..++..++",
+            "..++..+++"};
+
 OSDWidget::OSDWidget()
         : QWidget(NULL, "osd", WType_TopLevel |
                   WStyle_StaysOnTop |  WStyle_Customize | WStyle_NoBorder |
                   WStyle_Tool |WRepaintNoErase | WX11BypassWM)
 {
     baseFont = font();
+    m_button = NULL;
     int size = baseFont.pixelSize();
     if (size <= 0){
         size = baseFont.pointSize();
@@ -192,28 +216,6 @@ OSDWidget::OSDWidget()
 
 QPixmap& intensity(QPixmap &pict, float percent);
 
-#ifndef WIN32
-#if !defined(QT_MACOSX_VERSION) && !defined(QT_MAC)
-
-static XErrorHandler old_handler = 0;
-static Bool got_badwindow = False;
-static int
-BadWindow_ehandler (Display *dpy, XErrorEvent *error)
-{
-    if (error->error_code == BadWindow)
-    {
-        got_badwindow = True;
-        return 0;
-    }
-    else
-    {
-        return (*old_handler) (dpy, error);
-    }
-}
-
-#endif
-#endif
-
 bool OSDWidget::isScreenSaverActive()
 {
 #ifdef WIN32
@@ -226,6 +228,20 @@ bool OSDWidget::isScreenSaverActive()
     return false;
 }
 
+static const char * const close_h_xpm[] = {
+            "8 8 3 1",
+            "	    c None",
+            ".	    c #000000",
+            "+	    c none",
+            ".++++++.",
+            "+.++++.+",
+            "++.++.++",
+            "+++..+++",
+            "+++..+++",
+            "++.++.++",
+            "+.++++.+",
+            ".++++++."};
+
 void OSDWidget::showOSD(const QString &str, OSDUserData *data)
 {
     if (isScreenSaverActive()){
@@ -239,11 +255,18 @@ void OSDWidget::showOSD(const QString &str, OSDUserData *data)
     unsigned nScreens = screens();
     if (nScreen >= nScreens)
         nScreen = 0;
+    int SHADOW_OFFS = SHADOW_DEF;
     QRect rcScreen = screenGeometry(nScreen);
     rcScreen = QRect(0, 0,
-                     rcScreen.width() - SHADOW_OFFS - XOSD_MARGIN * 2 - data->Offset.value,
+                     rcScreen.width()  - SHADOW_OFFS - XOSD_MARGIN * 2 - data->Offset.value,
                      rcScreen.height() - SHADOW_OFFS - XOSD_MARGIN * 2 - data->Offset.value);
     QRect rc = p.boundingRect(rcScreen, AlignLeft | AlignTop | WordBreak, str);
+    if (rc.height() >= rcScreen.height() / 2){
+        rcScreen = QRect(0, 0,
+                         rcScreen.width() - SHADOW_OFFS - XOSD_MARGIN * 2 - data->Offset.value,
+                         rcScreen.height() - SHADOW_OFFS - XOSD_MARGIN * 2 - data->Offset.value);
+        rc = p.boundingRect(rcScreen, AlignLeft | AlignTop | WordBreak, str);
+    }
     p.end();
     if (data->EnableMessageShowContent.bValue && data->ContentLines.value){
         QFontMetrics fm(font());
@@ -255,13 +278,30 @@ void OSDWidget::showOSD(const QString &str, OSDUserData *data)
     int y = rcScreen.top();
     int w = rc.width() + 1;
     int h = rc.height() + 1;
-    if (data->Shadow.bValue){
-        w += SHADOW_OFFS;
-        h += SHADOW_OFFS;
-    }
+    int text_y = 0;
     if (data->Background.bValue){
         w += XOSD_MARGIN * 2;
         h += XOSD_MARGIN * 2;
+        if (m_button == NULL){
+            m_button = new QPushButton("", this);
+            m_button->setPixmap(QPixmap((const char **)close_h_xpm));
+            connect(m_button, SIGNAL(clicked()), this, SLOT(slotCloseClick()));
+        }
+        QSize s = m_button->sizeHint();
+        m_button->resize(s.height() - 4, s.height() - 4);
+        m_button->move(w - m_button->width() - 3, 2);
+        text_y = m_button->height() + 4;
+        h += text_y;
+        m_button->show();
+    }else{
+        if (m_button){
+            delete m_button;
+            m_button = NULL;
+        }
+    }
+    if (data->Shadow.bValue){
+        w += SHADOW_OFFS;
+        h += SHADOW_OFFS;
     }
     resize(QSize(w, h));
     switch (data->Position.value){
@@ -338,6 +378,7 @@ void OSDWidget::showOSD(const QString &str, OSDUserData *data)
     }
     p.setFont(font());
     p.setPen(QColor(data->Color.value));
+    rc.setTop(text_y);
     p.drawText(rc, AlignLeft | AlignTop | WordBreak, str);
     p.end();
     bgPict = pict;
@@ -357,11 +398,28 @@ void OSDWidget::mouseDoubleClickEvent(QMouseEvent*)
     emit dblClick();
 }
 
+void OSDWidget::slotCloseClick()
+{
+    emit closeClick();
+}
+
 #if 0
 i18n("male", "%1 is online")
 i18n("female", "%1 is online")
-i18n("male", "%1 typed")
-i18n("female", "%1 typed")
+i18n("male", "%1 is away")
+i18n("female", "%1 is away")
+i18n("male", "%1 is not available")
+i18n("female", "%1 is not available")
+i18n("male", "%1 doesn't want to be disturbed")
+i18n("female", "%1 doesn't want to be disturbed")
+i18n("male", "%1 is occupied")
+i18n("female", "%1 is occupied")
+i18n("male", "%1 is free for chat")
+i18n("female", "%1 is free for chat")
+i18n("male", "%1 is offline")
+i18n("female", "%1 is offline")
+i18n("male", "%1 is typing")
+i18n("female", "%1 is typing")
 #endif
 
 typedef map<unsigned, unsigned> TYPE_MAP;
@@ -381,14 +439,44 @@ void OSDPlugin::processQueue()
         OSDUserData *data = NULL;
         data = (OSDUserData*)contact->getUserData(user_data_id);
         switch (m_request.type){
-        case OSD_ALERT:
-            if (data->EnableAlert.bValue){
+        case OSD_ALERTONLINE:
+            if (data->EnableAlert.bValue && data->EnableAlertOnline.bValue){
                 unsigned style = 0;
                 const char *statusIcon = NULL;
-                if (contact->contactInfo(style, statusIcon) >= STATUS_ONLINE)
+                if (contact->contactInfo(style, statusIcon) == STATUS_ONLINE)
                     text = g_i18n("%1 is online", contact) .arg(contact->getName());
             }
             break;
+        case OSD_ALERTAWAY:
+            if (data->EnableAlert.bValue && data->EnableAlertAway.bValue){
+                text = g_i18n("%1 is away", contact) .arg(contact->getName());
+            }
+            break;
+        case OSD_ALERTNA:
+            if (data->EnableAlert.bValue && data->EnableAlertNA.bValue){
+                text = g_i18n("%1 is not available", contact) .arg(contact->getName());
+            }
+            break;
+        case OSD_ALERTDND:
+            if (data->EnableAlert.bValue && data->EnableAlertDND.bValue){
+                text = g_i18n("%1 doesn't want to be disturbed", contact) .arg(contact->getName());
+            }
+            break;
+        case OSD_ALERTOCCUPIED:
+            if (data->EnableAlert.bValue && data->EnableAlertOccupied.bValue){
+                text = g_i18n("%1 is occupied", contact) .arg(contact->getName());
+            }
+            break;
+        case OSD_ALERTFFC:
+            if (data->EnableAlert.bValue && data->EnableAlertFFC.bValue){
+                text = g_i18n("%1 is free for chat", contact) .arg(contact->getName());
+            }
+            break;
+        case OSD_ALERTOFFLINE:
+            if (data->EnableAlert.bValue && data->EnableAlertOffline.bValue && !(core->getManualStatus() == STATUS_OFFLINE)){
+                text = g_i18n("%1 is offline", contact) .arg(contact->getName());
+            }
+            break;
         case OSD_TYPING:
             if (data->EnableTyping.bValue){
                 unsigned style = 0;
@@ -466,11 +554,15 @@ void OSDPlugin::processQueue()
                 text += ":\n";
                 text += msg_text;
             }
+            break;
+        default:
+            break;
         }
         if (!text.isEmpty()){
             if (m_osd == NULL){
                 m_osd = new OSDWidget;
                 connect(m_osd, SIGNAL(dblClick()), this, SLOT(dblClick()));
+                connect(m_osd, SIGNAL(closeClick()), this, SLOT(closeClick()));
             }
             static_cast<OSDWidget*>(m_osd)->showOSD(text, data);
             m_timer->start(data->Timeout.value * 1000);
@@ -482,6 +574,32 @@ void OSDPlugin::processQueue()
     m_request.type = OSD_NONE;
 }
 
+void OSDPlugin::closeClick()
+{
+    if (m_request.type == OSD_MESSAGE){
+        for (list<msg_id>::iterator it = core->unread.begin(); it != core->unread.end(); ){
+            if ((*it).contact != m_request.contact){
+                ++it;
+                continue;
+            }
+            MessageID id;
+            id.id      = (*it).id;
+            id.contact = (*it).contact;
+            id.client  = (*it).client.c_str();
+            Event e(EventLoadMessage, &id);
+            Message *msg = (Message*)(e.process());
+            core->unread.erase(it);
+            if (msg){
+                Event e(EventMessageRead, msg);
+                e.process();
+                delete msg;
+            }
+            it = core->unread.begin();
+        }
+    }
+    timeout();
+}
+
 void OSDPlugin::dblClick()
 {
     Event e(EventDefaultAction, (void*)(m_request.contact));
@@ -501,7 +619,7 @@ void *OSDPlugin::processEvent(Event *e)
         contact = (Contact*)(e->param());
         if (contact->getIgnore()) break;
         osd.contact = contact->id();
-        osd.type    = OSD_ALERT;
+        osd.type    = OSD_ALERTONLINE;
         queue.push_back(osd);
         processQueue();
         break;
@@ -509,23 +627,54 @@ void *OSDPlugin::processEvent(Event *e)
     case EventMessageRead:
     case EventMessageReceived:
         msg = (Message*)(e->param());
-        if (msg->type() == MessageStatus)
-            break;
         contact = getContacts()->contact(msg->contact());
         if (contact == NULL)
             break;
         data = (OSDUserData*)(contact->getUserData(user_data_id));
         if (data == NULL)
             break;
-        osd.type    = OSD_MESSAGE;
         osd.contact = msg->contact();
-        if ((m_request.type == OSD_MESSAGE) && (m_request.contact == msg->contact())){
-            queue.push_front(osd);
-            m_timer->stop();
-            m_timer->start(100);
-        }else{
+        if (msg->type() == MessageStatus) {
+            StatusMessage *smsg = (StatusMessage*)msg;
+            switch (smsg->getStatus()) {
+            case STATUS_AWAY:
+                osd.type = OSD_ALERTAWAY;
+                break;
+            case STATUS_NA:
+                osd.type = OSD_ALERTNA;
+                break;
+            case STATUS_DND:
+                osd.type = OSD_ALERTDND;
+                break;
+            case 100:    /* STATUS_OCCUPIED, but defined in icqclient.h ! */
+                osd.type = OSD_ALERTOCCUPIED;
+                break;
+            case STATUS_FFC:
+                osd.type = OSD_ALERTFFC;
+                break;
+            case STATUS_OFFLINE:
+                osd.type = OSD_ALERTOFFLINE;
+                break;
+            case STATUS_ONLINE:
+                osd.type = OSD_NONE;
+                return NULL;
+            default:
+                log(L_DEBUG,"OSD: Unknown status %ld",smsg->getStatus());
+                osd.type = OSD_NONE;
+                return NULL;
+            }
             queue.push_back(osd);
             processQueue();
+        }else{
+            osd.type    = OSD_MESSAGE;
+            if ((m_request.type == OSD_MESSAGE) && (m_request.contact == msg->contact())){
+                queue.push_front(osd);
+                m_timer->stop();
+                m_timer->start(100);
+            }else{
+                queue.push_back(osd);
+                processQueue();
+            }
         }
         break;
     case EventContactStatus:
@@ -595,7 +744,7 @@ extern "C" BOOL __stdcall _DllMainCRTStartup( HINSTANCE hinstDLL, DWORD fdwReaso
 
 #endif
 
-#ifndef WIN32
+#ifndef _MSC_VER
 #include "osd.moc"
 #endif
 
