@@ -74,6 +74,7 @@ static DataDef _proxyData[] =
 ProxyData::ProxyData()
 {
     bInit = false;
+    load_data(_proxyData, this, NULL);
 }
 
 ProxyData::ProxyData(const ProxyData &d)
@@ -84,8 +85,17 @@ ProxyData::ProxyData(const ProxyData &d)
 
 ProxyData::ProxyData(const char *cfg)
 {
-    load_data(_proxyData, this, cfg);
-    bInit = true;
+    bInit = false;
+    if (cfg) {
+        Buffer config;
+        config << "[Title]\n" << cfg;
+        config.setWritePos(0);
+        config.getSection();
+        load_data(_proxyData, this, &config);
+        bInit = true;
+    }else{
+        load_data(_proxyData, this, NULL);
+    }
 }
 
 ProxyData::~ProxyData()
@@ -127,14 +137,21 @@ ProxyData& ProxyData::operator = (const ProxyData &d)
         bInit = false;
     }
     if (d.bInit){
-        load_data(_proxyData, this, save_data(_proxyData, (void*)(&d)).c_str());
+        Buffer cfg;
+        cfg << "[Title]\n" << save_data(_proxyData, (void*)(&d)).c_str();
+        cfg.setWritePos(0);
+        cfg.getSection();
+        load_data(_proxyData, this, &cfg);
         bInit = true;
         Default = d.Default;
+    }else{
+        load_data(_proxyData, this, NULL);
     }
+
     return *this;
 }
 
-ProxyData& ProxyData::operator = (const char *cfg)
+ProxyData& ProxyData::operator = (Buffer *cfg)
 {
     if (bInit){
         free_data(_proxyData, this);
@@ -284,7 +301,8 @@ void Proxy::error_state(const char *err, unsigned code)
             if (getNoShow()){
                 code = 0;
             }else{
-                m_client->m_reconnect = NO_RECONNECT;
+                if (m_client != (TCPClient*)(-1))
+                    m_client->m_reconnect = NO_RECONNECT;
             }
         }
         notify->error_state(err, code);
@@ -520,7 +538,7 @@ void SOCKS4_Listener::read_ready()
         m_state = Accept;
         if (notify)
             notify->bind_ready(port);
-        return;
+        break;
     case Accept:
         read(8);
         bIn >> b1 >> b2;
@@ -535,6 +553,9 @@ void SOCKS4_Listener::read_ready()
         }else{
             error_state("Bad state", 0);
         }
+        break;
+    default:
+        break;
     }
 }
 
@@ -865,7 +886,9 @@ void HTTPS_Proxy::connect_ready()
     << ":"
     << number(m_port).c_str()
     << " HTTP/1.0\r\n"
-    << "User-Agent: Mozilla/4.08 [en]] (WinNT; U ;Nav)\r\n";
+    << "User-Agent: "
+    << get_user_agent().c_str()
+    << "\r\n";
     send_auth();
     bOut << "\r\n";
     m_state = WaitConnect;
@@ -944,7 +967,8 @@ bool HTTPS_Proxy::readLine(string &s)
         bIn << c;
     }
     log_packet(bIn, false, m_plugin->ProxyPacket);
-    s.assign(bIn.data(0), bIn.size());
+    if(bIn.size())
+        s.assign(bIn.data(0), bIn.size());
     bIn.init(0);
     bIn.packetStart();
     return true;
@@ -1124,7 +1148,7 @@ void HTTP_Proxy::write(const char *buf, unsigned int size)
 // ______________________________________________________________________________________
 
 
-Plugin *createProxyPlugin(unsigned base, bool, const char *config)
+Plugin *createProxyPlugin(unsigned base, bool, Buffer *config)
 {
     Plugin *plugin = new ProxyPlugin(base, config);
     return plugin;
@@ -1144,7 +1168,7 @@ EXPORT_PROC PluginInfo* GetPluginInfo()
     return &info;
 }
 
-ProxyPlugin::ProxyPlugin(unsigned base, const char *config)
+ProxyPlugin::ProxyPlugin(unsigned base, Buffer *config)
         : Plugin(base)
 {
     data = config;
@@ -1268,8 +1292,10 @@ void *ProxyPlugin::processEvent(Event *e)
         if (data->code == ProxyErr){
             QString msg = i18n(data->err_str);
             if (data->err_str && *data->err_str){
-                if (data->args)
+                if (data->args){
                     msg = msg.arg(QString::fromUtf8(data->args));
+                    free(data->args);
+                }
             }
             ProxyError *err = new ProxyError(this, static_cast<TCPClient*>(data->client), msg);
             raiseWindow(err);
