@@ -24,6 +24,8 @@ extern CComModule _Module;
 #include "remote.h"
 #include "stl.h"
 
+using namespace std;
+
 class IPC
 {
 public:
@@ -136,7 +138,7 @@ bool IPC::process(const BSTR &in_str, BSTR *out_str)
         if (s[i] == SLOT_IN)
             continue;
         if (s[i] == SLOT_OUT){
-            CComBSTR res(mem);
+            CComBSTR res((int)mem);
             *out_str = res.Copy();
         }
         UnmapViewOfFile(mem);
