@@ -31,7 +31,7 @@
 #include <qpixmap.h>
 #include <time.h>
 
-Plugin *createRemotePlugin(unsigned base, bool, const char *config)
+Plugin *createRemotePlugin(unsigned base, bool, Buffer *config)
 {
     Plugin *plugin = new RemotePlugin(base, config);
     return plugin;
@@ -57,7 +57,7 @@ static DataDef remoteData[] =
         { "Path", DATA_STRING, 1, "auto:" },
         { "EnableMenu", DATA_BOOL, 1, DATA(1) },
 #else
-{ "Path", DATA_STRING, 1, "/tmp/sim.%user%" },
+        { "Path", DATA_STRING, 1, "/tmp/sim.%user%" },
 #endif
         { NULL, 0, 0, 0 }
     };
@@ -159,6 +159,12 @@ void IPC::process()
             continue;
         }
         unsigned short *mem = (unsigned short*)MapViewOfFile(hMem, FILE_MAP_ALL_ACCESS, 0, 0, 0);
+        if (mem == NULL){
+            log(L_WARN, "Map error");
+            s[i] = SLOT_NONE;
+            PulseEvent(hEventOut);
+            continue;
+        }
         unsigned short *p;
         for (p = mem; *p; p++)
             in += QChar(*p);
@@ -166,16 +172,20 @@ void IPC::process()
         bool bError = false;
         bool bRes = remote->command(in, out, bError);
         p = mem;
+        unsigned size = 0;
         if (!bError){
             if (bRes){
                 *(p++) = QChar('>').unicode();
             }else{
                 *(p++) = QChar('?').unicode();
             }
-            for (int n = 0; n < (int)(out.length()); n++)
-                *(p++) = out[n].unicode();
+            size = out.length();
+            if (size > 0x3F00)
+                size = 0x3F00;
+            memcpy(p, out.unicode(), size * sizeof(unsigned short));
+            size++;
         }
-        *(p++) = 0;
+        p[size] = 0;
         UnmapViewOfFile(mem);
         CloseHandle(hMem);
         s[i] = SLOT_OUT;
@@ -212,7 +222,7 @@ IPCLock::~IPCLock()
 
 #endif
 
-RemotePlugin::RemotePlugin(unsigned base, const char *config)
+RemotePlugin::RemotePlugin(unsigned base, Buffer *config)
         : Plugin(base)
 {
     load_data(remoteData, &data, config);
@@ -305,8 +315,10 @@ const unsigned CMD_FILE			= 10;
 const unsigned CMD_CONTACTS		= 11;
 const unsigned CMD_SENDFILE		= 12;
 const unsigned CMD_GROUP		= 13;
+const unsigned CMD_SHOW			= 14;
+const unsigned CMD_SMS			= 15;
 #ifdef WIN32
-const unsigned CMD_ICON			= 14;
+const unsigned CMD_ICON			= 16;
 #endif
 
 typedef struct cmdDef
@@ -329,11 +341,13 @@ static cmdDef cmds[] =
         { "HELP", "command help information", "HELP [<cmd>]", 0, 1 },
         { "ADD", "add contact", "ADD <protocol> <address> [<nick>] [<group>]", 2, 4 },
         { "DELETE", "delete contact", "DELETE [<address> | <nick>]", 1, 1 },
-        { "OPEN", "open contact", "ADD <protocol> <address> [<nick>] [<group>]", 2, 4 },
+        { "OPEN", "open contact", "OPEN <protocol> <address> [<nick>] [<group>]", 2, 4 },
         { "FILE", "process UIN file", "FILE <file>", 1, 1 },
         { "CONTACTS", "print contact list", "CONTACTS [<message_type>]", 0, 1 },
         { "SENDFILE", "send file", "SENDFILE <file> <contact>", 2, 2 },
         { "GROUP", "get group name", "GROUP id", 1, 1 },
+        { "SHOW", "open unread message", "SHOW", 0, 0 },
+        { "SMS", "send SMS", "SMS <phone> <message>", 2, 2 },
 #ifdef WIN32
         { "ICON", "get used icon", "ICON name", 1, 1 },
 #endif
@@ -346,7 +360,6 @@ static cmdDef cmds[] =
 { "DOCK", "show/hide dock", "DOCK [on|off]", 0, 1 },
 { "NOTIFY", "set notify mode", "NOTIFY [on|off]", 0, 1 },
 { "ICON", "get icon in xpm format", "ICON nIcon", 1, 1 },
-{ "OPEN", "open unread message", "OPEN", 0, 0 },
 { "POPUP", "show popup", "POPUP x y", 2, 2 },
 #endif
 
@@ -484,7 +497,7 @@ bool RemotePlugin::command(const QString &in, QString &out, bool &bError)
     for (; i < (int)(in.length()); i++){
         if (in[i] == ' ')
             break;
-        cmd += in[i];
+        cmd += in[i].upper();
     }
     for (; i < (int)(in.length()); ){
         for (; i < (int)(in.length()); i++)
@@ -581,7 +594,7 @@ bool RemotePlugin::command(const QString &in, QString &out, bool &bError)
                 vector<ContactInfo> contacts;
                 list<unsigned> groups;
                 while ((contact = ++it) != NULL){
-                    if (contact->getTemporary() || contact->getIgnore())
+                    if ((contact->getFlags() & CONTACT_TEMPORARY) || contact->getIgnore())
                         continue;
                     if (type){
                         Command cmd;
@@ -609,7 +622,7 @@ bool RemotePlugin::command(const QString &in, QString &out, bool &bError)
                                 index = getContacts()->groupIndex(grp->id());
                         }
                         QString grpIndex;
-                        grpIndex.sprintf("%08lX", index);
+                        grpIndex.sprintf("%08X", index);
                         info.key += grpIndex;
                     }
                     for (;;){
@@ -672,7 +685,15 @@ bool RemotePlugin::command(const QString &in, QString &out, bool &bError)
             string line;
             bool bOpen = false;
             unsigned uin = 0;
-            while (getLine(f, line)){
+            Buffer sf;
+            sf.init(f.size());
+            f.readBlock(sf.data(), f.size());
+            sf.setSize(f.size());
+            while (sf.readPos() < sf.size()){
+                string line;
+                sf.scan("\n", line);
+                if (!line.empty() && (line[line.length() - 1] == '\r'))
+                    line = line.substr(0, line.length() - 1);
                 if (line == "[ICQ Message User]")
                     bOpen = true;
                 if (line.substr(0, 4) == "UIN=")
@@ -900,6 +921,27 @@ bool RemotePlugin::command(const QString &in, QString &out, bool &bError)
             out += " not found";
             return false;
         }
+    case CMD_SHOW:{
+            Command cmd;
+            if (core->unread.size())
+                cmd->id = CmdUnread;
+            else return false;
+            Event e(EventCommandExec, cmd);
+            e.process();
+            return true;
+        }
+    case CMD_SMS:{
+            SMSMessage *m = new SMSMessage;
+            m->setPhone(args[0]);
+            m->setText(args[1]);
+            unsigned i;
+            for (i = 0; i < getContacts()->nClients(); i++){
+                Client *client = getContacts()->getClient(i);
+                if (client->send(m, NULL))
+                    return true;
+            }
+            return false;
+        }
     case CMD_HELP:
         if (args.size() == 0){
             for (c = cmds; c->cmd; c++){
@@ -909,6 +951,7 @@ bool RemotePlugin::command(const QString &in, QString &out, bool &bError)
                 out += "\n";
             }
         }else{
+            args[0] = args[0].upper();
             for (c = cmds; c->cmd; c++)
                 if (args[0] == c->cmd)
                     break;
@@ -1028,7 +1071,7 @@ extern "C" BOOL __stdcall _DllMainCRTStartup( HINSTANCE hinstDLL, DWORD fdwReaso
 
 #endif
 
-#ifndef WIN32
+#ifndef _MSC_VER
 #include "remote.moc"
 #endif
 
