@@ -44,6 +44,7 @@
 #ifdef WIN32
 #include <windows.h>
 #define socklen_t	int
+#define in_addr_t int
 #endif
 
 #ifndef INADDR_NONE
@@ -183,7 +184,7 @@ Processor *createTCPProcessor(const char *addr_str)
         }else{
             port = atol(addr.c_str());
         }
-        int ip = INADDR_NONE;
+        in_addr_t ip = INADDR_NONE;
         if (port){
             if (!host.empty()){
                 ip = inet_addr(host.c_str());
@@ -325,7 +326,7 @@ Processor *createProcessor(const char *addr_str)
     }
     int s = socket(PF_UNIX, SOCK_STREAM, 0);
     if (s < 0){
-	fprintf(stderr, "Can't create socket: %s\n", strerror(errno));
+        fprintf(stderr, "Can't create socket: %s\n", strerror(errno));
         return NULL;
     }
 
@@ -340,7 +341,7 @@ Processor *createProcessor(const char *addr_str)
     strcpy(sun_local.sun_path, local_name);
     unlink(local_name);
     if (bind(s, (struct sockaddr*)&sun_local, sizeof(sun_local)) < 0){
-	fprintf(stderr, "Can't bind socket %s: %s\n", local_name, strerror(errno));
+        fprintf(stderr, "Can't bind socket %s: %s\n", local_name, strerror(errno));
         return NULL;
     }
 
@@ -348,7 +349,7 @@ Processor *createProcessor(const char *addr_str)
     sun_remote.sun_family = AF_UNIX;
     strcpy(sun_remote.sun_path, addr.c_str());
     if (connect(s, (struct sockaddr*)&sun_remote, sizeof(sun_remote)) < 0){
-	fprintf(stderr, "Can't connect to %s: %s\n", addr.c_str(), strerror(errno));
+        fprintf(stderr, "Can't connect to %s: %s\n", addr.c_str(), strerror(errno));
         unlink(local_name);
         return NULL;
     }
@@ -378,8 +379,9 @@ static void usage(char *s)
 #ifndef WIN32
             "      -s socket	[/tmp/sim.%s] Control socket\n"
 #endif
-            "      -d                         Debug mode\n"
-	    "      -c command                 Command\n"
+            //            "      -d                         Debug mode\n"
+            "      -c command                 Command\n"
+            "      -h                         Show this help\n"
             "\n",
             s
 #ifndef WIN32
@@ -388,13 +390,17 @@ static void usage(char *s)
            );
 }
 
-int main(int argc, char **argv)
+int main(int, char **argv)
 {
     list<string> uins;
     const char *cmd  = NULL;
     const char *addr = NULL;
     for (char **p = argv + 1; *p; p++){
-        if (!strcmp(*p, "-d")){
+        if (!strcmp(*p, "-h")){
+            usage(argv[0]);
+            return 0;
+        }
+        if (!strcmp(*p, "-s")){
             p++;
             if (*p == NULL){
                 usage(argv[0]);
@@ -406,11 +412,11 @@ int main(int argc, char **argv)
         if (!strcmp(*p, "-c")){
             p++;
             if (*p == NULL){
-	 	usage(argv[0]);
-		return 1;
-	    }
+                usage(argv[0]);
+                return 1;
+            }
             cmd = *p;
-	    continue;
+            continue;
         }
         uins.push_back(*p);
     }
@@ -432,24 +438,25 @@ int main(int argc, char **argv)
         return 0;
     }
     if (cmd){
-	string out_str;
+        string out_str;
         if (!processor->process(cmd, out_str)){
-		fprintf(stderr, "Can't execute %s\n", cmd);
-		exit(1);
+            fprintf(stderr, "Can't execute %s\n", cmd);
+            exit(1);
         }
-	if (out_str.empty()){
-		fprintf(stderr, "No answer\n");
-		exit(1);
-	}
-	if (out_str[0] != '>'){
-		fprintf(stderr, "Execute %s fail\n", cmd);
-		exit(1);
-	}
-	printf("%s\n", out_str.c_str() + 1);
-	exit(0);
+        if (out_str.empty()){
+            fprintf(stderr, "No answer\n");
+            exit(1);
+        }
+        if (out_str[0] != '>'){
+            fprintf(stderr, "Execute %s fail\n", cmd);
+            exit(1);
+        }
+        printf("%s\n", out_str.c_str() + 1);
+        exit(0);
     }
     FILE *f = stdin;
     while (!feof(f) && !ferror(f)){
+        printf(">");
         char buf[4096];
         char *line = fgets(buf, sizeof(buf), f);
         if (line == NULL)
@@ -467,7 +474,7 @@ int main(int argc, char **argv)
         out_str = out_str.substr(1);
         if (bBad)
             printf("? ");
-        printf("%s\n>", out_str.c_str());
+        printf("%s\n", out_str.c_str());
     }
     return 0;
 }
