@@ -39,7 +39,7 @@
 #endif
 #endif
 
-Plugin *createShortcutsPlugin(unsigned base, bool, const char *config)
+Plugin *createShortcutsPlugin(unsigned base, bool, Buffer *config)
 {
     Plugin *plugin = new ShortcutsPlugin(base, config);
     return plugin;
@@ -420,7 +420,7 @@ static int X11EventFilter(XEvent *e)
 #endif
 #endif
 
-ShortcutsPlugin::ShortcutsPlugin(unsigned base, const char *config)
+ShortcutsPlugin::ShortcutsPlugin(unsigned base, Buffer *config)
         : Plugin(base)
 {
     load_data(shortcutsData, &data, config);
@@ -503,7 +503,7 @@ void *ShortcutsPlugin::processEvent(Event *e)
         }
     }
     if (e->type() == EventCommandRemove){
-        unsigned id = (unsigned)(e->param());
+        unsigned long id = (unsigned long)(e->param());
         MAP_STR::iterator it_key = oldKeys.find(id);
         if (it_key != oldKeys.end())
             oldKeys.erase(it_key);
@@ -580,7 +580,7 @@ void ShortcutsPlugin::releaseKeys()
     qApp->removeEventFilter(this);
 }
 
-void ShortcutsPlugin::applyKeys(unsigned id)
+void ShortcutsPlugin::applyKeys(unsigned long id)
 {
     Event eDef(EventGetMenuDef, (void*)id);
     CommandsDef *def = (CommandsDef*)(eDef.process());
@@ -695,7 +695,7 @@ void ShortcutsPlugin::applyKey(CommandDef *s)
     }
 }
 
-void ShortcutsPlugin::releaseKeys(unsigned id)
+void ShortcutsPlugin::releaseKeys(unsigned long id)
 {
     Event eDef(EventGetMenuDef, (void*)id);
     CommandsDef *def = (CommandsDef*)(eDef.process());
@@ -807,7 +807,7 @@ extern "C" BOOL __stdcall _DllMainCRTStartup( HINSTANCE hinstDLL, DWORD fdwReaso
 
 #endif
 
-#ifndef WIN32
+#ifndef _MSC_VER
 #include "shortcuts.moc"
 #endif
 
