@@ -19,6 +19,7 @@
 #define _SOUND_H
 
 #include "simapi.h"
+#include "stl.h"
 
 typedef struct SoundData
 {
@@ -40,11 +41,15 @@ typedef struct SoundUserData
 } SoundUserData;
 
 class CorePlugin;
+class QTimer;
+class QSound;
+class Exec;
 
-class SoundPlugin : public Plugin, public EventReceiver
+class SoundPlugin : public QObject, public Plugin, public EventReceiver
 {
+    Q_OBJECT
 public:
-    SoundPlugin(unsigned, bool, const char*);
+    SoundPlugin(unsigned, bool, Buffer*);
     virtual ~SoundPlugin();
 #ifdef USE_KDE
     PROP_BOOL(UseArts);
@@ -53,16 +58,27 @@ public:
     PROP_STR(StartUp);
     PROP_STR(FileDone);
     PROP_STR(MessageSent);
-    unsigned CmdSoundDisable;
+    unsigned long CmdSoundDisable;
     unsigned EventSoundChanged;
+protected slots:
+    void checkSound();
+    void childExited(int, int);
 protected:
-    unsigned user_data_id;
+    unsigned long user_data_id;
     virtual void *processEvent(Event*);
     virtual string getConfig();
     virtual QWidget *createConfigWindow(QWidget *parent);
     string fullName(const char*);
     string messageSound(unsigned type, SoundUserData *data);
     void playSound(const char*);
+    void processQueue();
+    string			m_current;
+    list<string>	m_queue;
+    QSound			*m_sound;
+    QTimer			*m_checkTimer;
+#ifndef WIN32
+    long             m_player;
+#endif
     SoundData	data;
     CorePlugin	*core;
     bool	    m_bChanged;
