@@ -31,20 +31,26 @@ SoundConfig::SoundConfig(QWidget *parent, SoundPlugin *plugin)
 {
     m_plugin = plugin;
     user_cfg = NULL;
-    bool bSound = QSound::available();
 #ifdef USE_KDE
-    bSound = false;
+    bool bSound = false;
     connect(chkArts, SIGNAL(toggled(bool)), this, SLOT(artsToggled(bool)));
     chkArts->setChecked(plugin->getUseArts());
 #else
     chkArts->hide();
+#ifndef WIN32
+    bool bSound = QSound::available();
 #endif
+#endif
+
+#ifdef WIN32
+    lblPlayer->hide();
+    edtPlayer->hide();
+#else
     if (bSound){
-        lblPlayer->hide();
-        edtPlayer->hide();
-    }else{
-        edtPlayer->setText(QString::fromLocal8Bit(plugin->getPlayer()));
+        lblPlayer->setText(i18n("Qt provides sound output so you just need to set a player if you don't like Qt's sound."));
     }
+    edtPlayer->setText(QString::fromLocal8Bit(plugin->getPlayer()));
+#endif
     string s;
     s = plugin->fullName(plugin->getStartUp());
     edtStartup->setText(QFile::decodeName(s.c_str()));
@@ -77,12 +83,18 @@ void SoundConfig::apply()
         void *data = getContacts()->getUserData(m_plugin->user_data_id);
         user_cfg->apply(data);
     }
-    bool bSound = QSound::available();
 #ifdef USE_KDE
     m_plugin->setUseArts(chkArts->isChecked());
-    bSound = false;
+    bool bSound = false;
+#else
+    /* If there is an external player selected, don't use Qt
+    Check first for edtPlayer->text().isEmpty() since QSound::available()
+    can take 5 seconds to return a value */
+    bool bSound = edtPlayer->text().isEmpty() && QSound::available();
 #endif
-    if (!bSound)
+    if (bSound)
+        m_plugin->setPlayer("");
+    else
         m_plugin->setPlayer(edtPlayer->text().local8Bit());
     m_plugin->setStartUp(QFile::encodeName(sound(edtStartup->text(), "startup.wav")));
     m_plugin->setFileDone(QFile::encodeName(sound(edtFileDone->text(), "startup.wav")));
@@ -103,7 +115,7 @@ void SoundConfig::artsToggled(bool)
     edtPlayer->setEnabled(!chkArts->isChecked());
 }
 
-#ifndef WIN32
+#ifndef _MSC_VER
 #include "soundconfig.moc"
 #endif
 
