@@ -27,6 +27,9 @@
 
 #include <windows.h>
 
+using namespace std;
+using namespace SIM;
+
 SpellFind::SpellFind(EditFile *edt)
         : SpellFindBase(NULL, NULL, false, WDestructiveClose)
 {
@@ -110,6 +113,6 @@ bool SpellFind::checkPath()
     return false;
 }
 
-#ifndef WIN32
+#ifndef _MSC_VER
 #include "spellfind.moc"
 #endif
