@@ -20,56 +20,7 @@
 #include "buffer.h"
 #include "socket.h"
 #include "toolbtn.h"
-
-#include "xpm/0.xpm"
-#include "xpm/1.xpm"
-#include "xpm/2.xpm"
-#include "xpm/3.xpm"
-#include "xpm/4.xpm"
-#include "xpm/5.xpm"
-#include "xpm/6.xpm"
-#include "xpm/7.xpm"
-#include "xpm/8.xpm"
-#include "xpm/9.xpm"
-#include "xpm/10.xpm"
-#include "xpm/11.xpm"
-#include "xpm/12.xpm"
-#include "xpm/13.xpm"
-#include "xpm/14.xpm"
-#include "xpm/15.xpm"
-#include "xpm/16.xpm"
-#include "xpm/17.xpm"
-#include "xpm/18.xpm"
-#include "xpm/19.xpm"
-#include "xpm/20.xpm"
-#include "xpm/21.xpm"
-#include "xpm/22.xpm"
-#include "xpm/23.xpm"
-#include "xpm/24.xpm"
-#include "xpm/25.xpm"
-#include "xpm/26.xpm"
-#include "xpm/27.xpm"
-#include "xpm/28.xpm"
-#include "xpm/29.xpm"
-#include "xpm/30.xpm"
-#include "xpm/31.xpm"
-#include "xpm/32.xpm"
-#include "xpm/33.xpm"
-#include "xpm/34.xpm"
-#include "xpm/35.xpm"
-#include "xpm/36.xpm"
-#include "xpm/37.xpm"
-#include "xpm/38.xpm"
-#include "xpm/39.xpm"
-#include "xpm/40.xpm"
-#include "xpm/41.xpm"
-#include "xpm/42.xpm"
-#include "xpm/43.xpm"
-#include "xpm/44.xpm"
-#include "xpm/45.xpm"
-#include "xpm/46.xpm"
-#include "xpm/47.xpm"
-#include "xpm/na.xpm"
+#include "icons.h"
 
 #include <time.h>
 
@@ -85,7 +36,7 @@
 const unsigned CHECK1_INTERVAL = 30 * 60;
 const unsigned CHECK2_INTERVAL = 120 * 60;
 
-Plugin *createWeatherPlugin(unsigned base, bool bInit, const char *config)
+Plugin *createWeatherPlugin(unsigned base, bool bInit, Buffer *config)
 {
     Plugin *plugin = new WeatherPlugin(base, bInit, config);
     return plugin;
@@ -109,9 +60,9 @@ static DataDef weatherData[] =
     {
         { "ID", DATA_STRING, 1, 0 },
         { "Location", DATA_STRING, 1, 0 },
-        { "Time", DATA_LONG, 1, 0 },
-        { "ForecastTime", DATA_LONG, 1, 0 },
-        { "Forecast", DATA_LONG, 1, DATA(2) },
+        { "Time", DATA_ULONG, 1, 0 },
+        { "ForecastTime", DATA_ULONG, 1, 0 },
+        { "Forecast", DATA_ULONG, 1, DATA(2) },
         { "Text", DATA_UTF, 1, 0 },
         { "Tip", DATA_UTF, 1, 0 },
         { "ForecastTip", DATA_UTF, 1, 0 },
@@ -121,6 +72,7 @@ static DataDef weatherData[] =
         { "Temperature", DATA_LONG, 1, 0 },
         { "FeelsLike", DATA_LONG, 1, 0 },
         { "DewPoint", DATA_LONG, 1, 0 },
+        { "Precipitance", DATA_LONG, 1, 0 },
         { "Humidity", DATA_LONG, 1, 0 },
         { "Pressure", DATA_LONG, 1, 0 },
         { "PressureD", DATA_STRING, 1, 0 },
@@ -131,7 +83,7 @@ static DataDef weatherData[] =
         { "Visibiliy", DATA_STRING, 1, 0 },
         { "Sun_raise", DATA_STRING, 1, 0 },
         { "Sun_set", DATA_STRING, 1, 0 },
-        { "Icon", DATA_LONG, 1, 0 },
+        { "Icon", DATA_ULONG, 1, 0 },
         { "UT", DATA_STRING, 1, 0 },
         { "US", DATA_STRING, 1, 0 },
         { "UP", DATA_STRING, 1, 0 },
@@ -142,27 +94,23 @@ static DataDef weatherData[] =
         { "MaxT", DATA_STRLIST, 1, 0 },
         { "DayIcon", DATA_STRLIST, 1, 0 },
         { "DayConditions", DATA_STRLIST, 1, 0 },
+		{ "UV_Intensity", DATA_LONG, 1, 0 },
+		{ "UV_Description", DATA_STRING, 1, 0 },
+		{ "MoonIcon", DATA_LONG, 1, 0 },
+		{ "MoonPhase", DATA_STRING, 1, 0 },
         { NULL, 0, 0, 0 }
     };
 
-WeatherPlugin::WeatherPlugin(unsigned base, bool bInit, const char *config)
+WeatherPlugin::WeatherPlugin(unsigned base, bool bInit, Buffer *config)
         : Plugin(base)
 {
     load_data(weatherData, &data, config);
-    memset(&m_handler, 0, sizeof(m_handler));
-    m_handler.startElement = p_element_start;
-    m_handler.endElement   = p_element_end;
-    m_handler.characters   = p_char_data;
     BarWeather = registerType();
     CmdWeather = registerType();
     EventWeather = registerType();
+    m_icons = getIcons()->addIconSet("icons/weather.jisp", true);
     Event eBar(EventToolbarCreate, (void*)BarWeather);
     eBar.process();
-    IconDef icon;
-    icon.name = "weather";
-    icon.xpm  = na;
-    Event eIcon(EventAddIcon, &icon);
-    eIcon.process();
     Command cmd;
     cmd->id = CmdWeather;
     cmd->text = I18N_NOOP("Not connected");
@@ -187,6 +135,7 @@ WeatherPlugin::~WeatherPlugin()
     Event eCmd(EventCommandRemove, (void*)CmdWeather);
     Event eBar(EventToolbarRemove, (void*)BarWeather);
     free_data(weatherData, &data);
+    getIcons()->removeIconSet(m_icons);
 }
 
 string WeatherPlugin::getConfig()
@@ -247,13 +196,12 @@ bool WeatherPlugin::done(unsigned code, Buffer &data, const char*)
     m_bWind = false;
     m_bUv	= false;
     m_bCC	= false;
-    m_context = xmlCreatePushParserCtxt(&m_handler, this, "", 0, "");
-    if (xmlParseChunk(m_context, data.data(), data.size(), 0)){
+    m_bMoon	= false;
+    reset();
+    if (!parse(data.data(), data.size(), false)){
         log(L_WARN, "XML parse error");
-        xmlFreeParserCtxt(m_context);
         return false;
     }
-    xmlFreeParserCtxt(m_context);
     time_t now;
     time(&now);
     setTime(now);
@@ -283,8 +231,36 @@ bool WeatherPlugin::parseTime(const char *str, int &h, int &m)
     string s = str;
     h = atol(getToken(s, ':').c_str());
     m = atol(getToken(s, ' ').c_str());
-    if (getToken(s, ' ') == "PM")
+    if ((getToken(s, ' ') == "PM") && (h < 12))
         h += 12;
+    if (h == 24)
+        h = 0;
+    return true;
+}
+
+bool WeatherPlugin::parseDateTime(const char *str, QDateTime &dt)
+{
+    int h, m, D, M, Y;
+    string daytime;
+
+    string s = str;
+    /* MM/DD/YY/ hh:mm */
+    M = atol(getToken(s, '/').c_str());
+    D = atol(getToken(s, '/').c_str());
+    Y = atol(getToken(s, ' ').c_str());
+    h = atol(getToken(s, ':').c_str());
+    m = atol(getToken(s, ' ').c_str());
+
+    if (getToken(s, ' ') == "PM"  && (h < 12))
+        h += 12;
+    /* 12:20 PM is 00:20 and 12:30 AM is 12:20
+       but what date is 12:20 pm 4/7/04? */
+    if (h == 24)
+        h = 0;
+    if (Y < 70)
+        Y += 2000;
+    dt.setDate(QDate(Y,M,D));
+    dt.setTime(QTime(h,m,0,0));
     return true;
 }
 
@@ -335,70 +311,24 @@ void WeatherPlugin::showBar()
     updateButton();
 }
 
-static const char **xpms[] =
-    {
-        xpm_0,
-        xpm_1,
-        xpm_2,
-        xpm_3,
-        xpm_4,
-        xpm_5,
-        xpm_6,
-        xpm_7,
-        xpm_8,
-        xpm_9,
-        xpm_10,
-        xpm_11,
-        xpm_12,
-        xpm_13,
-        xpm_14,
-        xpm_15,
-        xpm_16,
-        xpm_17,
-        xpm_18,
-        xpm_19,
-        xpm_20,
-        xpm_21,
-        xpm_22,
-        xpm_23,
-        xpm_24,
-        xpm_25,
-        xpm_26,
-        xpm_27,
-        xpm_28,
-        xpm_29,
-        xpm_30,
-        xpm_31,
-        xpm_32,
-        xpm_33,
-        xpm_34,
-        xpm_35,
-        xpm_36,
-        xpm_37,
-        xpm_38,
-        xpm_39,
-        xpm_40,
-        xpm_41,
-        xpm_42,
-        xpm_43,
-        xpm_44,
-        xpm_45,
-        xpm_46,
-        xpm_47
-    };
+static string weather_icon;
 
 void WeatherPlugin::updateButton()
 {
     if ((getTime() == 0) || (m_bar == NULL))
         return;
-    const char **xpm = xpms[getIcon()];
-    if (xpm){
-        IconDef icon;
-        icon.name = "weather";
-        icon.xpm  = xpm;
-        Event eIcon(EventAddIcon, &icon);
-        eIcon.process();
-    }
+    weather_icon = "weather";
+    weather_icon += number(getIcon());
+    Command cmd;
+    cmd->id      = CmdWeather;
+    cmd->text    = I18N_NOOP("Not connected");
+    cmd->icon    = weather_icon.c_str();
+    cmd->bar_id  = BarWeather;
+    cmd->bar_grp = 0x1000;
+    cmd->flags   = BTN_PICT | BTN_DIV;
+    Event eCmd(EventCommandChange, cmd);
+    eCmd.process();
+
     QString text = unquoteText(getButtonText());
     QString tip = getTipText();
     QString ftip = getForecastText();
@@ -411,16 +341,6 @@ void WeatherPlugin::updateButton()
         n = getForecast();
     for (m_day = 1; m_day <= getForecast(); m_day++){
         tip += forecastReplace(ftip);
-        const char **xpm = xpms[atol(getDayIcon(m_day))];
-        if (xpm){
-            string url = "weather";
-            url += number(m_day);
-            IconDef icon;
-            icon.name = url.c_str();
-            icon.xpm  = xpm;
-            Event eIcon(EventAddIcon, &icon);
-            eIcon.process();
-        }
         if (--n == 0){
             tip += "</td><td>";
             n = (getForecast() + 1) / 2;
@@ -430,10 +350,10 @@ void WeatherPlugin::updateButton()
         tip += "</td></tr></table>";
     tip += "<br>\nWeather data provided by weather.com";
     tip += QChar((unsigned short)174);
-    Command cmd;
-    cmd->id		= CmdWeather;
-    cmd->param	= m_bar;
-    Event e(EventCommandWidget, cmd);
+    Command cmdw;
+    cmdw->id	= CmdWeather;
+    cmdw->param	= m_bar;
+    Event e(EventCommandWidget, cmdw);
     CToolButton *btn = (CToolButton*)e.process();
     if (btn == NULL)
         return;
@@ -444,9 +364,7 @@ void WeatherPlugin::updateButton()
 
 static QString number(unsigned long n)
 {
-    char b[32];
-    sprintf(b, "%li", n);
-    return b;
+	return QString("%1").arg(n);
 }
 
 #if 0
@@ -458,10 +376,13 @@ i18n("Friday")
 i18n("Saturday")
 i18n("Sunday")
 
+i18n("weather", "Squalls")
 i18n("weather", "Overcast")
 i18n("weather", "Fog")
+i18n("weather", "Mist")
 i18n("weather", "Haze")
 i18n("weather", "Storm")
+i18n("weather", "T-Storm")
 i18n("weather", "Rain")
 i18n("weather", "Snow")
 i18n("weather", "Light Rain")
@@ -477,8 +398,13 @@ i18n("weather", "Cloudy")
 i18n("weather", "Mostly Cloudy")
 i18n("weather", "Partly Cloudy")
 i18n("weather", "Wind")
+i18n("weather", "Windy")
 i18n("weather", "Drizzle")
+i18n("weather", "Freezing Rain")
 i18n("weather", "Light Drizzle")
+i18n("weather", "Drifting Snow")
+i18n("weather", "Scattered")
+i18n("weather", "Smoke")
 i18n("weather", "steady")
 i18n("weather", "rising")
 i18n("weather", "falling")
@@ -499,8 +425,17 @@ i18n("weather", "E")
 i18n("weather", "ENE")
 i18n("weather", "NE")
 i18n("weather", "NNE")
+i18n("weather", "VAR")
 i18n("km")
 i18n("km/h")
+i18n("weather", "Low")
+i18n("weather", "Moderate")
+i18n("weather", "High")
+i18n("weather", "Showers in the Vicinity")
+i18n("weather", "Waning Crescent")
+i18n("weather", "Waxing Crescent")
+i18n("weather", "Waxing Gibbous")
+i18n("weather", "Waning Gibbous")
 #endif
 
 static QString i18n_conditions(const QString &str)
@@ -528,24 +463,55 @@ static QString i18n_conditions(const QString &str)
 QString WeatherPlugin::replace(const QString &text)
 {
     QString res = text;
-    res = res.replace(QRegExp("\\%t"), number(getTemperature()) + QChar((unsigned short)176) + getUT());
-    res = res.replace(QRegExp("\\%f"), number(getFeelsLike()) + QChar((unsigned short)176) + getUT());
-    res = res.replace(QRegExp("\\%d"), number(getDewPoint()) + QChar((unsigned short)176) + getUT());
+    QString sun_set, sun_raise, updated;
+#if COMPAT_QT_VERSION >= 0x030000
+    QTime tmp_time;
+    QDateTime dt;
+    int h,m;
+
+    parseTime(getSun_set(),h,m);
+    tmp_time.setHMS(h,m,0,0);
+    sun_set = tmp_time.toString(Qt::LocalDate);
+    sun_set = sun_set.left(sun_set.length() - 3);
+
+    parseTime(getSun_raise(),h,m);
+    tmp_time.setHMS(h,m,0,0);
+    sun_raise = tmp_time.toString(Qt::LocalDate);
+    sun_raise = sun_raise.left(sun_raise.length() - 3);
+
+    parseDateTime(getUpdated(),dt);
+    updated = dt.toString(Qt::LocalDate);
+    updated = updated.left(updated.length() - 3);
+#else
+    sun_set = getSun_set();
+    sun_raise = getSun_raise();
+    updated = getUpdated();
+#endif
+    /* double Expressions *before* single or better RegExp ! */
+    res = res.replace(QRegExp("\\%mp"), i18n("weather", getMoonPhase()));
+    res = res.replace(QRegExp("\\%mi"), number(getMoonIcon()));
+    res = res.replace(QRegExp("\\%pp"), number(getPrecipitance()));
+	res = res.replace(QRegExp("\\%ut"), i18n("weather", getUV_Description()));
+	res = res.replace(QRegExp("\\%ui"), number(getUV_Intensity()));
+    res = res.replace(QRegExp("\\%t"), QString::number((int)getTemperature()) + QChar((unsigned short)176) + getUT());
+    res = res.replace(QRegExp("\\%f"), QString::number((int)getFeelsLike()) + QChar((unsigned short)176) + getUT());
+    res = res.replace(QRegExp("\\%d"), QString::number((int)getDewPoint()) + QChar((unsigned short)176) + getUT());
     res = res.replace(QRegExp("\\%h"), number(getHumidity()) + "%");
     res = res.replace(QRegExp("\\%w"), number(getWind_speed()) + " " + i18n(getUS()));
     res = res.replace(QRegExp("\\%x"), QString::number(getWind_speed() * 10 / 36) + " " + i18n("m/s"));
-    res = res.replace(QRegExp("\\%g"), getWindGust() ? QString("(") + i18n("gust ") + number(getWindGust()) + i18n(getUS()) + ")" : "");
-    res = res.replace(QRegExp("\\%y"), getWindGust() ? QString("(") + i18n("gust ") + number(getWindGust() * 10 / 36) + " " + i18n("m/s") + ")" : "");
+    res = res.replace(QRegExp("\\%g"), getWindGust() ? QString("(") + i18n("gust ") + number(getWindGust()) + i18n(getUS()) + QString(")") : QString(""));
+    res = res.replace(QRegExp("\\%y"), getWindGust() ? QString("(") + i18n("gust ") + number(getWindGust() * 10 / 36) + QString(" ") + i18n("m/s") + QString(")") : QString(""));
     res = res.replace(QRegExp("\\%p"), number(getPressure()) + " " + i18n(getUP()));
     res = res.replace(QRegExp("\\%a"), number(getPressure() * 75 / 100));
     res = res.replace(QRegExp("\\%q"), i18n("weather", getPressureD()));
     res = res.replace(QRegExp("\\%l"), getLocation());
     res = res.replace(QRegExp("\\%b"), i18n("weather", getWind()));
-    res = res.replace(QRegExp("\\%u"), getUpdated());
-    res = res.replace(QRegExp("\\%r"), getSun_raise());
-    res = res.replace(QRegExp("\\%s"), getSun_set());
+    res = res.replace(QRegExp("\\%u"), updated);
+    res = res.replace(QRegExp("\\%r"), sun_raise);
+    res = res.replace(QRegExp("\\%s"), sun_set);
     res = res.replace(QRegExp("\\%c"), i18n_conditions(getConditions()));
-    res = res.replace(QRegExp("\\%v"), i18n("weather", getVisibility()) + (atol(getVisibility()) ? QString(" ") + i18n(getUD()) : ""));
+    res = res.replace(QRegExp("\\%v"), i18n("weather", getVisibility()) + (atol(getVisibility()) ? QString(" ") + i18n(getUD()) : QString("")));
+    res = res.replace(QRegExp("\\%i"), number(getIcon()));
     return res;
 }
 
@@ -557,35 +523,21 @@ QString WeatherPlugin::forecastReplace(const QString &text)
     QString temp;
     int minT = atol(getMinT(m_day));
     int maxT = atol(getMaxT(m_day));
-    if ((minT < 0) && (maxT < 0)){
-        int r = minT;
-        minT = maxT;
-        maxT = r;
-    }
-    if (minT < 0){
-        temp += "-";
-        minT = -minT;
-    }else if (minT > 0){
-        temp += "+";
-    }
-    temp += number((unsigned)minT).c_str();
+    temp += QString::number(minT);
     temp += QChar((unsigned short)176);
     temp += getUT();
-    if (maxT < 0){
+    if ((strcmp(getMaxT(m_day), "N/A")) && (maxT != -255)) {
         temp += "-";
-        maxT = -maxT;
-    }else if (maxT >= 0){
-        temp += "+";
+		temp += QString::number(maxT);
+        temp += QChar((unsigned short)176);
+        temp += getUT();
     }
-    temp += number((unsigned)maxT).c_str();
-    temp += QChar((unsigned short)176);
-    temp += getUT();
     string dd = getDay(m_day);
     string mon = getToken(dd, ' ');
     QString day = dd.c_str();
-    day += " ";
+    day += ". ";
     day += i18n(mon.c_str());
-    res = res.replace(QRegExp("\\%n"), number(m_day).c_str());
+    res = res.replace(QRegExp("\\%n"), getDayIcon(m_day));
     res = res.replace(QRegExp("\\%t"), temp);
     res = res.replace(QRegExp("\\%c"), i18n_conditions(getDayConditions(m_day)));
     res = res.replace(QRegExp("\\%w"), i18n(getWDay(m_day)));
@@ -607,15 +559,18 @@ QString WeatherPlugin::getTipText()
     QString str = getTip();
     if (str.isEmpty())
         str = i18n("%l<br><br>\n"
-                   "<img src=\"icon:weather\"> %c<br>\n"
+                   "<img src=\"icon:weather%i\"> %c<br>\n"
                    "Temperature: <b>%t</b> (feels like: <b>%f</b>)<br>\n"
                    "Humidity: <b>%h</b><br>\n"
+                   "Precipitance: <b>%pp %</b><br>\n"
                    "Pressure: <b>%p</b> (%q)<br>\n"
                    "Wind: <b>%b</b> <b>%w %g</b><br>\n"
                    "Visibility: <b>%v</b><br>\n"
                    "Dew Point: <b>%d</b><br>\n"
                    "Sunrise: %r<br>\n"
                    "Sunset: %s<br>\n"
+                   "<img src=\"icon:moon%mi\"> %mp<br>\n"
+				   "UV-Intensity is <b>%ut</b> with value <b>%ui</b> (of 11)<br>\n"
                    "<br>\n"
                    "Updated: %u<br>\n");
     return str;
@@ -645,6 +600,7 @@ static const char *tags[] =
         "suns",
         "tmp",
         "flik",
+        "ppcp",
         "hmid",
         "t",
         "icon",
@@ -652,12 +608,13 @@ static const char *tags[] =
         "s",
         "d",
         "ut",
+		"i",
         "us",
         "up",
         "ud",
         "gust",
         "vis",
-        "devp",
+        "dewp",
         "hi",
         "low",
         NULL,
@@ -678,8 +635,12 @@ void WeatherPlugin::element_start(const char *el, const char **attr)
         m_bWind = true;
         return;
     }
-    if (!strcmp(el, "uv")){
-        m_bUv = true;
+    if (!strcmp(el, "uv")) {
+		m_bUv = true;
+        return;
+    }
+    if (!strcmp(el, "moon")) {
+        m_bMoon = true;
         return;
     }
     if (!strcmp(el, "day")){
@@ -689,7 +650,7 @@ void WeatherPlugin::element_start(const char *el, const char **attr)
             string key = *(p++);
             string value = *(p++);
             if (key == "d"){
-                m_day = atol(value.c_str());
+                m_day = strtol(value.c_str(),NULL,10);
                 continue;
             }
             if (key == "dt"){
@@ -721,6 +682,11 @@ void WeatherPlugin::element_start(const char *el, const char **attr)
 
 void WeatherPlugin::element_end(const char *el)
 {
+    if (!strcmp(el, "day")){
+        if ((*getMinT(m_day) == 0) || (*getMaxT(m_day) == 0))
+            m_day--;
+        return;
+    }
     if (!strcmp(el, "obst")){
         setLocation(m_data.c_str());
         m_data = "";
@@ -756,28 +722,37 @@ void WeatherPlugin::element_end(const char *el)
         m_data = "";
         return;
     }
-    if (!strcmp(el, "devp") && m_bCC){
+    if (!strcmp(el, "dewp") && m_bCC){
         setDewPoint(atol(m_data.c_str()));
         m_data = "";
         return;
     }
+    if (!strcmp(el, "ppcp") && m_bCC){
+        setPrecipitance(atol(m_data.c_str()));
+        m_data = "";
+        return;
+    }
     if (!strcmp(el, "hmid") && m_bCC){
         setHumidity(atol(m_data.c_str()));
         m_data = "";
         return;
     }
     if (!strcmp(el, "low") && m_day){
+        if (m_data == "N/A")
+            m_data = "";
         setMinT(m_day, m_data.c_str());
         m_data = "";
         return;
     }
     if (!strcmp(el, "hi") && m_day){
+        if (m_data == "N/A")
+            m_data = "-255";
         setMaxT(m_day, m_data.c_str());
         m_data = "";
         return;
     }
     if (!strcmp(el, "t")){
-        if (!m_bBar && !m_bWind && !m_bUv){
+        if (!m_bBar && !m_bWind && !m_bUv && !m_bMoon){
             if (m_bCC){
                 setConditions(m_data.c_str());
             }else{
@@ -786,12 +761,25 @@ void WeatherPlugin::element_end(const char *el)
         }
         if (m_bWind && m_bCC)
             setWind(m_data.c_str());
+        if (m_bUv && m_bCC)
+            setUV_Description(m_data.c_str());
+        if (m_bMoon && m_bCC)
+            setMoonPhase(m_data.c_str());
+
+        m_data = "";
+        return;
+    }
+    if (!strcmp(el, "i")) {
+        if (m_bUv && m_bCC)
+            setUV_Intensity(strtol(m_data.c_str(),NULL,10));
         m_data = "";
         return;
     }
     if (!strcmp(el, "icon")){
-        if (m_bCC){
-            setIcon(atol(m_data.c_str()));
+        if (m_bMoon && m_bCC) {
+            setMoonIcon(atol(m_data.c_str()));
+        } else if (m_bCC){
+            setIcon(atoul(m_data.c_str()));
         }else{
             setDayIcon(m_day, m_data.c_str());
         }
@@ -860,6 +848,10 @@ void WeatherPlugin::element_end(const char *el)
         m_bUv = false;
         return;
     }
+    if (!strcmp(el, "moon")){
+        m_bMoon = false;
+        return;
+    }
 }
 
 void WeatherPlugin::char_data(const char *str, int len)
@@ -868,21 +860,6 @@ void WeatherPlugin::char_data(const char *str, int len)
         m_data.append(str, len);
 }
 
-void WeatherPlugin::p_element_start(void *data, const xmlChar *el, const xmlChar **attr)
-{
-    ((WeatherPlugin*)data)->element_start((char*)el, (const char**)attr);
-}
-
-void WeatherPlugin::p_element_end(void *data, const xmlChar *el)
-{
-    ((WeatherPlugin*)data)->element_end((char*)el);
-}
-
-void WeatherPlugin::p_char_data(void *data, const xmlChar *str, int len)
-{
-    ((WeatherPlugin*)data)->char_data((char*)str, len);
-}
-
 #ifdef WIN32
 #include <windows.h>
 
@@ -904,7 +881,7 @@ extern "C" BOOL __stdcall _DllMainCRTStartup( HINSTANCE hinstDLL, DWORD fdwReaso
 
 #endif
 
-#ifndef WIN32
+#ifndef _MSC_VER
 #include "weather.moc"
 #endif
 
