@@ -28,11 +28,14 @@
 #include <windows.h>
 #include <windowsx.h>
 
+using namespace std;
+using namespace SIM;
+
 const unsigned short ABE_FLOAT   = (unsigned short)(-1);
 
 static WinDockPlugin *dock = NULL;
 
-Plugin *createWinDockPlugin(unsigned base, bool, const char *config)
+Plugin *createWinDockPlugin(unsigned base, bool, Buffer *config)
 {
     Plugin *plugin = new WinDockPlugin(base, config);
     return plugin;
@@ -344,7 +347,7 @@ static DataDef winDockData[] =
         { NULL, 0, 0, 0 }
     };
 
-WinDockPlugin::WinDockPlugin(unsigned base, const char *config)
+WinDockPlugin::WinDockPlugin(unsigned base, Buffer *config)
         : Plugin(base), EventReceiver(DefaultPriority - 1)
 {
     dock = this;
