@@ -22,19 +22,19 @@
 
 typedef struct WinDockData
 {
-    Data	AutoHide;
-    Data	State;
-    Data	Height;
-    Data	Width;
+    SIM::Data	AutoHide;
+    SIM::Data	State;
+    SIM::Data	Height;
+    SIM::Data	Width;
 } WinDocData;
 
 class QTimer;
 
-class WinDockPlugin : public QObject, public Plugin, public EventReceiver
+class WinDockPlugin : public QObject, public SIM::Plugin, public SIM::EventReceiver
 {
     Q_OBJECT
 public:
-    WinDockPlugin(unsigned, const char*);
+    WinDockPlugin(unsigned, Buffer*);
     virtual ~WinDockPlugin();
     PROP_BOOL(AutoHide);
     PROP_USHORT(State);
@@ -45,9 +45,9 @@ protected slots:
     void slotSetState();
     void slotAutoHide();
 protected:
-    virtual void *processEvent(Event*);
+    virtual void *processEvent(SIM::Event*);
     virtual bool eventFilter(QObject*, QEvent*);
-    virtual string getConfig();
+    virtual std::string getConfig();
     QWidget *getMainWindow();
     unsigned CmdAutoHide;
     bool m_bInit;
