@@ -44,6 +44,7 @@
 
 #include "html.h"
 #include "core.h"
+#include "icons.h"
 
 #ifdef WIN32
 #include <winsock.h>
@@ -62,7 +63,7 @@
 #include <qregexp.h>
 #include <qfile.h>
 
-const unsigned MessageYahooFile	= 0x700;
+const unsigned long MessageYahooFile	= 0x700;
 
 static char YAHOO_PACKET_SIGN[] = "YMSG";
 
@@ -90,7 +91,6 @@ static DataDef yahooUserData[] =
         { "Group", DATA_STRING, 1, 0 },
         { "", DATA_BOOL, 1, 0 },			// bChecked
         { "", DATA_BOOL, 1, 0 },			// bTyping
-        { "Encoding", DATA_STRING, 1, 0 },
         { NULL, 0, 0, 0 }
     };
 
@@ -112,7 +112,7 @@ const DataDef *YahooProtocol::userDataDef()
     return yahooUserData;
 }
 
-YahooClient::YahooClient(Protocol *protocol, const char *cfg)
+YahooClient::YahooClient(Protocol *protocol, Buffer *cfg)
         : TCPClient(protocol, cfg)
 {
     load_data(yahooClientData, &data, cfg);
@@ -228,7 +228,7 @@ void YahooClient::packet_ready()
         }
     }
     log_packet(m_socket->readBuffer, false, YahooPlugin::YahooPacket);
-    process_packet();
+    scan_packet();
     m_socket->readBuffer.init(20);
     m_socket->readBuffer.packetStart();
     m_bHeader = true;
@@ -289,13 +289,6 @@ void YahooClient::connect_ready()
     }
 }
 
-class Params : public list<PARAM>
-{
-public:
-    Params() {}
-    const char *operator[](unsigned id);
-};
-
 const char *Params::operator [](unsigned id)
 {
     for (iterator it = begin(); it != end(); ++it){
@@ -305,19 +298,40 @@ const char *Params::operator [](unsigned id)
     return NULL;
 }
 
-void YahooClient::process_packet()
+void YahooClient::scan_packet()
 {
     Params params;
-    Params::iterator it;
+    int param7_cnt = 0;
+
     for (;;){
         string key;
         string value;
-        if (!m_socket->readBuffer.scan("\xC0\x80", key) || !m_socket->readBuffer.scan("\xC0\x80", value))
+        if (!(m_socket->readBuffer.scan("\xC0\x80", key) &&
+                m_socket->readBuffer.scan("\xC0\x80", value)))
             break;
         unsigned key_id = atol(key.c_str());
-        params.push_back(PARAM(key_id, value));
         log(L_DEBUG, "Param: %u %s", key_id, value.c_str());
+        /* There can be multiple buddies in an YAHOO_SERVICE_IDDEACT and
+           YAHOO_SERVICE_LOGON paket ... */
+        if ((key_id == 7) && ((m_service == YAHOO_SERVICE_IDDEACT) ||
+                              (m_service == YAHOO_SERVICE_LOGON))) {
+            if (param7_cnt) {
+                /* process the last buddie */
+                process_packet(params);
+                params.clear();
+                param7_cnt = 0;
+            } else {
+                param7_cnt = 1;
+            }
+        }
+        params.push_back(PARAM(key_id, value));
     }
+    process_packet(params);
+}
+
+void YahooClient::process_packet(Params &params)
+{
+    log(L_DEBUG,"Service type: %02X",m_service);
     switch (m_service){
     case YAHOO_SERVICE_VERIFY:
         if (m_pkt_status != 1){
@@ -443,18 +457,40 @@ void YahooClient::process_packet()
             break;
         }
     case YAHOO_SERVICE_FILETRANSFER:
-        if (params[4] && params[27] && params[28] && params[14] && params[20])
+        /*
+
+        	params[14] - can be empty when no message is send with the file...
+            params[20] - url - just for filetransfer through website
+        */
+        if (params[4] && params[27] && params[28])
             process_file(params[4], params[27], params[28], params[14], params[20], params[11]);
+        else
+            /* file-url-message */
+            process_fileurl(params[4],params[14],params[20]);
         break;
+    case YAHOO_SERVICE_ADDBUDDY:
+        if (params[1] && params[7] && params[65])
+            log(L_DEBUG,"%s added %s to group %s",params[1],params[7],params[65]);
+        else
+            log(L_DEBUG,"Please send paket to developer!");
+        break;
+    case YAHOO_SERVICE_REMBUDDY:
+        if (params[1] && params[7] && params[65])
+            log(L_DEBUG,"%s removed %s from group %s",params[1],params[7],params[65]);
+        else
+            log(L_DEBUG,"Please send paket to developer!");
+        break;
+    case YAHOO_SERVICE_CONFINVITE:
+        log(L_WARN, "Conferencing currently not implemented!");
     default:
-        log(L_WARN, "Unknown service %X", m_service);
+        log(L_WARN, "Unknown service %02X", m_service);
     }
 }
 
 class TextParser
 {
 public:
-    TextParser(YahooClient *client, YahooUserData *data);
+    TextParser(YahooClient *client, Contact *contact);
     QString parse(const char *msg);
 
     class Tag
@@ -483,6 +519,7 @@ class FaceSizeParser : public HTMLParser
 protected:
     void addText(const char *str, unsigned size);
     unsigned m_state;
+    Contact *m_contact;
     QString color;
     QString face;
     QString size;
@@ -494,7 +531,6 @@ protected:
     void put_style();
     void push_tag(const QString &tag);
     void pop_tag(const QString &tag);
-    YahooUserData *m_data;
     YahooClient   *m_client;
     QString m_text;
 };
@@ -559,9 +595,9 @@ QString TextParser::Tag::close_tag() const
     return res;
 }
 
-TextParser::TextParser(YahooClient *client, YahooUserData *data)
+TextParser::TextParser(YahooClient *client, Contact *contact)
 {
-    m_data     = data;
+    m_contact  = contact;
     m_client   = client;
     m_bChanged = false;
     m_state    = 0;
@@ -732,8 +768,8 @@ void TextParser::addText(const char *str, unsigned s)
     if (s == 0)
         return;
     QString text;
-    if (m_data){
-        text = m_client->toUnicode(str, m_data);
+    if (m_contact){
+        text = getContacts()->toUnicode(m_contact, str, s);
     }else{
         text = QString::fromUtf8(str, s);
     }
@@ -779,12 +815,14 @@ void YahooClient::process_message(const char *id, const char *msg, const char *u
     bool bUtf = false;
     if (utf && atol(utf))
         bUtf = true;
-    YahooUserData *data = NULL;
-    if (utf == NULL)
-        data = &this->data.owner;
+    Contact *contact = NULL;
+    if (utf == NULL){
+        if (findContact(id, NULL, contact) == NULL)
+            contact = getContacts()->owner();
+    }
     Message *m = new Message(MessageGeneric);
     m->setFlags(MESSAGE_RICHTEXT);
-    TextParser parser(this, data);
+    TextParser parser(this, contact);
     m->setText(parser.parse(msg));
     messageReceived(m, id);
 }
@@ -823,7 +861,7 @@ void YahooClient::contact_rejected(const char *id, const char *message)
     messageReceived(msg, id);
 }
 
-static bool _cmp(const char *s1, const char *s2)
+static bool equal(const char *s1, const char *s2)
 {
     if (s1 == NULL)
         s1 = "";
@@ -853,7 +891,7 @@ void YahooClient::processStatus(unsigned short service, const char *id,
         state = YAHOO_STATUS_OFFLINE;
     if ((state != data->Status.value) ||
             ((state == YAHOO_STATUS_CUSTOM) &&
-             (((away != 0) != data->bAway.bValue) || _cmp(_msg, data->AwayMessage.ptr)))){
+             (((away != 0) != data->bAway.bValue) || equal(_msg, data->AwayMessage.ptr)))){
 
         unsigned long old_status = STATUS_UNKNOWN;
         unsigned style  = 0;
@@ -932,7 +970,7 @@ void YahooClient::setStatus(unsigned status)
         }
         return;
     }
-    unsigned yahoo_status = YAHOO_STATUS_OFFLINE;
+    unsigned long yahoo_status = YAHOO_STATUS_OFFLINE;
     switch (status){
     case STATUS_ONLINE:
         yahoo_status = YAHOO_STATUS_AVAILABLE;
@@ -950,7 +988,7 @@ void YahooClient::setStatus(unsigned status)
     ar.contact  = NULL;
     ar.status   = status;
     ar.receiver = this;
-    ar.param	= (void*)status;
+    ar.param	= (void*)(unsigned long)status;
     Event eAR(EventARRequest, &ar);
     eAR.process();
 }
@@ -958,15 +996,27 @@ void YahooClient::setStatus(unsigned status)
 void YahooClient::process_file(const char *id, const char *fileName, const char *fileSize, const char *msg, const char *url, const char *msgid)
 {
     YahooFileMessage *m = new YahooFileMessage;
-    m->setDescription(toUnicode(fileName, NULL));
+    m->setDescription(getContacts()->toUnicode(NULL, fileName));
     m->setSize(atol(fileSize));
-    m->setUrl(url);
-    m->setMsgText(msg);
-    if (id)
+    if (url)
+        m->setUrl(url);
+    if (msg)
+        m->setServerText(msg);
+    if (msgid)
         m->setMsgID(atol(msgid));
     messageReceived(m, id);
 }
 
+void YahooClient::process_fileurl(const char *id, const char *msg, const char *url)
+{
+    UrlMessage *m = new UrlMessage(MessageUrl);
+
+    if (msg)
+        m->setServerText(msg);
+    m->setUrl(url);
+    messageReceived(m, id);
+}
+
 void YahooClient::disconnected()
 {
     m_values.clear();
@@ -1006,10 +1056,15 @@ void YahooClient::disconnected()
     }
 }
 
-bool YahooClient::isMyData(clientData *&_data, Contact*&)
+bool YahooClient::isMyData(clientData *&_data, Contact*&contact)
 {
     if (_data->Sign.value != YAHOO_SIGN)
         return false;
+    YahooUserData *data = (YahooUserData*)_data;
+    YahooUserData *my_data = findContact(data->Login.ptr, NULL, contact);
+    if (!my_data){
+        contact = NULL;
+    }
     return true;
 }
 
@@ -1085,10 +1140,11 @@ void YahooClient::loadList(const char *str)
                 QString grpName;
                 if (contact->getGroup()){
                     Group *grp = getContacts()->group(contact->getGroup());
-                    grpName = grp->getName();
+                    if (grp)
+                        grpName = grp->getName();
                 }
-                if (grpName != toUnicode(grp.c_str(), NULL))
-                    moveBuddy(data, toUnicode(grp.c_str(), NULL));
+                if (grpName != getContacts()->toUnicode(NULL, grp.c_str()))
+                    moveBuddy(data, getContacts()->toUnicode(NULL, grp.c_str()));
                 data->bChecked.bValue = true;
             }
         }
@@ -1102,16 +1158,19 @@ void YahooClient::loadList(const char *str)
                 QString grpName;
                 if (contact->getGroup()){
                     Group *grp = getContacts()->group(contact->getGroup());
-                    grpName = grp->getName();
+                    if (grp)
+                        grpName = grp->getName();
                 }
-                if (grpName != toUnicode(data->Group.ptr, NULL))
+                if (grpName != getContacts()->toUnicode(NULL, data->Group.ptr))
                     moveBuddy(data, grpName.utf8());
             }
         }
         if ((*itl).type == LR_DELETE){
             YahooUserData data;
+            load_data(yahooUserData, &data, NULL);
             set_str(&data.Login.ptr, (*itl).name.c_str());
             removeBuddy(&data);
+            free_data(yahooUserData, &data);
         }
     }
     m_requests.clear();
@@ -1142,7 +1201,7 @@ void YahooClient::loadList(const char *str)
         delete *itr;
 }
 
-YahooUserData *YahooClient::findContact(const char *id, const char *grpname, Contact *&contact, bool bSend)
+YahooUserData *YahooClient::findContact(const char *id, const char *grpname, Contact *&contact, bool bSend, bool bJoin)
 {
     ContactList::ContactIterator it;
     while ((contact = ++it) != NULL){
@@ -1154,14 +1213,16 @@ YahooUserData *YahooClient::findContact(const char *id, const char *grpname, Con
         }
     }
     it.reset();
-    while ((contact = ++it) != NULL){
-        if (contact->getName() == id){
-            YahooUserData *data = (YahooUserData*)contact->clientData.createData(this);
-            set_str(&data->Login.ptr, id);
-            set_str(&data->Group.ptr, grpname);
-            Event e(EventContactChanged, contact);
-            e.process();
-            return data;
+    if (bJoin){
+        while ((contact = ++it) != NULL){
+            if (contact->getName() == id){
+                YahooUserData *data = (YahooUserData*)contact->clientData.createData(this);
+                set_str(&data->Login.ptr, id);
+                set_str(&data->Group.ptr, grpname);
+                Event e(EventContactChanged, contact);
+                e.process();
+                return data;
+            }
         }
     }
     if (grpname == NULL)
@@ -1170,11 +1231,11 @@ YahooUserData *YahooClient::findContact(const char *id, const char *grpname, Con
     if (*grpname){
         ContactList::GroupIterator it;
         while ((grp = ++it) != NULL)
-            if (grp->getName() == toUnicode(grpname, NULL))
+            if (grp->getName() == getContacts()->toUnicode(NULL, grpname))
                 break;
         if (grp == NULL){
             grp = getContacts()->group(0, true);
-            grp->setName(toUnicode(grpname, NULL));
+            grp->setName(getContacts()->toUnicode(NULL, grpname));
             Event e(EventGroupChanged, grp);
             e.process();
         }
@@ -1205,7 +1266,7 @@ void YahooClient::messageReceived(Message *msg, const char *id)
                 delete msg;
                 return;
             }
-            contact->setTemporary(CONTACT_TEMP);
+            contact->setFlags(CONTACT_TEMP);
             Event e(EventContactChanged, contact);
             e.process();
         }
@@ -1570,6 +1631,7 @@ void YahooParser::tag_start(const QString &tag, const list<QString> &options)
 {
     if (tag == "img"){
         QString src;
+        QString alt;
         for (list<QString>::const_iterator it = options.begin(); it != options.end(); ++it){
             QString name = (*it);
             ++it;
@@ -1578,16 +1640,17 @@ void YahooParser::tag_start(const QString &tag, const list<QString> &options)
                 src = value;
                 break;
             }
+            if (name == "alt"){
+                alt = value;
+                break;
+            }
         }
-        if (src.left(10) != "icon:smile")
+        list<string> smiles = getIcons()->getSmile(src.latin1());
+        if (smiles.empty()){
+            text(alt);
             return;
-        bool bOK;
-        unsigned nSmile = src.mid(10).toUInt(&bOK, 16);
-        if (!bOK)
-            return;
-        const smile *p = smiles(nSmile);
-        if (p)
-            text(p->paste);
+        }
+        text(QString::fromUtf8(smiles.front().c_str()));
         return;
     }
     if (tag == "br"){
@@ -1854,7 +1917,7 @@ void *YahooClient::processEvent(Event *e)
     }
     if (e->type() == EventTemplateExpanded){
         TemplateExpand *t = (TemplateExpand*)(e->param());
-        sendStatus(YAHOO_STATUS_CUSTOM, fromUnicode(t->tmpl, NULL).c_str());
+        sendStatus(YAHOO_STATUS_CUSTOM, getContacts()->fromUnicode(NULL, t->tmpl).c_str());
     }
     if (e->type() == EventMessageCancel){
         Message *msg = (Message*)(e->param());
@@ -1917,8 +1980,8 @@ void *YahooClient::processEvent(Event *e)
                     addParam(49, "FILEXFER");
                     addParam(1, getLogin().utf8());
                     addParam(13, "2");
-                    addParam(27, fromUnicode(msg->getDescription(), data).c_str());
-                    addParam(53, fromUnicode(msg->getDescription(), data).c_str());
+                    addParam(27, getContacts()->fromUnicode(contact, msg->getDescription()).c_str());
+                    addParam(53, getContacts()->fromUnicode(contact, msg->getDescription()).c_str());
                     addParam(11, number(msg->getMsgID()).c_str());
                     sendPacket(YAHOO_SERVICE_P2PFILEXFER);
                 }
@@ -1943,9 +2006,11 @@ void *YahooClient::processEvent(Event *e)
     return NULL;
 }
 
-QWidget *YahooClient::searchWindow()
+QWidget *YahooClient::searchWindow(QWidget *parent)
 {
-    return new YahooSearch(this);
+    if (getState() != Connected)
+        return NULL;
+    return new YahooSearch(this, parent);
 }
 
 void YahooClient::setInvisible(bool bState)
@@ -1966,7 +2031,8 @@ void YahooClient::sendStatus(unsigned long _status, const char *msg)
     unsigned long service = YAHOO_SERVICE_ISAWAY;
     if (msg)
         status = YAHOO_STATUS_CUSTOM;
-    if (data.owner.Status.value == YAHOO_STATUS_AVAILABLE)
+    /* data.owner.Status contains sim-status, not protocol-status! */
+    if (data.owner.Status.value == STATUS_ONLINE)
         service = YAHOO_SERVICE_ISBACK;
     addParam(10, number(status).c_str());
     if ((status == YAHOO_STATUS_CUSTOM) && msg) {
@@ -2000,7 +2066,9 @@ void YahooClient::sendFile(FileMessage *msg, QFile *file, YahooUserData *data, u
     url += number(port);
     url += "/";
     string nn;
-    string ff = fromUnicode(fn, data);
+    Contact *contact;
+    findContact(data->Login.ptr, NULL, contact);
+    string ff = getContacts()->fromUnicode(contact, fn);
     for (const char *p = ff.c_str(); *p; p++){
         if (((*p >= 'a') && (*p <='z')) || ((*p >= 'A') && (*p < 'Z')) || ((*p >= '0') && (*p <= '9')) || (*p == '.')){
             nn += *p;
@@ -2014,10 +2082,10 @@ void YahooClient::sendFile(FileMessage *msg, QFile *file, YahooUserData *data, u
     addParam(49, "FILEXFER");
     addParam(1, getLogin().utf8());
     addParam(13, "1");
-    addParam(27, fromUnicode(fn, data).c_str());
+    addParam(27, getContacts()->fromUnicode(contact, fn).c_str());
     addParam(28, number(file->size()).c_str());
     addParam(20, url.c_str());
-    addParam(14, fromUnicode(m, data).c_str());
+    addParam(14, getContacts()->fromUnicode(contact, m).c_str());
     addParam(53, nn.c_str());
     addParam(11, number(++m_ft_id).c_str());
     addParam(54, "MSG1.0");
@@ -2030,7 +2098,7 @@ void YahooClient::sendFile(FileMessage *msg, QFile *file, YahooUserData *data, u
     }
 }
 
-static Message *createYahooFile(const char *cfg)
+static Message *createYahooFile(Buffer *cfg)
 {
     return new YahooFileMessage(cfg);
 }
@@ -2049,13 +2117,12 @@ static MessageDef defYahooFile =
 
 static DataDef yahoMessageFile[] =
     {
-        { "ServerText", DATA_STRING, 1, 0 },
         { "", DATA_STRING, 1, 0 },				// URL
         { "", DATA_ULONG, 1, 0 },
         { NULL, 0, 0, 0 }
     };
 
-YahooFileMessage::YahooFileMessage(const char *cfg)
+YahooFileMessage::YahooFileMessage(Buffer *cfg)
         : FileMessage(MessageYahooFile, cfg)
 {
     load_data(yahoMessageFile, &data, cfg);
@@ -2071,14 +2138,6 @@ string YahooFileMessage::save()
     return save_data(yahoMessageFile, &data);
 }
 
-QString YahooFileMessage::getText() const
-{
-    const char *serverText = getMsgText();
-    if ((serverText == NULL) || (*serverText == 0))
-        return Message::getText();
-    return YahooClient::toUnicode(serverText, client(), contact());
-}
-
 void YahooPlugin::registerMessages()
 {
     Command cmd;
@@ -2096,87 +2155,6 @@ void YahooPlugin::unregisterMessages()
     eFile.process();
 }
 
-QTextCodec *YahooClient::getCodec(const char *encoding)
-{
-    if ((encoding == NULL) || (*encoding == 0))
-        encoding = data.owner.Encoding.ptr;
-    return _getCodec(encoding);
-}
-
-QTextCodec *YahooClient::_getCodec(const char *encoding)
-{
-    QTextCodec *codec = NULL;
-    if (encoding)
-        codec = QTextCodec::codecForName(encoding);
-    if (codec == NULL){
-        codec = QTextCodec::codecForLocale();
-        const ENCODING *e;
-        for (e = YahooPlugin::core->encodings; e->language; e++){
-            if (!strcmp(codec->name(), e->codec))
-                break;
-        }
-        if (e->language && !e->bMain){
-            for (e++; e->language; e++){
-                if (e->bMain){
-                    codec = QTextCodec::codecForName(e->codec);
-                    break;
-                }
-            }
-        }
-        if (codec == NULL)
-            codec= QTextCodec::codecForLocale();
-    }
-    return codec;
-}
-
-QString YahooClient::toUnicode(const char *str, YahooUserData *client_data)
-{
-    if ((str == NULL) || (*str == 0))
-        return QString();
-    if (client_data != NULL)
-    {
-        QTextCodec *codec = getCodec(client_data->Encoding.ptr);
-        return codec->toUnicode(str, strlen(str));
-    }
-    else
-    {
-        QTextCodec *codec = getCodec(NULL);
-        return codec->toUnicode(str, strlen(str));
-    }
-}
-
-string YahooClient::fromUnicode(const QString &str, YahooUserData *client_data)
-{
-    string res;
-    if (str.isEmpty())
-        return res;
-    QString s = str;
-    s.replace(QRegExp("\r"), "");
-    s.replace(QRegExp("\n"), "\r\n");
-    QTextCodec *codec = getCodec(client_data ? client_data->Encoding.ptr : NULL);
-    QCString cstr = codec->fromUnicode(s);
-    res = (const char*)cstr;
-    return res;
-}
-
-QString YahooClient::toUnicode(const char *serverText, const char *clientName, unsigned contactId)
-{
-    Contact *c = getContacts()->contact(contactId);
-    if (c){
-        void *data;
-        ClientDataIterator it(c->clientData);
-        while ((data = ++it) != NULL){
-            if (it.client()->dataName(data) != clientName)
-                continue;
-            QString res = static_cast<YahooClient*>(it.client())->toUnicode(serverText, (YahooUserData*)data);
-            return res.replace(QRegExp("\r"), "");
-        }
-    }
-    QTextCodec *codec = _getCodec(NULL);
-    QString res = codec->toUnicode(serverText, strlen(serverText));
-    return res.replace(QRegExp("\r"), "");
-}
-
 ListRequest *YahooClient::findRequest(const char *name)
 {
     for (list<ListRequest>::iterator it = m_requests.begin(); it != m_requests.end(); ++it){
@@ -2623,7 +2601,7 @@ void YahooFileTransfer::connect()
         m_notify->createFile(m_msg->getDescription(), 0xFFFFFFFF, false);
 }
 
-#ifndef WIN32
+#ifndef _MSC_VER
 #include "yahooclient.moc"
 #endif
 
